
/* ************************************************************************
*  File: actobj.c                                             Part of ACT *
*  Usage: Loading/saving chars, booting/resetting world, internal funcs   *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include "act.h"

extern char *item_types[];
extern char *extra_bits[];
extern char *wear_bits[];
extern char *apply_types[];

char buf[MAX_STRING_LENGTH], buf1[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];

/* read all objects from obj file; generate index and prototypes */
char *parse_object(FILE * obj_f, int nr, FILE *actfile)
{
  static char line[256];
  int t[10], j;

  sprintf(buf2, "object #%d", nr);

  /* *** string data *** */
  fread_string(obj_f, buf2, actfile);				/* name */
  fread_string(obj_f, buf2, actfile);				/* short desc */
  fread_string(obj_f, buf2, actfile);				/* long desc */
  fread_string(obj_f, buf2, actfile);				/* action desc */

  /* *** numeric data *** */
  if (!get_line(obj_f, line, actfile) ||
   sscanf(line, " %d %d %d", t, t + 1, t + 2) != 3) {
    fprintf(stderr, "Format error in first numeric line, %s\n", buf2);
    exit(1);
  }
  fprintf(actfile, "%s ", select_to_string(t[0], item_types));
  fprintf(actfile, "%s ", bitvector_to_string(t[1], extra_bits));
  fprintf(actfile, "%s\n", bitvector_to_string(t[2], wear_bits));

  if (!get_line(obj_f, line, actfile) ||
   sscanf(line, "%d %d %d %d", t, t + 1, t + 2, t + 3) != 4) {
    fprintf(stderr, "Format error in second numeric line, %s\n", buf2);
    exit(1);
  }
  fprintf(actfile, "%d %d %d %d\n", t[0], t[1], t[2], t[3]);

  if (!get_line(obj_f, line, actfile) ||
   sscanf(line, "%d %d %d", t, t + 1, t + 2) != 3) {
    fprintf(stderr, "Format error in third numeric line, %s\n", buf2);
    exit(1);
  }
  fprintf(actfile, "%d %d %d\n", t[0], t[1], t[2]);	/* Wt, Cost, Cost/day */

  /* *** extra descriptions and affect fields *** */

  strcat(buf2, ", after numeric constants (expecting E/A/#xxx)");
  j = 0;

  for (;;) {
    if (!get_line(obj_f, line, actfile)) {
      fprintf(stderr, "Format error in %s\n", buf2);
      exit(1);
    }
    switch (*line) {
    case 'E':
      fprintf(actfile, "%s\n", line);
      fread_string(obj_f, buf2, actfile);			/* Keywords */
      fread_string(obj_f, buf2, actfile);			/* Description */
      break;
    case 'A':
      fprintf(actfile, "%s\n", line);
      get_line(obj_f, line, actfile);
      sscanf(line, " %d %d ", t, t + 1);
      fprintf(actfile, "%s %d\n", select_to_string(t[0], apply_types), t[1]);
      j++;
      break;
    case '$':
    case '#':
      fprintf(actfile, "%s\n", line);
      return line;
      break;
    default:
      fprintf(stderr, "Format error in %s\n", buf2);
      exit(1);
      break;
    }
  }
}

