
/* ************************************************************************
*  File: main.c                                               Part of ACT *
*  Usage: Loading/saving chars, booting/resetting world, internal funcs   *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include "act.h"


char *act_mode[] = {
	"wld",
	"mob",
	"obj"
} ;


#define MODE_WLD	0
#define MODE_MOB	1
#define MODE_OBJ	2



void discrete_load(FILE * fl, int mode, char *fn, int actmode)
{
  int nr = -1, last = 0;
  char line[256];
  FILE *actfile;

  actfile = get_outputfile(fn, actmode);
  for (;;) {
    /*
     * we have to do special processing with the obj files because they have
     * no end-of-record marker :(
     */
    if (mode != MODE_OBJ || nr < 0)
      if (!get_line(fl, line, actfile)) {
        fprintf(stderr, "Format error after %s #%d (couldnt get_line)\n", act_mode[mode], nr);
        exit(1);
      } else
	fprintf(actfile, "%s\n", line);
    if (*line == '$') {
      fclose(actfile);
      return;
    }

    if (*line == '#') {
      last = nr;
      if (sscanf(line, "#%d", &nr) != 1) {
        fprintf(stderr, "Format error after %s #%d(room number munged)\n", act_mode[mode], last);
        exit(1);
      }
      if (nr >= 99999) {
	fprintf(actfile, "$~\n");
	fclose(actfile);
        return;
      } else
        switch (mode) {
        case MODE_WLD:
          parse_room(fl, nr, actfile);
          break;
        case MODE_MOB:
          parse_mobile(fl, nr, actfile);
          break;
        case MODE_OBJ:
          strcpy(line, parse_object(fl, nr, actfile));
          break;
        }
    } else {
      fprintf(stderr, "Format error in %s file near %s #%d\n",
              act_mode[mode], act_mode[mode], nr);
      fprintf(stderr, "Offending line: '%s'\n", line);
      exit(1);
    }
  }
}

  
void usage(char *arg)
{
  printf("\nUsage: convert filename...\n");
  exit(0);
}



void main(int argc, char *argv[])
{
  FILE *fp;
  int index, loop, actmode;

  if (argc < 2)
    usage(argv[0]);

  printf("Working on file %s...", argv[1]);
  if ((fp = fopen(argv[1], "rt")) == NULL)
    printf("File not found!");
  else {
    discrete_load(fp, MODE_WLD, argv[1], MODE_ACT);
    printf("Done!\n");
    fclose(fp);
  }
}

