/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "quest.h"
#include "boards.h"
#include "dg_scripts.h"
#include "olc.h"
#include "shop.h"


/* extern variables */
extern struct str_app_type str_app[];
extern struct room_data *world;
extern struct obj_data *obj_proto;
extern const char *drinks[];
extern int drink_aff[][3];
extern struct index_data *obj_index;
extern char *position_types[];
extern bool checkgetput(struct char_data *ch, struct obj_data *obj);
extern byte ProximityCheck(struct char_data *ch, struct obj_data *obj, struct char_data *target, int door);
extern int has_key(struct char_data *ch, int key);
extern void do_doorcmd(struct char_data *ch, struct obj_data *obj, int door, int scmd);
int find_eq_pos(struct char_data *ch, struct obj_data *obj, char *arg);
char perform_wear(struct char_data *ch, struct obj_data *obj, int where);

int have_money(struct char_data *ch, int amount, int type)
{
  int Amount = 0;
  struct obj_data *obj, *next_obj;

  for (obj = ch->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    if (CAN_SEE_OBJ(ch, obj)) {
      if (GET_OBJ_TYPE(obj) == ITEM_MONEY) {
	if (GET_OBJ_VAL(obj, 1) == type) {
	  Amount += GET_OBJ_VAL(obj, 0);
	}
      }
    }
  }
  if (Amount >= amount)
    return TRUE;
  else
    return FALSE;
}

struct obj_data *
 find_money(struct char_data *ch, int type, int king)
{
  struct obj_data *obj, *next_obj;

  for (obj = ch->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    if (CAN_SEE_OBJ(ch, obj)) {
      if (GET_OBJ_TYPE(obj) == ITEM_MONEY) {
	if (GET_OBJ_VAL(obj, 1) == type) {
	  if (king < 0 || (king >= 0 && GET_OBJ_VAL(obj, 2) == king)) {
	    return obj;
	  }
	}
      }
    }
  }
  return NULL;
}

void remove_money(struct char_data *ch, int amount, int type, int king)
{
  int Amount = 0, rest;
  struct obj_data *obj, *obj2, *next_obj, *tmp_obj;
  struct char_data *tmp_ch;

  Amount = amount;
  for (obj = ch->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    if (CAN_SEE_OBJ(ch, obj)) {
      if (GET_OBJ_TYPE(obj) == ITEM_MONEY) {
	if (GET_OBJ_VAL(obj, 1) == type && (king < 0 || (king >= 0 && GET_OBJ_VAL(obj, 2) == king))) {
	  if (Amount == GET_OBJ_VAL(obj, 0)) {
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	    }
	    extract_obj(obj);
	    return;
	  } else if (Amount < GET_OBJ_VAL(obj, 0)) {
	    rest = GET_OBJ_VAL(obj, 0) - Amount;
	    obj2 = create_money(rest, type, GET_OBJ_VAL(obj, 2));
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	      obj_to_char(obj2, ch);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	      obj_to_obj(obj2, tmp_obj);
	    }
	    extract_obj(obj);
	    return;
	  } else {
	    Amount -= GET_OBJ_VAL(obj, 0);
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	    }
	    extract_obj(obj);
	  }
	}
      }
    }
  }
}


void remove_and_drop_money(struct char_data *ch, int amount, int type)
{
  int Amount = 0, rest, King;
  struct obj_data *obj, *obj2, *next_obj, *tmp_obj;
  struct char_data *tmp_ch;

  Amount = amount;
  for (obj = ch->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    if (CAN_SEE_OBJ(ch, obj)) {
      if (GET_OBJ_TYPE(obj) == ITEM_MONEY) {
	if (GET_OBJ_VAL(obj, 1) == type) {
	  if (Amount == GET_OBJ_VAL(obj, 0)) {
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	    }
	    put_obj_to_room(obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));

	    if (!drop_wtrigger(obj, ch)) {
	      extract_obj(obj);
	      return;
	    }
	    return;
	  } else if (Amount < GET_OBJ_VAL(obj, 0)) {
	    King = GET_OBJ_VAL(obj, 2);
	    rest = GET_OBJ_VAL(obj, 0) - Amount;
	    obj2 = create_money(rest, type, King);
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	      obj_to_char(obj2, ch);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	      obj_to_obj(obj2, tmp_obj);
	    }
	    extract_obj(obj);
	    obj = create_money(Amount, type, King);
	    put_obj_to_room(obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
	    return;
	  } else {
	    Amount -= GET_OBJ_VAL(obj, 0);
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	    }
	    put_obj_to_room(obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
	  }
	}
      }
    }
  }
}

void remove_and_destroy_money(struct char_data *ch, int amount, int type)
{
  int Amount = 0, rest, King;
  struct obj_data *obj, *obj2, *next_obj, *tmp_obj;
  struct char_data *tmp_ch;

  Amount = amount;
  for (obj = ch->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    if (CAN_SEE_OBJ(ch, obj)) {
      if (GET_OBJ_TYPE(obj) == ITEM_MONEY) {
	if (GET_OBJ_VAL(obj, 1) == type) {
	  if (Amount == GET_OBJ_VAL(obj, 0)) {
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	      extract_obj(obj);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	      extract_obj(obj);
	    }
	    return;
	  } else if (Amount < GET_OBJ_VAL(obj, 0)) {
	    King = GET_OBJ_VAL(obj, 2);
	    rest = GET_OBJ_VAL(obj, 0) - Amount;
	    obj2 = create_money(rest, type, King);
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	      obj_to_char(obj2, ch);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	      obj_to_obj(obj2, tmp_obj);
	    }
	    extract_obj(obj);
	    return;
	  } else {
	    Amount -= GET_OBJ_VAL(obj, 0);
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	      extract_obj(obj);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	      extract_obj(obj);
	    }
	  }
	}
      }
    }
  }
}
void remove_and_give_money(struct char_data *ch, struct char_data *vict, int amount, int type)
{
  int Amount = 0, rest, King;
  struct obj_data *obj, *obj2, *next_obj, *tmp_obj;
  struct char_data *tmp_ch;

  Amount = amount;
  for (obj = ch->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    if (CAN_SEE_OBJ(ch, obj)) {
      if (GET_OBJ_TYPE(obj) == ITEM_MONEY) {
	if (GET_OBJ_VAL(obj, 1) == type) {
	  if (Amount == GET_OBJ_VAL(obj, 0)) {
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	    }
	    obj_to_char(obj, vict);
	    return;
	  } else if (Amount < GET_OBJ_VAL(obj, 0)) {
	    King = GET_OBJ_VAL(obj, 2);
	    rest = GET_OBJ_VAL(obj, 0) - Amount;
	    obj2 = create_money(rest, type, King);
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	      obj_to_char(obj2, ch);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	      obj_to_obj(obj2, tmp_obj);
	    }
	    extract_obj(obj);
	    obj = create_money(Amount, type, King);
	    obj_to_char(obj, vict);
	    return;
	  } else {
	    Amount -= GET_OBJ_VAL(obj, 0);
	    if ((tmp_ch = obj->carried_by)) {
	      obj_from_char(obj);
	    } else if ((tmp_obj = obj->in_obj)) {
	      obj_from_obj(obj);
	    }
	    obj_to_char(obj, vict);
	  }
	}
      }
    }
  }
}

void create_and_give_money_to(struct char_data *ch, struct char_data *vict,
			      int amount, int type)
{
  int Amount = 0, rest, King;
  struct obj_data *obj, *obj2, *next_obj, *tmp_obj;
  struct char_data *tmp_ch;

  Amount = amount;
  King = number(0, NUM_KINGDOMS);
  /* weigh it to generic coins */
  if (number(0, 3))
    King = 0;

  for (obj = ch->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    if (CAN_SEE_OBJ(ch, obj)) {
      if (GET_OBJ_TYPE(obj) == ITEM_MONEY) {
	if (GET_OBJ_VAL(obj, 1) == type && King == GET_OBJ_VAL(obj, 2)) {
	  rest = GET_OBJ_VAL(obj, 0) + Amount;
	  obj2 = create_money(rest, type, King);
	  if ((tmp_ch = obj->carried_by)) {
	    obj_from_char(obj);
	    obj_to_char(obj2, vict);
	  } else if ((tmp_obj = obj->in_obj)) {
	    obj_from_obj(obj);
	    obj_to_obj(obj2, tmp_obj);
	  }
	  extract_obj(obj);
	  return;
	}
      }
    }
  }
  /* they didnt have more coins to add too */
  obj2 = create_money(Amount, type, King);
  obj_to_char(obj2, vict);
}

void create_and_give_money(struct char_data *vict, int amount, int type)
{
  int Amount = 0, rest, King;
  struct obj_data *obj, *obj2, *next_obj, *tmp_obj;
  struct char_data *tmp_ch;

  Amount = amount;
  King = number(0, NUM_KINGDOMS);
  /* weigh it to generic coins */
  if (number(0, 3))
    King = 0;

  for (obj = vict->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    if (CAN_SEE_OBJ(vict, obj)) {
      if (GET_OBJ_TYPE(obj) == ITEM_MONEY) {
	if (GET_OBJ_VAL(obj, 1) == type && King == GET_OBJ_VAL(obj, 2)) {
	  rest = GET_OBJ_VAL(obj, 0) + Amount;
	  obj2 = create_money(rest, type, King);
	  if ((tmp_ch = obj->carried_by)) {
	    obj_from_char(obj);
	    obj_to_char(obj2, vict);
	  } else if ((tmp_obj = obj->in_obj)) {
	    obj_from_obj(obj);
	    obj_to_obj(obj2, tmp_obj);
	  }
	  extract_obj(obj);
	  return;
	}
      }
    }
  }
  /* they didnt have more coins to add too */
  obj2 = create_money(Amount, type, King);
  obj_to_char(obj2, vict);
}

void perform_put(struct char_data *ch, struct obj_data *obj,
		 struct obj_data *cont)
{
  if (!drop_otrigger(obj, ch))
    return;
  else if (checkgetput(ch, obj))
    return;

  if (GET_OBJ_WEIGHT(cont) + GET_OBJ_WEIGHT(obj) > GET_OBJ_VAL(cont, 0))
    act("$p won't fit in $P.", FALSE, ch, obj, cont, TO_CHAR);
  else if (IS_SCABBARD(cont) &&
	   (!IS_SCABBARD_WEAPON(obj, cont) || (cont->contains != NULL)
	    || (GET_OBJ_SIZE(cont) < GET_OBJ_SIZE(obj))))
    act("$p won't fit in $P.", FALSE, ch, obj, cont, TO_CHAR);
  else {
    obj_from_char(obj);
    obj_to_obj(obj, cont);
    act("You put $p in $P.", FALSE, ch, obj, cont, TO_CHAR);
    act("$n puts $p in $P.", TRUE, ch, obj, cont, TO_PROXIMITY);
  }
}


/* The following put modes are supported by the code below:

   1) put <object> <container>
   2) put all.<object> <container>
   3) put all <container>

   <container> must be in inventory or on ground.
   Not anymore. Can now be in equipment. -Boccob

   all objects to be put into container must be in inventory.
   Can also be held. - Boccob
 */

ACMD(do_put)
{
  char arg1[MAX_INPUT_LENGTH];
  char arg2[MAX_INPUT_LENGTH];
  struct obj_data *obj, *next_obj, *cont;
  struct char_data *tmp_char;
  int obj_dotmode, cont_dotmode, found = 0, put_num, amount, i = 0;

  /* The command may not work, but it's easier to just save now, rather
     than in each of the various subfunctions */
  if (IS_PC(ch))
    Crash_crashsave(ch);

  two_arguments(argument, arg1, arg2);
  obj_dotmode = find_all_dots(arg1);
  cont_dotmode = find_all_dots(arg2);

  if (!*arg1)
    send_to_char("Put what in what?\r\n", ch);
  else if (cont_dotmode != FIND_INDIV)
    send_to_char("You can only put things into one container at a time.\r\n", ch);
#if 1
/****/
  else if (is_number(arg1)) {
    put_num = atoi(arg1);
    argument = two_arguments(argument, arg1, arg2);
    one_argument(argument, arg1);
    if (!*arg2)
      send_to_char("Put what in what?\r\n", ch);
    else if (!*arg1) {
      sprintf(buf, "What do you want to put %s in?\r\n", arg2);
      send_to_char(buf, ch);
    } else {
      generic_find(arg1, FIND_OBJ_INV | FIND_OBJ_EQUIP | FIND_OBJ_ROOM,
		   ch, &tmp_char, &cont);
      if (!cont) {
	sprintf(buf, "You don't see %s %s here.\r\n", AN(arg1), arg1);
	send_to_char(buf, ch);
      } else if (!IS_CONTAINER(cont))
	act("$p is not a container.", FALSE, ch, cont, 0, TO_CHAR);
/*
   else if (IS_SET(GET_OBJ_VAL(cont, 1), CONT_CLOSED) && (GET_OBJ_TYPE(cont) == ITEM_CONTAINER))
   send_to_char("You'd better open it first!\r\n", ch);
 */
      else {
	for (amount = 0; amount < put_num; amount++) {
	  if (!(obj = get_obj_in_list_vis(ch, arg2, ch->carrying))) {
	    if (!(obj = get_held_object_in_equip_vis(ch, arg, ch->equipment, &i))) {
	      sprintf(buf, "You aren't carrying %s %s.\r\n", AN(arg2), arg2);
	      send_to_char(buf, ch);
	      break;
	    }
	    obj_to_char(unequip_char(ch, i), ch);
	  }
	  if (obj == cont) {
	    send_to_char("You attempt to fold it into itself, but fail.\r\n", ch);
	    amount = put_num;
	  } else
	    perform_put(ch, obj, cont);
	}
      }
    }
  }
/****/
#endif
  else if (!*arg2) {
    sprintf(buf, "What do you want to put %s in?\r\n",
	    ((obj_dotmode == FIND_INDIV) ? "it" : "them"));
    send_to_char(buf, ch);
  } else {
    generic_find(arg2, FIND_OBJ_INV | FIND_OBJ_EQUIP | FIND_OBJ_ROOM, ch, &tmp_char, &cont);
    if (!cont) {
      sprintf(buf, "You don't see %s %s here.\r\n", AN(arg2), arg2);
      send_to_char(buf, ch);
    } else if (!IS_CONTAINER(cont))
      act("$p is not a container.", FALSE, ch, cont, 0, TO_CHAR);
    else if (IS_SET(GET_OBJ_VAL(cont, 1), CONT_CLOSED) &&
	     GET_OBJ_TYPE(cont) == ITEM_CONTAINER)
      send_to_char("You'd better open it first!\r\n", ch);
    else {
      if (obj_dotmode == FIND_INDIV) {	/* put <obj> <container> */
	if (!(obj = get_obj_in_list_vis(ch, arg1, ch->carrying))) {
	  if (!(obj = get_held_object_in_equip_vis(ch, arg1, ch->equipment, &i))) {
	    sprintf(buf, "You aren't carrying %s %s.\r\n", AN(arg1), arg1);
	    send_to_char(buf, ch);
	    return;
	  }
	  obj_to_char(unequip_char(ch, i), ch);
	}
	if (obj == cont)
	  send_to_char("You attempt to fold it into itself, but fail.\r\n", ch);
	else
	  perform_put(ch, obj, cont);
      } else {
	for (obj = ch->carrying; obj; obj = next_obj) {
	  next_obj = obj->next_content;
	  if (obj != cont && CAN_SEE_OBJ(ch, obj) &&
	      (obj_dotmode == FIND_ALL || isname(arg1, obj->name))) {
	    found = 1;
	    perform_put(ch, obj, cont);
	  }
	}
	if (!found) {
	  if (obj_dotmode == FIND_ALL)
	    send_to_char("You don't seem to have anything to put in it.\r\n", ch);
	  else {
	    sprintf(buf, "You don't seem to have any %ss.\r\n", arg1);
	    send_to_char(buf, ch);
	  }
	}
      }
    }
  }
}



int can_take_obj(struct char_data *ch, struct obj_data *obj)
{
  struct obj_data *obj2 = GET_EQ(ch, WEAR_WIELD);

  if (obj2 == NULL)
    obj2 = GET_EQ(ch, WEAR_HOLD);
  if (obj2 == NULL)
    obj2 = GET_EQ(ch, WEAR_WIELD2);
  if (obj2 == NULL)
    obj2 = GET_EQ(ch, WEAR_HOLD2);

  if ((IS_CARRYING_N(ch) >= CAN_CARRY_N(ch)) ||
      (obj2 && (IS_OBJ_STAT(obj2, ITEM_TWOHANDS) ||	/*Two handed */
	    IS_SET(obj2->obj_flags.extra2_flags, ITEM2_REMOVE_2HAND))) ||
      ((GET_EQ(ch, WEAR_HOLD) || GET_EQ(ch, WEAR_WIELD))	/*main hand */
       &&(GET_EQ(ch, WEAR_HOLD2) || GET_EQ(ch, WEAR_WIELD2)
	  || GET_EQ(ch, WEAR_SHIELD)) /*off hand */ )) {
    act("$p: you can't carry that many items.", FALSE, ch, obj, 0, TO_CHAR);
    return 0;
  } else if ((IS_CARRYING_W(ch) + GET_OBJ_WEIGHT(obj)) > CAN_CARRY_W(ch)) {
    act("$p: you can't carry that much weight.", FALSE, ch, obj, 0, TO_CHAR);
    return 0;
  } else if (!(CAN_WEAR(obj, ITEM_TAKE))) {
    act("$p: you can't take that!", FALSE, ch, obj, 0, TO_CHAR);
    return 0;
  } else if ((GET_OBJ_TYPE(obj) == ITEM_FURNITURE) && (obj->people) &&
	     (GET_LEVEL(ch) < LVL_GRGOD)) {
    act("$p: you can't take that, it's occupied!", FALSE, ch, obj, 0, TO_CHAR);
    return 0;
  } else if (obj->in_obj &&
	   ((obj->in_obj->carried_by && obj->in_obj->carried_by == ch) ||
	    (obj->in_obj->worn_by && obj->in_obj->worn_by == ch))) {
    return 1;
  } else if (GET_X_CORD(ch) != GET_OBJ_X_CORD(obj->in_obj ? obj->in_obj : obj) ||
     GET_Y_CORD(ch) != GET_OBJ_Y_CORD(obj->in_obj ? obj->in_obj : obj) ||
     GET_Z_CORD(ch) != GET_OBJ_Z_CORD(obj->in_obj ? obj->in_obj : obj)) {
    act("$p: you're not close enough to take that!", FALSE, ch, obj, 0, TO_CHAR);
    return 0;
  }
  return 1;
}


void get_check_money(struct char_data *ch, struct obj_data *obj)
{
  struct obj_data *tmp_obj, *obj2;
  struct char_data *tmp_ch;
  int Amount = 0, Type, King;

  if ((GET_OBJ_TYPE(obj) == ITEM_MONEY) && (GET_OBJ_VAL(obj, 0) > 0)) {
    Amount = GET_OBJ_VAL(obj, 0);
    Type = GET_OBJ_VAL(obj, 1);
    King = GET_OBJ_VAL(obj, 2);

    if ((tmp_obj = find_money(ch, GET_OBJ_VAL(obj, 1), King)) != NULL) {
      obj2 = create_money(Amount + GET_OBJ_VAL(tmp_obj, 1), Type, King);
      if ((tmp_ch = tmp_obj->carried_by)) {
	obj_from_char(tmp_obj);
	obj_to_char(obj2, ch);
      } else if ((tmp_obj = obj->in_obj)) {
	obj_from_obj(tmp_obj);
	obj_to_obj(obj2, tmp_obj);
      }
      extract_obj(obj);
    }
    if (GET_OBJ_VAL(obj, 0) > 1) {
      sprintf(buf, "There were %d coins.\r\n", GET_OBJ_VAL(obj, 0));
      send_to_char(buf, ch);
    }
  }
}


void perform_get_from_container(struct char_data *ch, struct obj_data *obj,
				struct obj_data *cont, int mode)
{
  struct obj_data *obj2 = GET_EQ(ch, WEAR_WIELD);

  if (obj2 == NULL)
    obj2 = GET_EQ(ch, WEAR_HOLD);
  if (obj2 == NULL)
    obj2 = GET_EQ(ch, WEAR_WIELD2);
  if (obj2 == NULL)
    obj2 = GET_EQ(ch, WEAR_HOLD2);

  if (checkgetput(ch, obj))
    return;

  if (mode == FIND_OBJ_INV || can_take_obj(ch, obj)) {
    if ((IS_CARRYING_N(ch) >= CAN_CARRY_N(ch)) ||
	(obj2 && (IS_OBJ_STAT(obj2, ITEM_TWOHANDS) ||	/*Two handed */
	    IS_SET(obj2->obj_flags.extra2_flags, ITEM2_REMOVE_2HAND))) ||
	((GET_EQ(ch, WEAR_HOLD) || GET_EQ(ch, WEAR_WIELD))	/*main hand */
	 &&(GET_EQ(ch, WEAR_HOLD2) || GET_EQ(ch, WEAR_WIELD2)
	    || GET_EQ(ch, WEAR_SHIELD)) /*off hand */ )) {
      act("$p: you can't hold any more items.", FALSE, ch, obj, 0, TO_CHAR);
    } else if (get_otrigger(obj, ch)) {
#if 0
      obj_from_obj(obj);
      act("You get $p from $P.", FALSE, ch, obj, cont, TO_CHAR);
      act("$n gets $p from $P.", TRUE, ch, obj, cont, TO_PROXIMITY);
      if ((where = find_eq_pos(ch, obj, 0)) >= 0) {
	if (where == WEAR_HOLD || where == WEAR_HOLD2)
	  perform_wear(ch, obj, where);
	else if (CAN_WEAR(obj, ITEM_TAKE)) {
	  perform_wear(ch, obj, WEAR_HOLD);
	} else
	  obj_to_char(obj, ch);
      } else
	obj_to_char(obj, ch);
#endif
      if (perform_wear(ch, obj, WEAR_HOLD)) {
	obj_from_obj(obj);
	act("You get $p from $P.", FALSE, ch, obj, cont, TO_CHAR);
	act("$n gets $p from $P.", TRUE, ch, obj, cont, TO_PROXIMITY);
      }
    }
  }
}


void get_from_container(struct char_data *ch, struct obj_data *cont,
			char *arg, int mode)
{
  struct obj_data *obj, *next_obj;
  int obj_dotmode, found = 0, was_closed = FALSE, was_locked = FALSE;
  int keynum = -1, can_open = FALSE;

  obj_dotmode = find_all_dots(arg);

  if ((cont->worn_by && cont->worn_by == ch) ||
      (cont->carried_by && cont->carried_by == ch))
    can_open = TRUE;

  if (IS_SET(GET_OBJ_VAL(cont, 1), CONT_CLOSED) && GET_OBJ_TYPE(cont) == ITEM_CONTAINER) {
    if ((cont->carried_by && cont->carried_by == ch) ||
	(cont->worn_by && cont->worn_by == ch)) {
      was_closed = TRUE;
      if (IS_SET(GET_OBJ_VAL(cont, 1), CONT_LOCKED))
	was_locked = TRUE;

      if (!can_open && was_closed) {
	act("$p appears to be closed.", FALSE, ch, cont, 0, TO_CHAR);
	return;
      } else if (!can_open && was_locked) {
	act("$p appears to be locked.", FALSE, ch, cont, 0, TO_CHAR);
	return;
      }
      if (was_locked) {
	keynum = DOOR_KEY(ch->in_room, cont, -1);
	if (!has_key(ch, keynum) && !IS_TRUSTED(ch)) {
	  act("Your $p is closed, locked, and you haven't a key handy.",
	      FALSE, ch, cont, 0, TO_CHAR);
	  return;
	} else
	  do_doorcmd(ch, cont, -1, SCMD_UNLOCK);
      }
      if (was_closed) {
	if (!(DOOR_IS_OPENABLE(ch->in_room, cont, -1))) {
	  act("$p is closed, and you can't seem to open it!", FALSE, ch, cont, 0, TO_CHAR);
	  return;
	} else
	  do_doorcmd(ch, cont, -1, SCMD_OPEN);
      }
    } else {
      act("$p appears to be closed.", FALSE, ch, cont, 0, TO_CHAR);
      return;
    }
  }
  if (obj_dotmode == FIND_INDIV) {
    if (!(obj = get_obj_in_list(arg, cont->contains))) {
      sprintf(buf, "There doesn't seem to be %s %s in $p.", AN(arg), arg);
      act(buf, FALSE, ch, cont, 0, TO_CHAR);
    } else
      perform_get_from_container(ch, obj, cont, mode);
  } else {
    if (obj_dotmode == FIND_ALLDOT && !*arg) {
      send_to_char("Get all of what?\r\n", ch);
      return;
    }
    for (obj = cont->contains; obj; obj = next_obj) {
      next_obj = obj->next_content;
      if (CAN_SEE_OBJ(ch, obj) &&
	  (obj_dotmode == FIND_ALL || isname(arg, obj->name))) {
	found = 1;
	perform_get_from_container(ch, obj, cont, mode);
      }
    }
    if (!found) {
      if (obj_dotmode == FIND_ALL)
	act("$p seems to be empty.", FALSE, ch, cont, 0, TO_CHAR);
      else {
	sprintf(buf, "You can't seem to find any %ss in $p.", arg);
	act(buf, FALSE, ch, cont, 0, TO_CHAR);
      }
    }
  }
  if (was_closed && cont)
    do_doorcmd(ch, cont, -1, SCMD_CLOSE);
  if (was_locked && cont)
    do_doorcmd(ch, cont, -1, SCMD_LOCK);
}


int perform_get_from_room(struct char_data *ch, struct obj_data *obj)
{
  int where = -1;

  if (checkgetput(ch, obj))
    return 1;

  if (can_take_obj(ch, obj) && get_otrigger(obj, ch)) {
    obj_from_room(obj);
    act("You pick up $p.", FALSE, ch, obj, 0, TO_CHAR);
    act("$n picks up $p.", TRUE, ch, obj, 0, TO_PROXIMITY);
    if ((where = find_eq_pos(ch, obj, 0)) >= 0) {
      if (where == WEAR_HOLD || where == WEAR_HOLD2)
	perform_wear(ch, obj, where);
      else if (CAN_WEAR(obj, ITEM_TAKE)) {
	perform_wear(ch, obj, WEAR_HOLD);
      } else
	obj_to_char(obj, ch);
    } else
      obj_to_char(obj, ch);
    return 1;
  }
  return 0;
}


void get_from_room(struct char_data *ch, char *arg)
{
  struct obj_data *obj, *next_obj;
  int dotmode, found = 0;

  dotmode = find_all_dots(arg);

  if (dotmode == FIND_INDIV) {
    if (!(obj = get_obj_in_list_vis(ch, arg, world[ch->in_room].contents))) {
      sprintf(buf, "You don't see %s %s here.\r\n", AN(arg), arg);
      send_to_char(buf, ch);
    } else {
      if (!ProximityCheck(ch, obj, NULL, -1)) {
	if (CAN_SEE_OBJ(ch, obj)) {
	  sprintf(buf, "%s %s is too far to get.\r\n", AN(arg), arg);
	  send_to_char(buf, ch);
	}
	return;
      }
      perform_get_from_room(ch, obj);
    }
  } else {
    if (dotmode == FIND_ALLDOT && !*arg) {
      send_to_char("Get all of what?\r\n", ch);
      return;
    }
    for (obj = world[ch->in_room].contents; obj; obj = next_obj) {
      next_obj = obj->next_content;
      if (CAN_SEE_OBJ(ch, obj) &&
	  (dotmode == FIND_ALL || isname(arg, obj->name))) {
	found = 1;
	if (!ProximityCheck(ch, obj, NULL, -1)) {
	  if (CAN_SEE_OBJ(ch, obj) && str_cmp(arg, "all")) {
	    sprintf(buf, "%s %s is too far to get.\r\n", AN(arg), arg);
	    send_to_char(buf, ch);
	  }
	  continue;
	}
	perform_get_from_room(ch, obj);
      }
    }
    if (!found) {
      if (dotmode == FIND_ALL)
	send_to_char("There doesn't seem to be anything here.\r\n", ch);
      else {
	sprintf(buf, "You don't see any %ss here.\r\n", arg);
	send_to_char(buf, ch);
      }
    }
  }
}



ACMD(do_get)
{
  char arg1[MAX_INPUT_LENGTH];
  char arg2[MAX_INPUT_LENGTH];

  int cont_dotmode, found = 0, mode, i;
  struct obj_data *cont;
  struct char_data *tmp_char;

  /* The command may not work, but it's easier to just save now, rather
     than in each of the various subfunctions */
  if (IS_PC(ch))
    Crash_crashsave(ch);

  two_arguments(argument, arg1, arg2);

  if (IS_CARRYING_N(ch) >= CAN_CARRY_N(ch))
    send_to_char("Your arms are already full!\r\n", ch);
  else if (!*arg1)
    send_to_char("Get what?\r\n", ch);
  else if (!*arg2)
    get_from_room(ch, arg1);
  else {
    cont_dotmode = find_all_dots(arg2);
    if (cont_dotmode == FIND_INDIV) {
      mode = generic_find(arg2, FIND_OBJ_INV | FIND_OBJ_EQUIP | FIND_OBJ_ROOM,
			  ch, &tmp_char, &cont);
      if (!cont) {
	sprintf(buf, "You don't have %s %s.\r\n", AN(arg2), arg2);
	send_to_char(buf, ch);
      } else if (!IS_CONTAINER(cont))
	act("$p is not a container.", FALSE, ch, cont, 0, TO_CHAR);
      else
	get_from_container(ch, cont, arg1, mode);
    } else {
      if (cont_dotmode == FIND_ALLDOT && !*arg2) {
	send_to_char("Get from all of what?\r\n", ch);
	return;
      }
      for (cont = ch->carrying; cont; cont = cont->next_content)
	if (CAN_SEE_OBJ(ch, cont) &&
	    (cont_dotmode == FIND_ALL || isname(arg2, cont->name))) {
	  if (IS_CONTAINER(cont)) {
	    found = 1;
	    get_from_container(ch, cont, arg1, FIND_OBJ_INV);
	  } else if (cont_dotmode == FIND_ALLDOT) {
	    found = 1;
	    act("$p is not a container.", FALSE, ch, cont, 0, TO_CHAR);
	  }
	}
      for (i = 0; i < NUM_WEARS; i++) {
	if (GET_EQ(ch, i))
	  cont = GET_EQ(ch, i);
	else
	  continue;

	if (CAN_SEE_OBJ(ch, cont) &&
	    (cont_dotmode == FIND_ALL || isname(arg2, cont->name))) {
	  if (IS_CONTAINER(cont)) {
	    found = 1;
	    get_from_container(ch, cont, arg1, FIND_OBJ_INV);
	  } else if (cont_dotmode == FIND_ALLDOT) {
	    found = 1;
	    act("$p is not a container.", FALSE, ch, cont, 0, TO_CHAR);
	  }
	}
      }
      for (cont = world[ch->in_room].contents; cont; cont = cont->next_content)
	if (CAN_SEE_OBJ(ch, cont) &&
	    (cont_dotmode == FIND_ALL || isname(arg2, cont->name))) {
	  if (IS_CONTAINER(cont)) {
	    get_from_container(ch, cont, arg1, FIND_OBJ_ROOM);
	    found = 1;
	  } else if (cont_dotmode == FIND_ALLDOT) {
	    act("$p is not a container.", FALSE, ch, cont, 0, TO_CHAR);
	    found = 1;
	  }
	}
      if (!found) {
	if (cont_dotmode == FIND_ALL)
	  send_to_char("You can't seem to find any containers.\r\n", ch);
	else {
	  sprintf(buf, "You can't seem to find any %ss here.\r\n", arg2);
	  send_to_char(buf, ch);
	}
      }
    }
  }
}

void perform_drop_money(struct char_data *ch, int amount,
			int type)
{
  if (amount <= 0)
    send_to_char("Heh heh heh.. we are jolly funny today, eh?\r\n", ch);
  else if (!have_money(ch, amount, type))
    send_to_char("You don't have that many coins!\r\n", ch);
  else {
    WAIT_STATE(ch, PULSE_VIOLENCE);	/* to prevent coin-bombing */
    remove_and_drop_money(ch, amount, type);
    send_to_char("You drop some money.\r\n", ch);
    sprintf(buf, "$n drops %s.", money_desc(amount, type, 0));
    act(buf, TRUE, ch, 0, 0, TO_PROXIMITY);
  }
}


int perform_drop(struct char_data *ch, struct obj_data *obj,
		 byte mode, char *sname, sh_int RDR)
{
  int value;

  if (!drop_otrigger(obj, ch))
    return 0;
  if ((mode == SCMD_DROP) && !drop_wtrigger(obj, ch))
    return 0;

  if (IS_OBJ_STAT(obj, ITEM_NODROP)) {
    sprintf(buf, "You can't %s $p, it must be CURSED!", sname);
    act(buf, FALSE, ch, obj, 0, TO_CHAR);
    return 0;
  }
  sprintf(buf, "You %s $p.", sname);
  act(buf, FALSE, ch, obj, 0, TO_CHAR);
  sprintf(buf, "$n %ss $p.", sname);
  act(buf, TRUE, ch, obj, 0, TO_PROXIMITY);
  obj_from_char(obj);

  switch (mode) {
  case SCMD_DROP:
    put_obj_to_room(obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
    return 0;
    break;
  case SCMD_JUNK:
    value = MAX(1, MIN(200, GET_OBJ_COST(obj) >> 4));
    extract_obj(obj);
    return value;
    break;
  default:
    log("SYSERR: Incorrect argument passed to perform_drop");
    break;
  }

  return 0;
}



ACMD(do_drop)
{
  struct obj_data *obj, *next_obj;
  sh_int RDR = 0;
  byte mode = SCMD_DROP;
  int dotmode, amount = 0, i = 0;
  char *sname;

  switch (subcmd) {
  case SCMD_JUNK:
    sname = "junk";
    mode = SCMD_JUNK;
    break;
  default:
    sname = "drop";
    break;
  }
  /* The command may not work, but it's easier to just save now, rather
     than in each of the various subfunctions */
  if (IS_PC(ch))
    Crash_crashsave(ch);

  argument = one_argument(argument, arg);

  if (!*arg) {
    sprintf(buf, "What do you want to %s?\r\n", sname);
    send_to_char(buf, ch);
    return;
  } else if (is_number(arg)) {
    amount = atoi(arg);
    argument = one_argument(argument, arg);
    if (!str_cmp("copper", arg) || !str_cmp("coppers", arg)) {
      perform_drop_money(ch, amount, MONEY_COPPER);
    } else if (!str_cmp("silver", arg) || !str_cmp("silvers", arg)) {
      perform_drop_money(ch, amount, MONEY_SILVER);
    } else if (!str_cmp("electrum", arg) || !str_cmp("electrum", arg)) {
      perform_drop_money(ch, amount, MONEY_ELECTRUM);
    } else if (!str_cmp("gold", arg) || !str_cmp("golds", arg)) {
      perform_drop_money(ch, amount, MONEY_GOLD);
    } else if (!str_cmp("platinum", arg) || !str_cmp("platinums", arg)) {
      perform_drop_money(ch, amount, MONEY_PLATINUM);
    } else {
      /* code to drop multiple items.  anyone want to write it? -je */
      send_to_char("Sorry, you can't do that to more than one item at a time.\r\n", ch);
    }
    return;
  } else {
    dotmode = find_all_dots(arg);

    /* Can't junk or donate all */
    if ((dotmode == FIND_ALL) && (subcmd == SCMD_JUNK) && !IS_TRUSTED(ch)) {
      send_to_char("Go to the dump if you want to junk EVERYTHING!\r\n", ch);
      return;
    }
    if (dotmode == FIND_ALL) {
      if (!ch->carrying)
	send_to_char("You don't seem to be carrying anything.\r\n", ch);
      else
	for (obj = ch->carrying; obj; obj = next_obj) {
	  next_obj = obj->next_content;
	  amount += perform_drop(ch, obj, mode, sname, RDR);
	}
    } else if (dotmode == FIND_ALLDOT) {
      if (!*arg) {
	sprintf(buf, "What do you want to %s all of?\r\n", sname);
	send_to_char(buf, ch);
	return;
      }
      if (!(obj = get_obj_in_list_vis(ch, arg, ch->carrying))) {
	sprintf(buf, "You don't seem to have any %ss.\r\n", arg);
	send_to_char(buf, ch);
      }
      while (obj) {
	next_obj = get_obj_in_list_vis(ch, arg, obj->next_content);
	amount += perform_drop(ch, obj, mode, sname, RDR);
	obj = next_obj;
      }
    } else {
      if (!(obj = get_obj_in_list(arg, ch->carrying))) {
	if (!(obj = get_held_object_in_equip_vis(ch, arg, ch->equipment, &i))) {
	  sprintf(buf, "You don't seem to have %s %s.\r\n", AN(arg), arg);
	  send_to_char(buf, ch);
	} else {
	  unequip_char(ch, i);
	  amount += perform_drop(ch, obj, mode, sname, RDR);
	}
      } else
	amount += perform_drop(ch, obj, mode, sname, RDR);
    }
  }
#if 0
  /* rewards? give me a break */
  if (amount && (subcmd == SCMD_JUNK)) {
    send_to_char("You have been rewarded by the gods!\r\n", ch);
    act("$n has been rewarded by the gods!", TRUE, ch, 0, 0, TO_PROXIMITY);
    ADD_MONEY(ch, amount);
  }
#endif
}


void perform_give(struct char_data *ch, struct char_data *vict,
		  struct obj_data *obj)
{
  int where = -1, i, free_hand = 2;

  if (IS_OBJ_STAT(obj, ITEM_NODROP)) {
    act("You can't let go of $p!!  Yeech!", FALSE, ch, obj, 0, TO_CHAR);
    return;
  }
  if (GET_EQ(vict, WEAR_WIELD)) {
    free_hand--;
    if (IS_OBJ_STAT(GET_EQ(vict, WEAR_WIELD), ITEM_TWOHANDS) ||
	NEEDS_TWO_HANDS(vict, GET_EQ(vict, WEAR_WIELD)))
      free_hand--;
  }
  if (GET_EQ(vict, WEAR_WIELD2)) {
    free_hand--;
    if (IS_OBJ_STAT(GET_EQ(vict, WEAR_WIELD2), ITEM_TWOHANDS) ||
	NEEDS_TWO_HANDS(vict, GET_EQ(vict, WEAR_WIELD2)))
      free_hand--;
  }
  if (GET_EQ(vict, WEAR_HOLD)) {
    free_hand--;
    if (IS_OBJ_STAT(GET_EQ(vict, WEAR_HOLD), ITEM_TWOHANDS))
      free_hand--;
  }
  if (GET_EQ(vict, WEAR_HOLD2)) {
    free_hand--;
    if (IS_OBJ_STAT(GET_EQ(vict, WEAR_HOLD2), ITEM_TWOHANDS))
      free_hand--;
  }
  if (IS_CARRYING_N(vict) >= CAN_CARRY_N(vict) || free_hand <= 0) {
    act("$N seems to have $S hands full.", FALSE, ch, 0, vict, TO_CHAR);
    return;
  }
  if (GET_OBJ_WEIGHT(obj) + IS_CARRYING_W(vict) > CAN_CARRY_W(vict)) {
    act("$E can't carry that much weight.", FALSE, ch, 0, vict, TO_CHAR);
    return;
  }
  if (!give_otrigger(obj, ch, vict) || !receive_mtrigger(vict, ch, obj))
    return;

  if (obj->worn_by != NULL) {
    for (i = 0; i < NUM_WEARS; i++) {
      if (GET_EQ(obj->worn_by, i) == obj) {
	obj_from_char(unequip_char(obj->worn_by, i));
	break;
      }
    }
  } else if (obj->in_obj)
    obj_from_obj(obj);
  else
    obj_from_char(obj);

  if ((where = find_eq_pos(vict, obj, 0)) >= 0) {
    if (where == WEAR_HOLD || where == WEAR_HOLD2)
      perform_wear(vict, obj, where);
    else if (CAN_WEAR(obj, ITEM_TAKE)) {
      perform_wear(vict, obj, WEAR_HOLD);
    } else
      obj_to_char(obj, vict);
  } else
    obj_to_char(obj, vict);
  act("You give $p to $N.", FALSE, ch, obj, vict, TO_CHAR);
  act("$n gives you $p.", FALSE, ch, obj, vict, TO_VICT);
  act("$n gives $p to $N.", TRUE, ch, obj, vict, TO_NOTVICT);
  autoquest_trigger_check(ch, vict, obj, AQ_RETURN_OBJ);
}

/* utility function for give */
struct char_data *
 give_find_vict(struct char_data *ch, char *arg)
{
  struct char_data *vict;

  if (!*arg) {
    send_to_char("To who?\r\n", ch);
    return NULL;
  } else if (IS_TRUSTED(ch)) {
    if (!(vict = get_char_vis(ch, arg))) {
      send_to_char(NOPERSON, ch);
      return NULL;
    } else
      return vict;
  } else if (!(vict = get_char_room_vis(ch, arg))) {
    send_to_char(NOPERSON, ch);
    return NULL;
  } else if (vict == ch && !IS_TRUSTED(ch)) {
    send_to_char("What's the point of that?\r\n", ch);
    return NULL;
  } else
    return vict;
}

void perform_give_money(struct char_data *ch, struct char_data *vict,
			int amount, int type)
{

  if (amount <= 0)
    send_to_char("Heh heh heh.. we are jolly funny today, eh?\r\n", ch);
  else if (!have_money(ch, amount, type) && !IS_TRUSTED(ch))
    send_to_char("You don't have that many coins!\r\n", ch);
  else {
    WAIT_STATE(ch, PULSE_VIOLENCE);	/* to prevent coin-bombing */
    if (!IS_TRUSTED(ch))
      remove_and_give_money(ch, vict, amount, type);
    else
      create_and_give_money_to(ch, vict, amount, type);
    send_to_char(OK, ch);
    sprintf(buf, "$n gives you %s.", money_desc(amount, type, 0));
    act(buf, FALSE, ch, 0, vict, TO_VICT);
    sprintf(buf, "$n gives %s to $N.", money_desc(amount, type, 0));
    act(buf, TRUE, ch, 0, vict, TO_NOTVICT);
    /* since we're done, let's alter amount for the bribe trigger */
    switch (type) {
    case MONEY_SILVER:
      amount *= 10;
      break;
    case MONEY_ELECTRUM:
      amount *= 100;
      break;
    case MONEY_GOLD:
      amount *= 1000;
      break;
    case MONEY_PLATINUM:
      amount *= 10000;
      break;
    }
    bribe_mtrigger(vict, ch, amount);
  }
}

ACMD(do_give)
{
  int amount, dotmode, i;
  struct char_data *vict;
  struct obj_data *obj, *next_obj;

  argument = one_argument(argument, arg);

  if (!*arg)
    send_to_char("Give what to who?\r\n", ch);
  else if (is_number(arg)) {
    amount = atoi(arg);
    argument = one_argument(argument, arg);
    if (!str_cmp("copper", arg) || !str_cmp("coppers", arg) || !str_cmp("c", arg)) {
      argument = one_argument(argument, arg);
      if ((vict = give_find_vict(ch, arg))) {
	perform_give_money(ch, vict, amount, MONEY_COPPER);
	return;
      }
    } else if (!str_cmp("silver", arg) || !str_cmp("silvers", arg) || !str_cmp("s", arg)) {
      argument = one_argument(argument, arg);
      if ((vict = give_find_vict(ch, arg))) {
	perform_give_money(ch, vict, amount, MONEY_SILVER);
	return;
      }
    } else if (!str_cmp("electrum", arg) || !str_cmp("electrum", arg) || !str_cmp("e", arg)) {
      argument = one_argument(argument, arg);
      if ((vict = give_find_vict(ch, arg))) {
	perform_give_money(ch, vict, amount, MONEY_ELECTRUM);
	return;
      }
    } else if (!str_cmp("gold", arg) || !str_cmp("golds", arg) || !str_cmp("g", arg)) {
      argument = one_argument(argument, arg);
      if ((vict = give_find_vict(ch, arg))) {
	perform_give_money(ch, vict, amount, MONEY_GOLD);
	return;
      }
    } else if (!str_cmp("platinum", arg) || !str_cmp("platinums", arg) || !str_cmp("p", arg)) {
      argument = one_argument(argument, arg);
      if ((vict = give_find_vict(ch, arg))) {
	perform_give_money(ch, vict, amount, MONEY_PLATINUM);
	return;
      }
    } else {
      /* code to give multiple items.  anyone want to write it? -je */
      send_to_char("You can't give more than one item at a time.\r\n", ch);
      return;
    }
  } else {
    one_argument(argument, buf1);
    if (!(vict = give_find_vict(ch, buf1)))
      return;
    dotmode = find_all_dots(arg);
    if (dotmode == FIND_INDIV) {
      if (!(obj = get_obj_in_list(arg, ch->carrying))) {
	i = WEAR_HOLD;
	if (!(obj = get_held_object_in_equip_vis(ch, arg, ch->equipment, &i))) {
	  sprintf(buf, "You don't seem to have %s %s.\r\n", AN(arg), arg);
	  send_to_char(buf, ch);
	  return;
	} else {
	  //unequip_char(ch, i);
	  perform_give(ch, vict, obj);
	}
      } else
	perform_give(ch, vict, obj);
    } else {
      if (dotmode == FIND_ALLDOT && !*arg) {
	send_to_char("All of what?\r\n", ch);
	return;
      }
      if (!ch->carrying)
	send_to_char("You don't seem to be holding anything.\r\n", ch);
      else
	for (obj = ch->carrying; obj; obj = next_obj) {
	  next_obj = obj->next_content;
	  if (CAN_SEE_OBJ(ch, obj) &&
	      ((dotmode == FIND_ALL || isname(arg, obj->name))))
	    perform_give(ch, vict, obj);
	}
    }
  }
}



void weight_change_object(struct obj_data *obj, int weight)
{
  struct obj_data *tmp_obj;
  struct char_data *tmp_ch;

  if (obj->in_room != NOWHERE) {
    GET_OBJ_WEIGHT(obj) += weight;
  } else if ((tmp_ch = obj->carried_by)) {
    obj_from_char(obj);
    GET_OBJ_WEIGHT(obj) += weight;
    obj_to_char(obj, tmp_ch);
  } else if ((tmp_obj = obj->in_obj)) {
    obj_from_obj(obj);
    GET_OBJ_WEIGHT(obj) += weight;
    obj_to_obj(obj, tmp_obj);
  } else {
    log("SYSERR: Unknown attempt to subtract weight from an object.");
  }
}



void name_from_drinkcon(struct obj_data *obj)
{
  int i;
  char *new_name;

  for (i = 0; (*((obj->name) + i) != ' ') && (*((obj->name) + i) != '\0'); i++);

  if (*((obj->name) + i) == ' ') {
    new_name = str_dup((obj->name) + i + 1);
    if (GET_OBJ_RNUM(obj) < 0 || obj->name != obj_proto[GET_OBJ_RNUM(obj)].name)
      FREE(obj->name);
    obj->name = new_name;
  }
}



void name_to_drinkcon(struct obj_data *obj, int type)
{
  char *new_name;
  extern struct obj_data *obj_proto;
  extern const char *drinknames[];

  CREATE(new_name, char, strlen(obj->name) + strlen(drinknames[type]) + 2);
  sprintf(new_name, "%s %s", drinknames[type], obj->name);
  if (GET_OBJ_RNUM(obj) < 0 || obj->name != obj_proto[GET_OBJ_RNUM(obj)].name)
    FREE(obj->name);
  obj->name = new_name;
}



ACMD(do_drink)
{
  struct obj_data *temp;
  struct affected_type af;
  struct char_data *tmp_char;
  int amount, weight;
  int on_ground = 0;

  one_argument(argument, arg);

  if (!*arg) {
    send_to_char("Drink from what?\r\n", ch);
    return;
  }
  generic_find(arg, FIND_OBJ_EQUIP | FIND_OBJ_INV, ch, &tmp_char, &temp);
  if (!temp) {
    if (!(temp = get_obj_in_list_vis(ch, arg, world[ch->in_room].contents))) {
      act("You can't find it!", FALSE, ch, 0, 0, TO_CHAR);
      return;
    } else
      on_ground = 1;
  }
  if ((GET_OBJ_TYPE(temp) != ITEM_DRINKCON) &&
      (GET_OBJ_TYPE(temp) != ITEM_FOUNTAIN)) {
    send_to_char("You can't drink from that!\r\n", ch);
    return;
  }
  if (on_ground && (GET_OBJ_TYPE(temp) == ITEM_DRINKCON)) {
    send_to_char("You have to be holding that to drink from it.\r\n", ch);
    return;
  }
  if ((GET_COND(ch, DRUNK) > 10) && (GET_COND(ch, THIRST) > 0)) {
    /* The pig is drunk */
    send_to_char("You can't seem to get close enough to your mouth.\r\n", ch);
    act("$n tries to drink but misses $s mouth!", TRUE, ch, 0, 0, TO_PROXIMITY);
    return;
  }
  if ((GET_COND(ch, FULL) > 20) && (GET_COND(ch, THIRST) > 0)) {
    send_to_char("Your stomach can't contain anymore!\r\n", ch);
    return;
  }
  if (!GET_OBJ_VAL(temp, 1)) {
    send_to_char("It's empty.\r\n", ch);
    return;
  }
  if (subcmd == SCMD_DRINK) {
    sprintf(buf, "$n drinks %s from $p.", drinks[GET_OBJ_VAL(temp, 2)]);
    act(buf, TRUE, ch, temp, 0, TO_PROXIMITY);

    sprintf(buf, "You drink the %s.\r\n", drinks[GET_OBJ_VAL(temp, 2)]);
    send_to_char(buf, ch);

    if (drink_aff[GET_OBJ_VAL(temp, 2)][DRUNK] > 0)
      amount = (25 - GET_COND(ch, THIRST)) / drink_aff[GET_OBJ_VAL(temp, 2)][DRUNK];
    else
      amount = number(3, 10);

  } else {
    act("$n sips from $p.", TRUE, ch, temp, 0, TO_PROXIMITY);
    sprintf(buf, "It tastes like %s.\r\n", drinks[GET_OBJ_VAL(temp, 2)]);
    send_to_char(buf, ch);
    amount = 1;
  }

  amount = MIN(amount, GET_OBJ_VAL(temp, 1));

  /* You can't subtract more than the object weighs */
  weight = MIN(amount, GET_OBJ_WEIGHT(temp));

  weight_change_object(temp, -weight);	/* Subtract amount */

  gain_condition(ch, DRUNK,
      (int) ((int) drink_aff[GET_OBJ_VAL(temp, 2)][DRUNK] * amount) / 4);

  gain_condition(ch, FULL,
       (int) ((int) drink_aff[GET_OBJ_VAL(temp, 2)][FULL] * amount) / 4);

  gain_condition(ch, THIRST,
     (int) ((int) drink_aff[GET_OBJ_VAL(temp, 2)][THIRST] * amount) / 4);

  if (GET_COND(ch, DRUNK) > 10)
    send_to_char("You feel drunk.\r\n", ch);

  if (GET_COND(ch, THIRST) > 20)
    send_to_char("You don't feel thirsty any more.\r\n", ch);

  if (GET_COND(ch, FULL) > 20)
    send_to_char("You are full.\r\n", ch);

  if (GET_OBJ_VAL(temp, 3)) {	/* The shit was poisoned ! */
    send_to_char("Oops, it tasted rather strange!\r\n", ch);
    act("$n chokes and utters some strange sounds.", TRUE, ch, 0, 0, TO_PROXIMITY);

    af.type = SPELL_POISON;
    af.duration = amount * 3;
    af.modifier = 0;
    af.location = APPLY_NONE;
    af.bitvector = AFF_POISON;
    affect_join(ch, &af, FALSE, FALSE, FALSE, FALSE);
  }
  /* empty the container, and no longer poison. */
  GET_OBJ_VAL(temp, 1) -= amount;
  if (!GET_OBJ_VAL(temp, 1)) {	/* The last bit */
    GET_OBJ_VAL(temp, 2) = 0;
    GET_OBJ_VAL(temp, 3) = 0;
    name_from_drinkcon(temp);
  }
  return;
}



ACMD(do_eat)
{
  struct obj_data *food;
  struct char_data *tmp_char;
  struct affected_type af;
  int amount;

  one_argument(argument, arg);

  if (!*arg) {
    send_to_char("Eat what?\r\n", ch);
    return;
  }
  generic_find(arg, FIND_OBJ_EQUIP | FIND_OBJ_INV, ch, &tmp_char, &food);
  if (!food) {
    sprintf(buf, "You don't seem to have %s %s.\r\n", AN(arg), arg);
    send_to_char(buf, ch);
    return;
  }
  if (subcmd == SCMD_TASTE && ((GET_OBJ_TYPE(food) == ITEM_DRINKCON) ||
			       (GET_OBJ_TYPE(food) == ITEM_FOUNTAIN))) {
    do_drink(ch, argument, 0, SCMD_SIP);
    return;
  }
  if ((GET_OBJ_TYPE(food) != ITEM_FOOD) && !IS_TRUSTED(ch)) {
    send_to_char("You can't eat THAT!\r\n", ch);
    return;
  }
  if (GET_COND(ch, FULL) > 20) {	/* Stomach full */
    act("You are too full to eat more!", FALSE, ch, 0, 0, TO_CHAR);
    return;
  }
  if (subcmd == SCMD_EAT) {
    act("You eat the $o.", FALSE, ch, food, 0, TO_CHAR);
    act("$n eats $p.", TRUE, ch, food, 0, TO_PROXIMITY);
  } else {
    act("You nibble a little bit of the $o.", FALSE, ch, food, 0, TO_CHAR);
    act("$n tastes a little bit of $p.", TRUE, ch, food, 0, TO_PROXIMITY);
  }

  amount = (subcmd == SCMD_EAT ? GET_OBJ_VAL(food, 0) : 1);

  gain_condition(ch, FULL, amount);

  if (GET_COND(ch, FULL) > 20)
    act("You are full.", FALSE, ch, 0, 0, TO_CHAR);

  if (GET_OBJ_VAL(food, 3) && (GET_LEVEL(ch) < LVL_IMMORT)) {
    /* The shit was poisoned ! */
    send_to_char("Oops, that tasted rather strange!\r\n", ch);
    act("$n coughs and utters some strange sounds.", FALSE, ch, 0, 0, TO_PROXIMITY);

    af.type = SPELL_POISON;
    af.duration = amount * 2;
    af.modifier = 0;
    af.location = APPLY_NONE;
    af.bitvector = AFF_POISON;
    affect_join(ch, &af, FALSE, FALSE, FALSE, FALSE);
  }
  if (subcmd == SCMD_EAT)
    extract_obj(food);
  else {
    if (!(--GET_OBJ_VAL(food, 0))) {
      send_to_char("There's nothing left now.\r\n", ch);
      extract_obj(food);
    }
  }
}


ACMD(do_pour)
{
  char arg1[MAX_INPUT_LENGTH];
  char arg2[MAX_INPUT_LENGTH];
  struct obj_data *from_obj = NULL, *to_obj = NULL;
  int amount;

  two_arguments(argument, arg1, arg2);

  if (subcmd == SCMD_POUR) {
    if (!*arg1) {		/* No arguments */
      act("From what do you want to pour?", FALSE, ch, 0, 0, TO_CHAR);
      return;
    }
    if (!(from_obj = get_obj_in_list_vis(ch, arg1, ch->carrying))) {
      act("You can't find it!", FALSE, ch, 0, 0, TO_CHAR);
      return;
    }
    if (GET_OBJ_TYPE(from_obj) != ITEM_DRINKCON) {
      act("You can't pour from that!", FALSE, ch, 0, 0, TO_CHAR);
      return;
    }
  }
  if (subcmd == SCMD_FILL) {
    if (!*arg1) {		/* no arguments */
      send_to_char("What do you want to fill?  And what are you filling it from?\r\n", ch);
      return;
    }
    if (!(to_obj = get_obj_in_list_vis(ch, arg1, ch->carrying))) {
      send_to_char("You can't find it!", ch);
      return;
    }
    if (GET_OBJ_TYPE(to_obj) != ITEM_DRINKCON) {
      act("You can't fill $p!", FALSE, ch, to_obj, 0, TO_CHAR);
      return;
    }
    if (!*arg2) {		/* no 2nd argument */
      act("What do you want to fill $p from?", FALSE, ch, to_obj, 0, TO_CHAR);
      return;
    }
    if (!(from_obj = get_obj_in_list_vis(ch, arg2, world[ch->in_room].contents))) {
      sprintf(buf, "There doesn't seem to be %s %s here.\r\n", AN(arg2), arg2);
      send_to_char(buf, ch);
      return;
    }
    if (GET_OBJ_TYPE(from_obj) != ITEM_FOUNTAIN) {
      act("You can't fill something from $p.", FALSE, ch, from_obj, 0, TO_CHAR);
      return;
    }
  }
  if (GET_OBJ_VAL(from_obj, 1) == 0) {
    act("The $p is empty.", FALSE, ch, from_obj, 0, TO_CHAR);
    return;
  }
  if (subcmd == SCMD_POUR) {	/* pour */
    if (!*arg2) {
      act("Where do you want it?  Out or in what?", FALSE, ch, 0, 0, TO_CHAR);
      return;
    }
    if (!str_cmp(arg2, "out")) {
      act("$n empties $p.", TRUE, ch, from_obj, 0, TO_PROXIMITY);
      act("You empty $p.", FALSE, ch, from_obj, 0, TO_CHAR);

      weight_change_object(from_obj, -GET_OBJ_VAL(from_obj, 1));	/* Empty */

      GET_OBJ_VAL(from_obj, 1) = 0;
      GET_OBJ_VAL(from_obj, 2) = 0;
      GET_OBJ_VAL(from_obj, 3) = 0;
      name_from_drinkcon(from_obj);

      return;
    }
    if (!(to_obj = get_obj_in_list_vis(ch, arg2, ch->carrying))) {
      act("You can't find it!", FALSE, ch, 0, 0, TO_CHAR);
      return;
    }
    if ((GET_OBJ_TYPE(to_obj) != ITEM_DRINKCON) &&
	(GET_OBJ_TYPE(to_obj) != ITEM_FOUNTAIN)) {
      act("You can't pour anything into that.", FALSE, ch, 0, 0, TO_CHAR);
      return;
    }
  }
  if (to_obj == from_obj) {
    act("A most unproductive effort.", FALSE, ch, 0, 0, TO_CHAR);
    return;
  }
  if ((GET_OBJ_VAL(to_obj, 1) != 0) &&
      (GET_OBJ_VAL(to_obj, 2) != GET_OBJ_VAL(from_obj, 2))) {
    act("There is already another liquid in it!", FALSE, ch, 0, 0, TO_CHAR);
    return;
  }
  if (!(GET_OBJ_VAL(to_obj, 1) < GET_OBJ_VAL(to_obj, 0))) {
    act("There is no room for more.", FALSE, ch, 0, 0, TO_CHAR);
    return;
  }
  if (subcmd == SCMD_POUR) {
    sprintf(buf, "You pour the %s into the %s.",
	    drinks[GET_OBJ_VAL(from_obj, 2)], arg2);
    send_to_char(buf, ch);
  }
  if (subcmd == SCMD_FILL) {
    act("You gently fill $p from $P.", FALSE, ch, to_obj, from_obj, TO_CHAR);
    act("$n gently fills $p from $P.", TRUE, ch, to_obj, from_obj, TO_PROXIMITY);
  }
  /* New alias */
  if (GET_OBJ_VAL(to_obj, 1) == 0)
    name_to_drinkcon(to_obj, GET_OBJ_VAL(from_obj, 2));

  /* First same type liq. */
  GET_OBJ_VAL(to_obj, 2) = GET_OBJ_VAL(from_obj, 2);

  /* Then how much to pour */
  GET_OBJ_VAL(from_obj, 1) -= (amount =
		      (GET_OBJ_VAL(to_obj, 0) - GET_OBJ_VAL(to_obj, 1)));

  GET_OBJ_VAL(to_obj, 1) = GET_OBJ_VAL(to_obj, 0);

  if (GET_OBJ_VAL(from_obj, 1) < 0) {	/* There was too little */
    GET_OBJ_VAL(to_obj, 1) += GET_OBJ_VAL(from_obj, 1);
    amount += GET_OBJ_VAL(from_obj, 1);
    GET_OBJ_VAL(from_obj, 1) = 0;
    GET_OBJ_VAL(from_obj, 2) = 0;
    GET_OBJ_VAL(from_obj, 3) = 0;
    name_from_drinkcon(from_obj);
  }
  /* Then the poison boogie */
  GET_OBJ_VAL(to_obj, 3) =
      (GET_OBJ_VAL(to_obj, 3) || GET_OBJ_VAL(from_obj, 3));

  /* And the weight boogie */
  weight_change_object(from_obj, -amount);
  weight_change_object(to_obj, amount);		/* Add weight */

  return;
}



void wear_message(struct char_data *ch, struct obj_data *obj, int where)
{
  char *wear_messages[][2] =
  {
    {"$n wears $p as a badge.",
     "You wear $p as a badge."},

    {"$n dons $p on $s head.",
     "You don $p on your head."},

    {"$n covers $s face with $p.",
     "You cover your face with $p."},

    {"$n places $p on $s eyes.",
     "You place $p on your eyes."},

    {"$n attach $p on to $s right ear.",
     "You attach $p on to your right ear."},

    {"$n attach $p on to $s left ear.",
     "You attach $p on to your left ear."},

    {"$n wears $p around $s neck.",
     "You wear $p around your neck."},

    {"$n wears $p on $s body.",
     "You wear $p on your body.",},

    {"$n wears $p about $s body.",
     "You wear $p around your body."},

    {"$n straps $p on $s back.",
     "You strap $p on your back."},

    {"$n wears $p on $s arms.",
     "You wear $p on your arms."},

    {"$n puts $p on around $s right wrist.",
     "You put $p on around your right wrist."},

    {"$n puts $p on around $s left wrist.",
     "You put $p on around your left wrist."},

    {"$n slides $p on to $s right ring finger.",
     "You slide $p on to your right ring finger."},

    {"$n slides $p on to $s left ring finger.",
     "You slide $p on to your left ring finger."},

    {"$n puts $p on $s hands.",
     "You put $p on your hands."},

    {"$n wears $p around $s waist.",
     "You wear $p around your waist."},

    {"$n attachs $p to $s belt.",
     "You attach $p to your belt."},

    {"$n attachs $p to $s belt.",
     "You attach $p to your belt."},

    {"$n attachs $p to $s belt.",
     "You attach $p to your belt."},

    {"$n straps $p around $s arm.",
     "You straps $p around your arm."},

    {"$n wields $p.",
     "You wield $p."},

    {"$n wields $p.",
     "You wield $p."},

    {"$n grabs $p.",
     "You grab $p."},

    {"$n grabs $p.",
     "You grab $p."},

    {"$n puts $p on $s legs.",
     "You put $p on your legs."},

    {"$n wears $p on $s feet.",
     "You wear $p on your feet."},

    {"should not see that.",
     "should not see that."}
  };

  act(wear_messages[where][0], TRUE, ch, obj, 0, TO_PROXIMITY);
  act(wear_messages[where][1], FALSE, ch, obj, 0, TO_CHAR);
}



char perform_wear(struct char_data *ch, struct obj_data *obj, int where)
{
  /*
   * ITEM_WEAR_TAKE is used for objects that do not require special bits
   * to be put into that position (e.g. you can hold any object, not just
   * an object with a HOLD bit.)
   */

  int free_hand = 2, need_hand = 0;
  int temp_size;
  int wear_bitvectors[] =
  {ITEM_GUILD_INSIGNIA,
   ITEM_WEAR_HEAD,
   ITEM_WEAR_FACE,
   ITEM_WEAR_EYES,
   ITEM_WEAR_EARRING,
   ITEM_WEAR_EARRING,
   ITEM_WEAR_NECK,
   ITEM_WEAR_BODY,
   ITEM_WEAR_ABOUT,
   ITEM_WEAR_BACK,
   ITEM_WEAR_ARMS,
   ITEM_WEAR_WRIST,
   ITEM_WEAR_WRIST,
   ITEM_WEAR_FINGER,
   ITEM_WEAR_FINGER,
   ITEM_WEAR_HANDS,
   ITEM_WEAR_WAIST,
   ITEM_ATTACH_BELT,
   ITEM_ATTACH_BELT,
   ITEM_ATTACH_BELT,
   ITEM_WEAR_SHIELD,
   ITEM_WEAR_WIELD,
   ITEM_WEAR_WIELD,
   ITEM_WEAR_HOLD,
   ITEM_WEAR_HOLD,
   ITEM_WEAR_LEGS,
   ITEM_WEAR_FEET};

  char *already_wearing[] =
  {
    "You're already wearing a badge.\r\n",
    "You're already wearing something on your head.\r\n",
    "You're already wearing something on your face.\r\n",
    "You're already wearing something on your eyes.\r\n",
    "YOU SHOULD NEVER SEE THIS MESSAGE.  PLEASE REPORT #1.\r\n",
    "You're already wearing something on both of your ears.\r\n",
    "You can't wear anything else around your neck.\r\n",
    "You're already wearing something on your body.\r\n",
    "You're already wearing something about your body.\r\n",
    "You're already wearing something on your back.\r\n",
    "You're already wearing something on your arms.\r\n",
    "YOU SHOULD NEVER SEE THIS MESSAGE.  PLEASE REPORT #2.\r\n",
    "You're already wearing something around both of your wrists.\r\n",
    "YOU SHOULD NEVER SEE THIS MESSAGE.  PLEASE REPORT #3.\r\n",
    "You're already wearing something on both of your ring fingers.\r\n",
    "You're already wearing something on your hands.\r\n",
    "You already have something around your waist.\r\n",
    "YOU SHOULD NEVER SEE THIS MESSAGE.  PLEASE REPORT #4.\r\n",
    "YOU SHOULD NEVER SEE THIS MESSAGE.  PLEASE REPORT #5.\r\n",
    "You can't wear anything else on your belt.\r\n",
    "You're already using a shield.\r\n",
    "You're already wielding a weapon.\r\n",
    "You're already wielding a weapon.\r\n",
    "You're already holding something.\r\n",
    "You're already holding something in both your hands.\r\n",
    "You're already wearing something on your legs.\r\n",
    "You're already wearing something on your feet.\r\n",
    "YOU SHOULD NEVER SEE THIS MESSAGE.  PLEASE REPORT #6.\r\n"
  };

  if (where <= 0)
    return 0;

  /* first, make sure that the wear position is valid. */
  if (!CAN_WEAR(obj, wear_bitvectors[where]) && !CAN_WEAR(obj, ITEM_TAKE)) {
    act("You can't wear $p there.", FALSE, ch, obj, 0, TO_CHAR);
    return 0;
  }
  /* for hold, neck, finger, and wrist, try pos 2 if pos 1 is already full */
  if ((where == WEAR_HOLD) || (where == WEAR_FINGER_R) ||
      (where == WEAR_WRIST_R) || (where == WEAR_EARRING_R) ||
      (where == WEAR_WIELD))
    if (GET_EQ(ch, where))
      where++;

  if ((where == WEAR_ATTACH_1))
    if (GET_EQ(ch, where))
      where++;

  if ((where == WEAR_ATTACH_2))
    if (GET_EQ(ch, where))
      where++;

  if (GET_EQ(ch, where)) {
    send_to_char(already_wearing[where], ch);
    return 0;
  }
  if (IS_OBJ_STAT(obj, ITEM_WHOLE_HEAD)) {
    if (GET_EQ(ch, WEAR_HEAD)) {
      send_to_char(already_wearing[WEAR_HEAD], ch);
      return 0;
    }
    if (GET_EQ(ch, WEAR_FACE)) {
      send_to_char(already_wearing[WEAR_FACE], ch);
      return 0;
    }
    if (GET_EQ(ch, WEAR_EYES)) {
      send_to_char(already_wearing[WEAR_EYES], ch);
      return 0;
    }
  }
  if (IS_OBJ_STAT(obj, ITEM_WHOLE_BODY)) {
    if (GET_EQ(ch, WEAR_BODY)) {
      send_to_char(already_wearing[WEAR_BODY], ch);
      return 0;
    }
    if (GET_EQ(ch, WEAR_ARMS)) {
      send_to_char(already_wearing[WEAR_ARMS], ch);
      return 0;
    }
    if (GET_EQ(ch, WEAR_LEGS)) {
      send_to_char(already_wearing[WEAR_LEGS], ch);
      return 0;
    }
  }
  if (IS_OBJ_STAT(obj, ITEM_ANTI_GOOD) && IS_GOOD(ch)) {
    send_to_char("You have a bad feeling and dont want to use it!\r\n", ch);
    return 0;
  }
  if (IS_OBJ_STAT(obj, ITEM_ANTI_EVIL) && IS_EVIL(ch)) {
    send_to_char("You have a bad feeling and dont want to use it!\r\n", ch);
    return 0;
  }
  if (GET_OBJ_SIZE(obj) == ITEM_SIZE_MAGICAL) {
    temp_size = GET_SIZE(ch);
  } else if (GET_OBJ_SIZE(obj) == ITEM_SIZE_SM) {
    if (GET_SIZE(ch) <= SIZE_SMALL) {
      temp_size = ITEM_SIZE_SMALL;
    } else {
      temp_size = ITEM_SIZE_MEDIUM;
    }
  } else if (GET_OBJ_SIZE(obj) == ITEM_SIZE_ML) {
    if (GET_SIZE(ch) <= SIZE_MEDIUM) {
      temp_size = ITEM_SIZE_MEDIUM;
    } else {
      temp_size = ITEM_SIZE_LARGE;
    }
  } else {
    temp_size = GET_OBJ_SIZE(obj);
  }

  if (where == ITEM_ATTACH_BELT && !GET_EQ(ch, WEAR_WAIST)) {
    send_to_char("You dont have a belt!\r\n", ch);
    return 0;
  }
  if (temp_size != ITEM_SIZE_MAGICAL) {
    switch (GET_OBJ_WEAR(obj)) {
    case ITEM_WEAR_FINGER:
    case ITEM_WEAR_NECK:
    case ITEM_WEAR_BODY:
    case ITEM_WEAR_HEAD:
    case ITEM_WEAR_LEGS:
    case ITEM_WEAR_FEET:
    case ITEM_WEAR_HANDS:
    case ITEM_WEAR_ARMS:
    case ITEM_WEAR_ABOUT:
    case ITEM_WEAR_WRIST:
    case ITEM_WEAR_EYES:
    case ITEM_WEAR_FACE:
    case ITEM_WEAR_SHIELD:
      if (temp_size != GET_SIZE(ch)) {
	send_to_char("This is not the right size for you!\r\n", ch);
	return 0;
      }
      break;
    case ITEM_WEAR_WAIST:
    case ITEM_WEAR_BACK:
    case ITEM_WEAR_EARRING:
      if (temp_size < GET_SIZE(ch) - 1 || temp_size > GET_SIZE(ch) + 1) {
	send_to_char("This is not the right size for you!\r\n", ch);
	return 0;
      }
      break;
    case ITEM_ATTACH_BELT:
      if (temp_size > GET_SIZE(ch)) {
	send_to_char("This is too big to attach to your belt!\r\n", ch);
	return 0;
      }
      break;
    case ITEM_WEAR_HOLD:
      if (temp_size > GET_SIZE(ch) + 1) {
	send_to_char("This is too big to hold!\r\n", ch);
	return 0;
      }
      break;
    }
  }
  if (GET_EQ(ch, WEAR_WIELD)) {
    free_hand--;
    if (IS_OBJ_STAT(GET_EQ(ch, WEAR_WIELD), ITEM_TWOHANDS) ||
	NEEDS_TWO_HANDS(ch, GET_EQ(ch, WEAR_WIELD)))
      free_hand--;
  }
  if (GET_EQ(ch, WEAR_WIELD2)) {
    free_hand--;
    if (IS_OBJ_STAT(GET_EQ(ch, WEAR_WIELD2), ITEM_TWOHANDS) ||
	NEEDS_TWO_HANDS(ch, GET_EQ(ch, WEAR_WIELD2)))
      free_hand--;
  }
  if (GET_EQ(ch, WEAR_HOLD)) {
    free_hand--;
    if (IS_OBJ_STAT(GET_EQ(ch, WEAR_HOLD), ITEM_TWOHANDS)	/* ||
								   NEEDS_TWO_HANDS(ch, GET_EQ(ch, WEAR_HOLD)) */ )
      free_hand--;
  }
  if (GET_EQ(ch, WEAR_HOLD2)) {
    free_hand--;
    if (IS_OBJ_STAT(GET_EQ(ch, WEAR_HOLD2), ITEM_TWOHANDS)	/* ||
								   NEEDS_TWO_HANDS(ch, GET_EQ(ch, WEAR_HOLD2)) */ )
      free_hand--;
  }
  if (GET_EQ(ch, WEAR_SHIELD))
    free_hand--;

  if (where == WEAR_SHIELD) {
    if (free_hand < 1) {
      send_to_char("You need a free hand to wear that!\r\n", ch);
      return 0;
    }
    if (GET_EQ(ch, WEAR_WIELD2) || GET_EQ(ch, WEAR_HOLD2)) {
      send_to_char("You already have something in your off-hand!\r\n", ch);
      return 0;
    }
  }
  if (where == WEAR_HOLD || where == WEAR_HOLD2) {
    if (free_hand < 1) {
      send_to_char("You need a free hand to hold that!\r\n", ch);
      return 0;
    }
    if (where == WEAR_HOLD) {
      if (IS_CRIPPLED(ch, LOCATION_WEAPON_HAND)) {
	send_to_char("Your hand is too damaged to hold anything.\r\n", ch);
	return 0;
      }
    } else {
      if (IS_CRIPPLED(ch, LOCATION_SHIELD_HAND)) {
	send_to_char("Your off hand is too damaged to hold anything.\r\n", ch);
	return 0;
      }
    }
  }
  if (where == WEAR_WIELD || where == WEAR_WIELD2) {
    if (free_hand < 1) {
      send_to_char("You need a free hand to wield that!\r\n", ch);
      return 0;
    }
    if (GET_OBJ_WEIGHT(obj) > str_app[STRENGTH_APPLY_INDEX(ch)].wield_w) {
      send_to_char("It's too heavy for you to use.\r\n", ch);
      return 0;
    }
    if (temp_size < GET_SIZE(ch) - 2) {
      send_to_char("This weapon is too small to wield!\r\n", ch);
      return 0;
    }
    if (temp_size > GET_SIZE(ch) + 2) {
      send_to_char("This weapon is too big to wield!\r\n", ch);
      return 0;
    }
    if (where == WEAR_WIELD) {
      if (IS_CRIPPLED(ch, LOCATION_WEAPON_HAND)) {
	send_to_char("Your hand is too damaged to wield anything.\r\n", ch);
	return 0;
      }
    } else {
      if (IS_CRIPPLED(ch, LOCATION_SHIELD_HAND)) {
	send_to_char("Your off hand is too damaged to wield anything.\r\n", ch);
	return 0;
      }
    }
    if (temp_size <= GET_SIZE(ch))
      need_hand = 1;
    if (temp_size == GET_SIZE(ch) + 1)
      need_hand = 2;
    if (IS_OBJ_STAT(obj, ITEM_TWOHANDS) || NEEDS_TWO_HANDS(ch, obj))
      need_hand = 2;

    if (free_hand < need_hand) {
      sprintf(buf, "Free: %d, Need: %d\r\n", free_hand, need_hand);
      send_to_char(buf, ch);
      send_to_char("You dont have enough free hands to wield that!\r\n", ch);
      return 0;
    }
  }
  if (!wear_otrigger(obj, ch, where))
    return 0;
  wear_message(ch, obj, where);
  obj_from_char(obj);
  equip_char(ch, obj, where);

  return 1;
}



int find_eq_pos(struct char_data *ch, struct obj_data *obj, char *arg)
{
  int where = -1;

  static char *keywords[NUM_WEARS + 1] =
  {
    "badge",
    "head",
    "face",
    "eyes",
    "earring_l",
    "earring_r",
    "neck",
    "body",
    "about",
    "back",
    "arms",
    "wrist_l",
    "wrist_r",
    "finger_l",
    "finger_r",
    "hands",
    "waist",
    "attach",
    "belt",
    "!RESERVED!",
    "shield",
    "!RESERVED!",
    "!RESERVED!",
    "!RESERVED!",
    "!RESERVED!",
    "legs",
    "feet",
    "\n"
  };

  if (!arg || !*arg) {
    if (CAN_WEAR(obj, ITEM_WEAR_HOLD))
      where = WEAR_HOLD;
    if (CAN_WEAR(obj, ITEM_GUILD_INSIGNIA))
      where = WEAR_BADGE;
    if (CAN_WEAR(obj, ITEM_WEAR_HEAD))
      where = WEAR_HEAD;
    if (CAN_WEAR(obj, ITEM_WEAR_FACE))
      where = WEAR_FACE;
    if (CAN_WEAR(obj, ITEM_WEAR_EYES))
      where = WEAR_EYES;
    if (CAN_WEAR(obj, ITEM_WEAR_EARRING))
      where = WEAR_EARRING_R;
    if (CAN_WEAR(obj, ITEM_WEAR_NECK))
      where = WEAR_NECK;
    if (CAN_WEAR(obj, ITEM_WEAR_BODY))
      where = WEAR_BODY;
    if (CAN_WEAR(obj, ITEM_WEAR_ABOUT))
      where = WEAR_ABOUT;
    if (CAN_WEAR(obj, ITEM_WEAR_BACK))
      where = WEAR_BACK;
    if (CAN_WEAR(obj, ITEM_WEAR_ARMS))
      where = WEAR_ARMS;
    if (CAN_WEAR(obj, ITEM_WEAR_WRIST))
      where = WEAR_WRIST_R;
    if (CAN_WEAR(obj, ITEM_WEAR_FINGER))
      where = WEAR_FINGER_R;
    if (CAN_WEAR(obj, ITEM_WEAR_HANDS))
      where = WEAR_HANDS;
    if (CAN_WEAR(obj, ITEM_WEAR_WAIST))
      where = WEAR_WAIST;
    if (CAN_WEAR(obj, ITEM_ATTACH_BELT) && GET_EQ(ch, WEAR_WAIST))
      where = WEAR_ATTACH_1;
    if (CAN_WEAR(obj, ITEM_WEAR_SHIELD))
      where = WEAR_SHIELD;
    if (CAN_WEAR(obj, ITEM_WEAR_LEGS))
      where = WEAR_LEGS;
    if (CAN_WEAR(obj, ITEM_WEAR_FEET))
      where = WEAR_FEET;
    if (CAN_WEAR(obj, ITEM_WEAR_WIELD))
      where = WEAR_WIELD;
    if (GET_EQ(ch, WEAR_WIELD) && where == WEAR_WIELD)
      where = WEAR_WIELD2;
    if (GET_EQ(ch, WEAR_HOLD) && where == WEAR_HOLD)
      where = WEAR_HOLD2;
  } else {
    if (((where = search_block(arg, keywords, FALSE)) < 0) || (*arg == '!')) {
      sprintf(buf, "'%s'?  What part of your body is THAT?\r\n", arg);
      send_to_char(buf, ch);
      return -1;
    }
    /* might come up as 2-3 due to additional keywords allowed */
    if (where == WEAR_ATTACH_2 || where == WEAR_ATTACH_3)
      where = WEAR_ATTACH_1;
  }

  return where;
}



ACMD(do_wear)
{
  char arg1[MAX_INPUT_LENGTH];
  char arg2[MAX_INPUT_LENGTH];
  struct obj_data *obj, *next_obj;
  int where, dotmode, items_worn = 0, i = 0;

  /* The command may not work, but it's easier to just save now, rather
     than in each of the various subfunctions */
  if (IS_PC(ch))
    Crash_crashsave(ch);

  two_arguments(argument, arg1, arg2);

  if (!*arg1) {
    send_to_char("Wear what?\r\n", ch);
    return;
  }
  dotmode = find_all_dots(arg1);

  if (*arg2 && (dotmode != FIND_INDIV)) {
    send_to_char("You can't specify the same body location for more than one item!\r\n", ch);
    return;
  }
  if (dotmode == FIND_ALL) {
    for (obj = ch->carrying; obj; obj = next_obj) {
      next_obj = obj->next_content;
      if (CAN_SEE_OBJ(ch, obj) && (where = find_eq_pos(ch, obj, 0)) >= 0 &&
	  GET_OBJ_TYPE(obj) != ITEM_FOOD) {
	items_worn++;
	perform_wear(ch, obj, where);
      }
    }
    if (!items_worn)
      send_to_char("You don't seem to have anything wearable.\r\n", ch);
  } else if (dotmode == FIND_ALLDOT) {
    if (!*arg1) {
      send_to_char("Wear all of what?\r\n", ch);
      return;
    }
    if (!(obj = get_obj_in_list_vis(ch, arg1, ch->carrying))) {
      sprintf(buf, "You don't seem to have any %ss.\r\n", arg1);
      send_to_char(buf, ch);
    } else
      while (obj) {
	next_obj = get_obj_in_list_vis(ch, arg1, obj->next_content);
	if ((where = find_eq_pos(ch, obj, 0)) >= 0)
	  perform_wear(ch, obj, where);
	else
	  act("You can't wear $p.", FALSE, ch, obj, 0, TO_CHAR);
	obj = next_obj;
      }
  } else {
    if (!(obj = get_obj_in_list(arg1, ch->carrying))) {
      if (!(obj = get_held_object_in_equip_vis(ch, arg1, ch->equipment, &i))) {
	sprintf(buf, "You don't seem to have %s %s.\r\n", AN(arg1), arg1);
	send_to_char(buf, ch);
      } else {
	obj_to_char(unequip_char(ch, i), ch);
	if ((where = find_eq_pos(ch, obj, arg2)) >= 0)
	  perform_wear(ch, obj, where);
	else if (!*arg2)
	  act("You can't wear $p.", FALSE, ch, obj, 0, TO_CHAR);
      }
    } else {
      if ((where = find_eq_pos(ch, obj, arg2)) >= 0)
	perform_wear(ch, obj, where);
      else if (!*arg2)
	act("You can't wear $p.", FALSE, ch, obj, 0, TO_CHAR);
    }
  }
}



ACMD(do_wield)
{
  struct obj_data *obj, *obj2;
  int i;

  one_argument(argument, arg);

  if (!*arg) {
    send_to_char("Wield what?\r\n", ch);
    return;
  } else if (!(obj = get_obj_in_list_vis(ch, arg, ch->carrying))) {
    if ((obj = get_object_in_equip_vis(ch, arg, ch->equipment, &i))) {
      if (obj == GET_EQ(ch, WEAR_HOLD)) {
	unequip_char(ch, WEAR_HOLD);
	if (!perform_wear(ch, obj, WEAR_WIELD))
	  perform_wear(ch, obj, WEAR_HOLD);
	return;
      } else if (obj == GET_EQ(ch, WEAR_HOLD2)) {
	unequip_char(ch, WEAR_HOLD2);
	if (!perform_wear(ch, obj, WEAR_WIELD))
	  perform_wear(ch, obj, WEAR_HOLD2);
	return;
      } else if (CAN_WEAR(obj, ITEM_WEAR_WIELD) && IS_OBJ_STAT(obj, ITEM_ONEORTWOHANDS)) {
	obj2 = read_object(GET_OBJ_VNUM(obj) + 1, VIRTUAL);
	if (!obj2) {
	  vlogf("Object #%d was set as 1/2 handed, but appears to not be.", GET_OBJ_NUM(obj));
	  return;
	}
	if (GET_EQ(ch, WEAR_WIELD)) {
	  unequip_char(ch, WEAR_WIELD);
	  extract_obj(obj);
	} else if (GET_EQ(ch, WEAR_WIELD2)) {
	  unequip_char(ch, WEAR_WIELD2);
	  extract_obj(obj);
	}
	obj_to_char(obj2, ch);
	SET_BIT((obj2)->obj_flags.extra2_flags, ITEM2_REMOVE_2HAND);
	perform_wear(ch, obj2, WEAR_WIELD);
	return;
      } else if (CAN_WEAR(obj, ITEM_WEAR_WIELD) && IS_OBJ_STAT(obj, ITEM_ONEORTWOHANDS)) {
	sprintf(buf, "%s %s is already being wielded two-handed.\r\n", AN(arg), arg);
	CAP(buf);
	send_to_char(buf, ch);
	return;
      } else {
	sprintf(buf, "%s %s can't be used two-handed.\r\n", AN(arg), arg);
	CAP(buf);
	send_to_char(buf, ch);
	return;
      }
    } else {
      sprintf(buf, "You don't seem to have %s %s.\r\n", AN(arg), arg);
      send_to_char(buf, ch);
      return;
    }
  }
  if (!CAN_WEAR(obj, ITEM_WEAR_WIELD)) {
    send_to_char("You can't wield that.\r\n", ch);
    return;
  }
  if (GET_OBJ_WEIGHT(obj) > str_app[STRENGTH_APPLY_INDEX(ch)].wield_w) {
    send_to_char("It's too heavy for you to use.\r\n", ch);
    return;
  }
  if (GET_EQ(ch, WEAR_WIELD))
    perform_wear(ch, obj, WEAR_WIELD2);
  else
    perform_wear(ch, obj, WEAR_WIELD);
}



ACMD(do_grab)
{
  struct obj_data *obj;

  one_argument(argument, arg);

  if (!*arg)
    send_to_char("Hold what?\r\n", ch);
  else if (!(obj = get_obj_in_list(arg, ch->carrying))) {
    sprintf(buf, "You don't seem to have %s %s.\r\n", AN(arg), arg);
    send_to_char(buf, ch);
  } else {
    if (!CAN_WEAR(obj, ITEM_WEAR_HOLD) && GET_OBJ_TYPE(obj) != ITEM_WAND &&
      GET_OBJ_TYPE(obj) != ITEM_STAFF && GET_OBJ_TYPE(obj) != ITEM_SCROLL
	&& GET_OBJ_TYPE(obj) != ITEM_POTION) {
      send_to_char("You can't hold that.\r\n", ch);
      return;
    }
    if (GET_EQ(ch, WEAR_HOLD))
      perform_wear(ch, obj, WEAR_HOLD2);
    else
      perform_wear(ch, obj, WEAR_HOLD);
  }
}



void perform_remove(struct char_data *ch, int pos)
{
  struct obj_data *obj;

  if (!(obj = GET_EQ(ch, pos)))
    log("Error in perform_remove: bad pos passed.");
  else if (IS_OBJ_STAT(obj, ITEM_NODROP))
    act("You can't seem to remove $p.", FALSE, ch, obj, 0, TO_CHAR);
  else if (IS_CARRYING_N(ch) >= CAN_CARRY_N(ch))
    act("$p: you can't carry that many items!", FALSE, ch, obj, 0, TO_CHAR);
  else {
    if (!remove_otrigger(obj, ch))
      return;
    if ((pos == WEAR_HOLD) || (pos == WEAR_HOLD2)) {
      put_obj_to_room(unequip_char(ch, pos), ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
      act("You remove $p and set it down.", FALSE, ch, obj, 0, TO_CHAR);
      act("$n removes $p and sets it down.", TRUE, ch, obj, 0, TO_PROXIMITY);
    } else if (perform_wear(ch, unequip_char(ch, pos), WEAR_HOLD) == 0) {
      put_obj_to_room(obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
      act("You remove $p and set it down.", FALSE, ch, obj, 0, TO_CHAR);
      act("$n removes $p and sets it down.", TRUE, ch, obj, 0, TO_PROXIMITY);
    }
  }
}



ACMD(do_remove)
{
  struct obj_data *obj;
  int i, dotmode, found = 0, msg;

  one_argument(argument, arg);

  if (!*arg) {
    send_to_char("Remove what?\r\n", ch);
    return;
  }
  /* lemme check for a board FIRST */
  for (obj = ch->carrying; obj; obj = obj->next_content) {
    if (GET_OBJ_TYPE(obj) == ITEM_BOARD) {
      found = 1;
      break;
    }
  }
  if (!obj) {
    for (obj = world[ch->in_room].contents; obj; obj = obj->next_content) {
      if (GET_OBJ_TYPE(obj) == ITEM_BOARD) {
	found = 1;
	break;
      }
    }
  }
  if (found) {
    if (!isdigit(*arg) || (!(msg = atoi(arg)))) {
      found = 0;
    } else {
      Board_remove_msg(GET_OBJ_VNUM(obj), ch, msg);
    }
  }
  if (!found) {
    dotmode = find_all_dots(arg);

    /* The command may not work, but it's easier to just save now, rather
       than in each of the various subfunctions */
    if (IS_PC(ch))
      Crash_crashsave(ch);

    if (dotmode == FIND_ALL) {
      found = 0;
      for (i = 0; i < NUM_WEARS; i++)
	if (GET_EQ(ch, i)) {
	  perform_remove(ch, i);
	  found = 1;
	}
      if (!found)
	send_to_char("You're not using anything.\r\n", ch);
    } else if (dotmode == FIND_ALLDOT) {
      if (!*arg)
	send_to_char("Remove all of what?\r\n", ch);
      else {
	found = 0;
	for (i = 0; i < NUM_WEARS; i++)
	  if (GET_EQ(ch, i) && CAN_SEE_OBJ(ch, GET_EQ(ch, i)) &&
	      isname(arg, GET_EQ(ch, i)->name)) {
	    perform_remove(ch, i);
	    found = 1;
	  }
	if (!found) {
	  sprintf(buf, "You don't seem to be using any %ss.\r\n", arg);
	  send_to_char(buf, ch);
	}
      }
    } else {
      if (!(obj = get_object_in_equip_vis(ch, arg, ch->equipment, &i))) {
	sprintf(buf, "You don't seem to be using %s %s.\r\n", AN(arg), arg);
	send_to_char(buf, ch);
      } else
	perform_remove(ch, i);
    }
  }
}

ACMD(do_loot)
{
  char arg1[MAX_INPUT_LENGTH];
  char arg2[MAX_INPUT_LENGTH];

  int found = 0, mode, i;

  struct char_data *tmp_char;
  struct obj_data *cont, *source;
  struct obj_data *obj, *next_obj;

  /* The command may not work, but it's easier to just save now, rather
     than in each of the various subfunctions */
  if (IS_PC(ch))
    Crash_crashsave(ch);

  two_arguments(argument, arg1, arg2);

  if (!*arg1 || !*arg2) {
    send_to_char("Syntax: loot <room|source>(on ground) <container>(equipped)\r\n", ch);
    return;
  }
  //mode = generic_find(arg2, FIND_OBJ_EQUIP, ch, &tmp_char, &cont);
  cont = get_object_in_equip_vis(ch, arg2, ch->equipment, &i);
  if (!cont) {
    sprintf(buf, "You don't see %s %s here.\r\n", AN(arg2), arg2);
    send_to_char(buf, ch);
    return;
  } else if (!IS_CONTAINER(cont)) {
    act("$p is not a container.", FALSE, ch, cont, 0, TO_CHAR);
    return;
  } else if (IS_SET(GET_OBJ_VAL(cont, 1), CONT_CLOSED) &&
	     GET_OBJ_TYPE(cont) == ITEM_CONTAINER) {
    send_to_char("You'd better open it first!\r\n", ch);
    return;
  }
  if (strcasecmp(arg1, "room") == 0) {	/*search room */
    for (obj = world[ch->in_room].contents; obj; obj = next_obj) {
      next_obj = obj->next_content;
      if (CAN_SEE_OBJ(ch, obj)) {
	found = 1;
	if (!ProximityCheck(ch, obj, NULL, -1)) {
	  if (CAN_SEE_OBJ(ch, obj) && str_cmp(arg, "all")) {
	    sprintf(buf, "%s %s is too far to get.\r\n", AN(arg), arg);
	    send_to_char(buf, ch);
	  }
	  continue;
	}
	if (checkgetput(ch, obj))
	  continue;
	if (get_otrigger(obj, ch)) {
	  obj_from_room(obj);
	  act("You pick up $p.", FALSE, ch, obj, 0, TO_CHAR);
	  act("$n picks up $p.", TRUE, ch, obj, 0, TO_PROXIMITY);
	  obj_to_char(obj, ch);
	}
      }
    }
    if (!found) {
      send_to_char("There doesn't seem to be anything here.\r\n", ch);
    }
  } else {
    mode = generic_find(arg1, FIND_OBJ_ROOM, ch, &tmp_char, &source);
    if (source == NULL) {
      send_to_char("You don't see that here.\r\n", ch);
      return;
    }
    for (obj = source->contains; obj; obj = next_obj) {
      next_obj = obj->next_content;
      if (CAN_SEE_OBJ(ch, obj))
	found = 1;
      if (checkgetput(ch, obj))
	continue;
      if (get_otrigger(obj, ch)) {
	obj_from_obj(obj);
	act("You get $p from $P.", FALSE, ch, obj, cont, TO_CHAR);
	act("$n gets $p from $P.", TRUE, ch, obj, cont, TO_PROXIMITY);
	obj_to_char(obj, ch);
      }
    }
    if (!found) {
      act("$p seems to be empty.", FALSE, ch, cont, 0, TO_CHAR);
    }
  }
  found = 0;
  for (obj = ch->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    found = 1;
    perform_put(ch, obj, cont);
  }
  if (!found) {
    send_to_char("You don't seem to have anything to put in it.\r\n", ch);
  }
}
