//******************************************************************************
//      GREY HAWK PROCS
// File Name         : kanthas_procs.c
// Programmers' Info : Kanthas(wau@intergate.bc.ca)
//******************************************************************************

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"

const int Ring_Level = 12;

/*   external vars  */
extern struct room_data *world;
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct time_info_data time_info;
extern char *dirs[];
extern int rev_dir[];
extern char *activating_commands[];

int DetermineCharges(char* Arg);
void ASSIGNMOB(int mob, SPECIAL(fname));
void ASSIGNOBJ(int obj, SPECIAL(fname));
void ASSIGNROOM(int room, SPECIAL(fname));
void ASSIGNZONE(int zone, SPECIAL(fname));


void assign_kanthas_procs(void)
{
	/* mob procs declerations */
	SPECIAL(mob_proc_quaggoth_race);

   /* obj procs declerations */
	SPECIAL(ring_proc_mammal_control);
	SPECIAL(ring_proc_ram);
	SPECIAL(ring_proc_shocking_grasp);
	SPECIAL(ring_proc_blinking);

   /* mob proc assignments */
   ASSIGNMOB(6217, mob_proc_quaggoth_race);

	/* obj proc assignments */
	ASSIGNOBJ(2014, ring_proc_mammal_control);
	ASSIGNOBJ(2017, ring_proc_ram);
	ASSIGNOBJ(2019, ring_proc_shocking_grasp);
	ASSIGNOBJ(2002, ring_proc_blinking);
}	//assign_kanthas_procs



/* DetermineCharges - determines charges selected by user */
int DetermineCharges(char* Arg)
{
   int Counter = 0, ArgLen;

   ArgLen = strlen(Arg);
   while (Arg[Counter] != '\0' && Counter < ArgLen)
   {
   	if (isdigit(Arg[Counter]))
      	return Arg[Counter] - '0';

      Counter++;
   }	//while
	return 0;
}	//DetermineCharges


/*************************************************************************/
/* mob procs */
/*************************************************************************/

SPECIAL(mob_proc_quaggoth_race)
{
  /* confirm race */
  if ((GET_RACE(ch) != RACE_QUAGGOTH) || (cmd))
    return FALSE;


  /* proc - start */
  if (IS_FIGHTING(ch) && GET_HIT(ch) < (GET_MAX_HIT(ch) / 4) && !IS_AFFECTED(ch, AFF_CURSE)) {
    act("$n's hair stands on end as $e screams out in rage.", FALSE, ch, 0, 0, PROXIMITY);
    GET_DAMROLL(ch) += 2;
    GET_HITROLL(ch) += 2;
    SET_BIT(AFF_FLAGS(ch), AFF_CURSE);
    return TRUE;
  }				//if
  /* proc - end */
  else if (!IS_FIGHTING(ch) && IS_AFFECTED(ch, AFF_CURSE)) {
    act("$n lets out a deep breath as $e overcomes $s rage.", FALSE, ch, 0, 0, PROXIMITY);
    GET_DAMROLL(ch) -= 2;
    GET_HITROLL(ch) -= 2;
    REMOVE_BIT(AFF_FLAGS(ch), AFF_CURSE);
    return TRUE;
  }				//else if

  return FALSE;
}				//quaggoth


/*************************************************************************/
/* obj procs */
/*************************************************************************/
/* obj - rings */

SPECIAL(ring_proc_mammal_control)
{
  int Level_Diff, Die;
  char mob_name[256], ring_name[256];
  struct char_data *victim = NULL, *vict, *vict_next;
  struct obj_data *obj;

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;
  if (!CMD_IS("invoke"))
    return FALSE;

  two_arguments(argument, ring_name, mob_name);

  /* double check we are speaking of the same object */
  if (!isname("mammal", ring_name))
    return FALSE;

  /* find the victim */
  for (vict = world[ch->in_room].people; vict; vict = vict_next) {
    vict_next = vict->next_in_room;	/* the dying victim bug */
    if (vict == ch)
      continue;
    if (isname(mob_name, GET_NAME(vict))) {
      victim = vict;
      break;
    }				//if
  }				//for

  if (!vict) {
    send_to_char(NOPERSON, ch);
    return TRUE;
  }				//if
  //get obj pointer

  obj = me;

  //chance of breaking
  if (!number(0, 200)) {
    act("$n puts too much strain on $p as it shatters.", FALSE, ch, obj, 0, PROXIMITY);
    act("You put too much strain on $p as it shatters.", FALSE, ch, obj, 0, TO_CHAR);
    if (ch->equipment[WEAR_FINGER_L] && ch->equipment[WEAR_FINGER_L] == obj)
      obj_to_char(unequip_char(ch, WEAR_FINGER_L), ch);
    else if (ch->equipment[WEAR_FINGER_R] && ch->equipment[WEAR_FINGER_R] == obj)
      obj_to_char(unequip_char(ch, WEAR_FINGER_R), ch);
    else
      log("SYSERR: ring_of_mammal_control() lost the object!");

    /* irregardless of where its worn, we already know it existed, so just
       junk it and stop whining */
    extract_obj(obj);
  }				//if

  Level_Diff = GET_LEVEL(ch) - GET_LEVEL(victim);
  Die = 10;

  /*level differece chance effects */
  if (Level_Diff >= 0)
    Die += Level_Diff / 2;
  else
    Die -= Level_Diff / 2;

  if (Die <= 0)
    Die = 1;

  //check for mammal and int
  if (!IS_MAMMAL(victim) || (GET_INT(victim) > 4))
    Die = 0;

  /* Fail to charm */
  if (dice(Die, 10) < 50) {
    if (Level_Diff <= -10)
      set_fighting(victim, ch);

    act("$n $p spa&+Wrk&nles briefly, but $N is not impressed.", FALSE, ch, obj, victim, TO_OTHERS);
    act("$n $p spa&+Wrk&nles briefly, but fails to take control of &=LRYOU&n. ", FALSE, ch, obj, victim, TO_VICT);
    act("Your $p spa&+Wrk&nles briefly, but $N is not impressed.", FALSE, ch, obj, victim, TO_CHAR);
    return TRUE;
  }				//if

  /*SUCCESS of charm */
  act("$n $p spa&+Wrk&nles briefly, and $N begins to follow him. ", FALSE, ch, obj, victim, TO_OTHERS);
  act("$n $p spa&+Wrk&nles briefly, and &=LRYOU&n begin to follow $m. ", FALSE, ch, obj, victim, TO_VICT);
  act("Your $p spa&+Wrk&nles briefly, and $N begins to follow you. ", FALSE, ch, obj, victim, TO_CHAR);
//  call_magic(ch, victim, NULL, find_spells_num("charm mammal"), GET_LEVEL(ch), CAST_SPELL, NULL);
  r_charm(Ring_Level, ch, NULL, victim, NULL, NULL, SPELL_CHARM_MAMMAL);
//(*routines) (int, struct char_data *, struct obj_data *, struct char_data *,
//	 struct obj_data *, struct location_data *, int)
//  CALL_ROUTINE(r_charm, GET_LEVEL(ch), victim, NULL, NULL);
//#define CALL_ROUTINE(spellname,level, vict,obj,loc)\
//		(spellname(level, NULL, NULL, vict, obj, loc, 0))
//  SET_BIT(AFF_FLAGS(victim), AFF_CHARM);
//  add_follower(victim, ch);
  return TRUE;
}				//mammal control


SPECIAL(ring_proc_ram)
{
  int charges_used = 1, vict_str, vict_svspell, cardinal_direction = -1;
  char vict_name[256], ring_name[256];
//  char vict_name[256], ring_name[256], charges[3];
  struct char_data *victim = NULL, *vict, *vict_next;
  struct obj_data *obj, *door = NULL;
  bool vict_savethrow, exit = FALSE;
  int open_door_chances[6] = {6, 8, 10, 16, 16, 17};

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;
  if (!CMD_IS("invoke"))
    return FALSE;

  two_arguments(argument, ring_name, vict_name);

  /* double check we are speaking of the same object */
  if (!isname("ram", ring_name))
    return FALSE;

  /* get obj pointer */
  obj = me;

  /* find the victim */
  for (vict = world[ch->in_room].people; vict; vict = vict_next) {
    vict_next = vict->next_in_room;	/* the dying victim bug */
    if (vict == ch)
      continue;
    if (isname(vict_name, GET_NAME(vict))) {
      victim = vict;
      break;
    }				//if
  }				//for

  /* if no victim then look for door */
  if (!vict) {
    cardinal_direction = search_block(vict_name, dirs, FALSE);
    if (world[ch->in_room].dir_option[cardinal_direction])
      exit = TRUE;
  }				//if

  /* inform char, victim could not be found */
  if (!vict && !exit) {
    send_to_char(NOPERSON, ch);
    return TRUE;
  }				//if

  /* range of ring is 30ft */
  if (vict && distBetweenCharAndVict(ch, victim) > 3) {
    act("$n's $p glows slightly and then disapates.", FALSE, ch, obj, victim, PROXIMITY);
    act("$N is too far away.", FALSE, ch, obj, victim, TO_CHAR);
    return TRUE;
  }	//if

  /* determine charges wanted */
//  one_argument(argument, charges);
  /* determine charges used */
//  if (charges)
//	 charges_used = atoi(charges);
	charges_used = DetermineCharges(argument);
  /* comfirm charges are within range */
  if (charges_used > 3)
    charges_used = 3;
  else if (charges_used < 1)
    charges_used = 1;
  /* check for obj charge availability */
  if (charges_used > GET_OBJ_VAL(obj, 1))
    charges_used = GET_OBJ_VAL(obj, 1);
  /* out of charges */
  if (!charges_used) {
    act("$n's $p glows slightly and then disapates.", FALSE, ch, obj, 0, PROXIMITY);
    act("Your $p glows slightly and then disapates.", FALSE, ch, obj, 0, TO_CHAR);
    return TRUE;
  }				//if
  /* reduce charges to obj */
  GET_OBJ_VAL(obj, 1) -= charges_used;

  /* if ring is used to open a door */
  if (!vict && exit) {
    if (DOOR_IS_OPEN(IN_ROOM(ch), door, cardinal_direction)) {
      act("$n's $p glows slighty and then disapates.", FALSE, ch, obj, 0, PROXIMITY);
      act("Your $p glows slighty and then disapates.", FALSE, ch, obj, 0, TO_CHAR);
      return TRUE;
    }				//if

    /* determine the chances of opening door */
    if (DOOR_IS_UNLOCKED(IN_ROOM(ch), door, cardinal_direction))
      charges_used += 3;

    /* roll for success to open door */
    if (number(1, 20) <= open_door_chances[charges_used - 1]) {
      /* success in opening door */
      act("$n's $p &+Lsummons forth a &+Cmagical&+L ram's head, which &+Wbashes &+Ldown a door. &n", FALSE, ch, obj, 0, PROXIMITY);
      act("&+LYour $p &+Lsummons forth a &+Cmagical&+L ram's head, which &+Wbashes &+Ldown a door. &n", FALSE, ch, obj, 0, TO_CHAR);
      OPEN_DOOR(IN_ROOM(ch), door, cardinal_direction);
    }				//if
    else			/* failure to open */
    {
    	act("$n's $p&+L summons forth a &+Cmagical&+L ram's head, which &+rfails &+Lto budge a door. &n", FALSE, ch, obj, 0, PROXIMITY);
    	act("&+LYour $p &+Lsummons forth a &+Cmagical&+L ram's head, which &+rfails &+Lto budge a door. &n", FALSE, ch, obj, 0, TO_CHAR);
    }	//else
    return TRUE;
  }				//if

  /* vict_svspell modifiers */
  vict_svspell = GET_SAVE(victim, SAVING_SPELL);
  /* str modifier */
  vict_str = GET_STR(victim);
  if (vict_str < 12)
    vict_svspell += 2;
  else if (vict_str > 20)
    vict_svspell -= 6;
  else if (vict_str > 17)
    vict_svspell -= 3;

  /* weight mod */
  if (GET_WEIGHT(victim) > 1000)
    vict_svspell -= 2;
  /* charge mod */
  vict_svspell += charges_used + 1;
  /* size mod */
  if (GET_SIZE(victim) < SIZE_MEDIUM)
    vict_svspell++;
  else if (GET_SIZE(victim) > SIZE_MEDIUM)
    vict_svspell -= 2;
  /* leg mod */

  /* sv_spell throw - determine if victim will be knocked down/saved */
  vict_savethrow = (dice(1, 20) >= vict_svspell) ? TRUE : FALSE;

  /* produce effects */
  if (vict_savethrow) {
    /* regular damage */
    if (charges_used < 3)
    {
		act("$n's $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L goat to pummel&n $N. ", FALSE, ch, obj, victim, TO_OTHERS);
		act("$n's $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L goat to pummel &=LRYOU&+L.&n ", FALSE, ch, obj, victim, TO_VICT);
		act("&+LYour&n $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L goat to pummel&n $N. ", FALSE, ch, obj, victim, TO_CHAR);
    }	//if
    else
    {
		act("$n's $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L ram to pummel&n $N. ", FALSE, ch, obj, victim, TO_OTHERS);
		act("$n's $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L ram to pummel &=LRYOU&+L.&n ", FALSE, ch, obj, victim, TO_VICT);
		act("&+LYour &n$p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L ram to pummel&n $N. ", FALSE, ch, obj, victim, TO_CHAR);
    }	//else
  }	//if

  else {
    /* knock down victim */
    if (charges_used < 3)
    {
		act("$n's $p &+Lsummons forth a &+Cmagical &+Lgoat's head, which &+Wslams &+Linto&n $N &+Wknocking&+L $M to the ground. &n", FALSE, ch, obj, victim, TO_OTHERS);
		act("$n's $p &+Lsummons forth a &+Cmagical &+Lgoat's head, which &+Wslams &+Linto &=LRYOU &+Wknocking&+L you to the ground. &n", FALSE, ch, obj, victim, TO_VICT);
		act("&+LYour &n$p &+Lsummons forth a &+Cmagical &+Lgoat's head, which &+Wslams &+Linto $N &+Wknocking&+L $M to the ground. &n", FALSE, ch, obj, victim, TO_CHAR);
    }	//if
    else
    {
		act("$n's $p &+Lsummons forth a &+Cmagical &+Lram's head, which &+Wslams &+Linto&n $N &+Wknocking&+L $M to the ground. &n", FALSE, ch, obj, victim, TO_OTHERS);
		act("$n's $p &+Lsummons forth a &+Cmagical &+wr&+Wam&+L's head, which &+Wslams &+Linto &=LRYOU &+Wknocking&+L you to the ground. &n", FALSE, ch, obj, victim, TO_VICT);
		act("&+LYour &n$p &+Lsummons forth a &+Cmagical &+Lram's head, which &+Wslams &+Linto $N &+Wknocking&+L $M to the ground. &n", FALSE, ch, obj, victim, TO_CHAR);
    }	//else
    SET_POS(victim, GET_STAT(ch) + POS_SITTING);
  }				//else

  /* set damage */
  damage(ch, victim, dice(charges_used, 6), -1, -1, -1);
  return TRUE;
}				//ram


SPECIAL(ring_proc_shocking_grasp)
{
  char ring_name[256];
  struct obj_data *obj;

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;
  if (!CMD_IS("invoke"))
    return FALSE;

  one_argument(argument, ring_name);
  obj = me;

  /* double check we are speaking of the same object */
  if (!isname("grasp", ring_name) && !isname("shocking", ring_name))
    return FALSE;

  /* recharge item after 1 hour */
  if (!GET_OBJ_VAL(obj, 1) && !GET_OBJ_TIMER(obj))
    GET_OBJ_VAL(obj, 1) = 3;

  /* modify charges to obj + call effects */
  if (GET_OBJ_VAL(obj, 1) > 0) {	/* apply shocking grasp effect */
    act("$n's $p glows brightly.", FALSE, ch, obj, 0, PROXIMITY);
    act("Your $p glows brightly.", FALSE, ch, obj, 0, TO_CHAR);
    call_magic(ch, ch, NULL, find_spells_num("shocking grasp"), Ring_Level, CAST_SPELL, NULL);
    GET_OBJ_VAL(obj, 1)--;
    if (!GET_OBJ_VAL(obj, 1))
      GET_OBJ_TIMER(obj) = 1;
  }	//if
  else /* no proc message */
  {
    act("$n's $p glows slightly and then disapates.", FALSE, ch, obj, 0, PROXIMITY);
    act("Your $p glows slightly and then disapates.", FALSE, ch, obj, 0, TO_CHAR);
  }	//else
  return TRUE;
}			//ring_shocking_grasp


SPECIAL(ring_proc_blinking)
{
  char keyword[256];
  struct obj_data *obj;

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;
  if (!CMD_IS("say") && !CMD_IS("'"))
    return FALSE;

  one_argument(argument, keyword);
  obj = me;

  /* double check we are speaking of the same object */
  if (!isname("blink", keyword))
    return FALSE;

  /* make sure someone is wearing it */
  if ((ch->equipment[WEAR_FINGER_L] && ch->equipment[WEAR_FINGER_L] != obj) &&
  	  (ch->equipment[WEAR_FINGER_R] && ch->equipment[WEAR_FINGER_R] != obj))
  	 return FALSE;

  /* recharge item after 1 hour */
  if (!GET_OBJ_VAL(obj, 1) && !GET_OBJ_TIMER(obj))
    GET_OBJ_VAL(obj, 1) = 1;

  /* modify charges to obj + call effects */
  if (GET_OBJ_VAL(obj, 1) > 0) {	/* apply shocking grasp effect */
    act("$n's $p glows brightly.", FALSE, ch, obj, 0, PROXIMITY);
    act("Your $p glows brightly.", FALSE, ch, obj, 0, TO_CHAR);
    call_magic(ch, ch, NULL, find_spells_num("blink"), Ring_Level, CAST_SPELL, NULL);
    GET_OBJ_VAL(obj, 1) = 0;
    GET_OBJ_TIMER(obj) = 1;
  }	//if
  else /* no proc message */
  {
    act("$n's $p glows slightly and then disapates.", FALSE, ch, obj, 0, PROXIMITY);
    act("Your $p glows slightly and then disapates.", FALSE, ch, obj, 0, TO_CHAR);
  }	//else
  return TRUE;
}	//ring
