/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#define __COMM_C__

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "house.h"
#include "events.h"
#include "chatlink.h"
#include "dg_scripts.h"
#include "ident.h"

#ifdef HAVE_ARPA_TELNET_H
#include <arpa/telnet.h>
#else
#include "telnet.h"
#endif

#ifndef INVALID_SOCKET
#define INVALID_SOCKET -1
#endif

/* externs */
extern int restricted;
extern int mini_mud;
extern int no_rent_check;
extern FILE *player_fl;
extern int DFLT_PORT;
extern char *DFLT_DIR;
extern int MAX_PLAYERS;
extern int MAX_DESCRIPTORS_AVAILABLE;

extern struct room_data *world;	/* In db.c */
extern struct player_special_data dummy_mob;	/* In db.c */
extern int top_of_world;	/* In db.c */
extern int top_of_zone_table;
extern struct zone_data *zone_table;
extern struct index_data *mob_index;
extern struct time_info_data time_info;		/* In db.c */
extern char *dirs[];
extern char help[];
extern int char_hearing_modifier(struct char_data *ch);
void set_minute(int pulse);

/* local globals */
struct descriptor_data *descriptor_list = NULL;		/* master desc list */
struct txt_block *bufpool = 0;	/* pool of large output buffers */
int buf_largecount = 0;		/* # of large buffers which exist */
int buf_overflows = 0;		/* # of overflows of output */
int buf_switches = 0;		/* # of switches from small to large buf */
int circle_shutdown = 0;	/* clean shutdown */
int circle_reboot = 0;		/* reboot the game after a shutdown */
int no_specials = 0;		/* Suppress ass. of special routines */
int total_bytes_written = 0;	/* bandwidth display    */
int max_players = 0;		/* max descriptors available */
int tics = 0;			/* for extern checkpointing */
int scheck = 0;			/* for syntax checking mode */
#ifdef MUDFTP
int ftp_control = -1;		/* mudFTP control socket, -1 is off. */
/* Type of incoming connection: normal or mudFTP? */
typedef enum {
  NDESC_NORMAL, NDESC_FTP
} ndesc_t;

#endif
int dg_act_check = 1;		/* toggle for act_trigger */
unsigned long dg_global_pulse = 0;	/* number of pulses since game start */
extern int nameserver_is_slow;	/* see config.c */
extern int auto_save;		/* see config.c */
extern int autosave_time;	/* see config.c */
struct timeval null_time;	/* zero-valued time structure */
unsigned long pulse = 0;	/* number of pulses since game started */
int port;			/* port mud is running on */

static bool fCopyOver;		/* Are we booting in copyover mode? */
int mother_desc;		/* Now a global */

/* functions in this file */
int get_from_q(struct txt_q *queue, char *dest, int *aliased);
void init_game(int port);
void signal_setup(void);
void game_loop(int mother_desc);
int init_socket(int port);
#ifndef MUDFTP
int new_descriptor(int s);
#else
int new_descriptor(int s, ndesc_t type);
void greet_ftp(struct descriptor_data *d);
#endif
int get_max_players(void);
int process_output(struct descriptor_data *t);
int process_input(struct descriptor_data *t);
void close_socket(struct descriptor_data *d);
struct timeval timediff(struct timeval a, struct timeval b);
struct timeval timeadd(struct timeval a, struct timeval b);
void flush_queues(struct descriptor_data *d);
void nonblock(socket_t s);
int perform_subst(struct descriptor_data *t, char *orig, char *subst);
int perform_alias(struct descriptor_data *d, char *orig);
void record_usage(void);
char *make_prompt(struct descriptor_data *point);
void check_idle_passwords(void);
void heartbeat(int pulse);
void init_descriptor(struct descriptor_data *newd, int desc);
int set_sendbuf(socket_t s);
void copyover_recover(void);

/* extern fcnts */
void boot_db(void);
void boot_world(void);
void affect_update(void);	/* In spells.c */
void point_update(void);	/* In limits.c */
void mobile_activity(int pulse);
void call_room_specs(void);
void call_obj_specs(void);
void string_add(struct descriptor_data *d, char *str);
void show_string(struct descriptor_data *d, char *input);
int isbanned(char *hostname);
void weather_and_time(int mode);
void unload_room_maps(int zone);
void write_mud_date_to_file(void);
void master_parser(char *buf, struct char_data *ch, struct room_data *room, struct obj_data *obj, struct char_data *mob);
char *aowdns(struct in_addr sin_addr);
void greyhawk_city_noises(void);

#ifdef __CXREF__
#undef FD_ZERO
#undef FD_SET
#undef FD_ISSET
#undef FD_CLR
#define FD_ZERO(x)
#define FD_SET(x, y) 0
#define FD_ISSET(x, y) 0
#define FD_CLR(x, y)
#endif

/* ============= Ansi color table and data structure  ============== */

struct ansi_color {
  const char *symbol;		/* Symbol used in the game. eg &+symbol */
  const char *fg_code;		/* ^[[fg_codem; = foreground color code */
  const char *bg_code;		/* Same except it's the background color */
} color_table[] =

{

  {
    "L", "30", "40"
  },				/* * Black            */
  {
    "R", "31", "41"
  },				/* * Red              */
  {
    "G", "32", "42"
  },				/* * Green            */
  {
    "Y", "33", "43"
  },				/* * Yellow           */
  {
    "B", "34", "44"
  },				/* * Blue             */
  {
    "M", "35", "45"
  },				/* * Magenta          */
  {
    "C", "36", "46"
  },				/* * Cyan             */
  {
    "W", "37", "47"
  },				/* * White            */
  {
    NULL, NULL, NULL
  }				/* * End of the table */
};

int find_color_entry(int c)
{
  int i = 0;
  char s;

  s = UPPER(c);

  while ((color_table[i].symbol != NULL) && (*color_table[i].symbol != s))
    i++;

  return i;
}

void select_terminal(struct descriptor_data *d, char *arg)
{
  int term;

  if ((term = (int) strtol(arg, NULL, 0)) == 0) {
    if (!*arg)			/* carriage return */
      term = TERM_ANSI;
    else
      term = TERM_UNDEFINED;
  }
  switch (term) {
  case TERM_GENERIC:
    d->term_type = TERM_GENERIC;
    break;
  case TERM_MSP:
    d->term_type = TERM_MSP;
    break;
  case TERM_ANSI:
    d->term_type = TERM_ANSI;
    break;
  case TERM_GREY:
    d->term_type = TERM_GREY;
    break;
  case TERM_CLIENT:
    d->term_type = TERM_CLIENT;
    break;
  default:
    SEND_TO_Q("Unknown terminal type!\n", d);
    SEND_TO_Q("Please re-enter term type (<CR> for ANSI, '1' for Generic, '3' for MSP, '4' for Zmud with plugin, '5' for client (experimental)): \r\n", d);
    return;
  }

  /* if it gets here, we have a valid term type, carry on... */
  STATE(d) = CON_GET_NAME;
  SEND_TO_Q("By what name shall the bards sing your praises\n(entering 'new' will generate a random name for you)\n", d);
}



/***********************************************************************
*  main game loop and related stuff                                    *
***********************************************************************/

int main(int argc, char **argv)
{
  char buf[512];
  int pos = 1;
  char *dir;

  /* Initialize dummy_mob variable. */
  memset(&dummy_mob, 0, sizeof(dummy_mob));


  port = DFLT_PORT;
  dir = DFLT_DIR;

  while ((pos < argc) && (*(argv[pos]) == '-')) {
    switch (*(argv[pos] + 1)) {
    case 'C':			/* -C<socket number> - recover from copyover, this is the control socket */
      fCopyOver = TRUE;
      mother_desc = atoi(argv[pos] + 2);
      break;

    case 'd':
      if (*(argv[pos] + 2))
	dir = argv[pos] + 2;
      else if (++pos < argc)
	dir = argv[pos];
      else {
	log("SYSERR: Directory arg expected after option -d.");
	raise(SIGSEGV);
      }
      break;
    case 'm':
      mini_mud = 1;
      no_rent_check = 1;
      log("Running in minimized mode & with no rent check.");
      break;
    case 'c':
      scheck = 1;
      log("Syntax check mode enabled.");
      break;
    case 'q':
      no_rent_check = 1;
      log("Quick boot mode -- rent check supressed.");
      break;
    case 'r':
      restricted = 1;
      log("Restricting game -- no new players allowed.");
      break;
    case 's':
      no_specials = 1;
      log("Suppressing assignment of special routines.");
      break;
    default:
      sprintf(buf, "SYSERR: Unknown option -%c in argument string.", *(argv[pos] + 1));
      log(buf);
      break;
    }
    pos++;
  }

  if (pos < argc) {
    if (!isdigit(*argv[pos])) {
      fprintf(stderr, "Usage: %s [-c] [-m] [-q] [-r] [-s] [-d pathname] [port #]\n", argv[0]);
      raise(SIGSEGV);
    } else if ((port = atoi(argv[pos])) <= 1024) {
      fprintf(stderr, "SYSERR: Illegal port number.\n");
      raise(SIGSEGV);
    }
  }
  if (chdir(dir) < 0) {
    perror("SYSERR: Fatal error changing to data directory");
    raise(SIGSEGV);
  }
  sprintf(buf, "Using %s as data directory.", dir);
  log(buf);

  if (scheck) {
    boot_world();
    log("Done.");
    raise(SIGSEGV);
  } else {
    sprintf(buf, "Running game on port %d.", port);
    log(buf);
    init_game(port);
  }


  return 0;
}



/* Init sockets, run game, and cleanup sockets */
void init_game(int port)
{

  circle_srandom(time(0));

  log("Finding player limit.");
  max_players = get_max_players();

  if (!fCopyOver) {		/* If copyover mother_desc is already set up */
    log("Opening mother connection.");
    mother_desc = init_socket(port);
  }
  event_init();

  boot_db();
#ifdef MUDFTP
  /* Bind to the mudFTP port, normal port + 6. */
  ftp_control = init_socket(port + 6);
#endif
  log("Signal trapping.");
  signal_setup();

  if (fCopyOver)		/* reload players */
    copyover_recover();

  log("Entering game loop.");

  game_loop(mother_desc);

  log("Closing all sockets.");
  while (descriptor_list)
    close_socket(descriptor_list);

  CLOSE_SOCKET(mother_desc);
  fclose(player_fl);

  if (circle_reboot) {
    log("Rebooting.");
    exit(52);			/* what's so great about HHGTTG, anyhow? */
  }
  log("Normal termination of game.");
}



/*
 * init_socket sets up the mother descriptor - creates the socket, sets
 * its options up, binds it, and listens.
 */
int init_socket(int port)
{
  int s, opt;
  struct sockaddr_in sa;

  /*
   * Should the first argument to socket() be AF_INET or PF_INET?  I don't
   * know, take your pick.  PF_INET seems to be more widely adopted, and
   * Comer (_Internetworking with TCP/IP_) even makes a point to say that
   * people erroneously use AF_INET with socket() when they should be using
   * PF_INET.  However, the man pages of some systems indicate that AF_INET
   * is correct; some such as ConvexOS even say that you can use either one.
   * All implementations I've seen define AF_INET and PF_INET to be the same
   * number anyway, so the point is (hopefully) moot.
   */

  if ((s = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
    perror("SYSERR: Error creating socket");
    raise(SIGSEGV);
  }
#if defined(SO_REUSEADDR)
  opt = 1;
  if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &opt, sizeof(opt)) < 0) {
    perror("SYSERR: setsockopt REUSEADDR");
    raise(SIGSEGV);
  }
#endif

  set_sendbuf(s);

#if defined(SO_LINGER)
  {
    struct linger ld;

    ld.l_onoff = 0;
    ld.l_linger = 0;
    if (setsockopt(s, SOL_SOCKET, SO_LINGER, (char *) &ld, sizeof(ld)) < 0) {
      perror("SYSERR: setsockopt LINGER");
      raise(SIGSEGV);
    }
  }
#endif

  sa.sin_family = AF_INET;
  sa.sin_port = htons(port);
  sa.sin_addr.s_addr = htonl(INADDR_ANY);

  if (bind(s, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
    perror("SYSERR: bind");
    CLOSE_SOCKET(s);
    raise(SIGSEGV);
  }
  nonblock(s);
  listen(s, 5);
  return s;
}


int get_max_players(void)
{
  int max_descs = 0;
  char *method;

/*
 * First, we'll try using getrlimit/setrlimit.  This will probably work
 * on most systems.
 */
#if defined (RLIMIT_NOFILE) || defined (RLIMIT_OFILE)
#if !defined(RLIMIT_NOFILE)
#define RLIMIT_NOFILE RLIMIT_OFILE
#endif
  {
    struct rlimit limit;

    /* find the limit of file descs */
    method = "rlimit";
    if (getrlimit(RLIMIT_NOFILE, &limit) < 0) {
      perror("SYSERR: calling getrlimit");
      raise(SIGSEGV);
    }
    /* set the current to the maximum */
    limit.rlim_cur = limit.rlim_max;
    if (setrlimit(RLIMIT_NOFILE, &limit) < 0) {
      perror("SYSERR: calling setrlimit");
      raise(SIGSEGV);
    }
#ifdef RLIM_INFINITY
    if (limit.rlim_max == RLIM_INFINITY)
      max_descs = MAX_PLAYERS + NUM_RESERVED_DESCS;
    else
      max_descs = MIN(MAX_PLAYERS + NUM_RESERVED_DESCS, limit.rlim_max);
#else
    max_descs = MIN(MAX_PLAYERS + NUM_RESERVED_DESCS, limit.rlim_max);
#endif
  }

#elif defined (OPEN_MAX) || defined(FOPEN_MAX)
#if !defined(OPEN_MAX)
#define OPEN_MAX FOPEN_MAX
#endif
  method = "OPEN_MAX";
  max_descs = OPEN_MAX;		/* Uh oh.. rlimit didn't work, but we have
				 * OPEN_MAX */
#elseif				/* defined (POSIX) */
  /*
   * Okay, you don't have getrlimit() and you don't have OPEN_MAX.  Time to
   * use the POSIX sysconf() function.  (See Stevens' _Advanced Programming
   * in the UNIX Environment_).
   */
  method = "POSIX sysconf";
  errno = 0;
  if ((max_descs = sysconf(_SC_OPEN_MAX)) < 0) {
    if (errno == 0)
      max_descs = MAX_PLAYERS + NUM_RESERVED_DESCS;
    else {
      perror("SYSERR: Error calling sysconf");
      raise(SIGSEGV);
    }
  }
#else
  /* if everything has failed, we'll just take a guess */
  max_descs = MAX_PLAYERS + NUM_RESERVED_DESCS;
#endif

  /* now calculate max _players_ based on max descs */
  max_descs = MIN(MAX_PLAYERS, max_descs - NUM_RESERVED_DESCS);

  if (max_descs <= 0) {
    sprintf(buf, "SYSERR: Non-positive max player limit!  (Set at %d using %s).",
	    max_descs, method);
    log(buf);
    raise(SIGSEGV);
  }
  sprintf(buf, "Setting player limit to %d using %s.", max_descs, method);
  log(buf);
  return max_descs;
}



/*
 * game_loop contains the main loop which drives the entire MUD.  It
 * cycles once every 0.10 seconds and is responsible for accepting new
 * new connections, polling existing connections for input, dequeueing
 * output and sending it out to players, and calling "heartbeat" functions
 * such as mobile_activity().
 */
void game_loop(int mother_desc)
{
  fd_set input_set, output_set, exc_set, null_set;
  struct timeval last_time, before_sleep, opt_time, process_time, now,
   timeout;
  char comm[MAX_INPUT_LENGTH];
  struct descriptor_data *d, *next_d;
  int missed_pulses, maxdesc, aliased, i;

  /* initialize various time values */
  null_time.tv_sec = 0;
  null_time.tv_usec = 0;
  opt_time.tv_usec = OPT_USEC;
  opt_time.tv_sec = 0;
  FD_ZERO(&null_set);

  gettimeofday(&last_time, (struct timezone *) 0);

  /* The Main Loop.  The Big Cheese.  The Top Dog.  The Head Honcho.  The.. */
  while (!circle_shutdown) {

    /* Sleep if we don't have any connections */
    if (descriptor_list == NULL) {
      log("No connections.  Going to sleep.");
      for (i = 0; i < top_of_zone_table; i++)
	if (!zone_table[i].unloaded)
	  unload_room_maps(i);
      FD_ZERO(&input_set);
      FD_SET(mother_desc, &input_set);
#ifdef MUDFTP
      /*
       * Don't bother checking for mudFTP connections since we need a
       * character to go along with them first anyway.
       */
#endif
      if (select(mother_desc + 1, &input_set, (fd_set *) 0, (fd_set *) 0, NULL) < 0) {
	if (errno == EINTR)
	  log("Waking up to process signal.");
	else
	  perror("Select coma");
      } else
	log("New connection.  Waking up.");
      gettimeofday(&last_time, (struct timezone *) 0);
    }
    /* Set up the input, output, and exception sets for select(). */
    FD_ZERO(&input_set);
    FD_ZERO(&output_set);
    FD_ZERO(&exc_set);
    FD_SET(mother_desc, &input_set);
#ifdef MUDFTP
    if (ftp_control >= 0)
      FD_SET(ftp_control, &input_set);
    maxdesc = MAX(mother_desc, ftp_control);
#else
    maxdesc = mother_desc;
#endif
    for (d = descriptor_list; d; d = d->next) {
      if (d->descriptor > maxdesc)
	maxdesc = d->descriptor;
      FD_SET(d->descriptor, &input_set);
      FD_SET(d->descriptor, &output_set);
      FD_SET(d->descriptor, &exc_set);
    }

    /*
     * At this point, we have completed all input, output and heartbeat
     * activity from the previous iteration, so we have to put ourselves
     * to sleep until the next 0.1 second tick.  The first step is to
     * calculate how long we took processing the previous iteration.
     */

    gettimeofday(&before_sleep, (struct timezone *) 0);		/* current time */
    process_time = timediff(before_sleep, last_time);

    /*
     * If we were asleep for more than one pass, count missed pulses and sleep
     * until we're resynchronized with the next upcoming pulse.
     */
    if (process_time.tv_sec == 0 && process_time.tv_usec < OPT_USEC) {
      missed_pulses = 0;
    } else {
      missed_pulses = process_time.tv_sec * PASSES_PER_SEC;
      missed_pulses += process_time.tv_usec / OPT_USEC;
      process_time.tv_sec = 0;
      process_time.tv_usec = process_time.tv_usec % OPT_USEC;
    }

    /* Calculate the time we should wake up */
    last_time = timeadd(before_sleep, timediff(opt_time, process_time));

    /* Now keep sleeping until that time has come */
    gettimeofday(&now, (struct timezone *) 0);
    timeout = timediff(last_time, now);

    /* Go to sleep */
    do {
      if (select(0, (fd_set *) 0, (fd_set *) 0, (fd_set *) 0, &timeout) < 0) {
	if (errno != EINTR) {
	  perror("SYSERR: Select sleep");
	  raise(SIGSEGV);
	}
      }
      gettimeofday(&now, (struct timezone *) 0);
      timeout = timediff(last_time, now);
    } while (timeout.tv_usec || timeout.tv_sec);

    /* Poll (without blocking) for new input, output, and exceptions */
    if (select(maxdesc + 1, &input_set, &output_set, &exc_set, &null_time) < 0) {
      perror("Select poll");
      return;
    }
    /* If there are new connections waiting, accept them. */
    if (FD_ISSET(mother_desc, &input_set))
#ifndef MUDFTP
      new_descriptor(mother_desc);
#else
      new_descriptor(mother_desc, NDESC_NORMAL);
    if (FD_ISSET(ftp_control, &input_set))
      new_descriptor(ftp_control, NDESC_FTP);
#endif

    /* Kick out the freaky folks in the exception set and marked for close */
    for (d = descriptor_list; d; d = next_d) {
      next_d = d->next;
      if (FD_ISSET(d->descriptor, &exc_set) || d->close_me) {
	FD_CLR(d->descriptor, &input_set);
	FD_CLR(d->descriptor, &output_set);
	close_socket(d);
      }
    }

    /* Process descriptors with input pending */
    for (d = descriptor_list; d; d = next_d) {
      next_d = d->next;
      if (FD_ISSET(d->descriptor, &input_set))
	if (process_input(d) < 0)
	  close_socket(d);
    }

    /* process descriptors with ident pending */
    for (d = descriptor_list; d; d = next_d) {
      next_d = d->next;
      if (waiting_for_ident(d))
	ident_check(d, pulse);
    }

    /* Process commands we just read from process_input */
    for (d = descriptor_list; d; d = next_d) {
      next_d = d->next;

      if (!waiting_for_ident(d) && (--(d->wait) <= 0) &&
	  get_from_q(&d->input, comm, &aliased)) {
	if (d->character) {
	  /* Reset the idle timer & pull char back from void if necessary */
	  d->character->char_specials.timer = 0;
	  if (!d->connected && GET_WAS_IN(d->character) != NOWHERE) {
	    if (d->character->in_room != NOWHERE)
	      char_from_room(d->character);
	    char_to_room(d->character, GET_WAS_IN(d->character), -1);
	    GET_WAS_IN(d->character) = NOWHERE;
	    act("$n has returned.", TRUE, d->character, 0, 0, PROXIMITY);
	  }
	}
	d->wait = 1;
	d->has_prompt = 0;

	if (d->str)		/* Writing boards, mail, etc. */
	  string_add(d, comm);
	else if (d->showstr_count)	/* Reading something w/ pager */
	  show_string(d, comm);
#ifndef MUDFTP
	else if (d->connected != CON_PLAYING)	/* In menus, etc. */
	  nanny(d, comm);
	else {			/* else: we're playing normally. */
#else
	else if (STATE(d) != CON_PLAYING) {	/* In menus, etc. */
	  do {
	    nanny(d, comm);
	  } while (STATE(d) == CON_FTP_DATA && get_from_q(&d->input, comm, &aliased));
	} else {		/* else: we're playing normally. */
#endif
	  if (aliased)		/* To prevent recursive aliases. */
	    d->has_prompt = 1;	/* To get newline before next cmd output. */
	  else {
	    if (perform_alias(d, comm))		/* Run it through aliasing system */
	      get_from_q(&d->input, comm, &aliased);
	  }
	  command_interpreter(d->character, comm);	/* Send it to interpreter */
	}
      }
    }

    /* Send queued output out to the operating system (ultimately to user) */
    for (d = descriptor_list; d; d = next_d) {
      next_d = d->next;
      if (*(d->output) && FD_ISSET(d->descriptor, &output_set)) {
	/* Output for this player is ready */
	if (process_output(d) < 0)
	  close_socket(d);
	else
	  d->has_prompt = 1;
      }
    }

    /* Print prompts for other descriptors who had no other output */
    for (d = descriptor_list; d; d = d->next) {
      if (!d->has_prompt) {
	write_to_descriptor(d->descriptor, make_prompt(d));
	d->has_prompt = 1;
      }
    }

    /* Kick out folks in the CON_CLOSE state */
    for (d = descriptor_list; d; d = next_d) {
      next_d = d->next;
      if (STATE(d) == CON_CLOSE)
	close_socket(d);
    }

    /*
     * Now, we execute as many pulses as necessary--just one if we haven't
     * missed any pulses, or make up for lost time if we missed a few
     * pulses by sleeping for too long.
     */
    missed_pulses++;

    if (missed_pulses <= 0) {
      log("SYSERR: **BAD** MISSED_PULSES NONPOSITIVE, TIME GOING BACKWARDS!!");
      missed_pulses = 1;
    }
    /* If we missed more than 30 seconds worth of pulses, just do 30 secs */
    if (missed_pulses > (30 * PASSES_PER_SEC)) {
      log("SYSERR: Missed more than 30 seconds worth of pulses");
      missed_pulses = 30 * PASSES_PER_SEC;
    }
    /* Now execute the heartbeat functions */
    while (missed_pulses--)
      heartbeat(++pulse);

    /* Update tics for deadlock protection */
    tics++;

    /* record the time */
    write_mud_date_to_file();
  }
}


void heartbeat(int pulse)
{
  static int mins_since_crashsave = 0;
  void process_events(void);

  dg_global_pulse++;

  process_events();

  if (!(pulse % PULSE_DG_SCRIPT))
    script_trigger_check();

  /* is there a better way to do this? */
  if (!(pulse % PASSES_PER_SEC)) {
    set_minute(pulse);
    dequeue_paths();
  }
  event_process();

  if (!(pulse % PULSE_ZONE))
    zone_update();

  if (!(pulse % (30 * PASSES_PER_SEC)))		/* 30 seconds */
    greyhawk_city_noises();

  if (!(pulse % (15 * PASSES_PER_SEC)))		/* 15 seconds */
    check_idle_passwords();

  if (!(pulse % PULSE_MOBILE))
    mobile_activity(pulse);	/*either a global or passed.. */

  if (!(pulse % PULSE_SPEC)) {
    call_room_specs();
    call_obj_specs();
  }
  if (!(pulse % (SECS_PER_MUD_HOUR * PASSES_PER_SEC * 24))) {
    check_path();
  }
  if (!(pulse % (SECS_PER_MUD_HOUR * PASSES_PER_SEC))) {
    weather_and_time(1);
    point_update();
    fflush(player_fl);
  }
  if (!(pulse % (SECS_PER_ROUND * PASSES_PER_SEC))) {
    affect_update();
  }
  if (auto_save && !(pulse % (60 * PASSES_PER_SEC))) {	/* 1 minute */
    if (++mins_since_crashsave >= autosave_time) {
      mins_since_crashsave = 0;
      Crash_save_all();
      House_save_all();
    }
  }
  if (!(pulse % (5 * 60 * PASSES_PER_SEC)))	/* 5 minutes */
    record_usage();

}


/* ******************************************************************
   *  general utility stuff (for local use)                            *
   ****************************************************************** */

/*
 *  new code to calculate time differences, which works on systems
 *  for which tv_usec is unsigned (and thus comparisons for something
 *  being < 0 fail).  Based on code submitted by ss@sirocco.cup.hp.com.
 */

/*
 * code to return the time difference between a and b (a-b).
 * always returns a nonnegative value (floors at 0).
 */
struct timeval timediff(struct timeval a, struct timeval b)
{
  struct timeval rslt;

  if (a.tv_sec < b.tv_sec)
    return null_time;
  else if (a.tv_sec == b.tv_sec) {
    if (a.tv_usec < b.tv_usec)
      return null_time;
    else {
      rslt.tv_sec = 0;
      rslt.tv_usec = a.tv_usec - b.tv_usec;
      return rslt;
    }
  } else {			/* a->tv_sec > b->tv_sec */
    rslt.tv_sec = a.tv_sec - b.tv_sec;
    if (a.tv_usec < b.tv_usec) {
      rslt.tv_usec = a.tv_usec + 1000000 - b.tv_usec;
      rslt.tv_sec--;
    } else
      rslt.tv_usec = a.tv_usec - b.tv_usec;
    return rslt;
  }
}

/* add 2 timevals */
struct timeval timeadd(struct timeval a, struct timeval b)
{
  struct timeval rslt;

  rslt.tv_sec = a.tv_sec + b.tv_sec;
  rslt.tv_usec = a.tv_usec + b.tv_usec;

  while (rslt.tv_usec >= 1000000) {
    rslt.tv_usec -= 1000000;
    rslt.tv_sec++;
  }

  return rslt;
}


void record_usage(void)
{
  int sockets_connected = 0, sockets_playing = 0;
  struct descriptor_data *d;
  char buf[256];

  for (d = descriptor_list; d; d = d->next) {
    sockets_connected++;
    if (!d->connected)
      sockets_playing++;
  }

  sprintf(buf, "nusage: %-3d sockets connected, %-3d sockets playing",
	  sockets_connected, sockets_playing);
  log(buf);

#ifdef RUSAGE
  {
    struct rusage ru;

    getrusage(0, &ru);
    sprintf(buf, "rusage: user time: %ld sec, system time: %ld sec, max res size: %ld",
	    ru.ru_utime.tv_sec, ru.ru_stime.tv_sec, ru.ru_maxrss);
    log(buf);
  }
#endif

}



/*
 * Turn off echoing (specific to telnet client)
 */
void echo_off(struct descriptor_data *d)
{
  char off_string[] =
  {
    (char) IAC,
    (char) WILL,
    (char) TELOPT_ECHO,
    (char) 0,
  };

  SEND_TO_Q(off_string, d);
}


/*
 * Turn on echoing (specific to telnet client)
 */
void echo_on(struct descriptor_data *d)
{
  char on_string[] =
  {
    (char) IAC,
    (char) WONT,
    (char) TELOPT_ECHO,
    (char) TELOPT_NAOFFD,
    (char) TELOPT_NAOCRD,
    (char) 0,
  };

  SEND_TO_Q(on_string, d);
}

void write_to_q(char *txt, struct txt_q *queue, int aliased)
{
  struct txt_block *new;

  CREATE(new, struct txt_block, 1);
  CREATE(new->text, char, strlen(txt) + 1);
  strcpy(new->text, txt);
  new->aliased = aliased;

  /* queue empty? */
  if (!queue->head) {
    new->next = NULL;
    queue->head = queue->tail = new;
  } else {
    queue->tail->next = new;
    queue->tail = new;
    new->next = NULL;
  }
}



int get_from_q(struct txt_q *queue, char *dest, int *aliased)
{
  struct txt_block *tmp;

  /* queue empty? */
  if (!queue->head)
    return 0;

  tmp = queue->head;
  strcpy(dest, queue->head->text);
  *aliased = queue->head->aliased;
  queue->head = queue->head->next;

  FREE(tmp->text);
  FREE(tmp);

  return 1;
}



/* Empty the queues before closing connection */
void flush_queues(struct descriptor_data *d)
{
  int dummy;

  if (d->large_outbuf) {
    d->large_outbuf->next = bufpool;
    bufpool = d->large_outbuf;
  }
  while (get_from_q(&d->input, buf2, &dummy));
}

/* Add a new string to a player's output queue */
void write_to_output(const char *txt, struct descriptor_data *t)
{
  int size;

  size = strlen(txt);

  /* if we're in the overflow state already, ignore this new output */
  if (t->bufptr < 0)
    return;

  /* if we have enough space, just write to buffer and that's it! */
  if (t->bufspace >= size) {
    strcpy(t->output + t->bufptr, txt);
    t->bufspace -= size;
    t->bufptr += size;
    return;
  }
  /*
   * If the text is too big to fit into even a large buffer, chuck the
   * new text and switch to the overflow state.
   */
  if (size + t->bufptr > LARGE_BUFSIZE - 1) {
    t->bufptr = -1;
    buf_overflows++;
    return;
  }
  buf_switches++;
  /* if the pool has a buffer in it, grab it */
  if (bufpool != NULL) {
    t->large_outbuf = bufpool;
    bufpool = bufpool->next;
  } else {			/* else create a new one */
    CREATE(t->large_outbuf, struct txt_block, 1);
    CREATE(t->large_outbuf->text, char, LARGE_BUFSIZE);
    buf_largecount++;
  }
  strcpy(t->large_outbuf->text, t->output);	/* copy to big buffer */
  t->output = t->large_outbuf->text;	/* make big buffer primary */
  strcat(t->output, txt);	/* now add new text */

  /* set the pointer for the next write */
  t->bufptr = strlen(t->output);

  /* calculate how much space is left in the buffer */
  t->bufspace = LARGE_BUFSIZE - 1 - t->bufptr;
}



/* ******************************************************************
   *  socket handling                                                  *
   ****************************************************************** */



#if defined(SO_SNDBUF)
/* Sets the kernel's send buffer size for the descriptor */
int set_sendbuf(socket_t s)
{
  int opt = MAX_SOCK_BUF;

  if (setsockopt(s, SOL_SOCKET, SO_SNDBUF, (char *) &opt, sizeof(opt)) < 0) {
    perror("SYSERR: setsockopt SNDBUF");
    return -1;
  }
#if 0
  if (setsockopt(s, SOL_SOCKET, SO_RCVBUF, (char *) &opt, sizeof(opt)) < 0) {
    perror("SYSERR: setsockopt RCVBUF");
    return -1;
  }
#endif

  return 0;
}
#endif

#ifndef MUDFTP
int new_descriptor(int s)
#else
int new_descriptor(int s, ndesc_t type)
#endif
{
  socket_t desc;
  int sockets_connected = 0;
  int i;
  struct descriptor_data *newd;
  struct sockaddr_in peer;
  extern char *GREETINGS;

  /* accept the new connection */
  i = sizeof(peer);
  if ((desc = accept(s, (struct sockaddr *) &peer, &i)) == INVALID_SOCKET) {
    perror("accept");
    return -1;
  }
  /* keep it from blocking */
  nonblock(desc);

  /* set the send buffer size if available on the system */
#if defined (SO_SNDBUF)
  if (set_sendbuf(desc) < 0) {
    CLOSE_SOCKET(desc);
    return 0;
  }
#endif

  /* make sure we have room for it */
  for (newd = descriptor_list; newd; newd = newd->next) {
    sockets_connected++;
    if (newd->ident_sock != -1)
      sockets_connected++;
  }


  if (sockets_connected >= max_players) {
    write_to_descriptor(desc, "The game is too full to accept any more players!\r\n");
    CLOSE_SOCKET(desc);
    return 0;
  }
  /* create a new descriptor */
  CREATE(newd, struct descriptor_data, 1);
  memset((char *) newd, 0, sizeof(struct descriptor_data));

  /* This dns lookup function seriously speeds us up */

  strncpy(newd->host, aowdns(peer.sin_addr), HOST_LENGTH);

  /* determine if the site is banned */
  if (isbanned(newd->host) == BAN_ALL) {
    CLOSE_SOCKET(desc);
    sprintf(buf2, "Connection attempt denied from [%s]", newd->host);
    mudlog(buf2, CMP, LVL_GOD, TRUE);
    FREE(newd);
    return 0;
  }
#if 0
  /* Log new connections - probably unnecessary, but you may want it */
  sprintf(buf2, "New connection from [%s@%s]", newd->login, newd->host);
  mudlog(buf2, CMP, LVL_GOD, FALSE);
#endif

  init_descriptor(newd, desc);

  /* prepend to list */
  newd->next = descriptor_list;
  descriptor_list = newd;
#ifndef MUDFTP
  SEND_TO_Q(GREETINGS, newd);
#else
  if (type == NDESC_FTP)
    greet_ftp(newd);
  else {
    SEND_TO_Q(GREETINGS, newd);
    SEND_TO_Q("Please wait for your identity to be verified...\r\n", newd);
    ident_start(newd, peer.sin_addr.s_addr);
  }
#endif

  return 0;
}

/* Initialize a descriptor */
void init_descriptor(struct descriptor_data *newd, int desc)
{
  static int last_desc = 0;	/* last descriptor number */
  struct sockaddr_in peer;

  newd->descriptor = desc;
  newd->connected = CON_TERM;
  newd->peer_port = peer.sin_port;
  newd->idle_tics = 0;
  newd->view = 0;
  newd->screen_length = 0;
  newd->wait = 1;
  newd->output = newd->small_outbuf;
  newd->bufspace = SMALL_BUFSIZE - 1;
  newd->next = descriptor_list;
  newd->login_time = time(0);

  if (++last_desc == 1000)
    last_desc = 1;
  newd->desc_num = last_desc;
}

/*
 * Send all of the output that we've accumulated for a player out to
 * the player's descriptor.
 * FIXME - This will be rewritten before 3.1, this code is dumb.
 */


int process_output(struct descriptor_data *t)
{
  static char buf[MAX_SOCK_BUF];
  char buffer[MAX_SOCK_BUF];
  bool flg2 = 1, flg = 1, bold = FALSE, blink = FALSE;
  int ibuf = 0;
  int i, j, k, bg = 0;
  static int result;

  if (!IS_ANSI_TERM(t))
    flg = FALSE;
  if (!IS_CLIENT_TERM(t))
    flg2 = FALSE;

  /* we may need this \r\n for later -- see below */
  strcpy(buf, "\r\n");

  /* now, append the 'real' output */
  strcpy(buf + 2, t->output);

  /* if we're in the overflow state, notify the user */
  if (t->bufptr < 0)
    strcat(buf, "**OVERFLOW**");

  /* add the extra CRLF if the person isn't in compact mode */
  if (!t->connected && t->character && !PRF_FLAGGED(t->character, PRF_COMPACT))
    strcat(buf + 2, "\r\n");

  /* add a prompt */
  strncat(buf + 2, make_prompt(t), MAX_PROMPT_LENGTH);

  ibuf = strlen(buf);
  j = 0;
  /* Go through and convert/strip color symbols -Ithor */
    for (i = 0; (i < ibuf) && (j < (sizeof(buffer))); i++) {
      if (buf[i] == '&') {
	i++;
	if (i >= ibuf)
	  continue;

	switch (buf[i]) {
	case '&':
	  buffer[j++] = '&';
	  break;

	case 'N':
	case 'n':
	  if (flg2) {
	    sprintf(&buffer[j], "\033[0m\033[37m");
	    j += 9;
	  } else if (flg) {
	    sprintf(&buffer[j], "\033[0m");
	    j += 4;
	  }
	  break;

	  /* Do not add a break below. This way, we ignore the client
	     specials, but keep the colour change */
	case '%':
	  i += 4;

	case '+':
	case '-':
	  bg = (buf[i] == '-');
	  i++;
	  if (i >= ibuf)
	    continue;

	  bold = bg ? 0 : (isupper(buf[i])) ? 1 : 0;
	  blink = !bg ? 0 : (isupper(buf[i])) ? 1 : 0;
	  k = find_color_entry(buf[i]);
	  if (color_table[k].symbol != NULL) {
	    if (flg) {
	      sprintf(&buffer[j], "\033[0m\033[%s%s%sm", bold ? "1;" : "", blink ? "5;" : "",
		 (bg ? color_table[k].bg_code : color_table[k].fg_code));
	      j += (9 + (bold ? 2 : 0) + (blink ? 2 : 0));
	    }
	  } else {
	    sprintf(&buffer[j], "&%c%c", (bg ? '-' : '+'), buf[i]);
	    j += 3;
	  }
	  break;

	case '=':
	  i++;
	  if (i >= ibuf)
	    continue;

	  blink = (isupper(buf[i]) ? 1 : 0);
	  bg = find_color_entry(buf[i]);
	  i++;
	  if (i >= ibuf)
	    continue;

	  bold = (isupper(buf[i]) ? 1 : 0);
	  k = find_color_entry(buf[i]);
	  if ((color_table[k].symbol != NULL)
	      && (color_table[bg].symbol != NULL)) {
	    if (flg) {
	      sprintf(&buffer[j], "\033[0m\033[%s%s%s;%sm", bold ? "1;" : "", blink ? "5;" : "",
		      color_table[bg].bg_code, color_table[k].fg_code);
	      j += (12 + (bold ? 2 : 0) + (blink ? 2 : 0));
	    }
	  } else {
	    sprintf(&buffer[j], "&=%c%c", buf[i - 1], buf[i]);
	    j += 4;
	  }
	  break;

	default:
	  sprintf(&buffer[j], "&%c", buf[i]);
	  j += 2;
	  break;
	}
      } else {
	buffer[j++] = buf[i];
      }
    }

    buffer[j] = '\0';

  /*
   * now, send the output.  If this is an 'interruption', use the prepended
   * CRLF, otherwise send the straight output sans CRLF.
   */
#ifndef MUDFTP
  if (t->has_prompt)
#else
  if (t->has_prompt && t->username == NULL && !t->connected)
#endif
    result = write_to_descriptor(t->descriptor, buffer);
  else
    result = write_to_descriptor(t->descriptor, buffer + 2);

  /* handle snooping: prepend "% " and send to snooper */
  if (t->snoop_by) {
    SEND_TO_Q("% ", t->snoop_by);
    SEND_TO_Q(t->output, t->snoop_by);
    SEND_TO_Q("%%", t->snoop_by);
  }
  /*
   * if we were using a large buffer, put the large buffer on the buffer pool
   * and switch back to the small one. 
   */
  if (t->large_outbuf) {
    t->large_outbuf->next = bufpool;
    bufpool = t->large_outbuf;
    t->large_outbuf = NULL;
    t->output = t->small_outbuf;
  }
  /* reset total bufspace back to that of a small buffer */
  t->bufspace = SMALL_BUFSIZE - 1;
  t->bufptr = 0;
  *(t->output) = '\0';

  return result;
}


int write_to_descriptor(socket_t desc, char *txt)
{
  int total, bytes_written;

  total = strlen(txt);

  do {
    if ((bytes_written = write(desc, txt, total)) < 0) {
#ifdef EWOULDBLOCK
      if (errno == EWOULDBLOCK)
	errno = EAGAIN;
#endif				/* EWOULDBLOCK */
      if (errno == EAGAIN)
	log("process_output: socket write would block, about to close");
      else
	perror("Write to socket");
      return -1;
    } else {
      txt += bytes_written;
      total -= bytes_written;
    }
  } while (total > 0);

  total_bytes_written += bytes_written;
  return 0;
}


/*
 * ASSUMPTION: There will be no newlines in the raw input buffer when this
 * function is called.  We must maintain that before returning.
 */
int process_input(struct descriptor_data *t)
{
  int buf_length, bytes_read, space_left, failed_subst;
  char *ptr, *read_point, *write_point, *nl_pos = NULL;
  char tmp[MAX_INPUT_LENGTH + 8];

  /* first, find the point where we left off reading data */
  buf_length = strlen(t->inbuf);
  read_point = t->inbuf + buf_length;
  space_left = MAX_RAW_INPUT_LENGTH - buf_length - 1;

  do {
    if (space_left <= 0) {
      log("process_input: about to close connection: input overflow");
      return -1;
    }
    if ((bytes_read = read(t->descriptor, read_point, space_left)) < 0) {
#ifdef EWOULDBLOCK
      if (errno == EWOULDBLOCK)
	errno = EAGAIN;
#endif				/* EWOULDBLOCK */
      if (errno != EAGAIN && errno != EINTR) {
	perror("process_input: about to lose connection");
	return -1;		/* some error condition was encountered on
				 * read */
      } else
	return 0;		/* the read would have blocked: just means no
				 * data there but everything's okay */
    } else if (bytes_read == 0) {
      log("EOF on socket read (connection broken by peer)");
      return -1;
    }
    /* at this point, we know we got some data from the read */

    *(read_point + bytes_read) = '\0';	/* terminate the string */

    /* search for a newline in the data we just read */
    for (ptr = read_point; *ptr && !nl_pos; ptr++)
      if (ISNEWL(*ptr))
	nl_pos = ptr;

    read_point += bytes_read;
    space_left -= bytes_read;

/*
 * on some systems such as AIX, POSIX-standard nonblocking I/O is broken,
 * causing the MUD to hang when it encounters input not terminated by a
 * newline.  This was causing hangs at the Password: prompt, for example.
 * I attempt to compensate by always returning after the _first_ read, instead
 * of looping forever until a read returns -1.  This simulates non-blocking
 * I/O because the result is we never call read unless we know from select()
 * that data is ready (process_input is only called if select indicates that
 * this descriptor is in the read set).  JE 2/23/95.
 */
#if !defined(POSIX_NONBLOCK_BROKEN)
  } while (nl_pos == NULL);
#else
  } while (0);

  if (nl_pos == NULL)
    return 0;
#endif				/* POSIX_NONBLOCK_BROKEN */

  /*
   * okay, at this point we have at least one newline in the string; now we
   * can copy the formatted data to a new array for further processing.
   */

  read_point = t->inbuf;

  while (nl_pos != NULL) {
    write_point = tmp;
    space_left = MAX_INPUT_LENGTH - 1;

    for (ptr = read_point; (space_left > 0) && (ptr < nl_pos); ptr++) {
      if (*ptr == '\b') {	/* handle backspacing */
	if (write_point > tmp) {
	  if (*(--write_point) == '$') {
	    write_point--;
	    space_left += 2;
	  } else
	    space_left++;
	}
      } else if (isascii(*ptr) && isprint(*ptr)) {
	if ((*(write_point++) = *ptr) == '$') {		/* copy one character */
	  *(write_point++) = '$';	/* if it's a $, double it */
	  space_left -= 2;
	} else
	  space_left--;
      }
    }

    *write_point = '\0';

    if ((space_left <= 0) && (ptr < nl_pos)) {
      char buffer[MAX_INPUT_LENGTH + 64];

      sprintf(buffer, "Line too long.  Truncated to:\r\n%s\r\n", tmp);
      if (write_to_descriptor(t->descriptor, buffer) < 0)
	return -1;
    }
    if (t->snoop_by) {
      SEND_TO_Q("% ", t->snoop_by);
      SEND_TO_Q(tmp, t->snoop_by);
      SEND_TO_Q("\r\n", t->snoop_by);
    }
    failed_subst = 0;

    if (*tmp == '!')
      strcpy(tmp, t->last_input);
    else if (*tmp == '^') {
      if (!(failed_subst = perform_subst(t, t->last_input, tmp)))
	strcpy(t->last_input, tmp);
    } else
      strcpy(t->last_input, tmp);

    if (!failed_subst)
      write_to_q(tmp, &t->input, 0);

    /* find the end of this line */
    while (ISNEWL(*nl_pos))
      nl_pos++;

    /* see if there's another newline in the input buffer */
    read_point = ptr = nl_pos;
    for (nl_pos = NULL; *ptr && !nl_pos; ptr++)
      if (ISNEWL(*ptr))
	nl_pos = ptr;
  }

  /* now move the rest of the buffer up to the beginning for the next pass */
  write_point = t->inbuf;
  while (*read_point)
    *(write_point++) = *(read_point++);
  *write_point = '\0';

  return 1;
}



/* perform substitution for the '^..^' csh-esque syntax orig is the
 * orig string, i.e. the one being modified.  subst contains the
 * substition string, i.e. "^telm^tell"
 */
int perform_subst(struct descriptor_data *t, char *orig, char *subst)
{
  char new[MAX_INPUT_LENGTH + 5];

  char *first, *second, *strpos;

  /*
   * first is the position of the beginning of the first string (the one
   * to be replaced
   */
  first = subst + 1;

  /* now find the second '^' */
  if (!(second = strchr(first, '^'))) {
    SEND_TO_Q("Invalid substitution.\r\n", t);
    return 1;
  }
  /* terminate "first" at the position of the '^' and make 'second' point
   * to the beginning of the second string */
  *(second++) = '\0';

  /* now, see if the contents of the first string appear in the original */
  if (!(strpos = strstr(orig, first))) {
    SEND_TO_Q("Invalid substitution.\r\n", t);
    return 1;
  }
  /* now, we construct the new string for output. */

  /* first, everything in the original, up to the string to be replaced */
  strncpy(new, orig, (strpos - orig));
  new[(strpos - orig)] = '\0';

  /* now, the replacement string */
  strncat(new, second, (MAX_INPUT_LENGTH - strlen(new) - 1));

  /* now, if there's anything left in the original after the string to
   * replaced, copy that too. */
  if (((strpos - orig) + strlen(first)) < strlen(orig))
    strncat(new, strpos + strlen(first), (MAX_INPUT_LENGTH - strlen(new) - 1));

  /* terminate the string in case of an overflow from strncat */
  new[MAX_INPUT_LENGTH - 1] = '\0';
  strcpy(subst, new);

  return 0;
}



void close_socket(struct descriptor_data *d)
{
  char buf[128];
  struct descriptor_data *temp;
  long target_idnum = -1;

  CLOSE_SOCKET(d->descriptor);
  flush_queues(d);

  if (d->ident_sock != INVALID_SOCKET)
    CLOSE_SOCKET(d->ident_sock);

  /* Forget snooping */
  if (d->snooping)
    d->snooping->snoop_by = NULL;

  if (d->snoop_by) {
    SEND_TO_Q("Your victim is no longer among us.\r\n", d->snoop_by);
    d->snoop_by->snooping = NULL;
  }
  if (d->character) {
    target_idnum = GET_IDNUM(d->character);
    if (d->connected == CON_PLAYING) {
      save_char(d->character, NOWHERE);
      act("$n has lost $s link.", TRUE, d->character, 0, 0, PROXIMITY);
      sprintf(buf, "Closing link to: %s.", GET_NAME(d->character));
      mudlog(buf, BRF, MAX(LVL_IMMORT, GET_INVIS_LEV(d->character)), TRUE);
      d->character->desc = NULL;
    } else {
      if (GET_NAME(d->character)) {
	sprintf(buf, "Losing player: %s.",
	     GET_NAME(d->character) ? GET_NAME(d->character) : "<null>");
	mudlog(buf, BRF, LVL_IMMORT, TRUE);
      }				/* else they dropped before con_getname, and we dont care */
      free_char(d->character);
    }
  }				/* else
				   mudlog("Losing descriptor without char.", CMP, LVL_IMMORT, TRUE);
				 */
  if (d->original && d->original->desc)
    d->original->desc = NULL;

  REMOVE_FROM_LIST(d, descriptor_list, next);

  if (d->showstr_head)
    FREE(d->showstr_head);
  if (d->showstr_count)
    FREE(d->showstr_vector);

  FREE(d);
}



void check_idle_passwords(void)
{
  struct descriptor_data *d, *next_d;

  for (d = descriptor_list; d; d = next_d) {
    next_d = d->next;
    if (STATE(d) != CON_PASSWORD && STATE(d) != CON_GET_NAME)
      continue;
    if (!d->idle_tics || d->idle_tics < 3) {
      d->idle_tics++;
      continue;
    } else {
      echo_on(d);
      SEND_TO_Q("\r\nTimed out... goodbye.\r\n", d);
      STATE(d) = CON_CLOSE;
    }
  }
}

#ifndef O_NONBLOCK
#define O_NONBLOCK O_NDELAY
#endif

void nonblock(socket_t s)
{
  int flags;

  flags = fcntl(s, F_GETFL, 0);
  flags |= O_NONBLOCK;
  if (fcntl(s, F_SETFL, flags) < 0) {
    perror("SYSERR: Fatal error executing nonblock (comm.c)");
    raise(SIGSEGV);
  }
}


/* ******************************************************************
   *  signal-handling functions (formerly signals.c).  
   ****************************************************************** */

RETSIGTYPE checkpointing(int sig)
{
  if (!tics) {
    log("SYSERR: CHECKPOINT shutdown: tics not updated");
    abort();
  } else
    tics = 0;
}


RETSIGTYPE reread_wizlists(int sig)
{
  void reboot_wizlists(void);

  mudlog("Signal received - rereading wizlists.", CMP, LVL_IMMORT, TRUE);
  reboot_wizlists();
}


RETSIGTYPE unrestrict_game(int sig)
{
  extern struct ban_list_element *ban_list;
  extern int num_invalid;

  mudlog("Received SIGUSR2 - completely unrestricting game (emergent)",
	 BRF, LVL_IMMORT, TRUE);
  ban_list = NULL;
  restricted = 0;
  num_invalid = 0;
}


RETSIGTYPE hupsig(int sig)
{
  log("SYSERR: Received SIGHUP, SIGINT, or SIGTERM.  Shutting down...");
  raise(SIGSEGV);		/* perhaps something more elegant should
				   * substituted */
}


/*
 * This is an implementation of signal() using sigaction() for portability.
 * (sigaction() is POSIX; signal() is not.)  Taken from Stevens' _Advanced
 * Programming in the UNIX Environment_.  We are specifying that all system
 * calls _not_ be automatically restarted for uniformity, because BSD systems
 * do not restart select(), even if SA_RESTART is used.
 */

#ifdef POSIX
#define my_signal(signo, func) signal(signo, func)
#else
sigfunc *my_signal(int signo, sigfunc * func)
{
  struct sigaction act, oact;

  act.sa_handler = func;
  sigemptyset(&act.sa_mask);
  act.sa_flags = 0;
#ifdef SA_INTERRUPT
  act.sa_flags |= SA_INTERRUPT;	/* SunOS */
#endif

  if (sigaction(signo, &act, &oact) < 0)
    return SIG_ERR;

  return oact.sa_handler;
}
#endif				/* POSIX */

void signal_setup(void)
{
  struct itimerval itime;
  struct timeval interval;

  /* user signal 1: reread wizlists.  Used by autowiz system. */
  my_signal(SIGUSR1, reread_wizlists);

  /*
   * user signal 2: unrestrict game.  Used for emergencies if you lock
   * yourself out of the MUD somehow.  (Duh...)
   */
  my_signal(SIGUSR2, unrestrict_game);

  /*
   * set up the deadlock-protection so that the MUD aborts itself if it gets
   * caught in an infinite loop for more than 3 minutes.
   */
  interval.tv_sec = 180;
  interval.tv_usec = 0;
  itime.it_interval = interval;
  itime.it_value = interval;
  setitimer(ITIMER_VIRTUAL, &itime, NULL);
  my_signal(SIGVTALRM, checkpointing);

  /* just to be on the safe side: */
  my_signal(SIGHUP, hupsig);
  my_signal(SIGINT, hupsig);
  my_signal(SIGTERM, hupsig);
  my_signal(SIGPIPE, SIG_IGN);
  my_signal(SIGALRM, SIG_IGN);
}


/* ****************************************************************
   *       Public routines for system-to-player-communication        *
   **************************************************************** */

void send_to_char(char *messg, struct char_data *ch)
{
  if (ch->desc && messg)
    SEND_TO_Q(messg, ch->desc);
}


void send_to_all(char *messg)
{
  struct descriptor_data *i;

  if (messg)
    for (i = descriptor_list; i; i = i->next)
      if (!i->connected)
	SEND_TO_Q(messg, i);
}


void send_to_outdoor(char *messg)
{
  struct descriptor_data *i;

  if (!messg || !*messg)
    return;

  for (i = descriptor_list; i; i = i->next)
    if (!i->connected && i->character && AWAKE(i->character) &&
      OUTSIDE(i->character) && !PLR_FLAGGED((i->character), PLR_WRITING))
      SEND_TO_Q(messg, i);
}

void send_to_zone_outdoor(int zone_rnum, char *messg)
{
  struct descriptor_data *i;

  if (!messg || !*messg)
    return;

  for (i = descriptor_list; i; i = i->next)
    if (!i->connected && i->character && AWAKE(i->character) &&
	(IN_ROOM(i->character) != NOWHERE) &&
	!IS_SET(ROOM_FLAGS(i->character->in_room), ROOM_INDOORS) &&
	(world[IN_ROOM(i->character)].zone == zone_rnum))
      SEND_TO_Q(messg, i);
}

void send_to_zone_indoor(int zone_rnum, char *messg)
{
  struct descriptor_data *i;

  if (!messg || !*messg)
    return;

  for (i = descriptor_list; i; i = i->next)
    if (!i->connected && i->character && AWAKE(i->character) &&
	(IN_ROOM(i->character) != NOWHERE) &&
	IS_SET(ROOM_FLAGS(i->character->in_room), ROOM_INDOORS) &&
	(world[IN_ROOM(i->character)].zone == zone_rnum))
      SEND_TO_Q(messg, i);
}

void send_to_room(char *messg, int room)
{
  struct char_data *i;

  if (messg)
    for (i = world[room].people; i; i = i->next_in_room)
      if (i->desc)
	SEND_TO_Q(messg, i->desc);
}



char *ACTNULL = "<NULL>";

#define CHECK_NULL(pointer, expression) \
  if ((pointer) == NULL) i = ACTNULL; else i = (expression);


/* higher-level communication: the act() function */
char *perform_act(char *orig, struct char_data *ch, struct obj_data *obj,
		  void *vict_obj, struct char_data *to, int type)
{
  register char *i = NULL, *buf;
  static char lbuf[MAX_STRING_LENGTH];
  struct char_data *dg_victim = NULL;
  struct obj_data *dg_target = NULL;
  char *dg_arg = NULL;

  buf = lbuf;

  for (;;) {
    if (*orig == '$') {
      switch (*(++orig)) {
      case 'n':
	i = PERS(ch, to, TRUE);
	break;
      case 'N':
	CHECK_NULL(vict_obj, PERS((struct char_data *) vict_obj, to, TRUE));
	dg_victim = (struct char_data *) vict_obj;
	break;
      case 'f':
	i = FirstWord(ch->player.name);
	break;
      case 'm':
	i = HMHR(ch);
	break;
      case 'M':
	CHECK_NULL(vict_obj, HMHR((struct char_data *) vict_obj));
	dg_victim = (struct char_data *) vict_obj;
	break;
      case 's':
	i = HSHR(ch);
	break;
      case 'S':
	CHECK_NULL(vict_obj, HSHR((struct char_data *) vict_obj));
	dg_victim = (struct char_data *) vict_obj;
	break;
      case 'e':
	i = HSSH(ch);
	break;
      case 'E':
	CHECK_NULL(vict_obj, HSSH((struct char_data *) vict_obj));
	dg_victim = (struct char_data *) vict_obj;
	break;
      case 'o':
	CHECK_NULL(obj, OBJN(obj, to));
	break;
      case 'O':
	CHECK_NULL(vict_obj, OBJN((struct obj_data *) vict_obj, to));
	dg_victim = (struct char_data *) vict_obj;
	break;
      case 'p':
	CHECK_NULL(obj, OBJS(obj, to));
	master_parser(i, to, NULL, obj, NULL);
	break;
      case 'P':
	CHECK_NULL(vict_obj, OBJS((struct obj_data *) vict_obj, to));
	master_parser(i, to, NULL, vict_obj, NULL);
	dg_victim = (struct char_data *) vict_obj;
	break;
      case 'a':
	CHECK_NULL(obj, SANA(obj));
	break;
      case 'A':
	CHECK_NULL(vict_obj, SANA((struct obj_data *) vict_obj));
	dg_victim = (struct char_data *) vict_obj;
	break;
      case 'T':
	CHECK_NULL(vict_obj, (char *) vict_obj);
	dg_victim = (struct char_data *) vict_obj;
	break;
      case 't':
	CHECK_NULL(obj, (char *) obj);
	break;
      case 'F':
	CHECK_NULL(vict_obj, fname((char *) vict_obj));
	dg_victim = (struct char_data *) vict_obj;
	break;
      case 'd':
	if (ch && to)
	  i = dirs[face_angle(angle_offset(to, NULL, ch, NULL))];
	else
	  i = NULL;
	break;
      case 'w':		/* complicated crap, I use it for dam_messages() */
	if (type == TO_VICT)
	  if (ch && to)
	    i = str_dup("you");
	  else
	    i = NULL;
	else if (type == TO_CHAR)
	  if (vict_obj && to)
	    i = PERS((struct char_data *) vict_obj, to, FALSE);
	  else
	    i = NULL;
	else if (type == TO_NOTVICT) {
	  if (ch && to)
	    i = PERS((struct char_data *) vict_obj, to, FALSE);
	  else
	    i = NULL;
	}
	break;
      case 'W':
	if (type == TO_VICT)
	  i = str_dup("r");	/* changes you to your */
	else
	  i = str_dup("'s");	/* changes joe to joe's */
	break;
      case '$':
	i = "$";
	break;
      default:
	log("SYSERR: Illegal $-code to act():");
	strcpy(buf1, "SYSERR: ");
	strcat(buf1, orig);
	log(buf1);
	break;
      }
      while ((*buf = *(i++)))
	buf++;
      orig++;
    } else if (!(*(buf++) = *(orig++)))
      break;
  }

  *(--buf) = '\r';
  *(++buf) = '\n';
  *(++buf) = '\0';

  if (to->desc && SENDOK(to))
    SEND_TO_Q(CAP(lbuf), to->desc);
  if ((IS_NPC(to) /* && dg_act_check */ ) && (to != ch))
    act_mtrigger(to, lbuf, ch, dg_victim, obj, dg_target, dg_arg);

  return &lbuf[0];
}

void act2(char *see_str, char *hear_str, int hide_invis, struct
	  char_data *ch, struct obj_data *obj, void *vict_obj,
	  int see_type, int hear_type)
{
  // this function is a bit more involved than act(), in that it
  // deals with actions that make noise.  For those, set hear_str
  // and hear_type, exactly as you would for normal acts(), but
  // specifying a string and type/distance for hearing.  If the
  // see_type is a type that sends to only 1 char (or to gods), then
  // it acts exactly as the old act(), and the new parameters are
  // ignored.  If hear_type is set to
  // anything but TO_NOONE (meaning that there is noise involved),
  // then hide_invis is used only by the noise portion of the act.
  // My reasoning is that if you specify that it makes noise, you
  // intend for people who can't see someone to be able to hear
  // them.  What this function ends up doing is exactly what the old
  // act() does, but anyone who doesn't get the see_str is tested to
  // see if they might hear the act instead (based on hear_type)
  // Setting the hear_type to TO_CHAR, TO_VICT, TO_GODS or OR'd with
  // TO_SLEEP is silly, and results are undefined.  Note that if
  // TO_DISTANCE isn't specified for hear_type, DIST_NOISE_NORM is
  // used.  If see_type is TO_OTHER, a distance of DIST_NORM_SEE is
  // used. If see type is anything else, "3" is used as before, and
  // might be multiplied based on TO_PROXIMITY* types.  As before, a
  // see_type (or hear_type) of TO_NOTVICT ignores all proximities.
  // Finally, this code retains the original bug of only checking
  // people in the same room.


  struct char_data *to = NULL, *vict = NULL;
  int sleep, chat_check;
  struct descriptor_data *d, *next_d;
  int see_dist = 3;
  int hear_dist = DIST_NOISE_NORM;

  // if TO_DISTANCE bits are set, negate with XOR
  if (!see_str || !*see_str)
    see_type = TO_NOONE;
  if (!hear_str || !*hear_str)
    hear_type = TO_NOONE;

  if (see_type == TO_OTHERS)
    see_dist = DIST_NORM_SEE;
  if (hear_type == TO_OTHERS)
    hear_dist = DIST_NOISE_NORM;

  if (hear_type & TO_DISTANCE) {
    hear_dist = (hear_type ^ TO_DISTANCE);
    hear_type = TO_OTHERS;
  }
  if (see_type & TO_DISTANCE) {
    see_dist = (see_type ^ TO_DISTANCE);
    see_type = TO_OTHERS;
  }
  if (!(dg_act_check = !(see_type & DG_NO_TRIG)))
    see_type &= ~DG_NO_TRIG;
  if (!(dg_act_check = !(hear_type & DG_NO_TRIG)))
    hear_type &= ~DG_NO_TRIG;

  /*
   * Warning: the following TO_SLEEP code is a hack.
   * 
   * I wanted to be able to tell act to deliver a message regardless of sleep
   * without adding an additional argument.  TO_SLEEP is 128 (a single bit
   * high up).  It's ONLY legal to combine TO_SLEEP with one other TO_x
   * command.  It's not legal to combine TO_x's with each other otherwise.
   * TO_SLEEP only works because its value "happens to be" a single bit;
   * do not change it to something else.  In short, it is a hack.
   */
  /* check if TO_SLEEP is there, and remove it if it is. */
  if ((sleep = (see_type & TO_SLEEP)))
    see_type &= ~TO_SLEEP;
  if ((chat_check = (see_type & CHAT_CHECK)))
    see_type &= ~CHAT_CHECK;

  // TO_CHAR ignores noise stuff
  if (see_type == TO_CHAR) {
    if (ch && SENDOK(ch))
      perform_act(see_str, ch, obj, vict_obj, ch, see_type);
    return;
  }
  // TO_GODS ignores noise stuff
  if (see_type == TO_GODS) {
    for (d = descriptor_list; d; d = next_d) {
      next_d = d->next;
      if (d->character && IS_GOD(d->character) && SENDOK(d->character)
	  && d->connected == CON_PLAYING)
	perform_act(see_str, ch, obj, vict_obj, d->character, see_type);
    }
    return;
  }
  // TO_VICT ignores noise stuff
  if (see_type == TO_VICT) {
    if ((to = (struct char_data *) vict_obj) && SENDOK(to)) {
#ifdef ELIZA
      char *text = perform_act(see_str, ch, obj, vict_obj, to, see_type);
      if (chat_check)
	chatperform(to, ch, text);
#else
      perform_act(see_str, ch, obj, vict_obj, to, see_type);
#endif
    }
    return;
  }
  if (ch && ch->in_room != NOWHERE)
    to = world[ch->in_room].people;
  else if (obj && obj->in_room != NOWHERE)
    to = world[obj->in_room].people;
  else {
    // log("SYSERR: no valid target to act()!"); random mobs hit this constantly
    return;
  }

  vict = NULL;
  switch (see_type) {
  case TO_PROXIMITY3:
  case TO_PROXIMITY2:
    see_dist *= (see_type == TO_PROXIMITY3) ? 3 : 2;
  case TO_PROXIMITY:
    see_type = TO_OTHERS;
    break;
  case TO_NOONE:
    see_dist = 0;
    break;
  default:
    vict = (struct char_data *) vict_obj;
  }

  switch (hear_type) {
  case TO_PROXIMITY3:
  case TO_PROXIMITY2:
    hear_dist *= (hear_type == TO_PROXIMITY3) ? 3 : 2;
  case TO_PROXIMITY:
    hear_type = TO_OTHERS;
    break;
  case TO_NOONE:
    hear_dist = 0;
  default:
    vict = (struct char_data *) vict_obj;
  }
  hear_dist = BOUNDED(0, hear_dist, 50);

  for (; to; to = to->next_in_room) {
    int tmp_sdist = see_dist;
    if (to->watching && (to->watching->vict == ch) &&
	(tmp_sdist < DIST_WATCH))
      tmp_sdist = DIST_WATCH;
    if (SENDOK(to) && (to != ch) && (to != vict)) {
      // first the see_str.
      // if a hear_type is specified, then make sure that acting
      // OBJECTS fall through to there when they can't be seen
      if (see_type && (!ch || CAN_SEE(to, ch)) &&
	  (ch || CAN_SEE_OBJ(to, obj) || !hear_type) &&
	  ((see_type != TO_OTHERS) ||
	   victWithinRangeofChar(ch, to, tmp_sdist)))
	perform_act(see_str, ch, obj, vict_obj, to, TO_NOTVICT);
      else
	// for hearing I have to use a special CAN_SEE that ignores
	// angles.  CAN_SEE_WHO(), perhaps.
	if (hear_type &&
	    !(hide_invis && ch && !CAN_SEE_WHO(to, ch)) &&
	    (!IS_TRUSTED(ch) || (GET_INVIS_LEV(ch) <= GET_LEVEL(to))) &&
	    ((hear_type != TO_OTHERS) ||
	     victWithinRangeofChar(ch, to, hear_dist +
				   char_hearing_modifier(to))))
	perform_act(hear_str, ch, obj, vict_obj, to,
		    TO_NOTVICT);
    }
  }				// for()

}


void act(char *str, int hide_invisible, struct char_data *ch,
	 struct obj_data *obj, void *vict_obj, int type)
{
  act2(str, NULL, hide_invisible, ch, obj, vict_obj, type, TO_NOONE);
}

int enter_player_game(struct descriptor_data *d);

 /* Reload players after a copyover */
void copyover_recover(void)
{
  struct descriptor_data *d;
  FILE *fp;
  char host[1024];
  struct char_file_u tmp_store;
  int desc, player_i;
  bool fOld;
  char name[MAX_INPUT_LENGTH];

  log("Copyover recovery initiated");

  fp = fopen(COPYOVER_FILE, "r");

  if (!fp) {			/* there are some descriptors open which will hang forever then ? */
    perror("copyover_recover:fopen");
    log("Copyover file not found. Exitting.\n\r");
    exit(1);
  }
  unlink(COPYOVER_FILE);	/* In case something crashes - doesn't prevent reading  */

  for (;;) {
    fOld = TRUE;
    fscanf(fp, "%d %s %s\n", &desc, name, host);
    if (desc == -1)
      break;

    /* Write something, and check if it goes error-free */
    if (write_to_descriptor(desc, "\n\rRestoring from copyover...\n\r") < 0) {
      close(desc);		/* nope */
      continue;
    }
    /* create a new descriptor */
    CREATE(d, struct descriptor_data, 1);
    memset((char *) d, 0, sizeof(struct descriptor_data));
    init_descriptor(d, desc);	/* set up various stuff */

    strcpy(d->host, host);
    d->next = descriptor_list;
    descriptor_list = d;

    d->connected = CON_CLOSE;

    /* Now, find the pfile */

    CREATE(d->character, struct char_data, 1);
    clear_char(d->character);
    CREATE(d->character->player_specials, struct player_special_data, 1);
    d->character->desc = d;

    if ((player_i = load_char(name, &tmp_store)) >= 0) {
      store_to_char(&tmp_store, d->character);
      GET_PFILEPOS(d->character) = player_i;
      if (!PLR_FLAGGED(d->character, PLR_DELETED))
	REMOVE_BIT(PLR_FLAGS(d->character), PLR_WRITING | PLR_MAILING | PLR_CRYO);
      else
	fOld = FALSE;
    } else
      fOld = FALSE;

    if (!fOld) {		/* Player file not found?! */
      write_to_descriptor(desc, "\n\rSomehow, your character was lost in the copyover. Sorry.\n\r");
      close_socket(d);
    } else {			/* ok! */
      write_to_descriptor(desc, "\n\rCopyover recovery complete.\n\r");
      enter_player_game(d);
      d->connected = CON_PLAYING;
      look_at_room(d->character, 0);
    }

  }

  fclose(fp);
}

void sort_desc_in_list(struct descriptor_data *d)
{
  struct descriptor_data *temp, *prev;

  //* If this is the only desc return
  if ((descriptor_list->character == d->character) && !d->next)
    return;

  //* remove the desc from list
  REMOVE_FROM_LIST(d, descriptor_list, next);

  //* find the correct spot fo the desc
  for (temp = descriptor_list, prev = NULL; temp;) {
    if (temp->character && (GET_NAME(temp->character) < GET_NAME(d->character))) {
      //* We've found the spot...put in here
      d->next = temp;
      if (!prev)
	descriptor_list = d;
      else
	prev->next = d;
      break;
    } else {
      //* If this is the end, stick desc here.
      if ((temp->character != d->character) && !temp->next) {
	d->next = NULL;
	temp->next = d;
	break;
      }
      //* Step to next in list
      prev = temp;
      temp = temp->next;
    }
  }
}
