/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/


#include "conf.h"
#include "sysdep.h"
#include <signal.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "spells.h"
#include "handler.h"
#include "interpreter.h"
#include "olc.h"
#include "db.h"

extern struct room_data *world;
extern struct attack_hit_type attack_hit_text[];
extern char *item_types[];
extern byte ProximityCheck(struct char_data *ch, struct obj_data *obj, struct char_data *target, int door);
extern int rev_dir[];
int ItemSave(struct obj_data *i, int dam_type);

char *item_condition(struct obj_data *obj)
{
  int value;
  static char buf[MAX_STRING_LENGTH];

  /* default conditon is 100, and reflects average quality. This can be
     raised, through _special_ and _expensive_ smiths, tailors, etc to a max
     of 125, reflecting exceptional quality and manufacture. Code only
     cares about 1-12, so lets chop the name down a bit...
   */

  value = BOUNDED(1, GET_OBJ_CONDITION(obj) / 10, 12);

  switch (value) {
  case 1:
    sprintf(buf, " [ruined] ");
    break;
  case 2:
    sprintf(buf, " [well worn] ");
    break;
  case 3:
    sprintf(buf, " [worn] ");
    break;
  case 4:
    sprintf(buf, " [partially worn] ");
    break;
  case 5:
    sprintf(buf, " [fair] ");
    break;
  case 6:
    sprintf(buf, " [used] ");
    break;
  case 7:
    sprintf(buf, " [good] ");
    break;
  case 8:
    sprintf(buf, " [very good] ");
    break;
  case 9:
    sprintf(buf, " [almost new] ");
    break;
  default:
    sprintf(buf, " ");
  }
  return buf;
}

const char *ItemDamType[5] =
{
  "burned from the blast!",
  "weakened from the intense cold!",
  "filled with electrical energy!",
  "corroded.",
  "damaged from the massive blow!"
};

int ItemSaveThrows[NUM_ITEM_TYPES][5] =
{
/*  fire cold elec blow acid */
  {0, 0, 0, 0, 0},		/* ITEM_UNUSED  */
  {75, 2, 5, 2, 10},		/* ITEM_SCROLL  */
  {14, 5, 50, 10, 10},		/* ITEM_WAND    */
  {12, 5, 15, 8, 10},		/* ITEM_STAFF   */
  {5, 5, 5, 5, 5},		/* ITEM_WEAPON  */
  {5, 5, 5, 5, 5},		/* ITEM_FIREWEAPON */
  {14, 5, 5, 10, 10},		/* ITEM_MISSILE */
  {10, 10, 10, 10, 10},		/* ITEM_TREASURE */
  {5, 5, 5, 5, 5},		/* ITEM_ARMOR   */
  {10, 5, 5, 10, 10},		/* ITEM_POTION  */
  {10, 10, 10, 10, 10},		/* ITEM_WORN    */
  {10, 10, 10, 10, 10},		/* ITEM_OTHER   */
  {10, 10, 10, 10, 10},		/* ITEM_TRASH   */
  {0, 0, 0, 0, 0},		/* ITEM_WALL    */
  {10, 5, 5, 5, 10},		/* ITEM_CONTAINER */
  {75, 2, 5, 5, 10},		/* ITEM_NOTE    */
  {10, 5, 5, 10, 10},		/* ITEM_DRINKCON */
  {0, 0, 0, 0, 0},		/* ITEM_KEY     */
  {10, 10, 10, 10, 10},		/* ITEM_FOOD    */
  {0, 0, 0, 0, 0},		/* ITEM_MONEY   */
  {10, 10, 10, 10, 10},		/* ITEM_PEN     */
  {2, 2, 2, 2, 2},		/* ITEM_BOAT    */
  {15, 5, 5, 10, 10},		/* ITEM_BOOK    */
  {5, 5, 5, 5, 5},		/* ITEM_CORPSE  */
  {0, 0, 0, 0, 0},		/* ITEM_TELEPORT */
  {0, 0, 0, 0, 0},		/* ITEM_TIMER   */
  {0, 0, 0, 0, 0},		/* ITEM_VEHICLE */
  {0, 0, 0, 0, 0},		/* ITEM_SHIP    */
  {0, 0, 0, 0, 0},		/* ITEM_SWITCH  */
  {10, 5, 5, 10, 10},		/* ITEM_QUIVER  */
  {10, 5, 5, 10, 10},		/* ITEM_PICK    */
  {10, 5, 5, 10, 10},		/* ITEM_INSTRUMENT */
  {20, 10, 10, 20, 20},		/* ITEM_FURNITURE */
  {0, 0, 0, 0, 0},		/* ITEM_STORAGE   */
  {10, 5, 5, 10, 10},		/* ITEM_SCABBARD */
  {10, 5, 5, 10, 10},		/* ITEM_SHIELD */
  {0, 0, 0, 0, 0},		/* ITEM_TROOP */
  {0, 0, 0, 0, 0},		/* ITEM_HOLY_SYMBOL */
  {0, 0, 0, 0, 0},		/* ITEM_PIT */
  {10, 5, 5, 10, 10}		/* ITEM_DOOR */
};

int DamageItem(struct char_data *ch, struct obj_data *o, int num)
{
  struct char_data *witness, *next_wit;
  extern int perc_dam[NUM_MATERIALS];
  int temp, cond;

  if (!o)
    return 0;

  for (witness = world[ch->in_room].people; witness; witness = next_wit) {
    next_wit = witness->next_in_room;
    if (witness == ch)
      continue;
    if (IS_HUMANOID(witness) && IS_NPC(witness) && o->in_room > 0) {
      sprintf(buf, "mcallforguards %s vandalis", fname(GET_NAME(ch)));
      command_interpreter(witness, buf);
      break;
    }
  }

  cond = GET_OBJ_CRAFTSMANSHIP(o);
  temp = (num * perc_dam[(int) GET_OBJ_MATERIAL(o)]) / 100;
  temp = (temp * (17 - cond)) / 10;

  GET_OBJ_CONDITION(o) -= temp;
  GET_OBJ_MAX_COND(o) -= temp / 3;
  if (GET_OBJ_CONDITION(o) < 0) {
    GET_OBJ_CONDITION(o) = 0;
    return TRUE;
  }
  return FALSE;
}

int DamageDoor(struct char_data *ch, int dir, int num)
{
  struct char_data *witness, *next_wit;

  if (!EXIT(ch, dir))
    return FALSE;

  for (witness = world[ch->in_room].people; witness; witness = next_wit) {
    next_wit = witness->next_in_room;
    if (witness == ch)
      continue;
    if (IS_HUMANOID(witness) && IS_NPC(witness)) {
      sprintf(buf, "mcallforguards %s vandalis", fname(GET_NAME(ch)));
      command_interpreter(witness, buf);
      break;
    }
  }
  EXIT(ch, dir)->hitpoints -= (num / 2);
  if (EXIT(ch, dir)->hitpoints < 0)
    return TRUE;
  return FALSE;
}

/* Returns actual amount of damage, max 10 */
int DamagedByAttack(struct obj_data *i, int dam_type)
{
  int num = 0;

  if ((GET_OBJ_TYPE(i) == ITEM_ARMOR) || (GET_OBJ_TYPE(i) == ITEM_WEAPON) ||
  (GET_OBJ_TYPE(i) == ITEM_WORN) || (GET_OBJ_TYPE(i) == ITEM_CONTAINER) ||
      (GET_OBJ_TYPE(i) == ITEM_INSTRUMENT) || (GET_OBJ_TYPE(i) == ITEM_BOAT)) {
    while (!ItemSave(i, dam_type)) {
      num += 1;
      if (num > 10)
	return (num);
    }
    return (num);
  } else {
    if (ItemSave(i, dam_type)) {
      return (0);
    } else {
      return (5);
    }
  }
}
int DamageOneItem(struct char_data *ch, int dam_type, struct obj_data *obj, int dam)
{
  int num = dam;

  if (!obj)
    return 0;


  //num = DamagedByAttack(obj, dam_type);
  /*
     sprintf(buf, "%s DamagedByAttack = %d\r\n", fname(obj->name), num);
     send_to_char(buf, ch);
   */
  if (DamageItem(ch, obj, num)) {
    return TRUE;
  }
  return FALSE;
}


void MakeScrap(struct char_data *ch, struct obj_data *obj, int dam_type)
{
  char buf[256];
  struct obj_data *t, *x;
  int pos;


  if (!ch || !obj || (ch->in_room == NOWHERE))
    return;

  act("$p falls to the ground in scraps.", TRUE, ch, obj, 0, TO_CHAR);
  act("$p falls to the ground in scraps.", TRUE, ch, obj, 0, TO_ROOM);

  t = read_object(9, VIRTUAL);
  if (!t)
    return;

  sprintf(buf, "Scraps from %s&n lie in a pile",
	  obj->short_description);
  t->description = str_dup(buf);
  t->short_description = str_dup("a pile of scraps");

  if (obj->carried_by) {
    obj_from_char(obj);
  } else if (obj->worn_by) {
    for (pos = 0; pos < NUM_WEARS; pos++)
      if (ch->equipment[pos] == obj)
	break;
    if (pos >= NUM_WEARS) {
      log("SYSERR: MakeScrap(), can't find worn object in equip");
      raise(SIGSEGV);
    }
    obj = unequip_char(ch, pos);
  }
  put_obj_to_room(t, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
  while (obj->contains) {
    x = obj->contains;
    obj_from_obj(x);
    put_obj_to_room(x, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
  }
#if 0
  if (OBJ_FALLING(t))
    falling_obj(t);
#endif
  extract_obj(obj);
}

void DamageAllStuff(struct char_data *ch, int dam_type, int dam)
{
  int j;
  struct obj_data *obj, *next;

  /* this procedure takes all of the items in equipment and inventory
     and damages the ones that should be damaged */

  /* equipment */

  for (j = 0; j < NUM_WEARS; j++) {
    if (ch->equipment[j]) {
      obj = ch->equipment[j];
      if (DamageOneItem(ch, dam_type, obj, dam)) {	/* TRUE == destroyed */
	if ((obj = unequip_char(ch, j)) != NULL) {
	  MakeScrap(ch, obj, dam_type);
	} else {
	  log("SYSERR: DamageAllStuff(): hmm, really wierd!");
	}
      }
    }
  }

  /* inventory */

  for (obj = ch->carrying; obj; obj = next) {
    next = obj->next_content;
    if (DamageOneItem(ch, dam_type, obj, dam))
      MakeScrap(ch, obj, dam_type);
  }
}

int ItemSave(struct obj_data *i, int dam_type)
{
  int num, j;
  struct char_data *bearer;

  if (ItemSaveThrows[(int) GET_OBJ_TYPE(i)][dam_type] == 0)
    return TRUE;
  if (ItemSaveThrows[(int) GET_OBJ_TYPE(i)][dam_type] == 20)
    return FALSE;

  if (i->carried_by)
    bearer = i->carried_by;
  else if (i->worn_by)
    bearer = i->worn_by;

  num = number(1, 100);
  if (num <= 1)
    return FALSE;
  if (num >= 20)
    return TRUE;

  for (j = 0; j < MAX_OBJ_AFFECT; j++)
    if (i->affected[j].location == APPLY_SAVING_SPELL)
      num += (i->affected[j].modifier * -5);
  if (i->affected[j].location != APPLY_NONE) {
    num += 5;
  }
#if 0
  if (i->affected[j].location == APPLY_HITROLL) {
    num += i->affected[j].modifier;
  }
  if (i->affected[j].location == APPLY_DAMROLL) {
    num += i->affected[j].modifier;
  }
  /* material types vs damage type */
  if (GET_OBJ_TYPE(i) == ITEM_ARMOR || GET_OBJ_TYPE(i) == ITEM_WORN ||
      GET_OBJ_TYPE(i) == ITEM_OTHER) {
    switch (dam_type) {
    case SPELLTYPE_FIRE:
      if (bearer && affected_by_spell(bearer, SPELL_PROTECT_FROM_FIRE))
	num += 10;
/* below here, add/subtract bonuses for material */
      break;
    case SPELLTYPE_COLD:
      if (bearer && affected_by_spell(bearer, SPELL_PROTECT_FROM_COLD))
	num += 10;
      break;
    case SPELLTYPE_ELECTRIC:
      if (bearer && affected_by_spell(bearer, SPELL_PROTECT_FROM_LIGHTNING))
	num += 10;
      break;
    case SPELLTYPE_ACID:
      if (bearer && affected_by_spell(bearer, SPELL_PROTECT_FROM_ACID))
	num += 10;
      break;
    }
  }
  if (i->cost > 250000)
    num += 2;
#endif
  if (num <= 1)
    return FALSE;
  if (num >= 20)
    return TRUE;

  if (num >= ItemSaveThrows[(int) GET_OBJ_TYPE(i)][dam_type]) {
    return TRUE;
  } else {
    return FALSE;
  }
}


void DamageStuff(struct char_data *v, int type, int dam)
{
  DamageAllStuff(v, type, dam);
}

void AttackItem(struct char_data *ch, struct obj_data *obj)
{
  struct obj_data *wielded = NULL;
  struct mob_attack_data *Attack_type, *Next_attack;
  int w_type = TYPE_HIT, apr, num;
  int Distance = 1, Direction = -1;

  if (!objWithinRangeofChar(ch, obj, Distance)) {
    send_to_char("That is much too far away!\r\n", ch);
    return;
  }
  Distance = distBetweenCharAndObj(ch, obj);
  Direction = angleBetweenCharAndObj(ch, obj);

  wielded = ((ch->char_specials.attacks_left == 1) && GET_EQ(ch, WEAR_WIELD2)) ?
      GET_EQ(ch, WEAR_WIELD2) : GET_EQ(ch, WEAR_WIELD);
  if (wielded && GET_OBJ_TYPE(wielded) == ITEM_WEAPON) {
    w_type = GET_OBJ_VAL(wielded, 3) + TYPE_HIT;
  } else if (IS_NPC(ch) && ch->mob_specials.mob_attack != NULL) {
    /* loop through mob attacks until we find the proper one */
    for (Attack_type = ch->mob_specials.mob_attack, apr = 0; Attack_type; Attack_type = Next_attack) {
      Next_attack = Attack_type->next;
      if (ch->char_specials.attacks_left == apr) {
	w_type = Attack_type->attack_type + TYPE_HIT;
	break;
      }
      apr++;
    }
  } else if (IS_HUMANOID(ch)) {
    w_type = TYPE_PUNCH;
  }
  if (!w_type)
    w_type = TYPE_HIT;

  w_type -= TYPE_HIT;
  num = number(0, 5);
  if (!DamageItem(ch, obj, num)) {
    if (IS_WEAPON(w_type + TYPE_HIT)) {
      sprintf(buf, "You %s the %s", attack_hit_text[w_type].singular, fname(obj->name));
      sprintf(buf2, "%s %s the %s", GET_NAME(ch), attack_hit_text[TYPE_CHIP - TYPE_HIT - 2].plural, fname(obj->name));
      act(buf2, TRUE, ch, 0, 0, TO_ROOM);
    } else {
      sprintf(buf, "You %s the %s", ItemDamType[w_type], fname(obj->name));
      sprintf(buf2, "%s %s the %s", GET_NAME(ch), ItemDamType[w_type], fname(obj->name));
      act(buf2, TRUE, ch, 0, 0, TO_ROOM);
    }
    send_to_char(buf, ch);
  } else {
    MakeScrap(ch, obj, w_type);
  }
  num -= 2;
  if (num < 0) {
    send_to_char(".\r\n", ch);
    return;
  }
  if (wielded) {
    if (!DamageItem(ch, wielded, num)) {
      sprintf(buf, " and %s your %s", attack_hit_text[TYPE_CHIP - TYPE_HIT - 2].singular, fname(wielded->name));
      send_to_char(buf, ch);
    } else {
      sprintf(buf, " and ruin your %s in the process", fname(wielded->name));
      send_to_char(buf, ch);
      MakeScrap(ch, wielded, w_type);
    }
  } else {
    send_to_char(" and cause sharp pains to your fists", ch);
  }

  send_to_char(".\r\n", ch);
}

void DestroyDoor(struct char_data *ch, int door)
{
  int other_room = 0;
  struct room_direction_data *back = 0;

  if (!EXIT(ch, door)) {
    log("SYSERR: DestroyDoor called without an exit!");
    return;
  }
  if ((other_room = EXIT(ch, door)->to_room) != NOWHERE)
    if ((back = world[other_room].dir_option[rev_dir[door]]))
      if (back->to_room != ch->in_room)
	back = 0;

  REMOVE_BIT(EXIT(ch, door)->exit_info, EX_LOCKED);
  REMOVE_BIT(EXIT(ch, door)->exit_info, EX_CLOSED);
  SET_BIT(EXIT(ch, door)->exit_info, EX_SMASHED);

  if (back) {
    REMOVE_BIT(VIRTUAL_EXIT(other_room, rev_dir[door])->exit_info, EX_LOCKED);
    REMOVE_BIT(VIRTUAL_EXIT(other_room, rev_dir[door])->exit_info, EX_CLOSED);
    SET_BIT(VIRTUAL_EXIT(other_room, rev_dir[door])->exit_info, EX_SMASHED);
  }
  sprintf(buf + strlen(buf), "Your onslaught destroys the %s!", (EXIT(ch, door)->keyword ? "$F" : "door"));
  act(buf, FALSE, ch, 0, EXIT(ch, door)->keyword, TO_PROXIMITY);

  /* Notify the other room */
  if (back) {
    sprintf(buf, "The %s is smashed open from the other side.\r\n",
	    (back->keyword ? fname(back->keyword) : "door"));
    if (world[EXIT(ch, door)->to_room].people) {
      act(buf, FALSE, world[EXIT(ch, door)->to_room].people, 0, 0, TO_PROXIMITY);
      act(buf, FALSE, world[EXIT(ch, door)->to_room].people, 0, 0, TO_CHAR);
    }
  }
  send_to_char("Door destroyed\r\n", ch);
}

void AttackDoor(struct char_data *ch, int dir)
{
  struct obj_data *wielded = NULL;
  struct mob_attack_data *Attack_type, *Next_attack;
  int w_type = TYPE_HIT, apr, num;

  if (!IS_SET(EXIT(ch, dir)->exit_info, EX_ISDOOR))
    return;
  if (!ProximityCheck(ch, NULL, NULL, dir)) {
    send_to_char("That is much too far away!\r\n", ch);
    return;
  }
  wielded = ((ch->char_specials.attacks_left == 1) && GET_EQ(ch, WEAR_WIELD2)) ?
      GET_EQ(ch, WEAR_WIELD2) : GET_EQ(ch, WEAR_WIELD);
  if (wielded && GET_OBJ_TYPE(wielded) == ITEM_WEAPON) {
    w_type = GET_OBJ_VAL(wielded, 3) + TYPE_HIT;
  } else if (IS_NPC(ch) && ch->mob_specials.mob_attack != NULL) {
    /* loop through mob attacks until we find the proper one */
    for (Attack_type = ch->mob_specials.mob_attack, apr = 0; Attack_type; Attack_type = Next_attack) {
      Next_attack = Attack_type->next;
      if (ch->char_specials.attacks_left == apr) {
	w_type = Attack_type->attack_type + TYPE_HIT;
	break;
      }
      apr++;
    }
  } else if (IS_HUMANOID(ch)) {
    w_type = TYPE_PUNCH;
  }
  if (!w_type)
    w_type = TYPE_HIT;

  w_type -= TYPE_HIT;
  num = number(0, 5);
  if (!DamageDoor(ch, dir, num)) {
    if (IS_WEAPON(w_type + TYPE_HIT)) {
      sprintf(buf, "You %s the %s", attack_hit_text[w_type].singular, fname(EXIT(ch, dir)->keyword));
      sprintf(buf2, "%s %s the %s", GET_NAME(ch), attack_hit_text[TYPE_CHIP - TYPE_HIT - 2].plural, fname(EXIT(ch, dir)->keyword));
      act(buf2, TRUE, ch, 0, 0, TO_ROOM);
    } else {
      sprintf(buf, "You %s the %s", ItemDamType[w_type], fname(EXIT(ch, dir)->keyword));
      sprintf(buf2, "%s %s the %s", GET_NAME(ch), ItemDamType[w_type], fname(EXIT(ch, dir)->keyword));
      act(buf2, TRUE, ch, 0, 0, TO_ROOM);
    }
    send_to_char(buf, ch);
    num -= 2;
    if (num < 0) {
      send_to_char(".\r\n", ch);
      return;
    }
    if (wielded) {
      if (!DamageItem(ch, wielded, num)) {
	sprintf(buf, " and %s your %s", attack_hit_text[TYPE_CHIP - TYPE_HIT - 2].singular, fname(wielded->name));
	send_to_char(buf, ch);
      } else {
	sprintf(buf, " and ruin your %s in the process", fname(wielded->name));
	send_to_char(buf, ch);
	MakeScrap(ch, wielded, w_type);
      }
    } else {
      send_to_char(" and cause sharp pains to your fists", ch);
    }

    send_to_char(".\r\n", ch);

  } else
    DestroyDoor(ch, dir);
}
