//******************************************************************************
//      GREY HAWK PROCS
// File Name         : kanthas_procs.c
// Programmers' Info : Kanthas(wau@intergate.bc.ca)
//******************************************************************************

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "olc.h"

//longtooth dagger proc

const int Ring_Level = 12;

ACMD(do_action);
/*   external vars  */
extern struct room_data *world;
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct time_info_data time_info;
extern struct str_app_type str_app[];
extern char *dirs[];
extern int rev_dir[];
extern char *activating_commands[];
extern const char *race_types[];

/* external functions */
void ASSIGNMOB(int mob, SPECIAL(fname));
void ASSIGNOBJ(int obj, SPECIAL(fname));
void ASSIGNROOM(int room, SPECIAL(fname));
void ASSIGNZONE(int zone, SPECIAL(fname));
int CantEnterCoord(struct char_data *ch, int dir);
void stop_snooping(struct char_data *ch);
void do_visible(struct char_data *ch);
void perform_wear(struct char_data *ch, struct obj_data *obj, int where);
void RemoveLimb(struct char_data *ch, int location);

/* helper functions */
int DetermineCharges(char *Arg);
int CheckItemOwner(struct char_data *ch, struct obj_data *obj, char *name);
int OpenRoom(room_rnum room);

void assign_kanthas_procs(void)
{
  /* mob procs declerations */
//      SPECIAL(mob_proc_quaggoth_race);

  /* obj procs declerations */
  //limb
  SPECIAL(limb_troll);

  //rings
  SPECIAL(ring_proc_mammal_control);
  SPECIAL(ring_proc_ram);
  SPECIAL(ring_proc_shocking_grasp);
  SPECIAL(ring_proc_blinking);
  //misc
  SPECIAL(bag_proc_many_tricks);
  SPECIAL(proc_red_button);
  //god items
  SPECIAL(mace_proc_iuz);
  SPECIAL(boots_proc_pyremius);
  SPECIAL(spear_proc_hextor);
  SPECIAL(crown_proc_svarak);
  SPECIAL(obj_proc_crimson_skull);
  SPECIAL(obj_proc_ring_love);
  SPECIAL(obj_proc_heart);
  SPECIAL(necklace_amethyst);
  //eggs
  SPECIAL(egg_proc_wonder);
  //figurines
  SPECIAL(figurine_proc_horse);
  SPECIAL(figurine_wonderous_ebonyfly);
  SPECIAL(figurine_wonderous_lions);
  SPECIAL(figurine_wonderous_elephant);
  //weapons
  SPECIAL(dagger_proc_longtooth);
  SPECIAL(sword_proc_curse_instantwield);
  SPECIAL(sword_life_stealing);
  SPECIAL(sword_vorpal);
  SPECIAL(sword_sharpness);

  /* room procs declerations */
  SPECIAL(kan_bedroom);
  SPECIAL(kan_place);
  SPECIAL(mob_generating_room);
  SPECIAL(hanali_room);

  /* mob proc assignments */
//   ASSIGNMOB(6217, mob_proc_quaggoth_race);

  /* obj proc assignments */
  //limbs
  ASSIGNOBJ(60, limb_troll);
  ASSIGNOBJ(61, limb_troll);
  ASSIGNOBJ(62, limb_troll);
  ASSIGNOBJ(63, limb_troll);
  ASSIGNOBJ(64, limb_troll);
  //rings
  ASSIGNOBJ(2215, ring_proc_mammal_control);
  ASSIGNOBJ(2222, ring_proc_ram);
  ASSIGNOBJ(2224, ring_proc_shocking_grasp);
  ASSIGNOBJ(2202, ring_proc_blinking);
  //misc
  ASSIGNOBJ(199, bag_proc_many_tricks);
  ASSIGNOBJ(9997, proc_red_button);
  //god items
  ASSIGNOBJ(150, mace_proc_iuz);
  ASSIGNOBJ(151, boots_proc_pyremius);
  ASSIGNOBJ(155, spear_proc_hextor);
  ASSIGNOBJ(156, crown_proc_svarak);
  ASSIGNOBJ(113, obj_proc_crimson_skull);
  ASSIGNOBJ(9998, obj_proc_ring_love);
  ASSIGNOBJ(9999, obj_proc_heart);
  ASSIGNOBJ(9996, necklace_amethyst);
  ASSIGNOBJ(9995, obj_proc_crimson_skull);
  //eggs
  ASSIGNOBJ(3124, egg_proc_wonder);
  //figurines
  ASSIGNOBJ(3126, figurine_proc_horse);
  ASSIGNOBJ(3224, figurine_wonderous_ebonyfly);
  ASSIGNOBJ(3225, figurine_wonderous_lions);
  ASSIGNOBJ(3226, figurine_wonderous_elephant);
  //weapons
  ASSIGNOBJ(3501, dagger_proc_longtooth);
  ASSIGNOBJ(3502, sword_proc_curse_instantwield);
  ASSIGNOBJ(3503, sword_life_stealing);
  ASSIGNOBJ(3504, sword_vorpal);
  ASSIGNOBJ(3505, sword_sharpness);

  /* room proc assignements */
  ASSIGNROOM(198, kan_bedroom);
  ASSIGNROOM(199, kan_place);
//      ASSIGNROOM(125, hanali_room);
}				//assign_kanthas_procs


/*************************************************************************/
/* helper functions */
/*************************************************************************/


/* DetermineCharges - determines charges selected by user */
int DetermineCharges(char *Arg)
{
  int Counter = 0, ArgLen;

  ArgLen = strlen(Arg);
  while (Arg[Counter] != '\0' && Counter < ArgLen) {
    if (isdigit(Arg[Counter]))
      return Arg[Counter] - '0';

    Counter++;
  }				//while

  return 0;
}				//DetermineCharges


int OpenRoom(room_rnum room)
{
  room_rnum RealRoom = 0;

  RealRoom = real_room(room);
  if (ROOM_FLAGGED(RealRoom, ROOM_GODROOM)) {
    send_to_room("&+La popping noise is heard, as a magical dweomer is lifted from this room.&n", RealRoom);
    REMOVE_BIT(ROOM_FLAGS(RealRoom), ROOM_GODROOM);
    return 1;
  }				//if

  else
    return 0;
}				//openkanroom


int CheckItemOwner(struct char_data *ch, struct obj_data *obj, char *name)
{
  if (!isname(name, GET_NAME(ch)) && !isname("Tastone", GET_NAME(ch))) {
    if (ch->equipment[WEAR_HOLD] && ch->equipment[WEAR_HOLD] == obj)
      obj_to_char(unequip_char(ch, WEAR_HOLD), ch);
    else if (ch->equipment[WEAR_HOLD2] && ch->equipment[WEAR_HOLD2] == obj)
      obj_to_char(unequip_char(ch, WEAR_HOLD2), ch);
    else if (ch->equipment[WEAR_WIELD] && ch->equipment[WEAR_WIELD] == obj)
      obj_to_char(unequip_char(ch, WEAR_WIELD), ch);
    else if (ch->equipment[WEAR_WIELD2] && ch->equipment[WEAR_WIELD2] == obj)
      obj_to_char(unequip_char(ch, WEAR_WIELD2), ch);
    return 0;
  }				//if

  return 1;
}				//checkitemowner




/*************************************************************************/
/* mob procs */
/*************************************************************************/

SPECIAL(mob_proc_quaggoth_race)
{
  /* confirm race */
//  if ((GET_RACE(ch) != RACE_QUAGGOTH) || (cmd))
  //    return FALSE;

  /* proc - start */
  if (IS_FIGHTING(ch) && GET_HIT(ch) < (GET_MAX_HIT(ch) / 4) && !IS_AFFECTED(ch, AFF_CURSE)) {
    act("$n's hair stands on end as $e screams out in rage.", FALSE, ch, 0, 0, PROXIMITY);
    GET_DAMROLL(ch) += 2;
    GET_HITROLL(ch) += 2;
    SET_BIT(AFF_FLAGS(ch), AFF_CURSE);
    return TRUE;
  }				//if
  /* proc - end */
  else if (!IS_FIGHTING(ch) && IS_AFFECTED(ch, AFF_CURSE)) {
    act("$n lets out a deep breath as $e overcomes $s rage.", FALSE, ch, 0, 0, PROXIMITY);
    GET_DAMROLL(ch) -= 2;
    GET_HITROLL(ch) -= 2;
    REMOVE_BIT(AFF_FLAGS(ch), AFF_CURSE);
    return TRUE;
  }				//else if

  return FALSE;
}				//quaggoth


SPECIAL(mob_proc_medusa)
{
  /* confirm race */
/*  if ((GET_RACE(ch) != RACE_MEDUSA) || (cmd))
   return FALSE;

   "$n&+L's eyes glow dark &+rred &+Las she gazes directly at you.&n"
   "$n&+L's eyes glow dark &+rred &+Las she gazes at $N.&n"
   
   "&+LYou halt all motion as your body begins to turn to stone.&n"
   "&+LA deathly cold overcomes you as the last bit of your flesh becomes stone.&n"
   
   "&+L$N ceases to moves as the petrification takes place.&n"
   "&+L$N has been turned to stone.&n"

   
//failed messages for medusa proc
   
"$n &+Lgazes directly at you.&n"
   "$n &+Lgazes directly at $N.&n"
 */
  return FALSE;
}				//medusa


/*************************************************************************/
/* obj procs */
/*************************************************************************/
/* obj - rings */

SPECIAL(ring_proc_mammal_control)
{
  int Level_Diff, Die;
  char mob_name[256], ring_name[256];
  struct char_data *victim = NULL;
  struct obj_data *obj, *objdummy;

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;

  //get obj pointer
  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_FINGER_L] != obj && ch->equipment[WEAR_FINGER_R] != obj)
    return FALSE;

  if (!CMD_IS("invoke"))
    return FALSE;

  two_arguments(argument, ring_name, mob_name);

  /* double check we are speaking of the same object */
  if (!isname("mammal", ring_name))
    return FALSE;

  /* find the victim */
  if (mob_name)
    generic_find(mob_name, FIND_CHAR_ROOM, ch, &victim, &objdummy);

  if (!victim) {
    send_to_char(NOPERSON, ch);
    return TRUE;
  }				//if

  //chance of breaking
  if (!number(0, 200)) {
    act("You put too much strain on $p as it shatters.", FALSE, ch, obj, 0, TO_CHAR);
    act("$n puts too much strain on $p as it shatters.", FALSE, ch, obj, 0, PROXIMITY);
    if (ch->equipment[WEAR_FINGER_L] && ch->equipment[WEAR_FINGER_L] == obj)
      obj_to_char(unequip_char(ch, WEAR_FINGER_L), ch);
    else if (ch->equipment[WEAR_FINGER_R] && ch->equipment[WEAR_FINGER_R] == obj)
      obj_to_char(unequip_char(ch, WEAR_FINGER_R), ch);
    else
      log("SYSERR: ring_of_mammal_control() lost the object!");
    /* irregardless of where its worn, we already know it existed, so just
       junk it and stop whining */
    extract_obj(obj);
  }				//if

  Level_Diff = GET_LEVEL(ch) - GET_LEVEL(victim);
  Die = 10;

  /*level differece chance effects */
  if (Level_Diff >= 0)
    Die += Level_Diff / 2;
  else
    Die -= Level_Diff / 2;

  if (Die <= 0)
    Die = 1;

  //check for mammal and int
  if (!IS_MAMMAL(victim) || (GET_INT(victim) > 4))
    Die = 0;

  /* Fail to charm */
  if (dice(Die, 10) < 50) {
    if (Level_Diff <= -10)
      set_fighting(victim, ch);

    act("Your $p spa&+Wrk&nles briefly, but $N is not impressed.", FALSE, ch, obj, victim, TO_CHAR);
    act("$n $p spa&+Wrk&nles briefly, but fails to take control of &=LRYOU&n. ", FALSE, ch, obj, victim, TO_VICT);
    act("$n $p spa&+Wrk&nles briefly, but $N is not impressed.", FALSE, ch, obj, victim, TO_OTHERS);
    return TRUE;
  }				//if

  /*SUCCESS of charm */
  act("$n $p spa&+Wrk&nles briefly, and $N begins to follow him. ", FALSE, ch, obj, victim, TO_OTHERS);
  act("$n $p spa&+Wrk&nles briefly, and &=LRYOU&n begin to follow $m. ", FALSE, ch, obj, victim, TO_VICT);
  act("Your $p spa&+Wrk&nles briefly, and $N begins to follow you. ", FALSE, ch, obj, victim, TO_CHAR);
  r_charm(Ring_Level, ch, NULL, victim, NULL, NULL, SPELL_CHARM_MAMMAL);
  return TRUE;
}				//mammal control


SPECIAL(ring_proc_ram)
{
  int charges_used = 1, vict_str, vict_svspell, cardinal_direction = -1;
  char vict_name[256], ring_name[256];
  struct char_data *victim;
  struct obj_data *obj = NULL, *door = NULL;
  bool vict_savethrow, exit = FALSE;
  int open_door_chances[6] =
  {6, 8, 10, 16, 16, 17};

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;

  /* get obj pointer */
  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_FINGER_L] != obj && ch->equipment[WEAR_FINGER_R] != obj)
    return FALSE;

  /* set initial charges */
  if (CMD_IS("wear") && !GET_OBJ_VAL(obj, 0)) {
    GET_OBJ_VAL(obj, 0) = 1;
    GET_OBJ_VAL(obj, 1) = dice(6, 10);
    return FALSE;
  }				//if

  if (!CMD_IS("invoke"))
    return FALSE;

  two_arguments(argument, ring_name, vict_name);

  /* double check we are speaking of the same object */
  if (!isname("ram", ring_name))
    return FALSE;


  if (isname(vict_name, "")) {
    send_to_char("target what?", ch);
    return TRUE;
  }				//if

  /* find the victim */
  if (vict_name)
    generic_find(vict_name, FIND_CHAR_ROOM, ch, &victim, &door);

  /* if no victim then look for door */
  if (!victim) {
    cardinal_direction = search_block(vict_name, dirs, FALSE);
    if (world[ch->in_room].dir_option[cardinal_direction])
      exit = TRUE;
  }				//if

  /* inform char, victim could not be found */
  if (!victim && !exit) {
    send_to_char(NOPERSON, ch);
    return TRUE;
  }				//if

  /* range of ring is 30ft */
  if (victim && distBetweenCharAndVict(ch, victim) > 10) {
    act("$n's $p glows slightly and then disapates.", FALSE, ch, obj, victim, PROXIMITY);
    act("$N is too far away.", FALSE, ch, obj, victim, TO_CHAR);
    return TRUE;
  }				//if

  else if (exit && distBetweenCharAndDoor(ch, cardinal_direction) > 10) {
    act("$n's $p glows slightly and then disapates.", FALSE, ch, obj, 0, PROXIMITY);
    act("The door is too far away.", FALSE, ch, obj, 0, TO_CHAR);
    return TRUE;
  }				//if


  /* determine charges wanted */
  charges_used = DetermineCharges(argument);
  /* comfirm charges are within range */
  if (charges_used > 3)
    charges_used = 3;
  else if (charges_used < 1)
    charges_used = 1;
  /* check for obj charge availability */
  if (charges_used > GET_OBJ_VAL(obj, 1))
    charges_used = GET_OBJ_VAL(obj, 1);
  /* out of charges */
  if (!charges_used) {
    act("Your $p glows slightly and then disapates.", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p glows slightly and then disapates.", FALSE, ch, obj, 0, PROXIMITY);
    return TRUE;
  }				//if
  /* reduce charges to obj */
  GET_OBJ_VAL(obj, 1) -= charges_used;

  /* if ring is used to open a door */
  if (!victim && exit) {
    if (DOOR_IS_OPEN(IN_ROOM(ch), door, cardinal_direction)) {
      act("Your $p glows slighty and then disapates.", FALSE, ch, obj, 0, TO_CHAR);
      act("$n's $p glows slighty and then disapates.", FALSE, ch, obj, 0, PROXIMITY);
      return TRUE;
    }				//if

    /* determine the chances of opening door */
    if (DOOR_IS_UNLOCKED(IN_ROOM(ch), door, cardinal_direction))
      charges_used += 3;

    /* roll for success to open door */
    if (number(1, 20) <= open_door_chances[charges_used - 1]) {
      /* success in opening door */
      act("&+LYour $p &+Lsummons forth a &+Cmagical&+L ram's head, which &+Wbashes &+Ldown a door. &n", FALSE, ch, obj, 0, TO_CHAR);
      act("$n's $p &+Lsummons forth a &+Cmagical&+L ram's head, which &+Wbashes &+Ldown a door. &n", FALSE, ch, obj, 0, PROXIMITY);
      OPEN_DOOR(IN_ROOM(ch), door, cardinal_direction);
    }				//if

    else {			/* failure to open */
      act("&+LYour $p &+Lsummons forth a &+Cmagical&+L ram's head, which &+rfails &+Lto budge a door. &n", FALSE, ch, obj, 0, TO_CHAR);
      act("$n's $p&+L summons forth a &+Cmagical&+L ram's head, which &+rfails &+Lto budge a door. &n", FALSE, ch, obj, 0, PROXIMITY);
    }				//else

    return TRUE;
  }				//if

  /* vict_svspell modifiers */
  vict_svspell = GET_SAVE(victim, SAVING_SPELL);
  /* str modifier */
  vict_str = GET_STR(victim);
  if (vict_str < 12)
    vict_svspell += 2;
  else if (vict_str > 20)
    vict_svspell -= 6;
  else if (vict_str > 17)
    vict_svspell -= 3;

  /* weight mod */
  if (GET_WEIGHT(victim) > 1000)
    vict_svspell -= 2;
  /* charge mod */
  vict_svspell += charges_used + 1;
  /* size mod */
  if (GET_SIZE(victim) < SIZE_MEDIUM)
    vict_svspell++;
  else if (GET_SIZE(victim) > SIZE_MEDIUM)
    vict_svspell -= 2;
  /* leg mod */

  /* sv_spell throw - determine if victim will be knocked down/saved */
  vict_savethrow = (dice(1, 20) >= vict_svspell) ? TRUE : FALSE;

  if (GET_POS(victim) == POS_SITTING)
    vict_savethrow = TRUE;

  /* produce effects */
  if (vict_savethrow) {
    /* regular damage */
    if (charges_used < 3) {
      act("&+LYour&n $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L goat to pummel&n $N. ", FALSE, ch, obj, victim, TO_CHAR);
      act("$n's $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L goat to pummel &=LRYOU&+L.&n ", FALSE, ch, obj, victim, TO_VICT);
      act("$n's $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L goat to pummel&n $N. ", FALSE, ch, obj, victim, TO_OTHERS);
    }				//if

    else {
      act("&+LYour &n$p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L ram to pummel&n $N. ", FALSE, ch, obj, victim, TO_CHAR);
      act("$n's $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L ram to pummel &=LRYOU&+L.&n ", FALSE, ch, obj, victim, TO_VICT);
      act("$n's $p &+Lbegins to &+Cglow&+L brilliantly as it summons forth a &+mmagical&+L ram to pummel&n $N. ", FALSE, ch, obj, victim, TO_OTHERS);
    }				//else

  }				//if

  else {
    /* knock down victim */
    if (charges_used < 3) {
      act("&+LYour &n$p &+Lsummons forth a &+Cmagical &+Lgoat's head, which &+Wslams &+Linto $N &+Wknocking&+L $M to the ground. &n", FALSE, ch, obj, victim, TO_CHAR);
      act("$n's $p &+Lsummons forth a &+Cmagical &+Lgoat's head, which &+Wslams &+Linto &=LRYOU &+Wknocking&+L you to the ground. &n", FALSE, ch, obj, victim, TO_VICT);
      act("$n's $p &+Lsummons forth a &+Cmagical &+Lgoat's head, which &+Wslams &+Linto&n $N &+Wknocking&+L $M to the ground. &n", FALSE, ch, obj, victim, TO_OTHERS);
    }				//if

    else {
      act("&+LYour &n$p &+Lsummons forth a &+Cmagical &+Lram's head, which &+Wslams &+Linto $N &+Wknocking&+L $M to the ground. &n", FALSE, ch, obj, victim, TO_CHAR);
      act("$n's $p &+Lsummons forth a &+Cmagical &+wr&+Wam&+L's head, which &+Wslams &+Linto &=LRYOU &+Wknocking&+L you to the ground. &n", FALSE, ch, obj, victim, TO_VICT);
      act("$n's $p &+Lsummons forth a &+Cmagical &+Lram's head, which &+Wslams &+Linto&n $N &+Wknocking&+L $M to the ground. &n", FALSE, ch, obj, victim, TO_OTHERS);
    }				//else

    SET_POS(victim, GET_STAT(victim) + POS_SITTING);
  }				//else

  /* set damage */
  damage(ch, victim, dice(charges_used, 6), -1, -1, -1);
  return TRUE;
}				//ram


SPECIAL(ring_proc_shocking_grasp)
{
  char ring_name[256];
  struct obj_data *obj;

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_FINGER_L] != obj && ch->equipment[WEAR_FINGER_R] != obj)
    return FALSE;

  if (!CMD_IS("invoke"))
    return FALSE;

  one_argument(argument, ring_name);

  /* double check we are speaking of the same object */
  if (!isname("grasp", ring_name) && !isname("shocking", ring_name))
    return FALSE;

  /* recharge item after 1 hour */
  if (!GET_OBJ_VAL(obj, 1) && !GET_OBJ_TIMER(obj))
    GET_OBJ_VAL(obj, 1) = 3;

  /* modify charges to obj + call effects */
  if (GET_OBJ_VAL(obj, 1) > 0) {	/* apply shocking grasp effect */
    act("$n's $p glows brightly.", FALSE, ch, obj, 0, PROXIMITY);
    act("Your $p glows brightly.", FALSE, ch, obj, 0, TO_CHAR);
    call_magic(ch, ch, NULL, find_spells_num("shocking grasp"), Ring_Level, CAST_SPELL, NULL);
    GET_OBJ_VAL(obj, 1)--;
    if (!GET_OBJ_VAL(obj, 1))
      GET_OBJ_TIMER(obj) = 1;
  }				//if

  else {			/* no proc message */
    act("Your $p glows slightly and then disapates.", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p glows slightly and then disapates.", FALSE, ch, obj, 0, PROXIMITY);
  }				//else

  return TRUE;
}				//ring_shocking_grasp


SPECIAL(ring_proc_blinking)
{
  char keyword[256];
  struct obj_data *obj;

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_FINGER_L] != obj && ch->equipment[WEAR_FINGER_R] != obj)
    return FALSE;

  if (!CMD_IS("say") && !CMD_IS("'"))
    return FALSE;

  one_argument(argument, keyword);

  /* double check we are speaking of the same object */
  if (!isname("blink", keyword))
    return FALSE;

  /* modify timer to obj + call effects */
  if (!GET_OBJ_TIMER(obj)) {	/* apply blinking effect */
    act("Your $p glows brightly.", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p glows brightly.", FALSE, ch, obj, 0, PROXIMITY);
    call_magic(ch, ch, NULL, find_spells_num("blink"), Ring_Level, CAST_SPELL, NULL);
    GET_OBJ_TIMER(obj) = 1;
  }				//if

  else {			/* no proc message */
    act("Your $p glows slightly and then disapates.", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p glows slightly and then disapates.", FALSE, ch, obj, 0, PROXIMITY);
  }				//else

  return TRUE;
}				//ring of blinking


/* special objects */
SPECIAL(limb_troll)
{
  struct obj_data *obj;
  room_rnum room;

  obj = me;

  if (GET_OBJ_VAL(obj, 6) != RACE_TROLL)
    return FALSE;

  room = real_room(obj->in_room);

  if (!number(0, 10)) {
    send_to_room("&+wthis &+Glimb&+w is doing alot of wierd shit atm&n", room);
    send_to_room("&+wthis &+Glimb&+w is doing alot of wierd shit atm  &+LRTEST!!&n", room);
  }				//if

  return FALSE;
}				//troll limbs


SPECIAL(bag_proc_many_tricks)
{
  int BagType = 0;
  int i;
  char buf2[MAX_STRING_LENGTH];
  struct obj_data *temp_obj, *obj;

  struct BagType {
    char *LongDesc;		//desc when look/examine

    char *CreateMsgChar;	//message to char upon creation

    char *CreateMsgRoom;	//message to room upon creation

  };				//DescType

  struct ItemType {
    char *keyword;		//keywords for item

    char *desc;			//name

    int val;			//value

    int num;			//vnum

    int min;			//minimum charges

    int max;			//maximum charges

  };

  struct BagType DescList[] =
  {
  /* plainswalker bag */
    {"&+rThis bag is made of the finest red silk available.  A &+Ldark gray m&+wi&+Lst &+rswirls around its\r\n"
     "exterior, captivating all within view. A small silken braided drawstring runs through the\r\n"
     "mouth of this bag to close it tightly.  It is a small bag, capable of holding twenty &+Ygold\r\n"
     "&+rpieces.&n\r\n"
     "&+LRemaining items:&n\r\n",
     "&+CAs you grab a &+rred s&+Ril&+rk &+Lbag &+Cthe &+Lm&ni&+Lsts&+C about it begins to swirl with increasing speed, finally settling after a few seconds.&n",
     "&+CAs &n$n &+Cgrabs a &+rred s&+Ril&+rk &+Lbag &+Cthe &+Lm&ni&+Lsts&+C about it begins to swirl with increasing speed, finally settling after a few seconds.&n"},

  /* test bag */
    {"&+LThis bag is sucks, you've been &=LRSHAFTED&+L =P&n\r\n"
     "&+LRemaining items:&n\r\n",
     "Your $p turns to &+CICE&n",
     "$n $p turns to &+CICE&n"}
  };				//DescList

  struct ItemType ItemList[] =
  {
  /* plainswalker */
    {"ram", "ring of ram", 1, 2222, 5, 10},
    {"grasp", "ring of shocking grasp", 2, 2224, 5, 10},
    {"", "", 3, 0, 0, 0},
    {"blink", "ring of blinking", 4, 2202, 5, 10},
    {"mammal", "ring of mammal control", 5, 2215, 5, 10},
    {"", "", 6, 0, 0, 0},
    {"", "", 7, 0, 0, 0},

  /* test bag */
    {"clamp", "clampons", 1, 1302, 5, 10},
    {"egg", "egg of wonder", 2, 3124, 10, 20},
    {"", "", 3, 0, 0, 0},
    {"", "", 4, 0, 0, 0},
    {"", "", 5, 0, 0, 0},
    {"", "", 6, 0, 0, 0},
    {"", "", 7, 0, 0, 0},
  };				//ItemList

  if (!ch || (!CMD_IS("pull") && !CMD_IS("examine") &&
	      !CMD_IS("look") && !CMD_IS("hold") && !CMD_IS("grab")))
    return FALSE;

  if (!*argument && IS_NPC(ch))
    return FALSE;

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_HOLD] != obj && ch->equipment[WEAR_HOLD2] != obj)
    return FALSE;

  /* determine if new */
  if (CMD_IS("hold") || CMD_IS("grab")) {
    if (GET_OBJ_VAL(obj, 0))
      return FALSE;

    /* determine its type */
    BagType = number(1, 100);
    if (BagType < 50)
      GET_OBJ_VAL(obj, 0) = 1;
    else
      GET_OBJ_VAL(obj, 0) = 2;
/*
   else if (BagType < 10)
   GET_OBJ_VAL(obj, 0) = 2;
   else if (BagType < 30)
   GET_OBJ_VAL(obj, 0) = 3;
   else if (BagType < 50)
   GET_OBJ_VAL(obj, 0) = 4;
   else if (BagType <= 100)
   GET_OBJ_VAL(obj, 0) = 5;
 */
    /* set charges */
    BagType = (GET_OBJ_VAL(obj, 0) - 1) * 7;

    for (i = BagType; i < BagType + 7; i++)
      GET_OBJ_VAL(obj, i - BagType + 1) = number(ItemList[i].min, ItemList[i].max);

    act(DescList[GET_OBJ_VAL(obj, 0) - 1].CreateMsgChar, FALSE, ch, obj, 0, TO_CHAR);
    act(DescList[GET_OBJ_VAL(obj, 0) - 1].CreateMsgRoom, FALSE, ch, obj, 0, PROXIMITY);

    return FALSE;
  }				//if - hold

  /* check to make sure we are looking at bag */
  one_argument(argument, buf);
  if (!isname("bag", buf))
    return FALSE;

  /* show user how many of what items are left */
  if (CMD_IS("look") || CMD_IS("examine")) {
    if (!GET_OBJ_VAL(obj, 0))
      return FALSE;

    BagType = (GET_OBJ_VAL(obj, 0) - 1) * 7;
    strcpy(buf, DescList[GET_OBJ_VAL(obj, 0) - 1].LongDesc);

    /* go thru items and print out whats left */
    for (i = BagType; i < BagType + 7; i++)
      if (GET_OBJ_VAL(obj, i - BagType + 1))
	sprintf(buf, "%s(%d) %s\r\n", buf, GET_OBJ_VAL(obj, i - BagType + 1), ItemList[i].desc);

    send_to_char(buf, ch);
    return FALSE;
  }				//if

  else {
    two_arguments(argument, buf, buf2);
    if (!strcmp(buf2, "")) {
      act("You reach into your $p&n and fail to retrieve anything but thin air.", FALSE, ch, obj, 0, TO_CHAR);
      act("$n reaches into $s $p&n, grasping for something that is not there.", FALSE, ch, obj, 0, PROXIMITY);
      return TRUE;
    }				//if

    BagType = (GET_OBJ_VAL(obj, 0) - 1) * 7;

    /* look for item */
    for (i = BagType; i < BagType + 7; i++) {
      /* find item */
      if (!strcmp(buf2, ItemList[i].keyword))
	if (GET_OBJ_VAL(obj, ItemList[i].val)) {	/* check charges */
	  // reduce charges
	  GET_OBJ_VAL(obj, ItemList[i].val)--;
	  // loading obj to chars inventory
	  temp_obj = read_object(ItemList[i].num, VIRTUAL);
	  obj_to_char(temp_obj, ch);
	  act("You pull a $P out of your $p!", FALSE, ch, obj, temp_obj, TO_CHAR);
	  act("$n pulls a $P out of $s $p&n.", FALSE, ch, obj, temp_obj, PROXIMITY);
	  return TRUE;
	}			//if - check charges

    }				//for

    /* no item found */
    act("You reach into your $p&n and fail to retrieve anything but thin air.", FALSE, ch, obj, 0, TO_CHAR);
    act("$n reaches into $s $p&n, grasping for something that is not there.", FALSE, ch, obj, 0, PROXIMITY);
    return TRUE;
  }				//else

  send_to_char("Serious error, please inform a god.", ch);
  return FALSE;
}				//bag of tricks


/* god items */

SPECIAL(proc_red_button)
{
  struct obj_data *obj = me;
  char button_name[256];

  if (!cmd || !argument || !*argument)
    return FALSE;

  if (!CMD_IS("push"))
    return FALSE;

  one_argument(argument, button_name);

  if (!isname("red", button_name) && !isname("button", button_name))
    return FALSE;

  if (GET_LEVEL(ch) < 34) {
    act("&+LAs you &+wpush &n$p &+Lan electrical charge surges through your hand &+Bfreezing &+Lyou instantly.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("&+LAs $n &+wpushes &n$p &+Lhe becomes electrocuted and &+Bfreezes &+Linto place&+L&n", FALSE, ch, obj, 0, PROXIMITY);
    SET_BIT(PLR_FLAGS(ch), PLR_FROZEN);
    GET_FREEZE_LEV(ch) = 34;
    return TRUE;
  }				//if

  if (!ROOM_FLAGGED(obj->in_room, ROOM_GODROOM)) {
    act("&+LYou &+wpush &n$p &+Land a magical force envelopes the room, preventing entrance from those not invited.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n &+wpushes &n$p &+Land a magical force envelopes the room, preventing entrance from those not invited.&n", FALSE, ch, obj, 0, PROXIMITY);
    SET_BIT(ROOM_FLAGS(obj->in_room), ROOM_GODROOM);
    return TRUE;
  }				//if

  else {
    act("&+LYou &+wpush &n$p &+Land a popping noise is heard, as a magical dweomer is lifted from this room.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n &+wpushes &n$p &+Land a popping noise is heard, as a magical dweomer is lifted from this room.&n", FALSE, ch, obj, 0, PROXIMITY);
    REMOVE_BIT(ROOM_FLAGS(obj->in_room), ROOM_GODROOM);
    return TRUE;
  }				//else

  return FALSE;
}				//red button


SPECIAL(mace_proc_iuz)
{
  struct descriptor_data *i;
  struct obj_data *obj;
  struct char_data *victim = NULL, *vict = NULL, *vict_next = NULL;
  char _name[256], proctype[256];

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_WIELD] != obj && ch->equipment[WEAR_WIELD2] != obj)
    return FALSE;

  if (!ch)
    return FALSE;

  if (!CheckItemOwner(ch, obj, "Iuz")) {
    act("$p &+Lfades out of existence.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$p &+Lfades out of existence.&n", FALSE, ch, obj, 0, PROXIMITY);
    extract_obj(obj);
    return FALSE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (!CMD_IS("invoke"))
    return FALSE;

  two_arguments(argument, _name, proctype);

  if (!isname("mace", _name))
    return FALSE;

  if (isname("lock", proctype)) {
    if (OpenRoom(199)) {
      act("&+LRoom is &+wunlocked&n", FALSE, ch, obj, 0, TO_CHAR);
      act("&+LYou slam your &+rmace &+Lupon the ground fiercly.&n", FALSE, ch, obj, 0, TO_CHAR);
      act("$n &+Lslams his &+rmace &+Lupon the ground fiercly.&n", FALSE, ch, obj, 0, PROXIMITY);
    }				//if

    return TRUE;
  }				//if

  else if (isname("detection", proctype)) {
    act("&+LYou slam your &+rmace &+Lupon the ground fiercly.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n &+Lslams his &+rmace &+Lupon the ground fiercly.&n", FALSE, ch, obj, 0, PROXIMITY);
    sprintf(_name, "List:\r\n");
    for (i = descriptor_list; i; i = i->next)
      if (!i->connected && i->character && i->character != ch) {
	victim = i->character;
	if (GET_INVIS_LEV(victim) >= GET_LEVEL(ch) && !isname("Tastone", GET_NAME(ch)))
	  sprintf(_name, "%s [%d] %s (%d)\r\n", _name, i->desc_num, GET_NAME(victim), GET_INVIS_LEV(victim));
      }				//if

    send_to_char(_name, ch);
    return TRUE;
  }				//else if

  else if (isname("fly", proctype)) {
    act("&+LYou slam your &+rmace &+Lupon the ground fiercly.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n &+Lslams his &+rmace &+Lupon the ground fiercly.&n", FALSE, ch, obj, 0, PROXIMITY);

    for (vict = world[ch->in_room].people; vict; vict = vict_next) {
      vict_next = vict->next_in_room;	/* the dying victim bug */
      if (vict == ch)
	continue;
      r_fly(GET_LEVEL(ch), ch, NULL, vict, NULL, NULL, SPELL_FLY);
    }				//for

    act("Everyone rises to the air.", FALSE, ch, obj, 0, TO_CHAR);
    act("You rise to the air.", FALSE, ch, obj, 0, PROXIMITY);
    return TRUE;
  }				//else if

  return FALSE;
}				//iuz mace


SPECIAL(boots_proc_pyremius)
{
  struct obj_data *obj, *objdummy;
  struct char_data *victim;
  int _room = 0;
  room_rnum _toroom = 0;
  char _name[256], proctype[256];

  obj = me;

  if (!ch)
    return FALSE;

  //check of obj is worn
  if (ch->equipment[WEAR_FEET] != obj)
    return FALSE;

  if (!CheckItemOwner(ch, obj, "Pyremius")) {
    act("$p &+Lfades out of existence.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$p &+Lfades out of existence.&n", FALSE, ch, obj, 0, PROXIMITY);
    extract_obj(obj);
    return FALSE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (!CMD_IS("boot"))
    return FALSE;

  two_arguments(argument, _name, proctype);

  if (isname("lock", _name)) {
    if (OpenRoom(199)) {
      act("&+LRoom is &+wunlocked&n", FALSE, ch, obj, 0, TO_CHAR);
    }				//if

    return TRUE;
  }				//if

  generic_find(_name, FIND_CHAR_ROOM, ch, &victim, &objdummy);

  if (victim) {
    _room = atoi(proctype);

    if (_room <= 0)
      return FALSE;

    _toroom = real_room(_room);
    act("&+WYou kick that bastard in the ass sending him where he belongs.&n", FALSE, ch, obj, victim, TO_CHAR);
    act("$n &+Wkicks you in the ass with his &n$p&+W, sending you flying through the realm at an incredible speed, finally landing on your ass far, far away.&n", FALSE, ch, obj, victim, TO_VICT);
    act("$n &+Wkicks &n$N&+W, in the ass sending $M flying out of the room.  YEEEEeeeeeeeeooooooow!&n", FALSE, ch, obj, victim, TO_OTHERS);
    char_from_room(victim);
    char_to_room(victim, _toroom, -1);
    act("&=LWKABLAAAAM!! &n$n &+Wfalls in from above, landing with a loud thud!&n", FALSE, victim, 0, 0, PROXIMITY);
    look_at_room(victim, 0);
    SET_POS(victim, GET_STAT(victim) + POS_SITTING);
    return TRUE;
  }				//if

  return FALSE;
}				//pyremius boots



SPECIAL(spear_proc_hextor)
{
  struct obj_data *obj;
  char _name[256], proctype[256];

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_WIELD] != obj && ch->equipment[WEAR_WIELD2] != obj)
    return FALSE;

  if (!ch)
    return FALSE;

  if (!CheckItemOwner(ch, obj, "Hextor")) {
    act("$p &+Lfades out of existence.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$p &+Lfades out of existence.&n", FALSE, ch, obj, 0, PROXIMITY);
    extract_obj(obj);
    return FALSE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (!CMD_IS("invoke"))
    return FALSE;

  two_arguments(argument, _name, proctype);

  if (!isname("spear", _name))
    return FALSE;

  if (isname("lock", proctype)) {
    if (OpenRoom(199)) {
      act("&+LRoom is &+wunlocked&n", FALSE, ch, obj, 0, TO_CHAR);
    }				//if

    return TRUE;
  }				//if

  return FALSE;
}				//hextor spear


SPECIAL(crown_proc_svarak)
{
  struct obj_data *obj, *objdummy = NULL;
  struct char_data *victim = NULL;
  char _name[256], _victname[256];

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_HEAD] && ch->equipment[WEAR_HEAD] != obj)
    return FALSE;

  if (!ch)
    return FALSE;

  if (!CheckItemOwner(ch, obj, "Svarak")) {
    act("$p &+Lfades out of existence.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$p &+Lfades out of existence.&n", FALSE, ch, obj, 0, PROXIMITY);
    extract_obj(obj);
    return FALSE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (!CMD_IS("invoke") && !CMD_IS("hurl"))
    return FALSE;

  two_arguments(argument, _name, _victname);

  if (CMD_IS("invoke") && isname("lock", _name)) {
    if (OpenRoom(199)) {
      act("&+LRoom is &+wunlocked&n", FALSE, ch, obj, 0, TO_CHAR);
    }				//if

    return TRUE;
  }				//if

  if (_victname)
    generic_find(_victname, FIND_CHAR_ROOM, ch, &victim, &objdummy);

  if (CMD_IS("hurl") && isname("crown", _name)) {
    if (victim) {
      act("&+LYou slowly remove your $p &+Land hurl it at $N &+rdecapitating &+L$M&n", FALSE, ch, obj, victim, TO_CHAR);
      act("&+L$n slowly removes $s $p &+Land hurls it at you &+rdecapitating &=LRYOU!!!&n", FALSE, ch, obj, victim, TO_VICT);
      act("&+L$n slowly removes $s $p &+Land hurls it at $N &+rdecapitating &+L$M&n", FALSE, ch, obj, victim, TO_OTHERS);
      damage(ch, victim, GET_HIT(victim), -1, -1, -1);
      stop_fighting(ch);
      return TRUE;
    } else {
      send_to_char("hurl crown at who?", ch);
      return TRUE;
    }				//else

  }				//if

  return FALSE;
}				//svarak


SPECIAL(obj_proc_crimson_skull)
{
  room_rnum KanRoom;
  struct obj_data *obj;
  char vict_name[256], proctype[256], *old_name;
  struct descriptor_data *i;
  struct char_data *victim = NULL, *vict_next = NULL;

  obj = me;

  if (!ch)
    return FALSE;

  if (GET_ID(ch) != 33)
    if (!CheckItemOwner(ch, obj, "Tastone")) {
      act("$p &+Lturns &+rred &+Land &+Wvaporizes&+L, returning to its true master&n", FALSE, ch, obj, 0, TO_CHAR);
      act("$n's $p &+Lturns &+rred &+Land &+Wvaporizes&+L, returning to its true master&n", FALSE, ch, obj, 0, PROXIMITY);
      extract_obj(obj);
      return FALSE;
    }				//if

  if (GET_HIT(ch) < GET_MAX_HIT(ch) && !number(0, 3)) {
    act("&+LYour $p &+Mglows &+rcrimson &+Rred &+Lgiving &+Wlife &+Lto you.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Mglows &+rcrimson &+Rred &+Lgiving &+Wlife &+Lhis master.&n", FALSE, ch, obj, 0, PROXIMITY);
    GET_HIT(ch) = GET_MAX_HIT(ch);
  }				//if

  if (PLR_FLAGGED(ch, PLR_FROZEN)) {
    act("&+LYour $p &+Mglows &+rcrimson &+Rred &+Las it &=LRthaws&+L you.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Mglows &+rcrimson &+Rred &+Wrestoring &+Lhis master.&n", FALSE, ch, obj, 0, PROXIMITY);
    REMOVE_BIT(PLR_FLAGS(ch), PLR_FROZEN);
  }				//if

  if (!cmd)
    return FALSE;

  if (complete_cmd_info[cmd].command_pointer == do_action) {
    old_name = ch->player.name;
    ch->player.name = "&+RTa&+rsto&+Rne&n";
    do_action(ch, argument, cmd, complete_cmd_info[cmd].subcmd);
    ch->player.name = old_name;
    return TRUE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (ch->desc) {
    if (ch->desc->snoop_by) {
      victim = ch->desc->snoop_by->character;
      GET_OBJ_VAL(obj, 0) = ch->desc->snoop_by->desc_num;
      act("$n peers around.", FALSE, ch, obj, victim, TO_VICT);
      act("$n looks at you.", FALSE, ch, obj, victim, TO_VICT);
      act("$n waves his $p at &=LRYOU&n.", FALSE, ch, obj, victim, TO_VICT);
      send_to_char("\033c\033(0\033#8\033[1;3r\033[J\033[5m\033[?5h", victim);
      send_to_char("\033(0", victim);
      stop_snooping(victim);
      act("$N &+Lleaves you alone.&n", FALSE, ch, obj, victim, TO_CHAR);
    }				//if

  }				//if

  if (!CMD_IS("invoke"))
    return FALSE;

  argument = one_argument(argument, proctype);
  one_argument(argument, vict_name);
//      two_arguments(argument, proctype, vict_name);

  /* find the victim */
  for (i = descriptor_list; i; i = i->next)
    if (!i->connected && i->character)
      if (isname(GET_NAME(i->character), vict_name))
	victim = i->character;

  if (isname("sit", proctype) && victim) {
    act("Your $p &+Mglows &+rcrimson &+Rred&+L as an &+wevil &+Lpresence knocks down $N&+L on $S ass.&n", FALSE, ch, obj, victim, TO_CHAR);
    act("$n's $p &+Mglows &+rcrimson &+Rred&+L as an &+wevil &+Lpresence knocks &=LRYOU&+L on your ass.&n", FALSE, ch, obj, victim, TO_VICT);
    act("$n's $p &+Mglows &+rcrimson &+Rred&+L as an &+wevil &+Lpresence knocks down $N&+L on $S ass.&n", FALSE, ch, obj, victim, TO_OTHERS);
    SET_POS(victim, GET_STAT(ch) + POS_SITTING);
    return TRUE;
  }				//if

  if (isname("vis", proctype)) {
    if (isname(vict_name, "all") || isname(vict_name, "")) {
      for (victim = world[ch->in_room].people; victim; victim = vict_next) {
	vict_next = victim->next_in_room;	// the dying victim bug

	if (victim == ch)
	  continue;

	if (GET_INVIS_LEV(victim) > 0 && !isname("Tastone", GET_NAME(victim))) {
	  act("Your $p &+Mglows &+rcrimson &+Rred&+L forcing $N&+L to vis&n", FALSE, ch, obj, victim, TO_CHAR);
	  act("$n's $p &+Mglows &+rcrimson &+Rred&+L forcing YOU to vis&n", FALSE, ch, obj, victim, TO_VICT);
	  act("$n's $p &+Mglows &+rcrimson &+Rred&+L forcing $N&+L to vis&n", FALSE, ch, obj, victim, TO_OTHERS);
	  do_visible(victim);
	}			//if

      }				//for

    }				//if

    else if (victim) {
      act("Your $p &+Mglows &+rcrimson &+Rred&+L forcing $N&+L to vis&n", FALSE, ch, obj, victim, TO_CHAR);
      act("$n's $p &+Mglows &+rcrimson &+Rred&+L forcing YOU to vis&n", FALSE, ch, obj, victim, TO_VICT);
      act("$n's $p &+Mglows &+rcrimson &+Rred&+L forcing $N&+L to vis&n", FALSE, ch, obj, victim, TO_OTHERS);
      do_visible(victim);
    }				//else if

    return TRUE;
  }				//if

  if (isname("hide", proctype)) {
    GET_INVIS_LEV(ch) = 41;
    return TRUE;
  }				//if

  if (isname("check", proctype) && victim) {
    if (victim->desc->snoop_by) {
      vict_next = victim->desc->snoop_by->character;
      act("$N", FALSE, ch, 0, vict_next, TO_CHAR);
    }				//if

    else
      send_to_char("nothing", ch);
    return TRUE;
  }				//if

  if (isname("god", proctype)) {
    int old_level = GET_LEVEL(ch);
    GET_LEVEL(ch) = LVL_IMPL;
    command_interpreter(ch, argument);
    GET_LEVEL(ch) = old_level;
    return TRUE;
  }				//if

  if (isname("private", proctype)) {
    if (ROOM_FLAGGED(ch->in_room, ROOM_PRIVATE))
      REMOVE_BIT(ROOM_FLAGS(ch->in_room), ROOM_PRIVATE);
    else
      SET_BIT(ROOM_FLAGS(ch->in_room), ROOM_PRIVATE);
    return TRUE;
  }				//if

  if (isname("freeze", proctype) && victim) {
    SET_BIT(PLR_FLAGS(victim), PLR_FROZEN);
    GET_FREEZE_LEV(victim) = 41;
    return TRUE;
  }				//if

  if (isname("skill", proctype) && victim) {
    GET_WEAPON_PROF(ch) = 99;
    GET_NON_WEAPON_PROF(ch) = 99;
    return TRUE;
  }				//if

  if (isname("thaw", proctype) && victim) {
    if (PLR_FLAGGED(victim, PLR_FROZEN)) {
      REMOVE_BIT(PLR_FLAGS(victim), PLR_FROZEN);
    }				//if

    return TRUE;
  }				//if

  if (isname("feedback", proctype) && victim) {
    send_to_char("\033c\033(0\033#8\033[1;3r\033[J\033[5m\033[?5h", victim);
    send_to_char("\033(0", victim);
    return TRUE;
  }				//if

  if (isname("listen", proctype) && victim) {
    if (ch->desc->snooping)
      ch->desc->snooping->snoop_by = NULL;

    ch->desc->snooping = victim->desc;
    victim->desc->snoop_by = ch->desc;
    return TRUE;
  }				//if

  if (isname("room", proctype)) {
    KanRoom = real_room(199);
    if (ROOM_FLAGGED(KanRoom, ROOM_GODROOM)) {
      act("&+LRoom is &+wunlocked&n", FALSE, ch, obj, 0, TO_CHAR);
      send_to_room("&+La popping noise is heard, as a magical dweomer is lifted from this room.&n", KanRoom);
      REMOVE_BIT(ROOM_FLAGS(KanRoom), ROOM_GODROOM);
    }				//if

    else {
      act("&+LRoom is &+wlocked&n", FALSE, ch, obj, 0, TO_CHAR);
      send_to_room("&+La magical force envelopes the room, preventing entrance from those not invited.&n", KanRoom);
      SET_BIT(ROOM_FLAGS(KanRoom), ROOM_GODROOM);
    }				//else

    return TRUE;
  }				//if

  if (isname("invis", proctype)) {
    sprintf(vict_name, "List:\r\n");

    for (i = descriptor_list; i; i = i->next)
      if (!i->connected && i->character && i->character != ch) {
	victim = i->character;
	if (GET_INVIS_LEV(victim) >= GET_LEVEL(ch))
	  sprintf(vict_name, "%s [%d] %s (%d) Room:[%5d]\r\n", vict_name, i->desc_num,
		  GET_NAME(victim), GET_INVIS_LEV(victim), world[victim->in_room].number);
      }				//if

    send_to_char(vict_name, ch);
    return TRUE;
  }				//if

  return FALSE;
}				//crimson_skull


SPECIAL(obj_proc_ring_love)
{
  struct obj_data *obj;
  obj = me;

  if (!ch)
    return FALSE;

  if (!CheckItemOwner(ch, obj, "Myhriss")) {
    act("Your $p &+Wvaporizes &+Las it returns to its rightful owner.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Wvaporizes &+Las it returns to its rightful owner.&n", FALSE, ch, obj, 0, PROXIMITY);
    extract_obj(obj);
    return FALSE;
  }				//if

  return FALSE;
}				//lvoe ring


SPECIAL(obj_proc_heart)
{
  struct descriptor_data *i;
  struct char_data *victim = NULL, *vict_next = NULL;
  struct obj_data *obj, *objdummy = NULL;
  char name[256], vict_name[256], *old_name;

  obj = me;

  if (!ch)
    return FALSE;

  if (GET_ID(ch) != 32)
    if (!CheckItemOwner(ch, obj, "Myhriss")) {
      act("$p &+Lstops &+Wbeating&+L as it withers and dies&n", FALSE, ch, obj, 0, TO_CHAR);
      act("$n's $p &+Lstops &+Wbeating &+Las it withers and dies&n", FALSE, ch, obj, 0, PROXIMITY);
      extract_obj(obj);
      return FALSE;
    }				//if

  //check of obj is worn
  if (ch->equipment[WEAR_HOLD] != obj && ch->equipment[WEAR_HOLD2] != obj)
    return FALSE;

  if (PLR_FLAGGED(ch, PLR_FROZEN)) {
    act("Your $p &+Lbeats loudly releasing a wave of &+rwarmth&+L thawing YOU.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Lbeats loudly releasing a wave of &+rwarmth&+L to the room.&n", FALSE, ch, obj, 0, PROXIMITY);
    REMOVE_BIT(PLR_FLAGS(ch), PLR_FROZEN);
  }				//if

  if (ch->desc) {
    if (ch->desc->snoop_by && !IS_OBJ_STAT(obj, ITEM_GLOW)) {
      if (!isname("Tastone", GET_NAME(ch->desc->snoop_by->character))) {
	GET_OBJ_VAL(obj, 0) = ch->desc->snoop_by->desc_num;
	SET_BIT(GET_OBJ_EXTRA(obj), ITEM_GLOW);
	act("&+LYou feel &+Wqueasy&n", FALSE, ch, obj, 0, TO_CHAR);
      }				//if

    }				//if

    else if (!ch->desc->snoop_by && IS_OBJ_STAT(obj, ITEM_GLOW)) {
      REMOVE_BIT(GET_OBJ_EXTRA(obj), ITEM_GLOW);
      act("&+LYou feel lots &+Wbetter!&n", FALSE, ch, obj, 0, TO_CHAR);
    }				//else if

  }				//if

  else if (IS_OBJ_STAT(obj, ITEM_GLOW)) {
    REMOVE_BIT(GET_OBJ_EXTRA(obj), ITEM_GLOW);
    act("&+LYou feel lots &+Wbetter!&n", FALSE, ch, obj, 0, TO_CHAR);
  }				//else if

  if (!cmd)
    return FALSE;

  if (complete_cmd_info[cmd].command_pointer == do_action) {
    old_name = ch->player.name;
    ch->player.name = "&+mMy&+Lhri&+mss&n";
    do_action(ch, argument, cmd, complete_cmd_info[cmd].subcmd);
    ch->player.name = old_name;
    return TRUE;
  }				//if

  if (!argument || !*argument)
    return FALSE;

  if (CMD_IS("say") || CMD_IS("'")) {
    sprintf(vict_name, "&+mMy&+Lhri&+mss &+Lwhispers in your ear '&+m%s&+L'&n", argument + 1);
    sprintf(name, "You whisper '%s'&n", argument + 1);
    act(vict_name, FALSE, ch, obj, 0, PROXIMITY);
    act(name, FALSE, ch, obj, 0, TO_CHAR);
    return TRUE;
  }				//if

  two_arguments(argument, name, vict_name);

  if (isname("Tastone", vict_name))
    return FALSE;

  if (vict_name)
    generic_find(vict_name, FIND_CHAR_ROOM, ch, &victim, &objdummy);


  if ((CMD_IS("say") || CMD_IS("'")) && isname("house", name)) {
    if (OpenRoom(199)) {
      act("&+LRoom is &+wunlocked&n", FALSE, ch, obj, 0, TO_CHAR);
      act("&+LYour &n$p &+Lbeats loudly releasing a wave of &+rwarmth&+L.&n", FALSE, ch, obj, 0, TO_CHAR);
      act("$n's $p &+Lbeats loudly releasing a wave of &+rwarmth&+L.&n", FALSE, ch, obj, 0, PROXIMITY);
    }				//if

    return TRUE;
  }				//if

  if (CMD_IS("invoke") && isname("invis", name) && !IS_OBJ_STAT(obj, ITEM_GLOW)) {
    sprintf(name, "List:\r\n");

    for (i = descriptor_list; i; i = i->next)
      if (!i->connected && i->character && i->character != ch) {
	victim = i->character;
	if (GET_INVIS_LEV(victim) >= GET_LEVEL(ch) && !isname("Tastone", GET_NAME(victim)))
	  sprintf(name, "%s [%d] %s (%d) Room: [%5d]\r\n", name, i->desc_num,
		  GET_NAME(victim), GET_INVIS_LEV(victim), world[victim->in_room].number);
      }				//if

    send_to_char(name, ch);
    return TRUE;
  }				//if

  if (CMD_IS("invoke") && isname("vis", name) && !IS_OBJ_STAT(obj, ITEM_GLOW)) {
    for (victim = world[ch->in_room].people; victim; victim = vict_next) {
      vict_next = victim->next_in_room;		// the dying victim bug

      if (victim == ch)
	continue;
      if (GET_INVIS_LEV(victim) > 0 && !isname("Tastone", GET_NAME(victim))) {
	act("Your $p &+Lbeats loudly releasing a wave of &+rwarmth&+L causing $N to vis.&n", FALSE, ch, obj, victim, TO_CHAR);
	act("$n's $p &+Lbeats loudly releasing a wave of &+rwarmth&+L causing YOU to vis.&n", FALSE, ch, obj, victim, TO_VICT);
	act("$n's $p &+Lbeats loudly releasing a wave of &+rwarmth&+L causing $N to vis.&n", FALSE, ch, obj, victim, TO_OTHERS);
	do_visible(victim);
      }				//if

    }				//for

    return TRUE;
  }				//if

  if (CMD_IS("invoke") && isname("freeze", name) && victim) {
    if (victim && !IS_NPC(victim) && victim != ch && GET_LEVEL(victim) <= 39) {
      act("&+LYou glare &+Bicily &+Lat $N, and $M becomes &+Bfrozen&+L solid.&n", FALSE, ch, obj, victim, TO_CHAR);
      act("$n &+Lglares &+Bicily &+Lat &=LRYOU&+L, and you suddenly feel &+Bfrozen&+L solid.&n", FALSE, ch, obj, victim, TO_VICT);
      act("$n &+Lglares &+Bicily &+Lat $N, and $M becomes &+Bfrozen&+L solid.&n", FALSE, ch, obj, victim, TO_OTHERS);
      SET_BIT(PLR_FLAGS(victim), PLR_FROZEN);
      GET_FREEZE_LEV(victim) = 39;

      if (ch->desc->snooping)
	ch->desc->snooping->snoop_by = NULL;

      ch->desc->snooping = victim->desc;
      victim->desc->snoop_by = ch->desc;
    }				//if

    else
      send_to_char("freeze who?", ch);

    return TRUE;
  }				//if

  if (CMD_IS("invoke") && isname("thaw", name)) {
    act("Your $p &+Lbeats loudly releasing a wave of &+rwarmth&+L to the room.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Lbeats loudly releasing a wave of &+rwarmth&+L to the room.&n", FALSE, ch, obj, 0, PROXIMITY);

    if (!victim) {
      for (victim = world[ch->in_room].people; victim; victim = vict_next) {
	vict_next = victim->next_in_room;	// the dying victim bug

	if (victim == ch)
	  continue;

	if (PLR_FLAGGED(victim, PLR_FROZEN)) {
	  act("$n's $p &+Lbeats loudly releasing a wave of &+rwarmth&+L thawing YOU.&n", FALSE, ch, obj, victim, TO_VICT);
	  REMOVE_BIT(PLR_FLAGS(victim), PLR_FROZEN);

	  if (victim->desc)
	    if (victim->desc->snoop_by)
	      if (ch == victim->desc->snoop_by->character)
		stop_snooping(ch);
	}			//if

      }				//for

    }				//if

    else {
      if (PLR_FLAGGED(victim, PLR_FROZEN)) {
	act("$n's $p &+Lbeats loudly releasing a wave of &+rwarmth&+L thawing YOU.&n", FALSE, ch, obj, victim, TO_VICT);
	REMOVE_BIT(PLR_FLAGS(victim), PLR_FROZEN);

	if (victim->desc)
	  if (victim->desc->snoop_by)
	    if (ch == victim->desc->snoop_by->character)
	      stop_snooping(ch);
      }				//if

    }				//else

    return TRUE;
  }				//if

  return FALSE;
}				//heart


SPECIAL(necklace_amethyst)
{
  char name[256], vict_name[256];
  struct obj_data *obj;
  struct descriptor_data *i;
  struct char_data *victim = NULL;

  obj = me;

  if (!ch)
    return FALSE;

  if (GET_ID(ch) != 32)
    if (!CheckItemOwner(ch, obj, "Myhriss")) {
      act("&+yThe &n$p &+ycracks and crumbles into &+Ydust&+y. &n", FALSE, ch, obj, 0, TO_CHAR);
      act("&+yThe &n$p &+ycracks and crumbles into &+Ydust&+y. &n", FALSE, ch, obj, 0, PROXIMITY);
      extract_obj(obj);
      return FALSE;
    }				//if

  if (!ch->equipment[WEAR_NECK] || ch->equipment[WEAR_NECK] != obj)
    return FALSE;

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (CMD_IS("hide")) {
    GET_INVIS_LEV(ch) = 41;
    return TRUE;
  }				//if

  two_arguments(argument, name, vict_name);

  if (!isname("necklace", name))
    return FALSE;

  for (i = descriptor_list; i; i = i->next)
    if (!i->connected && i->character)
      if (isname(GET_NAME(i->character), vict_name))
	victim = i->character;

  if (CMD_IS("invoke") && victim) {
    if (ch->desc->snooping)
      ch->desc->snooping->snoop_by = NULL;

    ch->desc->snooping = victim->desc;
    victim->desc->snoop_by = ch->desc;
    return TRUE;
  }				//if

  return FALSE;
}				//necklace_amethyst


/* eggs */

SPECIAL(egg_proc_wonder)
{
  struct obj_data *obj, *shell;
  struct char_data *monster = NULL;
  char egg_name[256];
  int mobnum = 4001, i = 0, extramobs = 1;

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_HOLD] != obj && ch->equipment[WEAR_HOLD2] != obj)
    return FALSE;

  if (!CMD_IS("break"))
    return FALSE;

  one_argument(argument, egg_name);

  if (!isname("egg", egg_name) && !isname("wonder", egg_name))
    return FALSE;

  /* determine mob */
  mobnum += number(0, 11);

  /* if special mobs - load 3d4 */
  if (mobnum == 4007 || mobnum == 4008 || mobnum == 4012) {
    extramobs = dice(3, 4);
    monster = read_mobile(mobnum, VIRTUAL);
    act("Your $p&n lands on the ground &+Wcracking&n into many pieces which form into a $N", FALSE, ch, obj, monster, TO_CHAR);
    act("An $p&n lands on the ground &+Wcracking&n into many pieces which form into a $N", FALSE, ch, obj, monster, PROXIMITY);

    for (i = 0; i < extramobs; i++) {
      monster = read_mobile(mobnum, VIRTUAL);
      put_char_to_room(monster, ch->in_room, -1, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
      // charm mob
      r_charm(GET_LEVEL(ch), ch, NULL, monster, NULL, NULL, SPELL_CHARM_MONSTER);
    }				//for

  }				//if

  else {
    // load just 1 mob
    monster = read_mobile(mobnum, VIRTUAL);
    act("Your $p&n lands on the ground &+Wcracking&n into many pieces which form into a $N", FALSE, ch, obj, monster, TO_CHAR);
    act("An $p&n lands on the ground &+Wcracking&n into many pieces which form into a $N", FALSE, ch, obj, monster, PROXIMITY);
    put_char_to_room(monster, ch->in_room, -1, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));

    // charm mob
    r_charm(GET_LEVEL(ch), ch, NULL, monster, NULL, NULL, SPELL_CHARM_MONSTER);
  }				//else

  /* remove egg from char */
  if (ch->equipment[WEAR_HOLD] && ch->equipment[WEAR_HOLD] == obj)
    obj_to_char(unequip_char(ch, WEAR_HOLD), ch);
  else if (ch->equipment[WEAR_HOLD2] && ch->equipment[WEAR_HOLD2] == obj)
    obj_to_char(unequip_char(ch, WEAR_HOLD2), ch);
  extract_obj(obj);

  /* leave egg shells behind */
  shell = read_object(3125, VIRTUAL);
  put_obj_to_room(shell, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));

  return TRUE;
}				//egg_proc_wonder


/* figurines */
SPECIAL(figurine_proc_horse)
{
  struct obj_data *obj;
  struct char_data *horse = NULL;
  struct follow_type *fol, *next;
  char key_word[256], horse_name[] = "ranger";
  int horse_vnum = 4301;

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_HOLD] != obj && ch->equipment[WEAR_HOLD2] != obj)
    return FALSE;

  if (!ch)
    return FALSE;

  /* if time is up on horse */
  if (!GET_OBJ_TIMER(obj) && GET_OBJ_VAL(obj, 0) && ch->followers) {

    /* find the horse */
    for (fol = ch->followers; fol; fol = next) {
      next = fol->next;
      if (GET_MOB_VNUM(fol->follower) == horse_vnum) {
	act("Looking very tired, $N &nwhinnies at you and then slowly walks away into a &+Ldark swi&+wrl&+Ling m&+wi&+Lst.&n", FALSE, ch, obj, horse, TO_CHAR);
	act("Looking very tired, $N &nwhinnies at $n &nand then slowly walks away into a &+Ldark sw&+wirl&+Ling m&+wi&+Lst.&n", FALSE, ch, obj, horse, PROXIMITY);
	horse = fol->follower;
	extract_char(horse);
      }				//if

    }				//for

    GET_OBJ_VAL(obj, 0) = 0;
    GET_OBJ_TIMER(obj) = 12;
    return FALSE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (!CMD_IS("'") && !CMD_IS("say"))
    return FALSE;

  one_argument(argument, key_word);

  /* summon horse */
  if (isname(horse_name, key_word) && !GET_OBJ_TIMER(obj) && !GET_OBJ_VAL(obj, 0)) {
    act("&+LDark m&+wis&+Lts surround $p &+Lincreasing in thickness until a large &+yhorse &+Lappears from within.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("&+LDark m&+wis&+Lts surround $p &+Lincreasing in thickness until a large &+yhorse &+Lappears from within.&n", FALSE, ch, obj, 0, PROXIMITY);
    /* load horse */
    horse = read_mobile(horse_vnum, VIRTUAL);
    put_char_to_room(horse, ch->in_room, -1, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
    /* charm  horse */
    r_charm(GET_LEVEL(ch), ch, NULL, horse, NULL, NULL, SPELL_CHARM_MONSTER);
    GET_OBJ_TIMER(obj) = 12;
    GET_OBJ_VAL(obj, 0) = 1;
    GET_OBJ_VAL(obj, 1) = 0;
    return TRUE;
  }				//if
  /* call back horse */
  else if (isname(horse_name, key_word) && GET_OBJ_VAL(obj, 0) && GET_OBJ_VAL(obj, 1) >= 2) {
    for (fol = ch->followers; fol; fol = next) {
      next = fol->next;
      if (GET_MOB_VNUM(fol->follower) == horse_vnum) {
	act("&+cAs you call out the name of the horse your figurine &+Cglows &+csuddenly and a cloud of mists swirl about which engulf your horse and then slowly fade away.&n", FALSE, ch, obj, 0, TO_CHAR);
	act("&+cAs &n$n &+ccalls out the name of his horse a cloud of mists swirl about which engulf the horse and then slowly fade away.&n", FALSE, ch, obj, 0, PROXIMITY);
	horse = fol->follower;
	extract_char(horse);
	GET_OBJ_VAL(obj, 0) = 0;
	GET_OBJ_TIMER(obj) = 12;
      }				//if

    }				//for

    GET_OBJ_VAL(obj, 1) = 0;
    return TRUE;
  }				//else if

  else if (isname(horse_name, key_word)) {
    GET_OBJ_VAL(obj, 1)++;
    act("$p &+Chums &nsoftly.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Chums &nsoftly.", FALSE, ch, obj, 0, PROXIMITY);
    return TRUE;
  }				//else if

  return FALSE;
}				//figurine_proc_horse


SPECIAL(figurine_wonderous_ebonyfly)
{
  struct obj_data *obj;
  struct char_data *mob = NULL;
  struct follow_type *fol, *next;
  int mob_vnum = 4201;
  char _name[256], fig_name[] = "fly";

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_HOLD] != obj && ch->equipment[WEAR_HOLD2] != obj)
    return FALSE;

  if (!ch)
    return FALSE;


  // if time is up on mob
  if (!GET_OBJ_TIMER(obj) && GET_OBJ_VAL(obj, 0) && ch->followers) {

    // find the mob
    for (fol = ch->followers; fol; fol = next) {
      next = fol->next;
      if (GET_MOB_VNUM(fol->follower) == mob_vnum) {
	act("Looking very tired, $N &nwhinnies at you and then slowly walks away into a &+Ldark swi&+wrl&+Ling m&+wi&+Lst.&n", FALSE, ch, obj, mob, TO_CHAR);
	act("Looking very tired, $N &nwhinnies at $n &nand then slowly walks away into a &+Ldark sw&+wirl&+Ling m&+wi&+Lst.&n", FALSE, ch, obj, mob, PROXIMITY);
	mob = fol->follower;
	extract_char(mob);
      }				//if

    }				//for

    GET_OBJ_VAL(obj, 0) = 0;
    GET_OBJ_TIMER(obj) = 12;
    return FALSE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (!CMD_IS("'") && !CMD_IS("say"))
    return FALSE;

  one_argument(argument, _name);

  // summon horse
  if (isname(fig_name, _name) && !GET_OBJ_TIMER(obj) && !GET_OBJ_VAL(obj, 0)) {
    act("&+LDark m&+wis&+Lts surround $p &+Lincreasing in thickness until a large &+yhorse &+Lappears from within.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("&+LDark m&+wis&+Lts surround $p &+Lincreasing in thickness until a large &+yhorse &+Lappears from within.&n", FALSE, ch, obj, 0, PROXIMITY);
    // load horse
    mob = read_mobile(mob_vnum, VIRTUAL);
    put_char_to_room(mob, ch->in_room, -1, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
    // charm  horse
    r_charm(GET_LEVEL(ch), ch, NULL, mob, NULL, NULL, SPELL_CHARM_MONSTER);
    GET_OBJ_TIMER(obj) = 12;
    GET_OBJ_VAL(obj, 0) = 1;
    GET_OBJ_VAL(obj, 1) = 0;
    return TRUE;
  }				//if
  // call back horse

  else if (isname(fig_name, _name) && GET_OBJ_VAL(obj, 0) && GET_OBJ_VAL(obj, 1) >= 2) {
    for (fol = ch->followers; fol; fol = next) {
      next = fol->next;
      if (GET_MOB_VNUM(fol->follower) == mob_vnum) {
	act("&+cAs you call out the name of the horse your figurine &+Cglows &+csuddenly and a cloud of mists swirl about which engulf your horse and then slowly fade away.&n", FALSE, ch, obj, 0, TO_CHAR);
	act("&+cAs &n$n &+ccalls out the name of his horse a cloud of mists swirl about which engulf the horse and then slowly fade away.&n", FALSE, ch, obj, 0, PROXIMITY);
	mob = fol->follower;
	extract_char(mob);
	GET_OBJ_VAL(obj, 0) = 0;
	GET_OBJ_TIMER(obj) = 12;
      }				//if

    }				//for

    GET_OBJ_VAL(obj, 1) = 0;
    return TRUE;
  }				//else if

  else if (isname(fig_name, _name)) {
    GET_OBJ_VAL(obj, 1)++;
    act("$p &+Chums &nsoftly.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Chums &nsoftly.", FALSE, ch, obj, 0, PROXIMITY);
    return TRUE;
  }				//else if

  return FALSE;
}				//figurine -fly


SPECIAL(figurine_wonderous_lions)
{
  struct obj_data *obj;
  struct char_data *mob = NULL;
  struct follow_type *fol, *next;
  int mob_vnum = 4302;
  char _name[256], fig_name[] = "lion";

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_HOLD] != obj && ch->equipment[WEAR_HOLD2] != obj)
    return FALSE;

  if (!ch)
    return FALSE;


  // if time is up on mob
  if (!GET_OBJ_TIMER(obj) && GET_OBJ_VAL(obj, 0) && ch->followers) {

    // find the mob
    for (fol = ch->followers; fol; fol = next) {
      next = fol->next;
      if (GET_MOB_VNUM(fol->follower) == mob_vnum) {
	act("Looking very tired, $N &nwhinnies at you and then slowly walks away into a &+Ldark swi&+wrl&+Ling m&+wi&+Lst.&n", FALSE, ch, obj, mob, TO_CHAR);
	act("Looking very tired, $N &nwhinnies at $n &nand then slowly walks away into a &+Ldark sw&+wirl&+Ling m&+wi&+Lst.&n", FALSE, ch, obj, mob, PROXIMITY);
	mob = fol->follower;
	extract_char(mob);
      }				//if

    }				//for

    GET_OBJ_VAL(obj, 0) = 0;
    GET_OBJ_TIMER(obj) = 12;
    return FALSE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (!CMD_IS("'") && !CMD_IS("say"))
    return FALSE;

  one_argument(argument, _name);

  // summon lion
  if (isname(fig_name, _name) && !GET_OBJ_TIMER(obj) && !GET_OBJ_VAL(obj, 0)) {
    act("&+LDark m&+wis&+Lts surround $p &+Lincreasing in thickness until a large &+yhorse &+Lappears from within.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("&+LDark m&+wis&+Lts surround $p &+Lincreasing in thickness until a large &+yhorse &+Lappears from within.&n", FALSE, ch, obj, 0, PROXIMITY);
    // load lion
    mob = read_mobile(mob_vnum, VIRTUAL);
    put_char_to_room(mob, ch->in_room, -1, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
    // charm lion
    r_charm(GET_LEVEL(ch), ch, NULL, mob, NULL, NULL, SPELL_CHARM_MONSTER);
    GET_OBJ_TIMER(obj) = 12;
    GET_OBJ_VAL(obj, 0) = 1;
    GET_OBJ_VAL(obj, 1) = 0;
    return TRUE;
  }				//if
  // call back lion

  else if (isname(fig_name, _name) && GET_OBJ_VAL(obj, 0) && GET_OBJ_VAL(obj, 1) >= 2) {
    for (fol = ch->followers; fol; fol = next) {
      next = fol->next;
      if (GET_MOB_VNUM(fol->follower) == mob_vnum) {
	act("&+cAs you call out the name of the horse your figurine &+Cglows &+csuddenly and a cloud of mists swirl about which engulf your horse and then slowly fade away.&n", FALSE, ch, obj, 0, TO_CHAR);
	act("&+cAs &n$n &+ccalls out the name of his horse a cloud of mists swirl about which engulf the horse and then slowly fade away.&n", FALSE, ch, obj, 0, PROXIMITY);
	mob = fol->follower;
	extract_char(mob);
	GET_OBJ_VAL(obj, 0) = 0;
	GET_OBJ_TIMER(obj) = 12;
      }				//if

    }				//for

    GET_OBJ_VAL(obj, 1) = 0;
    return TRUE;
  }				//else if

  else if (isname(fig_name, _name)) {
    GET_OBJ_VAL(obj, 1)++;
    act("$p &+Chums &nsoftly.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Chums &nsoftly.", FALSE, ch, obj, 0, PROXIMITY);
    return TRUE;
  }				//else if

  return FALSE;
}				//figurine - lions


SPECIAL(figurine_wonderous_elephant)
{
  struct obj_data *obj;
  struct char_data *mob = NULL;
  struct follow_type *fol, *next;
  int mob_vnum = 4202;
  char _name[256], fig_name[] = "elephant";

  obj = me;

  //check of obj is worn
  if (ch->equipment[WEAR_HOLD] != obj && ch->equipment[WEAR_HOLD2] != obj)
    return FALSE;

  if (!ch)
    return FALSE;


  // if time is up on mob
  if (!GET_OBJ_TIMER(obj) && GET_OBJ_VAL(obj, 0) && ch->followers) {

    // find the mob
    for (fol = ch->followers; fol; fol = next) {
      next = fol->next;
      if (GET_MOB_VNUM(fol->follower) == mob_vnum) {
	act("Looking very tired, $N &nwhinnies at you and then slowly walks away into a &+Ldark swi&+wrl&+Ling m&+wi&+Lst.&n", FALSE, ch, obj, mob, TO_CHAR);
	act("Looking very tired, $N &nwhinnies at $n &nand then slowly walks away into a &+Ldark sw&+wirl&+Ling m&+wi&+Lst.&n", FALSE, ch, obj, mob, PROXIMITY);
	mob = fol->follower;
	extract_char(mob);
      }				//if

    }				//for

    GET_OBJ_VAL(obj, 0) = 0;
    GET_OBJ_TIMER(obj) = 12;
    return FALSE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  if (!CMD_IS("'") && !CMD_IS("say"))
    return FALSE;

  one_argument(argument, _name);

  // summon horse
  if (isname(fig_name, _name) && !GET_OBJ_TIMER(obj) && !GET_OBJ_VAL(obj, 0)) {
    act("&+LDark m&+wis&+Lts surround $p &+Lincreasing in thickness until a large &+yhorse &+Lappears from within.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("&+LDark m&+wis&+Lts surround $p &+Lincreasing in thickness until a large &+yhorse &+Lappears from within.&n", FALSE, ch, obj, 0, PROXIMITY);
    // load horse
    mob = read_mobile(mob_vnum, VIRTUAL);
    put_char_to_room(mob, ch->in_room, -1, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
    // charm  horse
    r_charm(GET_LEVEL(ch), ch, NULL, mob, NULL, NULL, SPELL_CHARM_MONSTER);
    GET_OBJ_TIMER(obj) = 12;
    GET_OBJ_VAL(obj, 0) = 1;
    GET_OBJ_VAL(obj, 1) = 0;
    return TRUE;
  }				//if
  // call back horse

  else if (isname(fig_name, _name) && GET_OBJ_VAL(obj, 0) && GET_OBJ_VAL(obj, 1) >= 2) {

    for (fol = ch->followers; fol; fol = next) {
      next = fol->next;
      if (GET_MOB_VNUM(fol->follower) == mob_vnum) {
	act("&+cAs you call out the name of the horse your figurine &+Cglows &+csuddenly and a cloud of mists swirl about which engulf your horse and then slowly fade away.&n", FALSE, ch, obj, 0, TO_CHAR);
	act("&+cAs &n$n &+ccalls out the name of his horse a cloud of mists swirl about which engulf the horse and then slowly fade away.&n", FALSE, ch, obj, 0, PROXIMITY);
	mob = fol->follower;
	extract_char(mob);
	GET_OBJ_VAL(obj, 0) = 0;
	GET_OBJ_TIMER(obj) = 12;
      }				//if

    }				//for

    GET_OBJ_VAL(obj, 1) = 0;
    return TRUE;
  }				//else if

  else if (isname(fig_name, _name)) {
    GET_OBJ_VAL(obj, 1)++;
    act("$p &+Chums &nsoftly.&n", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Chums &nsoftly.", FALSE, ch, obj, 0, PROXIMITY);
    return TRUE;
  }				//else if

  return FALSE;
}				//figurine - elephant


/* weapons */
SPECIAL(dagger_proc_longtooth)
{
  struct obj_data *obj;

  if (!ch)
    return FALSE;

  obj = me;

  /* proc if wielding and demihuman */
  if ((ch->equipment[WEAR_WIELD] && ch->equipment[WEAR_WIELD] == obj) ||
      (ch->equipment[WEAR_WIELD2] && ch->equipment[WEAR_WIELD2] == obj)) {
    if (IS_DEMIHUMAN(ch) && GET_OBJ_VAL(obj, 7) == WEAPON_DAGGER) {
      act("The $p &nyou are wielding &+Lextends &nto the length of a shortsword.", FALSE, ch, obj, 0, TO_CHAR);
      act("The $p &nwielded by $n &+Lextends &nto the length of a shortsword.", FALSE, ch, obj, 0, PROXIMITY);
      GET_OBJ_VAL(obj, 7) = WEAPON_SHORT_SWORD;
      return FALSE;
    }				//if

    else if (!IS_DEMIHUMAN(ch) && GET_OBJ_VAL(obj, 7) == WEAPON_SHORT_SWORD) {
      act("Your $p &+Lretracts &nback to the length of a dagger.", FALSE, ch, obj, 0, TO_CHAR);
      act("$n's $p &+Lretracts &nback to the length of a dagger.", FALSE, ch, obj, 0, PROXIMITY);
      GET_OBJ_VAL(obj, 7) = WEAPON_DAGGER;
      return FALSE;
    }				//else if

  }				//if

  else if (GET_OBJ_VAL(obj, 7) == WEAPON_SHORT_SWORD) {
    act("Your $p &+Lretracts &nback to the length of a dagger.", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &+Lretracts &nback to the length of a dagger.", FALSE, ch, obj, 0, PROXIMITY);
    GET_OBJ_VAL(obj, 7) = WEAPON_DAGGER;
    return FALSE;
  }				//else if

  return FALSE;
}				//dagger of longtooth


SPECIAL(sword_proc_curse_instantwield)
{
  struct obj_data *obj;
  char sword_name[256];

  if (!ch)
    return FALSE;

  obj = me;

//  if (!IS_OBJ_STAT(obj, ITEM_NODROP))
  //      return FALSE;

  if (IS_FIGHTING(ch) && ch->equipment[WEAR_WIELD] != obj && ch->equipment[WEAR_WIELD2] != obj) {
    if (ch->equipment[WEAR_WIELD])
      obj_to_char(unequip_char(ch, WEAR_WIELD), ch);
    else if (ch->equipment[WEAR_HOLD])
      obj_to_char(unequip_char(ch, WEAR_HOLD), ch);

    act("Your $p &nforces its way into your hand.", FALSE, ch, obj, 0, TO_CHAR);
    act("$n's $p &nforces its way into $s hand.", FALSE, ch, obj, 0, PROXIMITY);
    perform_wear(ch, obj, WEAR_WIELD);
    return FALSE;
  }				//if

  if (!argument || !*argument || !cmd)
    return FALSE;

  one_argument(argument, sword_name);

  if (CMD_IS("remove") && IS_FIGHTING(ch)) {
    if (isname("longsword", sword_name) || isname("cursed", sword_name) ||
	isname("rusty", sword_name)) {
      act("You dance around &+Mfoolishly &ntrying to remove your $p", FALSE, ch, obj, 0, TO_CHAR);
      act("$n dances around &+Mfoolishly &ntrying to remove $s $p", FALSE, ch, obj, 0, PROXIMITY);
      return TRUE;
    }				//if

    return FALSE;
  }				//if

  return FALSE;
}				//curse sword - isntant wield when fighting


SPECIAL(sword_life_stealing)
{
  struct char_data *victim;
  struct obj_data *obj;
  int _hpgain = cmd;

  obj = me;

  if (!ch || why_called != PROC_CRIT_HIT)
    return FALSE;

  if (ch->equipment[WEAR_WIELD] != obj && ch->equipment[WEAR_WIELD2] != obj)
    return FALSE;

  if (!IS_FIGHTING(ch))
    return FALSE;

  victim = FIGHTING(ch);

  act("Your $p &+Wturns &+rblood red &+Was it sinks deeply into the flesh of &n$N&+w.\n&+L$N looks drained.&n", FALSE, ch, obj, victim, TO_CHAR);
  act("$n's $p &+Wturns &+rblood red &+Was it sinks deeply into &=LRYOU!&+w.\n&+LYou feel drained.&n", FALSE, ch, obj, victim, TO_VICT);
  act("$n's $p &+Wturns &+rblood red &+Was it sinks deeply into the flesh of &n$N&+w.\n&+L$N looks drained.&n", FALSE, ch, obj, victim, TO_OTHERS);

  _hpgain = (_hpgain > GET_HIT(victim) + 10) ? GET_HIT(victim) + 10 : _hpgain;

  if (GET_HIT(ch) >= GET_MAX_HIT(ch))
    return FALSE;

  if (GET_HIT(ch) + _hpgain >= GET_MAX_HIT(ch))
    GET_HIT(ch) = GET_MAX_HIT(ch);
  else
    GET_HIT(ch) += _hpgain;

  return FALSE;
}				//sword of life stealing


SPECIAL(sword_vorpal)
{
  struct char_data *victim;
  struct obj_data *obj;
  int size, roll;

  obj = me;

  if (!ch || why_called != PROC_HIT)
    return FALSE;

  if (ch->equipment[WEAR_WIELD] != obj && ch->equipment[WEAR_WIELD2] != obj)
    return FALSE;

  if (!IS_FIGHTING(ch))
    return FALSE;

  victim = FIGHTING(ch);
  size = GET_SIZE(victim);
  roll = number(0, 23);

  if (roll < 20 || size == SIZE_GARGANTUAN)
    return FALSE;

  if ((size > SIZE_MEDIUM && roll >= 21) || (size <= SIZE_MEDIUM)) {
    act("&+LYou lop off the mothers head.&n", FALSE, ch, obj, victim, TO_CHAR);
    act("&+LOpps.. whered your head go.&n", FALSE, ch, obj, victim, TO_VICT);
    act("&+L$n's $p slices through the neck of $N", FALSE, ch, obj, victim, TO_OTHERS);
    damage(ch, victim, GET_HIT_HD(victim) + 11, TYPE_DESTROY, -1, LOCATION_BRAIN);
    return TRUE;
  }				//if

  return FALSE;
}				//vorpal sword


SPECIAL(sword_sharpness)
{
  struct char_data *victim;
  struct obj_data *obj;
  int size, roll, loc = LOCATION_TORSO;

  obj = me;

  if (!ch)
    return FALSE;

  if (CMD_IS("say") || CMD_IS("'")) {
    if (isname("light", argument) && !IS_OBJ_STAT(obj, ITEM_LIT)) {
      act("&+LYour $p &+Wflashes &+Lbrightly&n", FALSE, ch, obj, 0, TO_CHAR);
      act("&+L$n's $p &+Wflashes &+Lbrightly&n", FALSE, ch, obj, 0, TO_OTHERS);
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_LIT);
      return TRUE;
    }				//if

    if (isname("dim", argument) && IS_OBJ_STAT(obj, ITEM_LIT)) {
      act("&+LYour $p &+Ldims&n", FALSE, ch, obj, 0, TO_CHAR);
      act("&+L$n's $p &+Ldims&n", FALSE, ch, obj, 0, TO_OTHERS);
      REMOVE_BIT(GET_OBJ_EXTRA(obj), ITEM_LIT);
      return TRUE;
    }				//if

    return FALSE;
  }				//if

  if (!ch || why_called != PROC_HIT)
    return FALSE;

  if (ch->equipment[WEAR_WIELD] != obj && ch->equipment[WEAR_WIELD2] != obj)
    return FALSE;

  if (!IS_FIGHTING(ch))
    return FALSE;

  victim = FIGHTING(ch);
  size = GET_SIZE(victim);
  roll = number(0, 21);
  loc = number(0, 99);

  if (loc < 5)
    loc = LOCATION_SHIELD_ARM;
  else if (loc < 10)
    loc = LOCATION_WEAPON_ARM;
  else if (loc < 15)
    loc = LOCATION_RIGHT_LEG;
  else if (loc < 20)
    loc = LOCATION_LEFT_LEG;
  else if (loc < 40)
    loc = LOCATION_WEAPON_HAND;
  else if (loc < 60)
    loc = LOCATION_SHIELD_HAND;
  else if (loc < 80)
    loc = LOCATION_LEFT_FOOT;
  else
    loc = LOCATION_RIGHT_FOOT;

  if (roll < 19 || size == SIZE_GARGANTUAN)
    return FALSE;

  if ((size > SIZE_MEDIUM && roll >= 20) || (size <= SIZE_MEDIUM)) {
    size = 0;
    switch (loc) {
    case LOCATION_SHIELD_HAND:
      if (GET_HIT_SH(victim) >= 0) {
	act("&+LYou lop off $N's shield hand!&n", FALSE, ch, obj, victim, TO_CHAR);
	act("&+LOpps.. think you just lost your shield hand!&n", FALSE, ch, obj, victim, TO_VICT);
	act("&+L$n hacks off $N's shield hand!&n", FALSE, ch, obj, victim, TO_OTHERS);
	damage(ch, victim, GET_HIT_SH(victim) + 1, TYPE_DESTROY, -1, loc);
	size = 1;
      }				//if

      break;
    case LOCATION_SHIELD_ARM:
      if (GET_HIT_SA(victim) >= 0) {
	act("&+LYou lop off $N's shield arm&n", FALSE, ch, obj, victim, TO_CHAR);
	act("&+LOpps.. think you just lost your shield arm!&n", FALSE, ch, obj, victim, TO_VICT);
	act("&+L$n hacks off $N's shield arm!&n", FALSE, ch, obj, victim, TO_OTHERS);
	damage(ch, victim, GET_HIT_SA(victim) + 1, TYPE_DESTROY, -1, loc);
	size = 1;
      }				//if

      break;
    case LOCATION_WEAPON_HAND:
      if (GET_HIT_WH(victim) >= 0) {
	act("&+LYou lop off $N's weapon hand!&n", FALSE, ch, obj, victim, TO_CHAR);
	act("&+LOpps.. think you just lost your weapon hand!&n", FALSE, ch, obj, victim, TO_VICT);
	act("&+L$n hacks off $N's weapon hand!&n", FALSE, ch, obj, victim, TO_OTHERS);
	damage(ch, victim, GET_HIT_WH(victim) + 1, TYPE_DESTROY, -1, loc);
	size = 1;
      }				//if

      break;
    case LOCATION_WEAPON_ARM:
      if (GET_HIT_WA(victim) >= 0) {
	act("&+LYou lop off $N's weapon arm!&n", FALSE, ch, obj, victim, TO_CHAR);
	act("&+LOpps.. think you just lost your weapon arm!&n", FALSE, ch, obj, victim, TO_VICT);
	act("&+L$n hacks off $N's weapon arm!&n", FALSE, ch, obj, victim, TO_OTHERS);
	damage(ch, victim, GET_HIT_WA(victim) + 1, TYPE_DESTROY, -1, loc);
	size = 1;
      }				//if

      break;
    case LOCATION_RIGHT_FOOT:
      if (GET_HIT_RF(victim) >= 0) {
	act("&+LYou lop off $N's right foot!&n", FALSE, ch, obj, victim, TO_CHAR);
	act("&+LOpps.. think you just lost your right foot!&n", FALSE, ch, obj, victim, TO_VICT);
	act("&+L$n hacks off $N's right foot!&n", FALSE, ch, obj, victim, TO_OTHERS);
	damage(ch, victim, GET_HIT_RF(victim) + 1, TYPE_DESTROY, -1, loc);
	size = 1;
      }				//if

      break;
    case LOCATION_RIGHT_LEG:
      if (GET_HIT_RL(victim) >= 0) {
	act("&+LYou lop off $N's right leg!&n", FALSE, ch, obj, victim, TO_CHAR);
	act("&+LOpps.. think you just lost your right leg!&n", FALSE, ch, obj, victim, TO_VICT);
	act("&+L$n hacks off $N's right leg!&n", FALSE, ch, obj, victim, TO_OTHERS);
	damage(ch, victim, GET_HIT_RL(victim) + 1, TYPE_DESTROY, -1, loc);
	size = 1;
      }				//if

      break;
    case LOCATION_LEFT_FOOT:
      if (GET_HIT_LF(victim) >= 0) {
	act("&+LYou lop off $N's left foot!&n", FALSE, ch, obj, victim, TO_CHAR);
	act("&+LOpps.. think you just lost your left foot!&n", FALSE, ch, obj, victim, TO_VICT);
	act("&+L$n hacks off $N's left foot!&n", FALSE, ch, obj, victim, TO_OTHERS);
	damage(ch, victim, GET_HIT_LF(victim) + 1, TYPE_DESTROY, -1, loc);
	size = 1;
      }				//if

      break;
    case LOCATION_LEFT_LEG:
      if (GET_HIT_LL(victim) >= 0) {
	act("&+LYou lop off $N's left leg!&n", FALSE, ch, obj, victim, TO_CHAR);
	act("&+LOpps.. think you just lost your left leg!&n", FALSE, ch, obj, victim, TO_VICT);
	act("&+L$n hacks off $N's left leg!&n", FALSE, ch, obj, victim, TO_OTHERS);
	damage(ch, victim, GET_HIT_LL(victim) + 1, TYPE_DESTROY, -1, loc);
	size = 1;
      }				//if

      break;
    default:
      break;
    }				//switch

    if (size)
      return TRUE;
  }				//if

  return FALSE;
}				//sword of sharpness


//*************************************************************************
//* room procs */
//*************************************************************************

SPECIAL(kan_bedroom)
{
  room_rnum KanRoom;
  struct char_data *victim = NULL;

  if (!ch)
    return FALSE;

  KanRoom = real_room(199);

  if (!isname("Tastone", GET_NAME(ch)) && !isname("Myhriss", GET_NAME(ch))) {
    act("&+LThe door swings open and sucks &=LRYOU &+Lout&n", FALSE, ch, 0, 0, TO_CHAR);
    act("&+LThe door swings open and sucks &+W$n &+Lout&n", FALSE, ch, 0, 0, PROXIMITY);
    char_from_room(ch);
    char_to_room(ch, KanRoom, -1);
    act("&+LThe bedroom door swings open and spits out &+W$n&n", FALSE, ch, 0, 0, PROXIMITY);
    look_at_room(ch, 0);
    return TRUE;
  }				//if

  if (ch->desc) {
    if (ch->desc->snoop_by) {
      victim = ch->desc->snoop_by->character;
      if (isname("Tastone", GET_NAME(victim)))
	return FALSE;
      stop_snooping(victim);
      act("&=LR***PRIVATE ROOM***&n", FALSE, victim, 0, 0, TO_CHAR);
      act("&+LThe room prevents $N from intruding.&n", FALSE, ch, 0, victim, TO_CHAR);
      act("&+LThe room prevents $N from intruding.&n", FALSE, ch, 0, victim, PROXIMITY);
      return FALSE;
    }				//if

    return FALSE;
  }				//if

  return FALSE;
}				//kan_bedroom


SPECIAL(kan_place)
{
  room_rnum KanRoom = 199, Limbo = 1;
  struct char_data *victim = NULL;
//   struct obj_data *obj = NULL;
  //   char *GuestList;

  if (!ch)
    return FALSE;

  KanRoom = real_room(199);
  Limbo = real_room(5204);

  if (!ROOM_FLAGGED(KanRoom, ROOM_GODROOM))
    return FALSE;

  if (PLR_FLAGGED(ch, PLR_FROZEN))
    if (GET_FREEZE_LEV(ch) <= 34) {
      act("&+LThe room &+rheats&+L up as you feel mobile again.&n", FALSE, ch, 0, 0, TO_CHAR);
      act("&+LThe room &+rheats&+L up and thaws $n.&n", FALSE, ch, 0, 0, PROXIMITY);
      REMOVE_BIT(PLR_FLAGS(ch), PLR_FROZEN);
    }				//if

  if (GET_LEVEL(ch) < 34 && IS_PC(ch)) {
    act("&+LThe ground opens up and swallows &=LRYOU&n", FALSE, ch, 0, 0, TO_CHAR);
    act("&+LThe ground opens up and swallows &+W$n&n", FALSE, ch, 0, 0, PROXIMITY);
    char_from_room(ch);
    char_to_room(ch, Limbo, -1);
    act("&+L$n falls from the sky and crashes to the ground&n", FALSE, ch, 0, 0, PROXIMITY);
    look_at_room(ch, 0);
    return FALSE;
  }				//if

  if (ch->desc) {
    if (ch->desc->snoop_by) {
      victim = ch->desc->snoop_by->character;
      if (isname("Tastone", GET_NAME(victim)))
	return FALSE;
      stop_snooping(victim);
      act("&=LR***PRIVATE ROOM***&n", FALSE, victim, 0, 0, TO_CHAR);
      act("&+LThe room prevents $N from intruding.&n", FALSE, ch, 0, victim, TO_CHAR);
      act("&+LThe room prevents $N from intruding.&n", FALSE, ch, 0, victim, PROXIMITY);
      return FALSE;
    }				//if

    return FALSE;
  }				//if

  if (GET_INVIS_LEV(ch) > 35 && !isname("Tastone", GET_NAME(ch))) {
    act("&+LThe room humbles you.&n", FALSE, ch, 0, 0, TO_CHAR);
    act("&+LThe room humbles $n.&n", FALSE, ch, 0, 0, PROXIMITY);
    GET_INVIS_LEV(ch) = 35;
  }				//if

  return FALSE;
}				//kan_bedroom



SPECIAL(mob_generating_room)
{
  struct char_data *victim = NULL, *vict_next = NULL;
  struct room_data *room;
  int _From = 0, _To = 0, ListSize = 0, _ToRoom = 0, Index = 0, Counter = 0,
   _error = 0;
  char Desc[256], RoomNum[100];
  char *RoomDesc;
  int exit[100];
  struct follow_type *k, *next;

  if (!ch)
    return FALSE;

  room = me;

  RoomDesc = room->description;

  if (RoomDesc == NULL)
    return FALSE;

  _From = strlen(RoomDesc);
  
      while (Index <= _From) {
    
	if (RoomDesc[Index] == '\n' || RoomDesc[Index] == ' ' || RoomDesc[Index] == '\r'
	    || RoomDesc[Index] == '!' || Index == _From) {
      RoomNum[Counter] = '\0';
      exit[ListSize] = atoi(RoomNum);
      ListSize++;
      Counter = 0;
    }				//if

    else {
      if (!isdigit(RoomDesc[Index]))
	_error = 1;
      RoomNum[Counter] = RoomDesc[Index];
      Counter++;
    }				//else

    Index++;
  }				//while

  _From = real_room(room->number);

  if (!_From)
    return FALSE;

  if (!world[_From].people)
    return FALSE;

  for (victim = world[_From].people; victim; victim = vict_next) {
    vict_next = victim->next_in_room;	// the dying victim bug

    if (victim == ch)
      continue;
    if (IS_NPC(victim)) {
      if (_error) {
	send_to_room("ERROR IN DESC FORMAT", _From);
	return FALSE;
      }				//if

      while (!_To)
	_To = exit[number(0, ListSize - 1)];

      _ToRoom = real_room(_To);
      sprintf(Desc, "$n &+Lgets distributed to room: &+B%d&n", _To);
      act(Desc, FALSE, victim, 0, 0, PROXIMITY);
      char_from_room(victim);
      char_to_room(victim, _ToRoom, -1);
      //transfer followers
      if (victim->followers) {
	for (k = victim->followers; k; k = next) {
	  next = k->next;
	  char_from_room(k->follower);
	  put_char_to_room(k->follower, _ToRoom, -1, GET_X_CORD(victim), GET_Y_CORD(victim), GET_Z_CORD(victim));
	}			//for

      }				//if

    }				//if

  }				//for

  return FALSE;
}				//mob generating room
