/*
 ***************************************************************************
 *  File: kingdom.c
 *  Usage: troops, castles, land aquisition
 *  Copyright  1994, 1995 - Duris Systems Ltd.
 ***************************************************************************
 */
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>

#include "events.h"
#include "spells.h"
#include "structs.h"
#include "comm.h"
#include "prototypes.h"
#include "db.h"
#include "interp.h"
#include "utils.h"
#include "kingdom.h"
#include "config.h"
#include "mm.h"
#include "justice.h"
#include "sound.h"
#include "assocs.h"
#include "specs.prototypes.h"

/* external variables */
extern struct kingdom_global kingdom[];
extern int mini_mode;
extern struct char_data *character_list;

extern struct descriptor_data *descriptor_list;
extern P_event current_event;
extern P_event event_type_list[LAST_EVENT];
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern P_room world;
extern const char *dirs[];
extern int MobSpellIndex[MAX_SKILLS];
extern int equipment_pos_table[CUR_MAX_WEAR][3];
extern int no_specials;
extern int spl_table[TOTALLVLS][MAX_CIRCLE];
extern int top_of_world;
extern const int rev_dir[];
extern struct zone_data *zone_table;
extern char *troop_types[];
extern char *troop_levels[];
extern char *troop_offense[];
extern char *troop_defense[];
extern int troop_costs[NUM_TROOP_TYPES][4];

/* misc crap */
struct mm_ds *dead_house_pool = NULL;
struct mm_ds *dead_construction_pool = NULL;
struct house_control_rec *first_house = NULL;
struct house_upgrade_rec *house_upgrade_list = NULL;
struct kingdom_global kingdom[MAX_KINGDOMS];
struct troop_info_rec *deployed_troops;

void parse_connection(P_house, P_char, char *);
char *kingdom_num_name(int, int);
void do_build_secret(house_control_rec * house, struct char_data *ch);
void update_kingdom_score(int kingdom_num, int int_arg, int type);
void write_pc_land(int room);

#define SHOPKEEPER_VNUM 11005
#define MAGE_GOLEM_VNUM 11004
#define CLERIC_GOLEM_VNUM 11003
#define WARRIOR_GOLEM_VNUM 11002
#define FOUNTAIN_VNUM 11000
#define CHEST_VNUM 115
#define CABINET_VNUM 11009
#define MOUTH_VNUM 11002
#define TELEPORTER_VNUM 11001
#define GUILD_BOARD_START 11100
#define GUILD_HOLY_FOUNT 11005
#define GUILD_UNHOLY_FOUNT 11006

const int upgrade_times[] =
{				/* these are measured in mud hours */
  0,				/* null */
  24,				/* add mage golem */
  24,				/* add cleric golem */
  24,				/* add warrior golem */
  6,				/* add chest */
  168,				/* add teleporter */
  6,				/* add fountain */
  48,				/* add inn */
  6,				/* add shop */
  1,				/* add board */
  24,				/* add room */
  3,				/* describe room */
  3,				/* name room */
  6,				/* magic mouth */
  12,				/* holy */
  12,				/* unholy */
  24,				/* heal room */
  48,				/* build initial house */
  60,				/* build initial guildhall */
  6,				/* cabinet */
  24,				/* secret entrance */
  0};				/* last record */

const int upgrade_costs[] =
{				/* costs in copper */
  0,				/* null */
  10000,			/* add mage golem */
  10000,			/* add cleric golem */
  10000,			/* add warrior golem */
  2500,				/* add chest */
  20000,			/* add teleporter */
  500,				/* add fo untain */
  5000,				/* add inn */
  10000,			/* add shop */
  500,				/* add board */
  1000,				/* add room */
  10,				/* describe room */
  50,				/* name room */
  500,				/* magic mouth */
  3000,				/* holy */
  3000,				/* unholy */
  2000,				/* heal room */
  5000,				/* initial house, shouldnt be called here */
  5000,				/* initial guild, shouldnt be called here */
  500,				/* cabinet */
  10000,			/* secret entrance */
  0};				/* last record */


void parse_connection(struct house_control_rec *house, struct char_data *ch, char *arg)
{
  int v1, v2, dir;
  char buf[MAX_INPUT_LENGTH];

  arg = one_argument(arg, buf);
  if (!buf || !is_number(buf)) {
    send_to_char("Format: construct connection room1 room2 direction.\r\n", ch);
    send_to_char("You appear to have left out room1s virtual number.\r\n", ch);
    return;
  } else
    v1 = atoi(buf);
  arg = one_argument(arg, buf);
  if (!buf || !is_number(buf)) {
    send_to_char("Format: construct connection room1 room2 direction.\r\n", ch);
    send_to_char("You appear to have left out room2s virtual number.\r\n", ch);
    return;
  } else
    v1 = atoi(buf);
  arg = one_argument(arg, buf);
  if (!(dir = old_search_block(buf, 0, strlen(buf), dirs, 0))) {
    send_to_char("Format: construct connection room1 room2 direction.\r\n", ch);
    send_to_char("You appear to have left out the direction.\r\n", ch);
    return;
  }
  connect_rooms(v1, v2, dir);
}

void connect_rooms(int v1, int v2, int dir)
{				/* create exits from room1<->room2 */

  int r1, r2, rdir;;

  r1 = real_room0(v1);
  r2 = real_room0(v2);
  rdir = rev_dir[dir];
  if (!r1 || !r2) {
    logit(LOG_HOUSE, "Something missing in connect_rooms(%d, %d, %d)", v1, v2, dir);
    return;
  }
  if (!world[r1].dir_option[dir])
    CREATE(world[r1].dir_option[dir], struct room_direction_data, 1);
  if (!world[r2].dir_option[rdir])
    CREATE(world[r2].dir_option[rdir], struct room_direction_data, 1);

  world[r1].dir_option[dir]->to_room = r2;
  world[r1].dir_option[dir]->exit_info = 0;
  world[r2].dir_option[rdir]->to_room = r1;
  world[r2].dir_option[rdir]->exit_info = 0;
  return;
}


/* Parse 3 args from hcontrol */
void hcontrol_parser(int autom, char *arg, int *type, char *name, int *direction, int *guild, char *keyword)
{
  char buf[MAX_STRING_LENGTH];

  buf[0] = '\0';

  /* determine type */
  arg = one_argument(arg, buf);
  if (is_abbrev(buf, "house"))
    *type = HCONTROL_HOUSE;
  else if (is_abbrev(buf, "guild"))
    *type = HCONTROL_GUILD;
  else if (is_abbrev(buf, "castle"))
    *type = HCONTROL_CASTLE;
  else {
    *type = HCONTROL_ERROR;
    return;
  }

  /* find name */
  arg = one_argument(arg, name);

  switch (*type) {
  case HCONTROL_HOUSE:
    *direction = 0;
    *guild = 0;
    break;
  case HCONTROL_GUILD:
  case HCONTROL_CASTLE:
    arg = one_argument(arg, buf);
    if (!(*direction = old_search_block(buf, 0, strlen(buf), dirs, 0)))
      return;
    arg = one_argument(arg, buf);
    *guild = atoi(buf);
    break;
  }
  if (*direction <= 0 || *direction >= 3)
    *direction = 0;

  return;
}




/******************************************************************
 *  Functions for house administration (creation, deletion, etc.  *
 *****************************************************************/

struct house_control_rec *find_house(int vnum)
{
  struct house_control_rec *house = NULL;
  int x;

  if (!first_house)
    return NULL;

  for (house = first_house; house; house = house->next) {
    if (house->vnum == vnum)
      return (house);
    for (x = 0; x < MAX_HOUSE_ROOMS; x++)
      if (house->room_vnums[x] == vnum)
	return (house);
  }

  return NULL;
}

/* copy of above, except it looks based on ownership */
struct house_control_rec *find_house_by_owner(char *name)
{
  struct house_control_rec *house = NULL;

  if (!first_house)
    return NULL;

  for (house = first_house; house; house = house->next)
    if (!strcmp(name, house->owner))
      return (house);

  return NULL;
}

int room_guild_no(int rnum)
{				/* returns guild # of guild room */
  struct house_control_rec *house;
  int x;
  for (house = first_house; house; house = house->next)
    for (x = 0; x < house->num_of_rooms; x++)
      if (real_room(house->room_vnums[x]) == rnum)
	return house->owner_guild;

  return 0;
}

struct house_control_rec *get_house_from_room(int rnum)
{
  struct house_control_rec *house;
  int x;
  for (house = first_house; house; house = house->next)
    for (x = 0; x < house->num_of_rooms; x++)
      if (real_room(house->room_vnums[x]) == rnum)
	return house;
  return 0;
}

/****************************************************/
/* "House Control" functions */

const char *HCONTROL_FORMAT =
"\r\n"
"Usage: hcontrol build house <owner> <entrance direction>\r\n"
"       hcontrol build castle <owner> <entrance dir> <assoc #>\r\n"
"       hcontrol build guild <owner> <entrance dir> <assoc #>\r\n"
"       hcontrol destroy <house vnum>\r\n"
"       hcontrol pay <house vnum>\r\n"
"       hcontrol show (main listing)\r\n"
"	hcontrol detail (lists rooms)\r\n"
"	hcontrol clear (removes teleporters)\r\n"
"	hcontrol restore <house vnum> (reboots building)\r\n"
"	hcontrol change <house vnum> <aspect/field> <new value>\r\n"
"Type is one of house, guild or castle\r\n"
"Owner is the primary controlling character\r\n"
"Entrance is the direction from your current room, into the structure\r\n"
"Field is one of: owner, guild, upgrade, type, room <x>, numrooms, land\r\n"
"House vnum is shown via show\r\n"
"\r\n";

void hcontrol_list_houses(struct char_data *ch, char *arg)
{
  int j;
  char timestr[256];
  char built_on[128], last_pay[128], own_name[128];
  struct house_control_rec *house;
  char buf[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];
  char the_type;

  if (!first_house) {
    send_to_char("No houses have been defined.\r\n", ch);
    return;
  }
  strcpy(buf, "\r\n"
	 "&+GAddress  Guild Upgr Build Date  Rooms  Owner       Last Paymt Type MainRoom&N\r\n");
  strcat(buf, "&+G-------  ----- ---- ----------  ----- ------------ ---------- ---- ------&N\r\n");

  send_to_char(buf, ch);
  buf2[0] = '\0';

  for (house = first_house; house; house = house->next) {
    buf[0] = '\0';

    if (*arg)
      if (is_number(arg)) {
	if ((atoi(arg)) != house->owner_guild)
	  continue;
      } else if (house->owner && str_cmp(house->owner, arg))
	continue;

    if (house->built_on) {
      strcpy(timestr, asctime(localtime(&(house->built_on))));
      *(timestr + 10) = 0;
      strcpy(built_on, timestr);
    } else
      strcpy(built_on, "Unknown");

    if (house->last_payment) {
      strcpy(timestr, asctime(localtime(&(house->last_payment))));
      *(timestr + 10) = 0;
      strcpy(last_pay, timestr);
    } else
      strcpy(last_pay, "None");

    if (house->owner)
      strcpy(own_name, house->owner);
    else
      strcpy(own_name, "(null)");

    if (house->type == HCONTROL_CASTLE)
      the_type = 'C';
    else if (house->type == HCONTROL_HOUSE)
      the_type = 'H';
    else if (house->type == HCONTROL_GUILD)
      the_type = 'G';
    else
      the_type = ' ';

    sprintf(buf + strlen(buf), "%7d %5d %3d  %-10s    %2d    %-12s %-10s  %c   %d",
      (int) house->vnum, (int) house->owner_guild, (int) house->upgrades,
	    built_on, (int) house->num_of_rooms, own_name, last_pay, the_type, house->room_vnums[0]);

    if (house->construction > 0)
      sprintf(buf + strlen(buf), " &+R*&N\r\n");
    else
      sprintf(buf + strlen(buf), "\r\n");

    if (house->num_of_guests > 0) {
      sprintf(buf + strlen(buf), "     &+LGuests (%d):&n ", house->num_of_guests);
      for (j = 0; j < house->num_of_guests; j++) {
	sprintf(buf + strlen(buf), "&+L%s&n ", house->guests[j]);
      }
      strcat(buf, "\r\n");
    }
    strcat(buf2 + strlen(buf2), buf);
  }
  page_string(ch->desc, buf2, 1);
}

/* For icon purposes, we don't want these built near a maps edge. We need
   only 2 squares surrounding free, but lets check 3 in case something changes
   later on. Pass this a real room!
 */
bool valid_build_location(int room, struct char_data *ch, int type)
{
  int zone_num, temp1, temp2;
  int vroom;

  zone_num = world[room].zone;

  if (ch) {
    if (IS_TRUSTED(ch))
      return TRUE;
    if (world[room].sector_type == SECT_INSIDE ||
	world[room].sector_type == SECT_WATER_SWIM ||
	world[room].sector_type == SECT_WATER_NOSWIM ||
	world[room].sector_type == SECT_NO_GROUND ||
	world[room].sector_type == SECT_UNDERWATER ||
	world[room].sector_type == SECT_UNDERWATER_GR ||
	world[room].sector_type == SECT_FIREPLANE ||
	world[room].sector_type == SECT_OCEAN ||
	world[room].sector_type == SECT_UNDRWLD_INSIDE ||
	world[room].sector_type == SECT_UNDRWLD_WATER ||
	world[room].sector_type == SECT_UNDRWLD_NOSWIM ||
	world[room].sector_type == SECT_UNDRWLD_NOGROUND ||
	world[room].sector_type == SECT_AIR_PLANE ||
	world[room].sector_type == SECT_WATER_PLANE ||
	world[room].sector_type == SECT_EARTH_PLANE ||
	world[room].sector_type == SECT_ETHEREAL ||
	world[room].sector_type == SECT_ASTRAL ||
	world[room].sector_type == SECT_CASTLE ||
	world[room].sector_type == SECT_CASTLE_WALL ||
	world[room].sector_type == SECT_CASTLE_GATE) {
      send_to_char("Gee bucko, can't you think of a better place to build this?\r\n", ch);
      return FALSE;
    }
    if ((world[ch->in_room].room_flags & INDOORS) && !UNDERWORLD(ch->in_room)) {
      send_to_char("Umm, this area is already indoors. We don't build things like that around these parts.\r\n", ch);
      return FALSE;
    }
    if (IS_SET(world[ch->in_room].room_flags, GUILD_ROOM)) {
      send_to_char("Not inside a guild. Puhlease!/r/n", ch);
      return FALSE;
    }
    if (CHAR_IN_TOWN(ch)) {
      send_to_char("Sorry, you can't build inside towns right now.\r\n", ch);
      return FALSE;
    }
    if ((EVIL_RACE(ch) && IS_GOOD_MAP(room)) || (!EVIL_RACE(ch) && IS_EVIL_MAP(room))) {
      send_to_char("You can't seem to find anyone stupid enough to build it for you at this location.\r\n", ch);
      return FALSE;
    }
  }
/****** this is wrong. room is a real, the +300 is only valid on vnum ******/
  vroom = world[room].number;
/*
   if ((room + 300 > top_of_world) || (room + 3 > top_of_world) ||
   (room - 3 < 0) || (room - 300 < 0))
   return FALSE;
 */
  if (!(real_room0(vroom + 300) && real_room0(vroom + 3) &&
	real_room0(vroom - 300) && real_room0(vroom - 3))) {
    send_to_char("This area is a little unstable.\n\r", ch);
    return FALSE;
  }
  if ((world[real_room(vroom + 300)].zone == zone_num) &&
      (world[real_room(vroom - 300)].zone == zone_num) &&
      (world[real_room(vroom + 3)].zone == zone_num) &&
      (world[real_room(vroom - 3)].zone == zone_num))
    return TRUE;
/**** end of errors *****/

  temp1 = GET_A_NUM(ch);
  temp2 = GET_A_BITS(ch);

  if (type == HCONTROL_CASTLE || type == HCONTROL_GUILD) {
    if (world[real_room(room)].kingdom_type == KINGDOM_TYPE_NPC ||
	world[real_room(room)].kingdom_type == KINGDOM_TYPE_TOWN) {
      send_to_char("This land has already been claimed by another kingdom.\r\n", ch);
      return FALSE;
    } else if (world[real_room(room)].kingdom_type == KINGDOM_TYPE_PC && (!IS_MEMBER(temp2) || !temp1)) {
      send_to_char("This land has already been claimed by another kingdom.\r\n", ch);
      return FALSE;
    }
  }
  return FALSE;
}

void construct_castle(struct house_control_rec *temp_house)
{
  int gate_room, rdir;
  int virt_house, real_house;
  int x;
  struct obj_data *entrance_obj;
  struct char_data *golem;
  char buff[MAX_STRING_LENGTH], golem_block = 'a';
  struct obj_data *temp_obj;

  virt_house = temp_house->vnum;
  real_house = real_room0(temp_house->vnum);
  if (!real_room0(temp_house->vnum)) {
    wizlog(AVATAR, "Castle/guild %d cannot be built, due to this being a bugged room!.\r\n", temp_house->vnum);
    return;
  }
  if (!valid_build_location(real_room0(temp_house->vnum), NULL, temp_house->type)
      && temp_house->type == HCONTROL_CASTLE) {
    wizlog(AVATAR, "Castles may not be built within 3 squares of a maps edge.");
    return;
  }
  /* now, we know where we are talking about, so lets get busy building it */

  SET_BIT(world[real_room0(temp_house->vnum)].room_flags, ROOM_ATRIUM);
  if (temp_house->type == HCONTROL_HOUSE) {
    temp_house->mode = HMODE_PRIVATE;
  }
  if (temp_house->type == HCONTROL_CASTLE) {
    world[real_house].sector_type = SECT_CASTLE;
    /* build the 8 walls */
    world[real_room0(virt_house - 99)].sector_type = SECT_CASTLE_WALL;
    world[real_room0(virt_house - 100)].sector_type = SECT_CASTLE_WALL;
    world[real_room0(virt_house - 101)].sector_type = SECT_CASTLE_WALL;
    world[real_room0(virt_house - 1)].sector_type = SECT_CASTLE_WALL;
    world[real_room0(virt_house + 1)].sector_type = SECT_CASTLE_WALL;
    world[real_room0(virt_house + 99)].sector_type = SECT_CASTLE_WALL;
    world[real_room0(virt_house + 100)].sector_type = SECT_CASTLE_WALL;
    world[real_room0(virt_house + 101)].sector_type = SECT_CASTLE_WALL;
    world[real_house].sector_type = SECT_CASTLE;

    /* assign default land ownership */
    acquire_land(real_room0(virt_house - 198), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house - 199), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house - 200), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house - 201), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house - 202), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house - 98), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house - 102), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house - 2), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house + 2), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house + 102), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house + 98), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house + 202), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house + 201), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house + 200), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house + 199), temp_house->owner_guild, KINGDOM_TYPE_PC);
    acquire_land(real_room0(virt_house + 198), temp_house->owner_guild, KINGDOM_TYPE_PC);

    switch (temp_house->exit_num) {
    case 0:			/* n */
      world[real_room0(virt_house + 100)].sector_type = SECT_CASTLE_GATE;
      break;
    case 1:			/* e */
      world[real_room0(virt_house - 1)].sector_type = SECT_CASTLE_GATE;
      break;
    case 2:			/* s */
      world[real_room0(virt_house - 100)].sector_type = SECT_CASTLE_GATE;
      break;
    case 4:			/* w */
      world[real_room0(virt_house + 1)].sector_type = SECT_CASTLE_GATE;
      break;
    }
  }
  /* BUILD ROOMS */
  for (x = 0; x < temp_house->num_of_rooms; x++)
    read_guild_room(temp_house->room_vnums[x], temp_house->owner_guild);

  /* fathom some castle shit if we need it */
  if (temp_house->type == HCONTROL_CASTLE) {
    switch (temp_house->exit_num) {
    case 0:			/* n */
      gate_room = (real_room0(temp_house->vnum + 100));
      break;
    case 1:			/* e */
      gate_room = (real_room0(temp_house->vnum - 1));
      break;
    case 2:			/* s */
      gate_room = (real_room0(temp_house->vnum - 100));
      break;
    case 4:			/* w */
      gate_room = (real_room0(temp_house->vnum + 1));
      break;
    }
  }
  /* link entrance room and atrium, if needed */
  if (temp_house->type == HCONTROL_CASTLE && gate_room)
    connect_rooms(world[gate_room].number, temp_house->room_vnums[0], temp_house->exit_num);

  /* houses and halls get entrance teleporter object */
  if ((temp_house->type == HCONTROL_HOUSE) ||
      (temp_house->type == HCONTROL_GUILD)) {
    /* inner door is loaded now */
    if ((entrance_obj = read_object(HOUSE_INNER_DOOR, VIRTUAL))) {
      entrance_obj->str_mask = (STRUNG_KEYS /*| STRUNG_DESC1 */ );
      sprintf(buff, "door %s", temp_house->owner);
      entrance_obj->name = strdup(buff);
      entrance_obj->value[0] = temp_house->vnum;
      nuke_doorways(real_room0(temp_house->room_vnums[0]));
      obj_to_room(entrance_obj, real_room0(temp_house->room_vnums[0]));
    }
    /* outer door is loaded after construction q is done */
    if (temp_house->construction == 0) {
      if (temp_house->type == HCONTROL_HOUSE) {
	if ((entrance_obj = read_object(HOUSE_OUTER_DOOR, VIRTUAL))) {
	  entrance_obj->str_mask = (STRUNG_KEYS | STRUNG_DESC1);
	  sprintf(buff, "A door leading into %s's house is here.", temp_house->owner);
	  entrance_obj->description = strdup(buff);
	  if (temp_house->owner) {
	    sprintf(buff, "door %s", temp_house->owner);
	    entrance_obj->name = strdup(buff);
	  } else {
	    entrance_obj->name = strdup("door");
	    logit(LOG_HOUSE, "house #%d has a NULL owner", temp_house->vnum);
	  }
	  entrance_obj->value[0] = temp_house->room_vnums[0];
	  nuke_doorways(real_room0(temp_house->vnum));
	  obj_to_room(entrance_obj, real_room0(temp_house->vnum));
	  if (temp_house->secret_entrance != 1)
	    REMOVE_BIT(entrance_obj->extra_flags, ITEM_NOSHOW);
	}
      }
      if (temp_house->type == HCONTROL_GUILD) {
	if ((entrance_obj = read_object(HOUSE_OUTER_DOOR, VIRTUAL))) {
	  entrance_obj->str_mask = (STRUNG_KEYS /*| STRUNG_DESC1 */ );
	  sprintf(buff, "door guild building %s", temp_house->owner);
	  entrance_obj->name = strdup(buff);
	  entrance_obj->value[0] = temp_house->room_vnums[0];
	  nuke_doorways(real_room0(temp_house->vnum));
	  obj_to_room(entrance_obj, real_room0(temp_house->vnum));
	  if (temp_house->secret_entrance != 1)
	    REMOVE_BIT(entrance_obj->extra_flags, ITEM_NOSHOW);
	}
      }
    }
  }
  /* castles get gates */
  if (temp_house->type == HCONTROL_CASTLE) {
    /* setup door from atrium into house */
    if (gate_room) {
      SET_BIT(world[gate_room].dir_option[temp_house->exit_num]->exit_info, EX_ISDOOR);
      SET_BIT(world[gate_room].dir_option[temp_house->exit_num]->exit_info, EX_CLOSED);
      world[gate_room].dir_option[temp_house->exit_num]->keyword = strdup("portcullis gate");
      /* setup exit from first room back into atrium */
      rdir = rev_dir[temp_house->exit_num];
      if (world[world[gate_room].dir_option[temp_house->exit_num]->to_room].dir_option[rdir]) {
	SET_BIT(world[world[gate_room].dir_option[temp_house->exit_num]->to_room].dir_option[rdir]->exit_info, EX_ISDOOR);
	SET_BIT(world[world[gate_room].dir_option[temp_house->exit_num]->to_room].dir_option[rdir]->exit_info, EX_CLOSED);
	world[world[gate_room].dir_option[temp_house->exit_num]->to_room].dir_option[rdir]->keyword = strdup("portcullis gate");
      } else
	logit(LOG_HOUSE, "Castle %d has invalid dir info", temp_house->vnum);
    }
  }
  /* error control */
  if (temp_house->upgrades > 99)
    temp_house->upgrades = 0;
  /* inn */
  if (temp_house->room_vnums[0]) {
    world[real_room0(temp_house->inn_vnum)].funct = inn;
  }
  /* double check for houses */
  if (temp_house->type == HCONTROL_HOUSE)
    world[real_room0(temp_house->inn_vnum)].funct = inn;

  /* guild board */
  if (temp_house->owner_guild && (temp_house->type == HCONTROL_GUILD || temp_house->type == HCONTROL_CASTLE)) {
    temp_obj = read_object(GUILD_BOARD_START + temp_house->owner_guild, VIRTUAL);
    if (temp_obj) {
      if (temp_house->board_vnum) {
	obj_to_room(temp_obj, real_room0(temp_house->board_vnum));
      } else {
	obj_to_room(temp_obj, real_room0(temp_house->room_vnums[1]));
      }
    }
  }
  /* fountain */
  if (temp_house->fountain_vnum) {
    temp_obj = read_object(FOUNTAIN_VNUM, VIRTUAL);
    if (temp_obj) {
      obj_to_room(temp_obj, real_room0(temp_house->fountain_vnum));
    }
  }
  /* holy */
  if (temp_house->holy_fount_vnum) {
    temp_obj = read_object(GUILD_HOLY_FOUNT, VIRTUAL);
    if (temp_obj)
      obj_to_room(temp_obj, real_room0(temp_house->holy_fount_vnum));
  }
  if (temp_house->unholy_fount_vnum) {
    temp_obj = read_object(GUILD_UNHOLY_FOUNT, VIRTUAL);
    if (temp_obj)
      obj_to_room(temp_obj, real_room0(temp_house->unholy_fount_vnum));
  }
  if (temp_house->heal_vnum) {
    SET_BIT(world[real_room0(temp_house->heal_vnum)].room_flags, HEAL);
  }
  /* teleporters */
  if (temp_house->teleporter1_room) {
    temp_obj = read_object(TELEPORTER_VNUM, VIRTUAL);
    if (temp_obj) {
      temp_obj->value[0] = temp_house->teleporter1_dest;
      obj_to_room(temp_obj, real_room0(temp_house->teleporter1_room));
    }
  }
  if (temp_house->teleporter2_room) {
    temp_obj = read_object(TELEPORTER_VNUM, VIRTUAL);
    if (temp_obj) {
      temp_obj->value[0] = temp_house->teleporter2_dest;
      obj_to_room(temp_obj, real_room0(temp_house->teleporter2_room));
    }
  }
  if (temp_house->shop_vnum) {
    stock_guild_shop(temp_house->shop_vnum);
  }
  /* chest */
  if (temp_house->mouth_vnum) {
    temp_obj = read_object(MOUTH_VNUM, VIRTUAL);
    if (temp_obj) {
      temp_obj->value[0] = temp_house->owner_guild;
      obj_to_room(temp_obj, real_room0(temp_house->mouth_vnum));
    }
  }
  /* set up golems */
  if ((temp_house->type == HCONTROL_GUILD) ||
      (temp_house->type == HCONTROL_CASTLE)) {
    for (x = 0; x < temp_house->wizard_golems; x++) {
      golem = read_mobile(MAGE_GOLEM_VNUM, VIRTUAL);
      if (!golem)
	break;

      if (temp_house->exit_num == -1)
	golem_block = 'a';
      else
	golem_block = *dirs[temp_house->exit_num];

      sprintf(buff, "golem assoc%d_%c_0", temp_house->owner_guild, golem_block);
      golem->player.name = strdup(buff);
      char_to_room(golem, real_room0(temp_house->room_vnums[0]), 0);
    }
    for (x = 0; x < temp_house->warrior_golems; x++) {
      golem = read_mobile(WARRIOR_GOLEM_VNUM, VIRTUAL);
      if (!golem)
	break;

      if (temp_house->exit_num == -1)
	golem_block = 'a';
      else
	golem_block = *dirs[temp_house->exit_num];

      sprintf(buff, "golem assoc%d_%c_0", temp_house->owner_guild, golem_block);
      golem->player.name = strdup(buff);
      char_to_room(golem, real_room0(temp_house->room_vnums[0]), 0);
    }
    for (x = 0; x < temp_house->cleric_golems; x++) {
      golem = read_mobile(CLERIC_GOLEM_VNUM, VIRTUAL);
      if (!golem)
	break;

      if (temp_house->exit_num == -1)
	golem_block = 'a';
      else
	golem_block = *dirs[temp_house->exit_num];

      sprintf(buff, "golem assoc%d_%c_0", temp_house->owner_guild, golem_block);
      golem->player.name = strdup(buff);
      char_to_room(golem, real_room0(temp_house->room_vnums[0]), 0);
    }
  }
}

void destroy_castle(struct house_control_rec *temp_house)
{
  int real_house, virt_house;
  int temp, i;

  virt_house = temp_house->vnum;
  real_house = real_room0(virt_house);
  if (!real_house) {
    return;
  }
  /* remove doorway objects */
  nuke_doorways(real_house);
  nuke_doorways(real_room0(temp_house->room_vnums[0]));

  world[real_house].dir_option[temp_house->exit_num] = 0;
  for (temp = 0; temp < temp_house->num_of_rooms; temp++) {
    for (i = 0; i < NUMB_EXITS; i++)
      world[real_room0(temp_house->room_vnums[temp])].dir_option[i] = 0;
  }
  if (temp_house->type == HCONTROL_HOUSE) {
    REMOVE_BIT(world[real_house].room_flags, ROOM_HOUSE);
    REMOVE_BIT(world[real_house].room_flags, NO_TELEPORT);
    REMOVE_BIT(world[real_house].room_flags, NO_SUMMON);
    REMOVE_BIT(world[real_house].room_flags, NO_GATE);
    REMOVE_BIT(world[real_house].room_flags, TWILIGHT);
    return;
  }
  if (temp_house->type == HCONTROL_CASTLE) {
    REMOVE_BIT(world[real_house].room_flags, ROOM_HOUSE);
    REMOVE_BIT(world[real_house].room_flags, NO_TELEPORT);
    REMOVE_BIT(world[real_house].room_flags, NO_SUMMON);
    REMOVE_BIT(world[real_house].room_flags, NO_GATE);
    REMOVE_BIT(world[real_house].room_flags, TWILIGHT);
    world[real_house].sector_type = SECT_FIELD;

    /* remove the 8 walls */
    world[real_room0(virt_house - 99)].sector_type = SECT_FIELD;
    world[real_room0(virt_house - 100)].sector_type = SECT_FIELD;
    world[real_room0(virt_house - 101)].sector_type = SECT_FIELD;
    world[real_room0(virt_house - 1)].sector_type = SECT_FIELD;
    world[real_room0(virt_house + 1)].sector_type = SECT_FIELD;
    world[real_room0(virt_house + 99)].sector_type = SECT_FIELD;
    world[real_room0(virt_house + 100)].sector_type = SECT_FIELD;
    world[real_room0(virt_house + 101)].sector_type = SECT_FIELD;
/*    world[world[real_house].dir_option[temp_house->exit_num]->to_room].sector_type = SECT_FIELD; */
  }
}

void hcontrol_build_house(struct char_data *ch, char *arg)
{
  char name[MAX_INPUT_LENGTH];
  char keyword[MAX_INPUT_LENGTH];
  struct house_control_rec *temp_house;
  int virt_house, real_house, real_atrium, virt_atrium, i;
  int guild = 0, direction = 0, type = 0;
  int autom = TRUE;

  name[0] = '\0';

  if (IS_TRUSTED(ch))		/* is this a construct or hcontrol command? */
    autom = FALSE;

  hcontrol_parser(autom, arg, &type, name, &direction, &guild, keyword);
  if ((type == HCONTROL_ERROR) || !name || !name[0] /*|| !direction || !keyword || !keyword[0] */ ) {
    if (IS_TRUSTED(ch)) {
      send_to_char(HCONTROL_FORMAT, ch);
      return;
    }
    /* when automated, only error out if type is invalid */

    else {			/*if (type == HCONTROL_ERROR) */
      send_to_char("Error constructing. Try again.\r\n", ch);
      return;
    }
  }
  /* gets the array back on track */
  /* direction --; */

  real_atrium = ch->in_room;
  virt_atrium = world[real_atrium].number;

  if (!dead_house_pool)
    dead_house_pool = mm_create("HOUSE",
				sizeof(struct house_control_rec),
				offsetof(struct house_control_rec,
					 next), 1);

  temp_house = mm_get(dead_house_pool);

  if (type == HCONTROL_HOUSE)
    direction = -1;		/* so we won't link the rooms */
  else if (type == HCONTROL_GUILD /* && direction < 0 */ )
    direction = -1;
  else if (type == HCONTROL_CASTLE /*&& direction < 0 */ )
    direction = 0;

  if (autom)
    temp_house->construction = 1;
  else
    temp_house->construction = 0;

  virt_house = hcontrol_setup_room(temp_house, virt_atrium, direction);
  real_house = real_room0(virt_house);
  if (!real_house) {
    send_to_char("Couldnt allocate house room!\r\n", ch);
    return;			/* couldn't allocate a room! */
  }
  if (type == HCONTROL_GUILD)
    direction = 0;

  temp_house->mode = HMODE_OPEN;
  temp_house->type = type;
  temp_house->vnum = virt_atrium;
  temp_house->exit_num = direction;
  temp_house->built_on = time(0);
  temp_house->last_payment = 0;
  temp_house->owner = str_dup(name);
  temp_house->num_of_guests = 0;
  temp_house->size = 1;
  temp_house->upgrades = 0;
  temp_house->mouth_vnum = 0;
  temp_house->holy_fount_vnum = 0;
  temp_house->unholy_fount_vnum = 0;
  temp_house->secret_entrance = 0;
  if (temp_house->type == HCONTROL_HOUSE)
    temp_house->inn_vnum = virt_house;
  else
    temp_house->inn_vnum = 0;
  temp_house->warrior_golems = 0;
  temp_house->wizard_golems = 0;
  temp_house->cleric_golems = 0;
  temp_house->heal_vnum = 0;
  for (i = 0; i < MAX_CONTROLLED_LAND; i++)
    temp_house->controlled_land[i] = -1;
  if (temp_house->type == HCONTROL_HOUSE || temp_house->type == HCONTROL_GUILD)
    temp_house->entrance_keyword = strdup(J_NAME(ch));
  else
    temp_house->entrance_keyword = 0;
  /* houses start with 1 room, guilds/castles start with 2 */
  if (temp_house->type == HCONTROL_HOUSE)
    temp_house->num_of_rooms = 1;
  else
    temp_house->num_of_rooms = 2;
  temp_house->owner_guild = guild;	/* GUILD */
  temp_house->sack_list = 0;

  for (i = 0; i < MAX_HOUSE_ROOMS; i++)
    temp_house->room_vnums[i] = -1;

  temp_house->room_vnums[0] = virt_house;	/* first room */
  if ((temp_house->type == HCONTROL_GUILD) ||
      (temp_house->type == HCONTROL_CASTLE))
    temp_house->room_vnums[1] = hcontrol_setup_room(temp_house, virt_house, temp_house->exit_num);

  for (i = 0; i < MAX_GUESTS; i++)
    temp_house->guests[i] = NULL;

  temp_house->next = first_house;
  first_house = temp_house;

  if (autom && type != HCONTROL_CASTLE)
    writeConstructionQ();

  construct_castle(temp_house);

  if (!writeHouse(temp_house))
    wizlog(56, "Problem saving house %d", temp_house->vnum);
}


void hcontrol_destroy_house(struct char_data *ch, char *arg)
{
  int real_atrium, real_house;
  int house_nr, i;
  struct house_control_rec *house = NULL, *temp_house;
  char buf[MAX_STRING_LENGTH], housefile[MAX_STRING_LENGTH];

  if (!*arg) {
    if (ch)
      send_to_char(HCONTROL_FORMAT, ch);
    return;
  }
  if (is_number(arg)) {
    if (!(house = find_house(atoi(arg)))) {
      if (ch)
	send_to_char("Unknown house.\r\n", ch);
      return;
    }
  } else {
    if (ch)
      send_to_char(HCONTROL_FORMAT, ch);
    return;
  }

  real_house = real_room0(house->vnum);
  if (!real_room0(house->vnum)) {
    logit(LOG_HOUSE, "House had invalid vnum!");
    return;
  } else
    REMOVE_BIT(world[real_house].room_flags, ROOM_ATRIUM);

  house_nr = house->vnum;
  for (i = 0; i < house->num_of_rooms; i++) {
    if (!delete_guild_room(house->room_vnums[i]))
      logit(LOG_HOUSE, "Could not delete room number %d!", house->room_vnums[i]);
  }

  temp_house = first_house;

  if (!temp_house)
    return;

  destroy_castle(temp_house);	/* Does most of the flag and sector work */

  if (house == temp_house) {	/* first record */
    first_house = temp_house->next;
    str_free(temp_house->owner);
    for (i = 0; i < MAX_GUESTS; i++)
      str_free(temp_house->guests[i]);
    mm_release(dead_house_pool, temp_house);
  } else {
    while (temp_house->next) {
      if (temp_house->next == house) {
	struct house_control_rec *tmp_house;
	tmp_house = (temp_house->next)->next;

	str_free(temp_house->next->owner);
	for (i = 0; i < MAX_GUESTS; i++)
	  str_free(temp_house->next->guests[i]);
	mm_release(dead_house_pool, temp_house->next);

	temp_house->next = tmp_house;
	break;
      }
      temp_house = temp_house->next;
    }
  }

  sprintf(housefile, "%d", temp_house->vnum);
  if (ch)
    send_to_char("House deleted.\r\n", ch);
  if (!deleteHouse(housefile))
    wizlog(56, "Problem deleting house %s", housefile);

  /*
   * Now, reset the ROOM_ATRIUM flag on all existing houses' atriums,
   * just in case the house we just deleted shared an atrium with another
   * house.
   */
  for (house = first_house; house; house = house->next)
    if ((real_atrium = real_room(house->room_vnums[0])) >= 0)
      SET_BIT(world[real_atrium].room_flags, ROOM_ATRIUM);
}


void hcontrol_pay_house(struct char_data *ch, char *arg)
{
  struct house_control_rec *house;
  char buf[MAX_STRING_LENGTH];

  if (!*arg)
    send_to_char(HCONTROL_FORMAT, ch);
  else if (!(house = find_house(atoi(arg))))
    send_to_char("Unknown house.\r\n", ch);
  else {
    sprintf(buf, "Payment for house %s collected by %s.", arg, GET_NAME(ch));
    wizlog(56, buf);

    house->last_payment = time(0);
    send_to_char("Payment recorded.\r\n", ch);
    if (!writeHouse(house))
      wizlog(56, "Problem saving house %d", house->vnum);
  }
}

void hcontrol_clear_house(struct char_data *ch, char *arg)
{
  struct house_control_rec *house;

  if (!arg) {
    send_to_char("Supply a house vnum please.\r\n", ch);
    return;
  }
  house = find_house(atoi(arg));
  if (!house)
    return;
  house->teleporter1_room = 0;
  house->teleporter1_dest = 0;
  house->teleporter2_room = 0;
  house->teleporter2_dest = 0;
  send_to_char("Teleporters cleared.\r\n", ch);
  writeHouse(house);
  return;
}

void hcontrol_restore_house(struct char_data *ch, char *arg)
{
  struct house_control_rec *house;
  char id[80];


  if (!arg) {
    send_to_char("Supply a house vnum please.\r\n", ch);
    return;
  }
  house = find_house(atoi(arg));
  if (!house) {
    send_to_char("House not found.\r\n", ch);
    return;
  }
  sprintf(id, "%d", house->vnum);
  destroy_castle(house);
  restoreHouse(id);
}

void hcontrol_detail_house(struct char_data *ch, char *arg)
{
  char buf[256];
  struct house_control_rec *house;
  int x;

  if (!arg) {
    send_to_char("Supply a house vnum please.\r\n", ch);
    return;
  }
  house = find_house(atoi(arg));
  if (!house) {
    send_to_char("Can't find that house.\r\n", ch);
    return;
  }
  sprintf(buf, "&+RHouse:&n &+Y%d&n\r\n", atoi(arg));
  send_to_char(buf, ch);
  for (x = 0; x < house->num_of_rooms; x++) {
    if (x < (house->num_of_rooms - 2)) {
      sprintf(buf, "&+RRoom:&n %d  &+RRoom:&n %d  &+RRoom:&N %d\r\n ", house->room_vnums[x],
	      house->room_vnums[x + 1], house->room_vnums[x + 2]);
      x += 2;
    } else if (x < (house->num_of_rooms - 1)) {
      sprintf(buf, "&+RRoom:&n %d  &+RRoom:&n %d\r\n", house->room_vnums[x],
	      house->room_vnums[x + 1]);
      x += 1;
    } else
      sprintf(buf, "&+RRoom:&n %d\r\n", house->room_vnums[x]);
    send_to_char(buf, ch);
  }

  sprintf(buf, "Exit Number:%d\r\n", house->exit_num);
  send_to_char(buf, ch);
  sprintf(buf, "Number of room:%d\r\n", house->num_of_rooms);
  send_to_char(buf, ch);

  if (house->type == HCONTROL_CASTLE) {
    send_to_char("&+RControlled Lands:&n\r\n", ch);
    for (x = 0; x < MAX_CONTROLLED_LAND; x++)
      if (house->controlled_land[x] != -1) {
	sprintf(buf, "%d ", house->controlled_land[x]);
	send_to_char(buf, ch);
      }
  }
  send_to_char("\r\n", ch);
}

void hcontrol_change_house(struct char_data *ch, char *arg)
{
  struct house_control_rec *house;
  char arg0[MAX_STRING_LENGTH], arg1[MAX_STRING_LENGTH];
  char arg2[MAX_STRING_LENGTH], arg3[MAX_STRING_LENGTH];
  char arg4[MAX_STRING_LENGTH];
  int x;

  if (!arg) {
    send_to_char("Supply a house vnum please.\r\n", ch);
    return;
  }
  half_chop(arg, arg1, arg0);
  half_chop(arg0, arg2, arg3);
  half_chop(arg3, arg3, arg4);

  if (!*arg1 || !*arg2 || !*arg3 || !*arg) {
    send_to_char(HCONTROL_FORMAT, ch);
    return;
  }
  /* arg1 vnum arg2 field arg3 value */

  house = find_house(atoi(arg1));
  if (!house) {
    send_to_char("Use build to create a new house type.\r\n", ch);
    return;
  }
  /* field check */
  if (is_abbrev(arg2, "owner")) {
    house->owner = strdup(arg3);
  } else if (is_abbrev(arg2, "guild")) {
    if (!is_number(arg3)) {
      send_to_char("Value for that field appears wrong.\r\n", ch);
      return;
    }
    house->owner_guild = atoi(arg3);
  } else if (is_abbrev(arg2, "upgrades")) {
    if (!is_number(arg3)) {
      send_to_char("Value for that field appears wrong.\r\n", ch);
      return;
    }
    house->upgrades = atoi(arg3);
  } else if (is_abbrev(arg2, "land")) {
    if (!is_number(arg3)) {
      send_to_char("Value for that field appears wrong.\r\n", ch);
      return;
    }
    for (x = 0; x < MAX_CONTROLLED_LAND; x++) {
      if (house->controlled_land[x] == -1) {
	house->controlled_land[x] = atoi(arg3);
	break;
      } else if (house->controlled_land[x] == atoi(arg3)) {
	house->controlled_land[x] = -1;
	break;
      }
    }
  } else if (is_abbrev(arg2, "numrooms")) {
    if (!is_number(arg3)) {
      send_to_char("Value for that field appears wrong.\r\n", ch);
      return;
    }
    house->num_of_rooms = atoi(arg3);
  } else if (is_abbrev(arg2, "room")) {
    if (!is_number(arg3)) {
      send_to_char("Value for that field appears wrong.\r\n", ch);
      return;
    }
    for (x = 0; x < house->num_of_rooms; x++) {
      if (house->room_vnums[x] == -1) {
	house->room_vnums[x] = atoi(arg3);
	break;
      } else if (house->room_vnums[x] == atoi(arg3)) {
	house->room_vnums[x] = -1;
	break;
      }
    }
    send_to_char("Be sure to change num_of_rooms!!!\r\n", ch);
  } else if (is_abbrev(arg2, "type")) {
    if (is_abbrev(arg3, "house"))
      house->type = HCONTROL_HOUSE;
    else if (is_abbrev(arg3, "guild"))
      house->type = HCONTROL_GUILD;
    else if (is_abbrev(arg3, "castle"))
      house->type = HCONTROL_CASTLE;
  }
  if (!writeHouse(house))
    wizlog(56, "Problem saving house %d", house->vnum);

  send_to_char("Data changed.\r\n", ch);
}

/* The hcontrol command itself, used by imms to create/destroy houses */
void do_hcontrol(struct char_data *ch, char *arg, int cmd)
{
  char arg1[MAX_INPUT_LENGTH], arg2[MAX_INPUT_LENGTH];

  half_chop(arg, arg1, arg2);

  if (is_abbrev(arg1, "build"))
    hcontrol_build_house(ch, arg2);
  else if (is_abbrev(arg1, "destroy"))
    hcontrol_destroy_house(ch, arg2);
  else if (is_abbrev(arg1, "pay"))
    hcontrol_pay_house(ch, arg2);
  else if (is_abbrev(arg1, "show"))
    hcontrol_list_houses(ch, arg2);
  else if (is_abbrev(arg1, "detail"))
    hcontrol_detail_house(ch, arg2);
  else if (is_abbrev(arg1, "clear"))
    hcontrol_clear_house(ch, arg2);
  else if (is_abbrev(arg1, "change"))
    hcontrol_change_house(ch, arg2);
  else if (is_abbrev(arg1, "restore"))
    hcontrol_restore_house(ch, arg2);
  else
    send_to_char(HCONTROL_FORMAT, ch);
}

/* Simply determines how much a house will cost a player to initially build */
int house_cost(struct char_data *ch)
{
  int cost = 0;

  if (!ch)
    return cost;

  /* Base cost is x plat */
  cost = 2000;

  /* Modify by justice */
  if (CHAR_IN_JUSTICE_AREA(ch)) {
    if (PC_TOWN_JUSTICE_FLAGS(ch, CHAR_IN_JUSTICE_AREA(ch)) == JUSTICE_IS_CITIZEN) {
      cost /= 3;
    } else if (PC_TOWN_JUSTICE_FLAGS(ch, CHAR_IN_JUSTICE_AREA(ch)) == JUSTICE_IS_CITIZEN_BUY) {
      cost /= 2;
    } else if (PC_TOWN_JUSTICE_FLAGS(ch, CHAR_IN_JUSTICE_AREA(ch)) == JUSTICE_IS_NORMAL) {
      cost /= 1.5;
    } else if (PC_TOWN_JUSTICE_FLAGS(ch, CHAR_IN_JUSTICE_AREA(ch)) == JUSTICE_IS_OUTCAST) {
      cost *= 2;
    }
  }
  /* Modify by terrain */
  if (world[ch->in_room].sector_type == SECT_MOUNTAIN)
    cost *= 3;
  if (world[ch->in_room].sector_type == SECT_HILLS)
    cost *= 2;
  if (world[ch->in_room].sector_type == SECT_FOREST)
    cost *= 1.5;

  return cost;
}

/* Simply determines how much a guildhall will cost to initially build */
int guild_cost(struct char_data *ch)
{
  int cost = 0;

  if (!ch)
    return cost;

  /* Base cost is x plat */
  cost = 7000;

  /* Modify by justice */
  if (CHAR_IN_JUSTICE_AREA(ch)) {
    if (PC_TOWN_JUSTICE_FLAGS(ch, CHAR_IN_JUSTICE_AREA(ch)) == JUSTICE_IS_CITIZEN) {
      cost /= 1.5;
    } else if (PC_TOWN_JUSTICE_FLAGS(ch, CHAR_IN_JUSTICE_AREA(ch)) == JUSTICE_IS_CITIZEN_BUY) {
      cost /= 1.25;
    } else if (PC_TOWN_JUSTICE_FLAGS(ch, CHAR_IN_JUSTICE_AREA(ch)) == JUSTICE_IS_NORMAL) {
      cost *= 1.5;
    } else if (PC_TOWN_JUSTICE_FLAGS(ch, CHAR_IN_JUSTICE_AREA(ch)) == JUSTICE_IS_OUTCAST) {
      cost *= 2;
    }
  }
  /* Modify by terrain */
  if (world[ch->in_room].sector_type == SECT_MOUNTAIN)
    cost *= 3;
  if (world[ch->in_room].sector_type == SECT_HILLS)
    cost *= 2;
  if (world[ch->in_room].sector_type == SECT_FOREST)
    cost *= 1.5;

  return cost;
}

/* The house command, used by mortal house owners to assign guests */
void do_house(struct char_data *ch, char *arg, int cmd)
{
  int cost;
  char arg1[MAX_INPUT_LENGTH], buf[MAX_INPUT_LENGTH];
  struct house_control_rec *house;

  if (IS_NPC(ch))
    return;			/* NPCs bad */

  one_argument(arg, arg1);

  if (arg1 && !strcmp(arg1, "cost")) {
    if (!valid_build_location(ch->in_room, ch, 0))
      return;
    cost = house_cost(ch);
    cost *= 1000;
    sprintf(buf, "Building a house here would cost %s,", coin_stringv(cost));
    send_to_char(buf, ch);
    cost = guild_cost(ch);
    cost *= 1000;
    sprintf(buf, " and a guildhall would be %s.\r\n", coin_stringv(cost));
    send_to_char(buf, ch);
    return;
  } else if (arg1 && !strcmp(arg1, "sell")) {
    send_to_char("Real estate agents have joined a union, and as a result declared a strike. Try again next week.\r\n", ch);
    return;
    half_chop(arg1, arg, arg1);
    if (!IS_SET(world[ch->in_room].room_flags, ROOM_HOUSE)) {
      send_to_char("You must be in your house to sell it.\r\n", ch);
      return;
    } else if (!(house = find_house(world[ch->in_room].number))) {
      send_to_char("Um.. this house seems to be screwed up.\r\n", ch);
      return;
    } else if (str_cmp(house->owner, J_NAME(ch))) {
      send_to_char("Only the primary owner may sell a house.\r\n", ch);
      return;
    }
    if (!*arg1) {
      /* open market sell */
      sprintf(buf, "%d", house->vnum);
      AddEvent(EVENT_SPECIAL, number(10, 100), TRUE, open_market_house, buf);
      send_to_char("Your building has been placed upon the open market. Good luck!\r\n", ch);
      return;
    } else {
      if (!is_number(arg1)) {
	house->owner = strdup(arg1);
	sprintf(buf, "Buildings deed transfered to %s.\r\n", house->owner);
	send_to_char(buf, ch);
	return;
      } else {
	send_to_char("Eh?!? Who the hell is that?\r\n", ch);
	return;
      }
    }
#if 0
    /* no guests anymore. Kepp though, just in case someday... */
  } else if (!IS_SET(world[ch->in_room].room_flags, ROOM_HOUSE)) {
    send_to_char("You must be in your house to set guests.\r\n", ch);
    return;
  } else if (!(house = find_house(world[ch->in_room].number))) {
    send_to_char("Um.. this house seems to be screwed up.\r\n", ch);
    return;
  } else if (str_cmp(house->owner, GET_NAME(ch))) {
    send_to_char("Only the primary owner can set guests.\r\n", ch);
    return;
  } else if (!*arg1) {
    strcpy(buf, "&+GGuests of your house:&N\r\n");
    if (house->num_of_guests == 0)
      strcat(buf, " None.\r\n");
    else
      for (j = 0; j < house->num_of_guests; j++) {
	sprintf(buf + strlen(buf), "%s\r\n", house->guests[j]);
      }
    send_to_char(buf, ch);

    return;
  } else {
    for (j = 0; j < house->num_of_guests; j++) {
      if (!str_cmp(house->guests[j], arg1)) {
	for (; j < house->num_of_guests; j++)
	  house->guests[j] = house->guests[j + 1];

	house->num_of_guests--;
	send_to_char("Guest deleted.\r\n", ch);

	if (!writeHouse(house))
	  wizlog(56, "Problem saving house %d", house->vnum);

	return;
      }
    }

    j = house->num_of_guests;
    if (j < MAX_GUESTS) {
      if (GET_MONEY(ch) < 100000) {
	send_to_char("Adding a guest will cost you a 100 platinum tax.\r\n", ch);
	return;
      }
      SUB_MONEY(ch, 100000, 0);
      house->guests[j] = str_dup(arg1);
      send_to_char("Guest added, 100 platinum tax paid.\r\n", ch);
      house->num_of_guests++;

      if (!writeHouse(house))
	wizlog(56, "Problem saving house %d", house->vnum);

      return;
    } else {
      send_to_char("You may not add any more guests to your house.\r\n", ch);

      return;
    }
#endif
  }
}

/* note: arg passed must be house vnum, so there. */
int House_can_enter(struct char_data *ch, int vnum, int exitnumb)
{
  int j;
  struct house_control_rec *house;

  /* exitnumb is -1 if we're dealing with teleporter entrance object,
     as opposed to a true door */

  house = find_house(vnum);

  if (IS_TRUSTED(ch))
    return TRUE;
  if (!house && (exitnumb != -1))
    return TRUE;
  if (IS_NPC(ch) && IS_SET(ch->only.npc->spec[2], MOB_SPEC_JUSTICE))
    return TRUE;
  if (house && (exitnumb != -1) && (exitnumb != house->exit_num))
    return TRUE;
  if (house) {
    switch (house->mode) {
    case HMODE_PRIVATE:
      if (!str_cmp(house->owner, J_NAME(ch)))
	return TRUE;
      for (j = 0; j < house->num_of_guests; j++)
	if (!str_cmp(house->guests[j], J_NAME(ch)))
	  return TRUE;
      return FALSE;
      break;
    case HMODE_OPEN:
      return TRUE;
      break;
    }
  }
  return FALSE;
}
/* guild room data stored as follows:
   filename is 
   Name~
   Desc~
   exit info (same as .wld file)
 */

void read_guild_room(int vnum, int guild)
{
  FILE *hfile;
  char buffer[MAX_STRING_LENGTH];
  int room_nr, x;
  char *tmp;
  int dir, temp;

  sprintf(buffer, "%s/House/HouseRoomData/%d", SAVE_DIR, vnum);
  if (!(hfile = fopen(buffer, "r")))
    return;
  room_nr = real_room0(vnum);
  if (!room_nr) {
    logit(LOG_HOUSE, "Room %d didn't exist?", vnum);
    return;
  }
  fscanf(hfile, " %d \n", &temp);
  if (temp != guild) {
    logit(LOG_HOUSE, "read_guild_room: room #%d belonged to guild #%d instead of %d",
	  vnum, temp, guild);
    return;
  }
  world[room_nr].zone = temp;

  tmp = fread_string(hfile);
  world[room_nr].name = tmp;
  tmp = fread_string(hfile);
  world[room_nr].description = tmp;
  SET_BIT(world[room_nr].room_flags, ROOM_HOUSE);
  SET_BIT(world[room_nr].room_flags, NO_TELEPORT);
  SET_BIT(world[room_nr].room_flags, NORECALL);
  SET_BIT(world[room_nr].room_flags, NO_SUMMON);
  SET_BIT(world[room_nr].room_flags, NO_GATE);
  SET_BIT(world[room_nr].room_flags, TWILIGHT);
  SET_BIT(world[room_nr].room_flags, GUILD_ROOM);
  for (x = 0; x < NUMB_EXITS; x++)
    world[room_nr].dir_option[x] = 0;
  for (;;) {
    fscanf(hfile, " %s \n", buffer);
    if (*buffer == 'D') {
      dir = atoi(buffer + 1);
      CREATE(world[room_nr].dir_option[dir], struct room_direction_data, 1);
      world[room_nr].dir_option[dir]->general_description = fread_string(hfile);
      world[room_nr].dir_option[dir]->keyword = fread_string(hfile);
      fscanf(hfile, " %d ", &temp);
      if (temp) {
	world[room_nr].dir_option[dir]->exit_info = EX_ISDOOR;
	if (temp == 2)
	  world[room_nr].dir_option[dir]->exit_info |= EX_PICKABLE;
	if (temp == 3)
	  world[room_nr].dir_option[dir]->exit_info |= EX_PICKPROOF;
	if (temp > 3)
	  world[room_nr].dir_option[dir]->exit_info |= EX_SECRET;
      } else {
	world[room_nr].dir_option[dir]->exit_info = 0;
      }
      fscanf(hfile, " %d ", &temp);
      world[room_nr].dir_option[dir]->key = temp;
      fscanf(hfile, " %d ", &temp);
      world[room_nr].dir_option[dir]->to_room = real_room(temp);
#if 0
      if (world[world[room_nr].dir_option[dir]->to_room].zone !=
	  world[room_nr].zone)
	logit(LOG_HOUSE, "read_guild_room: room #%d belonged to guild #%d instead of %d",
	      vnum, world[room_nr].zone, guild);
#endif
      *buffer = 'X';
    } else
      break;
  }
  fclose(hfile);
};

void write_guild_room(int vnum, int guild)
{
  FILE *hfile;
  int x, room_nr, info;
  char buffer[MAX_STRING_LENGTH];

  room_nr = real_room0(vnum);
  if (!room_nr) {
    logit(LOG_HOUSE, "House %d appears bogus in write_guild_room", vnum);
    return;
  }
  sprintf(buffer, "%s/House/HouseRoomData/%d", SAVE_DIR, vnum);
  if (!(hfile = fopen(buffer, "w"))) {
    logit(LOG_HOUSE, "Cannot open %s for writing guild rooms.", buffer);
    return;
  }
  fprintf(hfile, "%d\n", guild);
  if (strstr(world[room_nr].name, "Unnamed"))
    world[room_nr].name = strdup("A Barren Room");
  fprintf(hfile, "%s~\n", world[room_nr].name);
  fprintf(hfile, "%s~\n", world[room_nr].description);
  for (x = 0; x < NUMB_EXITS; x++) {
    if (world[room_nr].dir_option[x] && world[world[room_nr].dir_option[x]->to_room].number >= 48000 &&
	world[world[room_nr].dir_option[x]->to_room].number <= 49999) {
      fprintf(hfile, "D%d\n", x);
      if (!world[room_nr].dir_option[x]->general_description)
	fprintf(hfile, "~\n");
      else
	fprintf(hfile, "%s~\n", world[room_nr].dir_option[x]->general_description);
      if (!world[room_nr].dir_option[x]->keyword)
	fprintf(hfile, "~\n");
      else
	fprintf(hfile, "%s~\n", world[room_nr].dir_option[x]->keyword);
      info = 0;
      if (IS_SET(world[room_nr].dir_option[x]->exit_info, EX_ISDOOR))
	info = 1;
      if (IS_SET(world[room_nr].dir_option[x]->exit_info, EX_PICKABLE))
	info = 2;
      if (IS_SET(world[room_nr].dir_option[x]->exit_info, EX_PICKPROOF))
	info = 3;
      if (IS_SET(world[room_nr].dir_option[x]->exit_info, EX_SECRET))
	info = 4;
      fprintf(hfile, "%d %d %d\n", info,
	      world[room_nr].dir_option[x]->key,
	      world[world[room_nr].dir_option[x]->to_room].number);
    }
  }
  fclose(hfile);
};

int delete_guild_room(int vnum)
{
  char Gbuf1[MAX_STRING_LENGTH], Gbuf2[MAX_STRING_LENGTH];

  if (vnum < 0) {
    logit(LOG_EXIT, "invalid guild room %d in delete_guild_room", vnum);
    raise(SIGSEGV);
  }
  sprintf(Gbuf1, "%s/House/HouseRoomData/%d", SAVE_DIR, vnum);
  strcpy(Gbuf2, Gbuf1);
  strcat(Gbuf2, ".bak");
  unlink(Gbuf1);
  unlink(Gbuf2);

  return TRUE;
}

int hcontrol_setup_room(struct house_control_rec *house, int cur_room, int exit_dir)
{
  /* setup the default name, dir, etc for room */
  int x, n, nr;

  for (x = START_HOUSE_VNUM; x < END_HOUSE_VNUM; x++) {
    if (strstr(world[real_room0(x)].name, "Unnamed"))
      break;
  }
  if (!strstr(world[real_room0(x)].name, "Unnamed"))
    return -1;			/* no more rooms! */

  nr = real_room0(x);
  if (!nr)
    return -1;

  world[nr].name = strdup("A Barren Room");
  SET_BIT(world[nr].room_flags, ROOM_HOUSE);
  SET_BIT(world[nr].room_flags, NO_TELEPORT);
  SET_BIT(world[nr].room_flags, NO_SUMMON);
  SET_BIT(world[nr].room_flags, NO_GATE);
  SET_BIT(world[nr].room_flags, TWILIGHT);
  if (house->type == HCONTROL_GUILD)
    SET_BIT(world[nr].room_flags, GUILD_ROOM);
  world[nr].description = strdup("This room is empty.\r\n");
  /* setup exits to/from atrium to first room */
  if (exit_dir >= 0)
    connect_rooms(cur_room, x, exit_dir);
  for (n = 0; n < house->num_of_rooms; n++) {
    if (house->room_vnums[n] == world[nr].number)
      break;
    else {
      house->room_vnums[house->num_of_rooms] = world[nr].number;
      break;
    }
  }
  return x;
}

struct house_control_rec *house_ch_is_in(struct char_data *ch)
{

  int x, flag;
  struct house_control_rec *temp_house;

  temp_house = first_house;
  flag = 0;

  while (!flag && temp_house) {
    for (x = 0; x < temp_house->num_of_rooms; x++) {
      if (temp_house->room_vnums[x] == world[ch->in_room].number)
	flag = 1;
    }
    if (!flag)
      temp_house = temp_house->next;
  }
  return temp_house;
}
/* Construction Commands */


/* CONSTRUCT syntax */

const char *CONSTRUCT_FORMAT =
"\r\n"
" Syntax\r\n"
" ------\r\n"
"  construct room exit_dir door|nodoor [door keyword]	<&+BG&n|&+YC&n>\r\n"
"  construct name <room name>				<A>\r\n"
"  construct describe					<A>\r\n"
"  construct cleric | mage | warrior			<&+BG&n|&+YC&n>\r\n"
"  construct inn					<&+BG&n|&+YC&n>\r\n"
"  construct chest					<&+BG&n|&+YC&n>\r\n"
"  construct cabinet					<&+LH&n>\r\n"
"  construct teleporter <target>			<&+BG&n|&+YC&n>\r\n"
"  construct fountain					<&+BG&n|&+YC&n>\r\n"
"  construct mouth					<&+BG&n|&+YC&n>\r\n"
"  construct board					<&+BG&n|&+YC&n>\r\n"
"  construct holy					<&+BG&n|&+YC&n>\r\n"
"  construct unholy					<&+BG&n|&+YC&n>\r\n"
"  construct heal					<&+BG&n|&+YC&n>\r\n"
"  construct shop					<&+BG&n|&+YC&n>\r\n"
"  construct secret entrance				<&+BG&n|&+LH&n>\r\n"
"  <A> command allowed for all types <G> guildhalls only\r\n"
"  <C> castle only <H> house only\r\n"
"  \r\n";

void do_construct(struct char_data *ch, char *arg, int cmd)
{
  struct house_control_rec *house, *house2;
  uint temp;
  char buff[MAX_STRING_LENGTH];
  struct house_upgrade_rec *cur_rec;

  if (!ch)
    return;

  if (!arg) {
    send_to_char(CONSTRUCT_FORMAT, ch);
    return;
  }
  arg = one_argument(arg, buff);

  if (is_abbrev(buff, "show") && IS_TRUSTED(ch)) {	/* show construction q */
    do_show_q(ch);
    return;
  }
  if (is_abbrev(buff, "house")) {
    do_construct_house(ch, arg);
    return;
  }
  if (is_abbrev(buff, "guild")) {
    do_construct_guild(ch, arg);
    return;
  }
  house = house_ch_is_in(ch);
  if (!house) {
    send_to_char("You cannot construct here.\r\n", ch);
    return;
  }
  if ((GET_A_NUM(ch) != house->owner_guild) && (!IS_TRUSTED(ch)) &&
      (house->type == HCONTROL_GUILD)) {
    send_to_char("You don't have the authority to build here.\r\n", ch);
    return;
  }
  temp = GET_M_BITS(GET_A_BITS(ch), A_RK_MASK);
  if (!(IS_LEADER(temp) || IS_TRUSTED(ch)) && (house->type == HCONTROL_GUILD)) {
    send_to_char("You are not high enough rank to build here.\r\n", ch);
    return;
  } else if (GET_LEVEL(ch) < 20) {
    send_to_char("Ye must achieve more experience in the world to build on to yer abode.\r\n", ch);
    return;
  }
  cur_rec = house_upgrade_list;
  while (cur_rec) {
    house2 = find_house(cur_rec->vnum);
    if (house2 && house == house2)
      if (!str_cmp(house2->owner, J_NAME(ch)) && !IS_TRUSTED(ch)) {
	send_to_char("You are already building something. Finish it first.\r\n", ch);
	return;
      }
    cur_rec = cur_rec->next;
  }
  /* Okay, valid place/person, parse the arg */

  if (is_abbrev(buff, "process")) {	/* process the q */
    if (IS_TRUSTED(ch))
      process_construction_q();
    return;
  }
  if (is_abbrev(buff, "connect")) {	/* connect 2 rooms, god only to */
    if (IS_TRUSTED(ch))		/* prevent goofyness */
      parse_connection(house, ch, arg);
    return;
  }
  if (is_abbrev(buff, "name")) {	/* name current room */
    do_name_room(house, ch, arg);
    return;
  }
  if (is_abbrev(buff, "describe")) {	/* describe current room */
    do_describe_room(house, ch, arg);
    return;
  }
  if (is_abbrev(buff, "cabinet")) {
    do_build_chest(house, ch);
    return;
  }
  if (house->type == HCONTROL_GUILD || house->type == HCONTROL_HOUSE) {
    if (is_abbrev(buff, "secret")) {	/* construct secret */
      do_build_secret(house, ch);
      return;
    }
  }
  /* next are castle/guild only */
  if (house->type == HCONTROL_GUILD || house->type == HCONTROL_CASTLE) {
    if (is_abbrev(buff, "shop")) {
      do_build_shop(house, ch);
      return;
    }
    if (is_abbrev(buff, "teleporter")) {
      do_build_teleporter(house, ch, arg);
      return;
    }
    if (is_abbrev(buff, "room")) {	/* construct room */
      do_construct_room(house, ch, arg);
      return;
    }
    if (is_abbrev(buff, "chest")) {
      do_build_chest(house, ch);
      return;
    }
    if (is_abbrev(buff, "holy")) {
      do_build_holy(house, ch);
      return;
    }
    if (is_abbrev(buff, "unholy")) {
      do_build_unholy(house, ch);
      return;
    }
    if (is_abbrev(buff, "cleric")) {
      do_build_golem(house, ch, HCONTROL_ADD_CLERIC_GOLEM);
      return;
    }
    if (is_abbrev(buff, "mage")) {
      do_build_golem(house, ch, HCONTROL_ADD_MAGE_GOLEM);
      return;
    }
    if (is_abbrev(buff, "warrior")) {
      do_build_golem(house, ch, HCONTROL_ADD_WARRIOR_GOLEM);
      return;
    }
    if (is_abbrev(buff, "heal")) {
      do_build_heal(house, ch);
      return;
    }
    if (is_abbrev(buff, "fountain")) {
      do_build_fountain(house, ch);
      return;
    }
    if (is_abbrev(buff, "mouth")) {
      do_build_mouth(house, ch);
      return;
    }
    if (is_abbrev(buff, "board")) {	/* this one simple */
      house->board_vnum = world[ch->in_room].number;
      writeHouse(house);
      send_to_char("Board location moved to here.\r\n", ch);
      return;
    }
    if (is_abbrev(buff, "inn")) {
      do_build_inn(house, ch);
      return;
    }
  }
  send_to_char(CONSTRUCT_FORMAT, ch);
  return;
}

int charge_char(struct char_data *ch, int plat)
{
  char buff[256];

  if (IS_TRUSTED(ch))
    return TRUE;

  if (!plat) {
    send_to_char("Ye don't have the funds.\r\n", ch);
    return FALSE;
  }
  plat *= 1000;

  if (GET_MONEY(ch) < plat) {
    sprintf(buff, "Ye don't have the %s required.\r\n", coin_stringv(plat));
    send_to_char(buff, ch);
    return FALSE;
  }
  SUB_MONEY(ch, plat, 0);
  return TRUE;
}

house_upgrade_rec *get_con_rec()
{

  if (!dead_construction_pool)
    dead_construction_pool = mm_create("CONSTRUCTION",
				       sizeof(struct house_upgrade_rec),
			    offsetof(struct house_upgrade_rec, next), 1);
  return mm_get(dead_construction_pool);
}
void do_build_heal(struct house_control_rec *house, struct char_data *ch)
{
  struct house_upgrade_rec *con_rec;

  if (house->heal_vnum) {
    send_to_char("Heal Room moved to here.\r\n", ch);
    house->heal_vnum = world[ch->in_room].number;
    writeHouse(house);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_HEAL]))
    return;

  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = HCONTROL_ADD_HEAL;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Healing room construction begun.\r\n", ch);
}

void do_build_secret(struct house_control_rec *house, struct char_data *ch)
{
  struct house_upgrade_rec *con_rec;

  if (house->secret_entrance) {
    send_to_char("Entrance is already hidden.\r\n", ch);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_SECRET]))
    return;
  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = HCONTROL_ADD_SECRET;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Secret entrance construction begun.\r\n", ch);
}

void do_build_holy(struct house_control_rec *house, struct char_data *ch)
{
  struct house_upgrade_rec *con_rec;

  if (house->holy_fount_vnum) {
    send_to_char("Holy water fountain location moved to here.\r\n", ch);
    house->holy_fount_vnum = world[ch->in_room].number;
    writeHouse(house);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_HOLY]))
    return;
  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = HCONTROL_ADD_HOLY;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Holy water fountain construction begun.\r\n", ch);
}

void do_build_unholy(struct house_control_rec *house, struct char_data *ch)
{
  struct house_upgrade_rec *con_rec;

  if (house->unholy_fount_vnum) {
    send_to_char("Unholy water fountain location moved to here.\r\n", ch);
    house->unholy_fount_vnum = world[ch->in_room].number;
    writeHouse(house);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_UNHOLY]))
    return;
  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = HCONTROL_ADD_UNHOLY;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Unholy water fountain construction started.\r\n", ch);
}
void do_build_mouth(struct house_control_rec *house, struct char_data *ch)
{
  struct house_upgrade_rec *con_rec;

  if (house->mouth_vnum) {
    house->mouth_vnum = world[ch->in_room].number;
    send_to_char("Mouth location moved to here.\r\n", ch);
    writeHouse(house);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_MOUTH])) {
    return;
  }
  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = HCONTROL_ADD_MOUTH;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Magic Mouth Construction begun.\r\n", ch);
}

void do_build_teleporter(struct house_control_rec *house, struct char_data *ch, char *arg)
{
  struct house_upgrade_rec *con_rec;
  struct char_data *target;
  struct house_control_rec *thouse;

  if (house->teleporter1_room && house->teleporter2_room) {
    send_to_char("You can only have 2 magic portals per place.\r\n", ch);
    return;
  }
  target = get_char_vis(ch, arg);
  if (!target) {
    send_to_char("I don't see that target.\r\n", ch);
    return;
  }
  if (GET_A_NUM(target) != GET_A_NUM(ch)) {
    send_to_char("Target must be in your guild.\r\n", ch);
    return;
  }
  thouse = house_ch_is_in(target);
  if (!thouse) {
    send_to_char("Target is not in a valid guild hall.\r\n", ch);
    return;
  }
  if (thouse->owner_guild != house->owner_guild) {
    send_to_char("Target house is not owned by your guild.\r\n", ch);
    return;
  }
  if (thouse->teleporter1_room && thouse->teleporter2_room) {
    send_to_char("Target house has no room for teleporters.\r\n", ch);
    return;
  }
  if ((thouse->num_of_rooms < 9) || (house->num_of_rooms < 9)) {
    send_to_char("You have to have at least 10 rooms in each house to build.\r\n", ch);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_TELEPORTER]))
    return;

  /* okay, build it */

  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = HCONTROL_ADD_TELEPORTER;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->door = world[target->in_room].number;
  con_rec->exit_dir = thouse->vnum;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Teleporter construction begun.\r\n", ch);
}

void stock_guild_shop(int loc)
{
  int items[] =
  {
    204, 364, 412, 3050, 95547, 6089, 53, 203, 5725, 95540, 399, 0
  };

  struct obj_data *tobj;
  struct char_data *tmob;
  int x;

  tmob = read_mobile(SHOPKEEPER_VNUM, VIRTUAL);
  if (!tmob)
    return;
  char_to_room(tmob, real_room0(loc), 0);
  x = 0;
  while (items[x] != 0) {
    tobj = read_object(items[x], VIRTUAL);
    if (tobj)
      obj_to_char(tobj, tmob);
    x++;
  }
}
void do_build_shop(struct house_control_rec *house, struct char_data *ch)
{
  struct house_upgrade_rec *con_rec;

  if (house->shop_vnum) {
    house->shop_vnum = world[ch->in_room].number;
    writeHouse(house);
    send_to_char("Shop Location moved to here.\r\n", ch);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_SHOP]))
    return;
  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = HCONTROL_ADD_SHOP;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Shop Construction begun.\r\n", ch);
}

void do_build_chest(struct house_control_rec *house, struct char_data *ch)
{
  struct house_upgrade_rec *con_rec;

  if (house->type == HCONTROL_GUILD)
    if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_CHEST])) {
      return;
    }
  if (house->type == HCONTROL_CASTLE)
    if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_CHEST])) {
      return;
    }
  if (house->type == HCONTROL_HOUSE)
    if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_CABINET])) {
      return;
    }
  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  if (house->type == HCONTROL_GUILD)
    con_rec->type = HCONTROL_ADD_CHEST;
  else if (house->type == HCONTROL_CASTLE)
    con_rec->type = HCONTROL_ADD_CHEST;
  else if (house->type == HCONTROL_HOUSE)
    con_rec->type = HCONTROL_ADD_CABINET;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Chest construction begun.\r\n", ch);
}

void do_build_fountain(struct house_control_rec *house, struct char_data *ch)
{
  struct house_upgrade_rec *con_rec;

  if (house->fountain_vnum) {
    house->fountain_vnum = world[ch->in_room].number;
    writeHouse(house);
    send_to_char("Fountain location moved to here.\r\n", ch);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_FOUNTAIN])) {
    return;
  }
  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = HCONTROL_ADD_FOUNTAIN;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Fountain construction begun.\r\n", ch);
}

void do_build_inn(struct house_control_rec *house, struct char_data *ch)
{
  struct house_upgrade_rec *con_rec;

  if (house->inn_vnum) {
    house->inn_vnum = world[ch->in_room].number;
    writeHouse(house);
    send_to_char("Inn location moved to here.\r\n", ch);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_INN])) {
    return;
  }
  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = HCONTROL_ADD_INN;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Inn construction begun.\r\n", ch);
}
void do_build_golem(struct house_control_rec *house, struct char_data *ch, int type)
{
  struct house_upgrade_rec *con_rec;

  if ((house->wizard_golems + house->warrior_golems + house->cleric_golems) == 4) {
    send_to_char("This castle has the max (4) of golems.\r\n", ch);
    return;
  }
  if (!charge_char(ch, upgrade_costs[type]))
    return;
  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->type = type;
  con_rec->time = time(0);
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Golem construction begun.\r\n", ch);
}

void do_describe_room(struct house_control_rec *house, struct char_data *ch, char *arg)
{
  struct house_upgrade_rec *con_rec;

  if (!charge_char(ch, upgrade_costs[HCONTROL_DESC_ROOM]))
    return;
  send_to_char("Enter new room description, save with /s, /h for help.\r\n", ch);
  ch->desc->str = NULL;
  ch->desc->max_str = MAX_STRING_LENGTH;

  con_rec = get_con_rec();
  con_rec->door_keyword = strdup("\n");
  *con_rec->door_keyword = 0;
  ch->desc->str = &(con_rec->door_keyword);
  con_rec->vnum = house->vnum;
  con_rec->time = time(0);
  con_rec->type = HCONTROL_DESC_ROOM;
  con_rec->location = world[ch->in_room].number;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Room description begun.\r\n", ch);
}

void do_name_room(struct house_control_rec *house, struct char_data *ch, char *arg)
{
  struct house_upgrade_rec *con_rec;

  arg++;
  if (!*arg || !arg)
    return;

  if (!charge_char(ch, upgrade_costs[HCONTROL_NAME_ROOM]))
    return;

  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->time = time(0);
  con_rec->type = HCONTROL_NAME_ROOM;
  con_rec->location = world[ch->in_room].number;
  con_rec->door_keyword = strdup(arg);
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
  send_to_char("Room naming begun.\r\n", ch);
}

/* construct one room of the house */
void do_construct_room(struct house_control_rec *house, struct char_data *ch, char *arg)
{
  char buff[MAX_STRING_LENGTH];
  int exit_dir;
  int door;
  int room_in_progress = 0;
  char *door_keyword;
  struct house_upgrade_rec *con_rec;

/*  if (!IS_TRUSTED(ch)) {
   send_to_char("No new rooms right now. Try back later...\r\n", ch);
   return;
   } */

  con_rec = house_upgrade_list;

  while (con_rec) {
    if ((con_rec->vnum == house->vnum) &&
	(con_rec->type == HCONTROL_ADD_ROOM)) {
      room_in_progress++;
    }
    con_rec = con_rec->next;
  }

  if ((house->num_of_rooms + room_in_progress) >= (MAX_HOUSE_ROOMS)) {
    send_to_char("This house has max # of rooms already.\n\r", ch);
    return;
  }
  arg = one_argument(arg, buff);
  if (!(exit_dir = old_search_block(buff, 0, strlen(buff), dirs, 0))) {
    send_to_char(CONSTRUCT_FORMAT, ch);
    return;
  };
  exit_dir--;			/* to put array back on track */
  if (world[ch->in_room].dir_option[exit_dir]) {	/* already exit there */
    send_to_char("There is already an exit there.\r\n", ch);
    return;
  }
  arg = one_argument(arg, buff);
  door = 0;
#if 1
/* buggy */
  if (is_abbrev(buff, "door")) {
    door = 1;
    if (arg[0])
      door_keyword = strdup(arg);
    else
      door_keyword = strdup("none");
  }
#endif
  if (world[ch->in_room].number == house->room_vnums[0]) {
    send_to_char("This is your golem room, you cannot build rooms here.\r\n", ch);
    return;
  }
  if (!charge_char(ch, upgrade_costs[HCONTROL_ADD_ROOM])) {
    return;
  }
  sprintf(buff, "Building a room, exit %s", dirs[exit_dir]);
  send_to_char(buff, ch);
  if (door) {
    sprintf(buff, " with door keyword %s.\r\n", door_keyword);
    send_to_char(buff, ch);
  } else
    send_to_char(" with no door.\r\n", ch);

  con_rec = get_con_rec();
  con_rec->vnum = house->vnum;
  con_rec->time = time(0);
  con_rec->type = HCONTROL_ADD_ROOM;
  con_rec->location = world[ch->in_room].number;
  con_rec->exit_dir = exit_dir;
  con_rec->door = door;
  if (door)
    con_rec->door_keyword = door_keyword;
  con_rec->next = house_upgrade_list;
  house_upgrade_list = con_rec;
  writeConstructionQ();
}

void do_show_q(struct char_data *ch)
{
  struct house_upgrade_rec *cur_rec;
  char buff[MAX_STRING_LENGTH];
  char temp[20];

  cur_rec = house_upgrade_list;

  send_to_char("House    Type    Location   Complete Time\r\n", ch);
  send_to_char("-----    ----    --------   -------------\r\n", ch);
  while (cur_rec) {
    switch (cur_rec->type) {
    case HCONTROL_ADD_ROOM:
      sprintf(temp, "%s", "Room");
      break;
    case HCONTROL_NAME_ROOM:
      sprintf(temp, "%s", "Name");
      break;
    case HCONTROL_DESC_ROOM:
      sprintf(temp, "%s", "Descr");
      break;
    case HCONTROL_ADD_MAGE_GOLEM:
    case HCONTROL_ADD_CLERIC_GOLEM:
    case HCONTROL_ADD_WARRIOR_GOLEM:
      sprintf(temp, "%s", "Golem");
      break;
    case HCONTROL_ADD_INN:
      sprintf(temp, "%s", "Inn");
      break;
    case HCONTROL_ADD_MOUTH:
      sprintf(temp, "%s", "Mouth");
      break;
    case HCONTROL_ADD_SHOP:
      sprintf(temp, "%s", "Shop");
      break;
    case HCONTROL_ADD_CHEST:
      sprintf(temp, "%s", "Chest");
      break;
    case HCONTROL_ADD_CABINET:
      sprintf(temp, "%s", "Cabinet");
      break;
    case HCONTROL_ADD_TELEPORTER:
      sprintf(temp, "%s", "Tele");
      break;
    case HCONTROL_ADD_FOUNTAIN:
      sprintf(temp, "%s", "Fount");
      break;
    case HCONTROL_ADD_HOLY:
      sprintf(temp, "%s", "Holy");
      break;
    case HCONTROL_ADD_UNHOLY:
      sprintf(temp, "%s", "Unholy");
      break;
    case HCONTROL_ADD_HEAL:
      sprintf(temp, "%s", "Heal");
      break;
    case HCONTROL_INITIAL_HOUSE:
      sprintf(temp, "%s", "Home");
      break;
    case HCONTROL_INITIAL_GUILD:
      sprintf(temp, "%s", "GuildHall");
      break;
    default:
      sprintf(temp, "%s", "????");
    }
    sprintf(buff, "%5d     %s    %5d    %14lu\r\n", cur_rec->vnum, temp,
	    cur_rec->location,
	    (upgrade_times[cur_rec->type] - (time(0) - cur_rec->time) / SECS_PER_MUD_HOUR));
    send_to_char(buff, ch);
    cur_rec = cur_rec->next;
  }
}

void process_construction_q()
{
  struct house_upgrade_rec *cur_rec, *prev_rec;
  struct house_control_rec *house = NULL, *house2 = NULL;
  int flag, temp;
  char golem_block = 'A';
  char buff[MAX_STRING_LENGTH];
  struct char_data *golem;
  struct obj_data *temp_obj, *entrance_obj;

  cur_rec = house_upgrade_list;
  prev_rec = NULL;

  while (cur_rec) {
    flag = 0;
    if (mini_mode || (time(0) - cur_rec->time) > (upgrade_times[cur_rec->type] * SECS_PER_MUD_HOUR)) {
      switch (cur_rec->type) {	/* time to build */
      case HCONTROL_ADD_ROOM:
	house = find_house(cur_rec->vnum);
	if (house) {
	  temp = real_room0(hcontrol_setup_room(house, cur_rec->location, cur_rec->exit_dir));
	  if (!temp)
	    return;
	  if (cur_rec->door && cur_rec->door_keyword) {		/* setup door */
	    if (world[real_room0(cur_rec->location)].dir_option[cur_rec->exit_dir]) {
	      SET_BIT(world[real_room0(cur_rec->location)].dir_option[cur_rec->exit_dir]->exit_info, EX_ISDOOR);
	      world[real_room0(cur_rec->location)].dir_option[cur_rec->exit_dir]->keyword = strdup(cur_rec->door_keyword);
	    }
	    if (world[real_room0(temp)].dir_option[rev_dir[cur_rec->exit_dir]]) {
	      SET_BIT(world[real_room0(temp)].dir_option[rev_dir[cur_rec->exit_dir]]->exit_info, EX_ISDOOR);
	      world[temp].dir_option[rev_dir[cur_rec->exit_dir]]->keyword = strdup(cur_rec->door_keyword);
	    }
	  }			/* setup door */
	  house->num_of_rooms++;
	  writeHouse(house);
	}			/* if house */
	break;
      case HCONTROL_NAME_ROOM:
	house = find_house(cur_rec->vnum);
	if (house) {
	  world[real_room0(cur_rec->location)].name =
	      strdup(cur_rec->door_keyword);
	  writeHouse(house);
	}
	break;
      case HCONTROL_DESC_ROOM:
	house = find_house(cur_rec->vnum);
	if (house && cur_rec->door_keyword) {
	  world[real_room0(cur_rec->location)].description =
	      strdup(cur_rec->door_keyword);
	  writeHouse(house);
	}
	break;
      case HCONTROL_ADD_SECRET:
	house = find_house(cur_rec->vnum);
	if (house) {
	  house->secret_entrance = 1;
	  writeHouse(house);
	}
	break;
      case HCONTROL_ADD_CLERIC_GOLEM:
	house = find_house(cur_rec->vnum);
	if (house) {
	  if ((house->cleric_golems +
	       house->warrior_golems +
	       house->wizard_golems) == 4)
	    break;
	  house->cleric_golems++;
	  writeHouse(house);
	  golem = read_mobile(CLERIC_GOLEM_VNUM, VIRTUAL);
	  if (!golem)
	    break;
	  if (house->exit_num == -1)
	    golem_block = 'a';
	  else
	    golem_block = *dirs[house->exit_num];
	  sprintf(buff, "golem assoc%d_%c_0", house->owner_guild, golem_block);
	  golem->player.name = strdup(buff);
	  char_to_room(golem, real_room0(house->room_vnums[0]), 0);
	}
	break;
      case HCONTROL_ADD_WARRIOR_GOLEM:
	house = find_house(cur_rec->vnum);
	if (!house)
	  break;
	if ((house->cleric_golems +
	     house->warrior_golems +
	     house->wizard_golems) == 4)
	  break;
	house->warrior_golems++;
	writeHouse(house);
	golem = read_mobile(WARRIOR_GOLEM_VNUM, VIRTUAL);
	if (!golem)
	  break;
	if (house->exit_num == -1)
	  golem_block = 'a';
	else
	  golem_block = *dirs[house->exit_num];
	sprintf(buff, "golem assoc%d_%c_0", house->owner_guild, golem_block);
	golem->player.name = strdup(buff);
	char_to_room(golem, real_room0(house->room_vnums[0]), 0);
	break;
      case HCONTROL_ADD_MAGE_GOLEM:
	house = find_house(cur_rec->vnum);
	if (house) {
	  if ((house->cleric_golems +
	       house->warrior_golems +
	       house->wizard_golems) == 4)
	    break;
	  house->wizard_golems++;
	  writeHouse(house);
	  golem = read_mobile(MAGE_GOLEM_VNUM, VIRTUAL);
	  if (!golem)
	    break;
	  if (house->exit_num == -1)
	    golem_block = 'a';
	  else
	    golem_block = *dirs[house->exit_num];
	  sprintf(buff, "golem assoc%d_%c_0", house->owner_guild, golem_block);
	  golem->player.name = strdup(buff);
	  char_to_room(golem, real_room0(house->room_vnums[0]), 0);
	}
	break;
      case HCONTROL_ADD_INN:
	house = find_house(cur_rec->vnum);
	if (house) {
	  if (house->inn_vnum)
	    break;
	  house->inn_vnum = cur_rec->location;
	  world[real_room0(house->inn_vnum)].funct = inn;
	  writeHouse(house);
	}
	break;
      case HCONTROL_ADD_FOUNTAIN:
	house = find_house(cur_rec->vnum);
	if (house) {
	  if (house->fountain_vnum)
	    break;
	  house->fountain_vnum = cur_rec->location;
	  writeHouse(house);
	  temp_obj = read_object(FOUNTAIN_VNUM, VIRTUAL);
	  if (temp_obj) {
	    obj_to_room(temp_obj, real_room0(cur_rec->location));
	  }
	}
	break;
      case HCONTROL_ADD_HOLY:
	house = find_house(cur_rec->vnum);
	if (house) {
	  if (house->holy_fount_vnum)
	    break;
	  house->holy_fount_vnum = cur_rec->location;
	  writeHouse(house);
	  temp_obj = read_object(GUILD_HOLY_FOUNT, VIRTUAL);
	  if (temp_obj)
	    obj_to_room(temp_obj, real_room0(cur_rec->location));
	}
	break;
      case HCONTROL_ADD_UNHOLY:
	house = find_house(cur_rec->vnum);
	if (house) {
	  if (house->unholy_fount_vnum)
	    break;
	  house->unholy_fount_vnum = cur_rec->location;
	  writeHouse(house);
	  temp_obj = read_object(GUILD_UNHOLY_FOUNT, VIRTUAL);
	  if (temp_obj)
	    obj_to_room(temp_obj, real_room0(cur_rec->location));
	}
	break;
      case HCONTROL_ADD_HEAL:
	house = find_house(cur_rec->vnum);
	if (house) {
	  if (house->heal_vnum)
	    break;
	  house->heal_vnum = cur_rec->location;
	  writeHouse(house);
	  SET_BIT(world[real_room0(cur_rec->location)].room_flags, HEAL);
	  break;
	}
	break;
      case HCONTROL_ADD_SHOP:
	house = find_house(cur_rec->vnum);
	if (house) {
	  stock_guild_shop(cur_rec->location);
	  house->shop_vnum = cur_rec->location;
	  writeHouse(house);
	}
	break;
      case HCONTROL_ADD_CHEST:
	house = find_house(cur_rec->vnum);
	temp_obj = read_object(CHEST_VNUM, VIRTUAL);
	if (house && temp_obj) {
	  obj_to_room(temp_obj, real_room0(cur_rec->location));
	  writeHouse(house);
	}
	break;
      case HCONTROL_ADD_CABINET:
	house = find_house(cur_rec->vnum);
	temp_obj = read_object(CABINET_VNUM, VIRTUAL);
	if (house && temp_obj) {
	  obj_to_room(temp_obj, real_room0(cur_rec->location));
	  writeHouse(house);
	}
	break;
      case HCONTROL_ADD_MOUTH:
	house = find_house(cur_rec->vnum);
	temp_obj = read_object(MOUTH_VNUM, VIRTUAL);
	if (house && temp_obj) {
	  if (house->mouth_vnum)
	    break;
	  house->mouth_vnum = cur_rec->location;
	  writeHouse(house);
	  temp_obj->value[0] = house->owner_guild;
	  obj_to_room(temp_obj, real_room0(cur_rec->location));
	}
	break;
      case HCONTROL_INITIAL_HOUSE:
	house = find_house(cur_rec->vnum);
	if (house) {
	  if ((entrance_obj = read_object(HOUSE_OUTER_DOOR, VIRTUAL))) {
	    entrance_obj->str_mask = (STRUNG_KEYS | STRUNG_DESC1);
	    entrance_obj->name = strdup(house->owner);
	    sprintf(buff, "A door leading into %s's house is here.", house->owner);
	    entrance_obj->description = strdup(buff);
	    entrance_obj->value[0] = house->room_vnums[0];
	    nuke_doorways(real_room0(house->vnum));
	    obj_to_room(entrance_obj, real_room0(house->vnum));
	  }
	  send_to_room("Construction of a new home completed.\r\n", real_room0(house->vnum));
	  house->owner_guild = 0;
	  house->upgrades = 0;
	  house->inn_vnum = house->room_vnums[0];
	  world[real_room0(house->inn_vnum)].funct = inn;
	  house->construction = 0;
	  writeHouse(house);
	}
	break;
      case HCONTROL_INITIAL_GUILD:
	house = find_house(cur_rec->vnum);
	if (house) {
	  if ((entrance_obj = read_object(HOUSE_OUTER_DOOR, VIRTUAL))) {
	    entrance_obj->str_mask = (STRUNG_KEYS);
	    sprintf(buff, "door guild building %s", house->owner);
	    entrance_obj->name = strdup(buff);
	    entrance_obj->value[0] = house->room_vnums[0];
	    nuke_doorways(real_room0(house->vnum));
	    obj_to_room(entrance_obj, real_room0(house->vnum));
	  }
	  house->owner_guild = cur_rec->guild;
	  send_to_room("Construction of a new guildhall completed.\r\n", real_room0(house->vnum));
	  house->upgrades = 0;
	  house->construction = 0;
	  writeHouse(house);
	}
	break;
      case HCONTROL_ADD_TELEPORTER:
	house = find_house(cur_rec->vnum);
	house2 = find_house(cur_rec->exit_dir);
	if (house->teleporter1_room && house->teleporter2_room)
	  break;
	if (house2->teleporter2_room && house2->teleporter2_room)
	  break;
	if (house->teleporter1_room) {
	  house->teleporter2_room = cur_rec->location;
	  house->teleporter2_dest = cur_rec->door;
	} else {
	  house->teleporter1_room = cur_rec->location;
	  house->teleporter1_dest = cur_rec->door;
	}
	writeHouse(house);
	if (house2->teleporter1_room) {
	  house2->teleporter2_room = cur_rec->door;
	  house2->teleporter2_dest = cur_rec->location;
	} else {
	  house2->teleporter1_room = cur_rec->door;
	  house2->teleporter1_dest = cur_rec->location;
	}
	writeHouse(house2);
	temp_obj = read_object(TELEPORTER_VNUM, VIRTUAL);
	if (!temp_obj)
	  break;
	temp_obj->value[0] = cur_rec->door;
	obj_to_room(temp_obj, real_room0(cur_rec->location));
	temp_obj = read_object(TELEPORTER_VNUM, VIRTUAL);
	temp_obj->value[0] = cur_rec->location;
	obj_to_room(temp_obj, real_room0(cur_rec->door));
	break;
      }				/* switch */
      /* count the upgrades */
      if (house)
	house->upgrades += 1;
      if (house2)
	house2->upgrades += 1;
      /* now delete this rec from Q */
      flag = 1;			/* don't advance cur_rec */
      if (prev_rec == NULL) {	/* first rec */
	prev_rec = cur_rec;
	cur_rec = cur_rec->next;
	house_upgrade_list = cur_rec;
	mm_release(dead_construction_pool, prev_rec);
	prev_rec = NULL;
      } else {
	prev_rec->next = cur_rec->next;
	mm_release(dead_construction_pool, cur_rec);
	cur_rec = prev_rec->next;
      }
      writeConstructionQ();
    }				/* if time to build */
    if (!flag) {
      prev_rec = cur_rec;
      cur_rec = cur_rec->next;
    }
  }
  writeConstructionQ();
}

void clear_sacks(struct char_data *ch)
{
  struct house_control_rec *house;
  struct house_sack_rec *sack_rec;

  house = first_house;
  while (house) {
    sack_rec = house->sack_list;
    while (sack_rec) {
      if (!strcmp(sack_rec->name, GET_NAME(ch))) {
	sack_rec->name = strdup("xxxx");
      }
      sack_rec = sack_rec->next;
    }
    house = house->next;
  }
}

/* sack commands */
void do_sack(struct char_data *ch, char *arg, int cmd)
{
  struct house_control_rec *house;
  struct char_data *mob;
  struct house_sack_rec *sack_rec;
  char buf[256];

  arg = one_argument(arg, buf);
  if ((buf[0] != '\000') &&
      is_abbrev(buf, "off")) {
    send_to_char("You stop your rampage.\r\n", ch);
    REMOVE_BIT(ch->specials.affected_by4, AFF4_SACKING);
    clear_sacks(ch);
    return;
  }
  if ((buf[0] != '\000') &&
      IS_TRUSTED(ch) &&
      is_abbrev(buf, "show")) {
    house = first_house;
    send_to_char("House     Sacker     Time Left\r\n", ch);
    while (house) {
      sack_rec = house->sack_list;
      while (sack_rec) {
	sprintf(buf, "%d    %s       %d\r\n", house->vnum, sack_rec->name, (int) (168 *
										  SECS_PER_MUD_HOUR - (time(0) - sack_rec->time)) / SECS_PER_MUD_HOUR);
	send_to_char(buf, ch);
	sack_rec = sack_rec->next;
      }
      house = house->next;
    }
    return;
  }
  house = house_ch_is_in(ch);
  if (!house) {
    send_to_char("Sack what? You're not in a guildhall or castle!\r\n", ch);
    return;
  }
  if (GET_A_NUM(ch) == house->owner_guild) {
    send_to_char("Sack your own property!  You can't be that bored.\r\n", ch);
    return;
  }
  if (IS_AFFECTED4(ch, AFF4_SACKING)) {
    send_to_char("You're already sacking something.\r\n", ch);
    send_to_char("Use 'sack off' to stop.\r\n", ch);
    return;
  }
  if (world[ch->in_room].number == house->room_vnums[0]) {
    send_to_char("You need to move slightly farther in to begin sacking.\r\n", ch);
    return;
  }
  for (mob = world[real_room0(house->room_vnums[0])].people; mob; mob = mob->next_in_room) {
    if (isname("golem", GET_NAME(mob))) {
      send_to_char("The golems might have something to say about that, go kill em.\r\n", ch);
      return;
    }
  }
  /* valid sack command */
  SET_BIT(ch->specials.affected_by4, AFF4_SACKING);
  CREATE(sack_rec, struct house_sack_rec, 1);
  sack_rec->name = strdup(GET_NAME(ch));
  sack_rec->time = time(0);
  sack_rec->next = house->sack_list;
  house->sack_list = sack_rec;

  send_to_char("&+RRAMPAGE!  DESTROY!&N You are now sacking this hall.\r\n", ch);

  logit(LOG_HOUSE, "House %d started sacking by %s", house->vnum, GET_NAME(ch));
}

struct char_data *get_char_online(char *name)
{
  struct char_data *i;

  for (i = character_list; i; i = i->next)
    if (isname(name, GET_NAME(i)))
      return i;
  return 0;
}
void nuke_portal(int rnum)
{

  struct obj_data *obj;

  obj = get_obj_in_list_num(real_object(TELEPORTER_VNUM), world[rnum].contents);
  if (!obj)
    return;
  obj_from_room(obj);
  extract_obj(obj, TRUE);
}

void nuke_doorways(int rnum)
{

  struct obj_data *obj = NULL;

  obj = get_obj_in_list_num(real_object(HOUSE_INNER_DOOR), world[rnum].contents);
  if (obj) {
    obj_from_room(obj);
    extract_obj(obj, TRUE);
  }
  /* dont second guess, just look for and yank any type */
  obj = get_obj_in_list_num(real_object(HOUSE_OUTER_DOOR), world[rnum].contents);
  if (obj) {
    obj_from_room(obj);
    extract_obj(obj, TRUE);
  }
  return;
}

void sack_house(struct house_control_rec *house, struct char_data *ch)
{
  struct house_sack_rec *sack_rec, *prev;
  int temp1, temp2;
  /* struct obj_data *tobj;  for finding objects to be nuked */
  struct house_control_rec *thouse;
  /* When house is sacked we're going to:
     1) remove all golems 
     2) set guild_no of house to new guild
     3) clear sacking list completely
     4) remove teleporter objects, and teleporters
     5) remove board
     6) remove magic mouth
   */
  house->warrior_golems = 0;
  house->wizard_golems = 0;
  house->cleric_golems = 0;
/* first teleporter */
  if (house->teleporter1_room) {
    temp1 = house->teleporter1_room;
    temp2 = house->teleporter1_dest;
    nuke_portal(real_room0(temp1));
    nuke_portal(real_room0(temp2));
    thouse = get_house_from_room(real_room0(temp2));
    if (thouse) {		/* should always be true, but just in case */
      if (thouse->teleporter1_room == temp2) {
	thouse->teleporter1_room = 0;
	thouse->teleporter1_dest = 0;
      } else {
	thouse->teleporter2_room = 0;
	thouse->teleporter2_dest = 0;
      }
      writeHouse(thouse);
    }
    house->teleporter1_room = 0;
    house->teleporter1_dest = 0;
  }
/* second teleporter */
  if (house->teleporter2_room) {
    temp1 = house->teleporter2_room;
    temp2 = house->teleporter2_dest;
    nuke_portal(real_room0(temp1));
    nuke_portal(real_room0(temp2));
    thouse = get_house_from_room(real_room0(temp2));
    if (thouse) {
      if (thouse->teleporter1_room == temp2) {
	thouse->teleporter1_room = 0;
	thouse->teleporter1_dest = 0;
      } else {
	thouse->teleporter2_room = 0;
	thouse->teleporter2_dest = 0;
      }
      writeHouse(thouse);
    }
    house->teleporter2_room = 0;
    house->teleporter2_dest = 0;
  }
/* mouth */
  if (house->mouth_vnum) {
    house->mouth_vnum = 0;
  }
/* board */

  sack_rec = house->sack_list;
  house->sack_list = 0;
  while (sack_rec) {
    prev = sack_rec->next;
    free((char *) sack_rec);
    sack_rec = prev;
  }
  house->owner_guild = GET_A_NUM(ch);
  writeHouse(house);
  return;
}

void check_sacks(struct house_control_rec *house)
{
  struct house_sack_rec *sack_rec, *prev;
  struct char_data *ch;
  struct house_control_rec *thouse;
  struct descriptor_data *i;
  char buf[128];

  if (!house)
    return;			/* howd we get here? */
  if (!house->sack_list)
    return;			/* nothing going on */

  sack_rec = house->sack_list;
  prev = NULL;

  while (sack_rec) {
    /* okay, check to make sure player is still in the house */
    ch = get_char_online(sack_rec->name);
    if (!ch || !IS_AFFECTED4(ch, AFF4_SACKING)) {	/* not online or not sacking */
      if (prev) {
	prev->next = sack_rec->next;
	free((char *) sack_rec);
	sack_rec = prev->next;
      } else {			/* delete first */
	house->sack_list = sack_rec->next;
	free((char *) sack_rec);
	sack_rec = house->sack_list;
      }
      break;
    }
    thouse = house_ch_is_in(ch);
    if (thouse != house) {	/* left hosue */
      REMOVE_BIT(ch->specials.affected_by4, AFF4_SACKING);
      clear_sacks(ch);
      send_to_char("How can you sack something when you're not there?\r\n", ch);
      break;
    }
    /* if been there 168 mud hours (a week) SACK IT */
    send_to_char("&+RYou continue your RAMPAGE.&N\r\n", ch);
    sprintf(buf, "&+CGuild Alert! %s is SACKING the hall!\r\n", sack_rec->name);
    logit(LOG_HOUSE, "%s sacking hall %d", sack_rec->name, house->vnum);
    for (i = descriptor_list; i; i = i->next)
      if (!i->connected &&
	  !is_silent(i->character, TRUE) &&
	  IS_SET(i->character->specials.act, PLR_GCC) &&
	  (GET_A_NUM(i->character) == house->owner_guild))
	send_to_char(buf, i->character);

    if ((time(0) - sack_rec->time) > (164 * SECS_PER_MUD_HOUR)) {
      sack_house(house, ch);
      REMOVE_BIT(ch->specials.affected_by4, AFF4_SACKING);
      clear_sacks(ch);
      logit(LOG_PLAYER, "%s sacked hall %d", GET_NAME(ch), house->vnum);
      send_to_char("Sacking complete, welcome home!\r\n", ch);
      return;
    }
    prev = sack_rec;
    sack_rec = sack_rec->next;
  }
}

/* timed housing calls */

void do_housekeeping(void)
{
  struct house_control_rec *house;

  if (!current_event)
    return;
  process_construction_q();
  house = first_house;
  while (house) {
    check_sacks(house);
    house = house->next;
  }
  current_event->timer = 1;
}

void do_construct_house(struct char_data *ch, char *arg)
{
  int cost;
  char buf[MAX_STRING_LENGTH];
  struct house_upgrade_rec *con_rec;
  struct house_control_rec *house;

  if (!valid_build_location(ch->in_room, ch, HCONTROL_HOUSE))
    return;

  /* one per char */
  for (house = first_house; house; house = house->next)
    if (house->owner && !str_cmp(house->owner, J_NAME(ch))) {
      send_to_char("You already own a house. Sell your other one before buying another.\r\n", ch);
      return;
    }
  if (find_house(world[ch->in_room].number)) {
    send_to_char("The area is already crowded enough with buildings as it is.\r\n", ch);
    return;
  }
  /* check construction queue as well */

  con_rec = house_upgrade_list;

  while (con_rec) {
    if ((con_rec->vnum == world[ch->in_room].number) &&
	((con_rec->type == HCONTROL_INITIAL_HOUSE) ||
	 (con_rec->type == HCONTROL_INITIAL_GUILD))) {
      send_to_char("The area is already crowded enough with buildings as it is.\r\n", ch);
      return;
    }
    con_rec = con_rec->next;
  }

  cost = house_cost(ch);

  if (!charge_char(ch, cost))
    return;

  con_rec = get_con_rec();

  con_rec->vnum = world[ch->in_room].number;
  con_rec->type = HCONTROL_INITIAL_HOUSE;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->door_keyword = strdup(J_NAME(ch));
  con_rec->next = house_upgrade_list;

  house_upgrade_list = con_rec;

  sprintf(buf, "house %s n 0", J_NAME(ch));

  hcontrol_build_house(ch, buf);
  send_to_char("House construction begun - this is going to take a while.\r\n", ch);

  return;
}

void do_construct_guild(struct char_data *ch, char *arg)
{
  int cost, temp1, temp2;
  char buf[MAX_STRING_LENGTH];
  struct house_upgrade_rec *con_rec;

  temp1 = GET_A_NUM(ch);
  temp2 = GET_A_BITS(ch);
  if (!IS_MEMBER(temp2) || !temp1) {
    send_to_char("Sorry, only guildmembers may initiate construction.\r\n", ch);
    return;
  }
  if (!GT_NORMAL(temp2)) {
    send_to_char("You need to be a senior member to order new construction!\r\n", ch);
    return;
  }
  if (!valid_build_location(ch->in_room, ch, HCONTROL_GUILD))
    return;

  if (find_house(world[ch->in_room].number)) {
    send_to_char("The area is already crowded enough with buildings as it is.\r\n", ch);
    return;
  }
  /* check construction queue as well */

  con_rec = house_upgrade_list;

  while (con_rec) {
    if ((con_rec->vnum == world[ch->in_room].number) &&
	((con_rec->type == HCONTROL_INITIAL_HOUSE) ||
	 (con_rec->type == HCONTROL_INITIAL_GUILD))) {
      send_to_char("The area is already crowded enough with buildings as it is.\r\n", ch);
      return;
    }
    con_rec = con_rec->next;
  }

  cost = guild_cost(ch);

  if (!charge_char(ch, cost))
    return;

  con_rec = get_con_rec();

  con_rec->vnum = world[ch->in_room].number;
  con_rec->type = HCONTROL_INITIAL_GUILD;
  con_rec->time = time(0);
  con_rec->location = world[ch->in_room].number;
  con_rec->guild = GET_A_NUM(ch);
  con_rec->door_keyword = strdup(J_NAME(ch));
  con_rec->next = house_upgrade_list;

  house_upgrade_list = con_rec;

  sprintf(buf, "guild %s n %d", J_NAME(ch), GET_A_NUM(ch));

  hcontrol_build_house(ch, buf);
  send_to_char("GuildHall construction begun - this is going to take a while.\r\n", ch);

  return;
}

void do_stathouse(struct char_data *ch, char *argument, int cmd)
{
  return;
}

void open_market_house(void)
{
  struct house_control_rec *house;
  struct char_data *ch;
  char buf[MAX_STRING_LENGTH];
  int price = 0, room;

  if (!current_event || (current_event->type != EVENT_SPECIAL)) {
    logit(LOG_EXIT, "Call to open_market_house() with invalid event");
    raise(SIGSEGV);
  }
  /* fathom our location */
  room = atoi((char *) current_event->target.t_arg);
  if (room <= 0) {
    logit(LOG_EXIT, "open_market_house() in invalid room.");
    raise(SIGSEGV);
  }
  if ((house = find_house(room))) {
    ch = get_char_online(house->owner);
    if (ch) {
      /* random price */
      price = number(10, 1000);
      /* convert to plat */
      price *= 1000;
      /* a bit more if it was a guildhall */
      if (house->type == HCONTROL_GUILD)
	price *= 3;
      /* and castles too */
      if (house->type == HCONTROL_CASTLE)
	price *= 5;
      sprintf(buf, "Your building has been sold! You receive %s for it.\r\n", coin_stringv(price));
      send_to_char(buf, ch);
      ADD_MONEY(ch, price);
    }
  }
  /* kill the house and file */
  sprintf(buf, "%d", room);
  hcontrol_destroy_house(NULL, buf);

  return;
}

char *kingdom_num_name(int kingdom_type, int kingdom_num)
{
  static char buf[MAX_STRING_LENGTH];
  int i;
  FILE *f;

  switch (kingdom_type) {
  case KINGDOM_TYPE_TOWN:
    sprintf(buf, "%s", town_name_list[kingdom_num]);
    break;
  case KINGDOM_TYPE_PC:
    sprintf(buf, "%sasc.%d", ASC_DIR, i);
    f = fopen(buf, "r");
    if (f) {
      fgets(buf, MAX_STR_NORMAL, f);
      strcat(buf, "&n");
      fclose(f);
    }
    break;
  case KINGDOM_TYPE_NPC:
    sprintf(buf, "NPC Owned");
    break;
  default:
    sprintf(buf, "None");
    break;
  }
  return buf;
}

void update_kingdoms()
{
  struct house_control_rec *house = NULL;
  FILE *f;
  char Gbuf1[MAX_STR_NORMAL];
  char Gbuf2[MAX_STR_NORMAL];
  char buf[ASC_SAVE_SIZE];
  int i, dummy2, dummy3, dummy4, dummy5, pc, gc, sc, cc;

  return;			/* not ready yet */

  if (!first_house)
    return;

  for (house = first_house; house; house = house->next) {
    if (house->type != HCONTROL_CASTLE)
      continue;

    /* 1: collect taxes */
    /* 1a: Determine amount earned */
    pc = gc = sc = cc = 0;
    /* 1b: open association file */
    sprintf(Gbuf1, "%sasc.%u", ASC_DIR, house->owner_guild);
    f = fopen(Gbuf1, "r");
    if (!f)
      continue;
    /* 1c: goto cash, copy on the way */
    fgets(Gbuf2, MAX_STR_NORMAL, f);
    strcpy(buf, Gbuf2);
    for (i = 0; i < 9; i++) {
      fgets(Gbuf2, MAX_STR_NORMAL, f);
      strcat(buf, Gbuf2);
    }
    /* 1d: update association cash */
    sprintf(Gbuf2, "%i %i %i %i\n", dummy2 + pc, dummy3 + gc, dummy4 + sc, dummy5 + cc);
    strcat(buf, Gbuf2);
    /* 1e: copy past members */
    while (fgets(Gbuf2, MAX_STR_NORMAL, f))
      strcat(buf, Gbuf2);
    fclose(f);
    /* 1f: now write the buffer with money changes */
    f = fopen(Gbuf1, "w");
    if (!f)
      continue;
    fwrite(buf, strlen(buf), 1, f);
    fclose(f);

    /* 2: charge maintenance costs */
    /* 2a: determine amount to charge */
    pc = gc = sc = cc = 0;
    /* 2b: open association file */
    sprintf(Gbuf1, "%sasc.%u", ASC_DIR, house->owner_guild);
    f = fopen(Gbuf1, "r");
    if (!f)
      continue;
    /* 2c: goto cash, copy on the way */
    fgets(Gbuf2, MAX_STR_NORMAL, f);
    strcpy(buf, Gbuf2);
    for (i = 0; i < 9; i++) {
      fgets(Gbuf2, MAX_STR_NORMAL, f);
      strcat(buf, Gbuf2);
    }
    /* 2d: check if association has the money */
    fscanf(f, "%i %i %i %i\n", &dummy2, &dummy3, &dummy4, &dummy5);
    if (dummy2 < pc || dummy3 < gc || dummy4 < sc || dummy5 < cc) {
      fclose(f);
      continue;
    }
    /* 2e: update association cash */
    sprintf(Gbuf2, "%i %i %i %i\n", dummy2 - pc, dummy3 - gc, dummy4 - sc, dummy5 - cc);
    strcat(buf, Gbuf2);
    /* 2f: copy past members */
    while (fgets(Gbuf2, MAX_STR_NORMAL, f))
      strcat(buf, Gbuf2);
    fclose(f);
    /* 2g: now write the buffer with money changes */
    f = fopen(Gbuf1, "w");
    if (!f)
      continue;
    fwrite(buf, strlen(buf), 1, f);
    fclose(f);

    /* utilize resources */
    /* check/increase size */
    /* write house file */
    writeHouse(house);
  }
  return;
}

void check_for_kingdom_trespassing(struct char_data *ch)
{

  /* If room is kingdom owned, check if ch is a member */
  /* send troops if not */
  return;
}

void print_troop_name_to_char(struct char_data *ch, int type, int level, int offense, int defense, int plural)
{
  char Gbuf1[MAX_STRING_LENGTH];

  if (!ch)
    return;

  sprintf(Gbuf1, "%s %s%s with %s weaponry and %s protection",
	  troop_levels[level], troop_types[type],
	  (plural && type != TROOP_CAVALRY) ? "'s" : "",
	  troop_offense[offense], troop_defense[defense]);
  send_to_char(Gbuf1, ch);
}

void print_decree_usage(struct char_data *ch)
{
  char Gbuf1[MAX_STRING_LENGTH];
  char Gbuf2[MAX_STRING_LENGTH];

  if (!ch)
    return;

  sprintf(Gbuf1, "&+RDecree&n\r\n"
	  "&+r======&n\r\n");
  send_to_char(Gbuf1, ch);

  if (IS_TRUSTED(ch))
    sprintf(Gbuf2, "&+RUsage:&n decree <asc_num> <&+Moption&n> ...\r\n"
	    "&+r------------------------------------&n\r\n");
  else
    sprintf(Gbuf2, "&+RUsage:&n decree <&+Moption&n> ...\r\n"
	    "&+r--------------------------&n\r\n");

  sprintf(Gbuf1, Gbuf2);

  sprintf(Gbuf2, "&+mOptions:&n\r\n"
	  "<&+Macquire&n>   Acquire new land\r\n"
	  "<&+Mpurchase&n>  Purchase kingdom components\r\n"
	  "<&+Mschedule&n>  Display scheduled purchases/troop events\r\n"
	  "<&+Mstatus&n>    Display quick kingdom status\r\n"
	  "<&+Mtroopinfo&n> Display cummulative troop information\r\n");

  strcat(Gbuf1, Gbuf2);
  if (IS_TRUSTED(ch)) {
    sprintf(Gbuf2, "&+cAdministrative Options:&n\r\n"
	  "<&+Mliberate&n>  Relinquishes kingdom control from room\r\n");
    strcat(Gbuf1, Gbuf2);
  }
  send_to_char(Gbuf1, ch);
}

/* This function checks to see if the guild can afford a charge
 * in platinum. If it can and charge_guild is TRUE then it will
 * be deducted from the guild account. If guild_charge is FALSE
 * then it will only check to see if the guild can afford it, no
 * charge applied.
 */

int can_afford_charge(int assoc_num, int cost, int charge_guild)
{
  char Gbuf1[MAX_STR_NORMAL];
  char Gbuf2[MAX_STR_NORMAL];
  char buf[ASC_SAVE_SIZE];
  int pl, go, si, co, i, tally;
  FILE *f;

  sprintf(Gbuf1, "%sasc.%u", ASC_DIR, assoc_num);
  f = fopen(Gbuf1, "r");
  if (!f) {
    wizlog(57, "The association file for assoc %d is busted!\r\n", assoc_num);
    return;
  }
  fgets(Gbuf2, MAX_STR_NORMAL, f);
  strcpy(buf, Gbuf2);
  for (i = 0; i < 9; i++) {
    fgets(Gbuf2, MAX_STR_NORMAL, f);
    strcat(buf, Gbuf2);
  }
  fscanf(f, "%i %i %i %i\n", &pl, &go, &si, &co);

  /* for the sake of time, just check for plat right now, we can
     use lower coins later */
  /*if(cost <= (pl+(int)(go/10)+(int)(si/100)+(int)(co/1000))) { */
  if (cost <= pl) {
    if (charge_guild) {
      pl -= cost;
      sprintf(Gbuf2, "%i %i %i %i\n", pl, go, si, co);
      strcat(buf, Gbuf2);
      while (fgets(Gbuf2, MAX_STR_NORMAL, f))
	strcat(buf, Gbuf2);
      fclose(f);
      f = fopen(Gbuf1, "w");
      fwrite(buf, strlen(buf), 1, f);
    }
    fclose(f);
    return TRUE;
  } else {
    fclose(f);
    return FALSE;
  }
}

int cost_per_unit(int type, int level, int offense, int defense)
{
  /* TODO: do bounds checking here */
  return troop_costs[type][level] +
      troop_costs[type][defense] +
      troop_costs[type][offense];
}

void order_troop_event_func(void)
{
  int kingdom_num, type, level, offense, defense, amount, god_mode,
   cost_unit, total_cost;
  char name[MAX_STR_NORMAL];

  if (!current_event || (current_event->type != EVENT_ORDER_TROOP)) {
    logit(LOG_EXIT, "order_troop_event crapped out.");
    raise(SIGSEGV);
  }
  sscanf(((const char *) current_event->target.t_arg), "%d %d %d %d %d %d %d %s", &kingdom_num, &type, &level, &offense, &defense, &amount, &god_mode, name);

  if (god_mode) {
    kingdom[kingdom_num].score->virtual_troops[type][level][offense][defense] += amount;
    kingdom[kingdom_num].score->troops_in_barracks[type][level][offense][defense] += amount;
    wizlog(57, "Troops received for kingdom #%d due to %s's request.", kingdom_num, name);
    return;
  }
  cost_unit = cost_per_unit(type, level, offense, defense);

  total_cost = cost_unit * amount;

  if (can_afford_charge(kingdom_num, total_cost, TRUE)) {
    /*TODO: tell kingdom here : messengerboy */
    kingdom[kingdom_num].score->virtual_troops[type][level][offense][defense] += amount;
    kingdom[kingdom_num].score->troops_in_barracks[type][level][offense][defense] += amount;
  } else {
    /* TODO: tell kingdom here: messengerboy */
    wizlog(57, "Troops rejected to kingdom #%d due to lack of funds.", kingdom_num);
  }
}

void do_decree_purchase_troops(struct char_data *ch, char *arg, int kingdom_num, int god_mode)
{
  char Gbuf1[MAX_STRING_LENGTH];
  char Gbuf2[MAX_STR_NORMAL];
  char type[MAX_STR_NORMAL];
  char level[MAX_STR_NORMAL];
  char offense[MAX_STR_NORMAL];
  char defense[MAX_STR_NORMAL];
  char amount[MAX_STR_NORMAL];
  char order_troop[MAX_STR_NORMAL];
  char buf[ASC_SAVE_SIZE];
  char imm[20];
  FILE *f;
  int i, type_pos, level_pos, offense_pos, defense_pos, amount_num = 0,
   cost_unit = 0, total_cost = 0;
  int pl, go, si, co, immediate = FALSE;

  if (!ch)
    return;

  if (arg && *arg) {
    if (god_mode) {
      half_chop(arg, Gbuf1, Gbuf2);
      if (is_abbrev(Gbuf1, "immediate")) {
	immediate = TRUE;
	half_chop(arg, Gbuf1, arg);
      }
    }
    half_chop(arg, type, arg);
    if (arg && *arg)
      half_chop(arg, level, arg);
    if (arg && *arg)
      half_chop(arg, offense, arg);
    if (arg && *arg)
      half_chop(arg, defense, arg);
    if (type && level && offense && defense && *type && *level && *offense && *defense) {
      /* ok, we have all arguments, let's begin checking them */
      for (type_pos = 0; type_pos <= NUM_TROOP_TYPES; type_pos++) {
	if (is_abbrev(type, troop_types[type_pos]))
	  break;
      }
      for (level_pos = 0; level_pos <= NUM_TROOP_LEVELS - 1; level_pos++) {
	if (is_abbrev(level, troop_levels[level_pos]))
	  break;
      }
      for (defense_pos = 0; defense_pos <= NUM_TROOP_DEFENSE; defense_pos++) {
	if (is_abbrev(defense, troop_defense[defense_pos]))
	  break;
      }
      for (offense_pos = 0; offense_pos <= NUM_TROOP_OFFENSE; offense_pos++) {
	if (is_abbrev(offense, troop_offense[offense_pos]))
	  break;
      }
      if (arg && *arg)
	half_chop(arg, amount, arg);
      if (amount && *amount && is_number(amount))
	amount_num = atoi(amount);
      if (type_pos < NUM_TROOP_TYPES && level_pos < NUM_TROOP_LEVELS - 1 &&
	  offense_pos < NUM_TROOP_OFFENSE && defense_pos < NUM_TROOP_DEFENSE) {
	/* syntax has passed! Let's see if they can buy the troops */

	/* calculate cost */
	cost_unit = cost_per_unit(type_pos, level_pos, offense_pos, defense_pos);
	if (!amount_num) {
	  print_troop_name_to_char(ch, type_pos, level_pos, offense_pos, defense_pos, TRUE);
	  sprintf(Gbuf1, " will cost your kingdom %d &+Wplatinum&n per unit.\r\n",
		  cost_unit);
	  send_to_char(Gbuf1, ch);
	  return;
	}
	total_cost = cost_unit * amount_num;

	/* Let's check for the right amount of cash */

	if (god_mode || can_afford_charge(kingdom_num, total_cost, FALSE)) {
	  sprintf(Gbuf1, "Congratulations! You have just purchased %d ", amount_num);
	  send_to_char(Gbuf1, ch);
	  print_troop_name_to_char(ch, type_pos, level_pos, offense_pos, defense_pos, amount_num > 1 ? TRUE : FALSE);
	  sprintf(Gbuf1, " for %d &+Wplatinum&n.\r\n"
	  "Your troops should arrive in 1 week, payment on delivery!\r\n",
		  total_cost);
	  send_to_char(Gbuf1, ch);

	  /* ***START BUY EVENT HERE*** */
	  sprintf(order_troop, "%d %d %d %d %d %d %d %s",
		  kingdom_num, type_pos, level_pos, offense_pos, defense_pos, amount_num, god_mode, GET_NAME(ch));

	  if (immediate)
	    AddEvent(EVENT_ORDER_TROOP, 5, TRUE, order_troop_event_func, order_troop);
	  else
	    AddEvent(EVENT_ORDER_TROOP, (500 * 24 * 7), TRUE, order_troop_event_func, order_troop);
	  return;
	} else {
	  sprintf(Gbuf1, "%d ", amount_num);
	  send_to_char(Gbuf1, ch);
	  print_troop_name_to_char(ch, type_pos, level_pos, offense_pos, defense_pos, amount_num > 1 ? TRUE : FALSE);
	  sprintf(Gbuf1, " will cost your kingdom %d &+Wplatinum&n.\r\n"
		  "&+RYour kingdom is too poor!&n\r\n",
		  total_cost);
	  send_to_char(Gbuf1, ch);
	  return;
	}
      }
    }
  }
  /* command failed due to syntax, print usage */
  if (god_mode)
    sprintf(imm, "[&+Bimmediate&n] ");
  else
    *imm = '\0';
  sprintf(Gbuf1, "&+RDecree Purchase Troops&n\r\n"
	  "&+r======================&n\r\n"
	  "&+RUsage:&n decree purchase troops %s<&+Ltype&n> <&+Clevel&n> <&+Boffense&n> <&+Mdefense&n> [# of troops]&n\r\n"
	  "&+r------------------------------------------------------------------------------&n\r\n"
    "&+yNote: Yout may leave out # of troops to get a price quote.&n\r\n"
	  "&+LTypes available to your kingdom:&n\r\n", imm);
  for (i = 0; i < NUM_TROOP_TYPES; i++) {
    strcat(Gbuf1, " <");
    strcat(Gbuf1, troop_types[i]);
    strcat(Gbuf1, ">\r\n");
  }
  strcat(Gbuf1, "&+CLevels available:&n\r\n");
  for (i = 0; i < NUM_TROOP_LEVELS - 1; i++) {
    strcat(Gbuf1, " <");
    strcat(Gbuf1, troop_levels[i]);
    strcat(Gbuf1, ">\r\n");
  }
  strcat(Gbuf1, "&+BOffense available:&n\r\n");
  for (i = 0; i < NUM_TROOP_OFFENSE; i++) {
    strcat(Gbuf1, " <");
    strcat(Gbuf1, troop_offense[i]);
    strcat(Gbuf1, ">\r\n");
  }
  strcat(Gbuf1, "&+MDefense available:&n\r\n");
  for (i = 0; i < NUM_TROOP_DEFENSE; i++) {
    strcat(Gbuf1, " <");
    strcat(Gbuf1, troop_defense[i]);
    strcat(Gbuf1, ">\r\n");
  }
  send_to_char(Gbuf1, ch);

}

void do_decree_purchase(struct char_data *ch, char *arg, int kingdom_num, int god_mode)
{
  char Gbuf1[MAX_STRING_LENGTH];
  char rest[MAX_STRING_LENGTH];

  if (!ch)
    return;
  if (arg && *arg) {
    half_chop(arg, Gbuf1, rest);
    if (is_abbrev(Gbuf1, "troops")) {
      do_decree_purchase_troops(ch, rest, kingdom_num, god_mode);
      return;
    }
  }
  sprintf(Gbuf1, "&+RDecree Purchase&n\r\n"
	  "&+r===============&n\r\n"
	  "&+RUsage:&n decree purchase <&+Moptions&n> ...\r\n"
	  "&+r-------------------------------------------&n\r\n"
	  "&+mAvailable options:&n\r\n"
	  "<&+Mtroops&n>  Purchase kingdom troops\r\n");
  send_to_char(Gbuf1, ch);
}

void do_decree_troopinfo(struct char_data *ch, char *arg, int kingdom_num, int god_mode)
{
  if (!ch)
    return;

  send_to_char("Decree troopinfo under construction\r\n", ch);

}

void troop_victory(struct troop_info_rec *troop_info)
{

}

int troops_from_room(struct troop_values *troops, int kingdom_num, int room, int destination)
{
  int w, x, y, z, count = 0;
  struct troop_info_rec *troop_info, *d_troop_list;

  if (!kingdom_num)
    return FALSE;
  if (!IS_KINGDOM(kingdom_num))
    return FALSE;
  if (room < 0 || room == NOWHERE || room > top_of_world)
    return FALSE;
  if (!world[room].troop_info || world[room].troop_info->kingdom_num != kingdom_num)
    return FALSE;

  troop_info = world[room].troop_info;
  /* do troops exist? */
  TROOP_LOOP {
    if (troops->troops[w][x][y][z] >
	troop_info->troops[w][x][y][z])
      return FALSE;
  }
  switch (destination) {
  case TROOP_TO_BARRACKS:
    TROOP_LOOP {
      kingdom[kingdom_num].score->troops_in_barracks[w][x][y][z] += troops->troops[w][x][y][z];
      troop_info->troops[w][x][y][z] -= troops->troops[w][x][y][z];
      count += troop_info->troops[w][x][y][z];
    }
    break;
  case TROOP_TO_TRANSIT:
    TROOP_LOOP {
      kingdom[kingdom_num].score->troops_in_transit[w][x][y][z] += troops->troops[w][x][y][z];
      troop_info->troops[w][x][y][z] -= troops->troops[w][x][y][z];
      count += troop_info->troops[w][x][y][z];
    }
    break;
  case TROOP_TO_DEATH:
    TROOP_LOOP {
      kingdom[kingdom_num].score->virtual_troops[w][x][y][z] -= troops->troops[w][x][y][z];
      troop_info->troops[w][x][y][z] -= troops->troops[w][x][y][z];
      count += troop_info->troops[w][x][y][z];
    }
    break;
  default:
    return FALSE;
  }
  /* Check to see if all troops are removed */
  if (!count) {
    /*TODO: remove any acquire events here! */
    if (troop_info->is_fighting) {
      troop_victory(troop_info->is_fighting);
    }
    world[room].troop_info = NULL;
    if (troop_info == deployed_troops) {
      deployed_troops = troop_info->next;
      free(troop_info);
    } else {
      for (d_troop_list = deployed_troops; d_troop_list && d_troop_list->next != troop_info; d_troop_list = d_troop_list->next);
      if (d_troop_list && d_troop_list->next == troop_info) {
	d_troop_list->next = troop_info->next;
	free(troop_info);
      }
    }
  }
  return TRUE;
}

int troops_to_room(struct troop_values *troops, int kingdom_num, int room, int source)
{
  int w, x, y, z;
  struct troop_info_rec *troop_info;

  if (!kingdom_num)
    return FALSE;
  if (!IS_KINGDOM(kingdom_num))
    return FALSE;
  if (room < 0 || room == NOWHERE || room > top_of_world)
    return FALSE;
  if (!IS_MAP_ROOM(room))
    return FALSE;
  if (world[room].troop_info && world[room].troop_info->kingdom_num != kingdom_num)
    return FALSE;

  /* init the troop struct */
  wizlog(57, "Entering troops_to_room...");
  switch (source) {
  case TROOP_FROM_BARRACKS:
    /* do troops exist? */
    TROOP_LOOP {
      if (troops->troops[w][x][y][z] >
	  kingdom[kingdom_num].score->troops_in_barracks[w][x][y][z]) {
	wizlog(57, "amount of troops failed at %d %d %d %d", w, x, y, z);
	return FALSE;
      }
    }
    if (!(world[room].troop_info)) {
      CREATE(troop_info, struct troop_info_rec, 1);
      troop_info->kingdom_num = kingdom_num;
      troop_info->time_of_occupation = 1;
      troop_info->last_movement = 0;
      troop_info->is_acquiring_land = 0;
      troop_info->is_fighting = NULL;
      troop_info->next = deployed_troops;
      troop_info->in_room = room;
      deployed_troops = troop_info;
      world[room].troop_info = troop_info;
      TROOP_LOOP troop_info->troops[w][x][y][z] = 0;
    } else {
      troop_info = world[room].troop_info;
    }
    TROOP_LOOP {
      kingdom[kingdom_num].score->troops_in_barracks[w][x][y][z] -= troops->troops[w][x][y][z];
      troop_info->troops[w][x][y][z] += troops->troops[w][x][y][z];
    }
    wizlog(57, "Troops to room successful");
    return TRUE;
    break;
  case TROOP_FROM_TRANSIT:
    TROOP_LOOP {
      if (troops->troops[w][x][y][z] >
	  kingdom[kingdom_num].score->troops_in_transit[w][x][y][z])
	return FALSE;
    }
    if (!(world[room].troop_info)) {
      CREATE(troop_info, struct troop_info_rec, 1);
      troop_info->kingdom_num = kingdom_num;
      troop_info->time_of_occupation = 0;
      troop_info->last_movement = 0;
      troop_info->is_acquiring_land = 0;
      troop_info->is_fighting = NULL;
      troop_info->next = NULL;
      world[room].troop_info = troop_info;
      TROOP_LOOP troop_info->troops[w][x][y][z] = 0;
    } else {
      troop_info = world[room].troop_info;
    }
    TROOP_LOOP {
      kingdom[kingdom_num].score->troops_in_transit[w][x][y][z] -= troops->troops[w][x][y][z];
      troop_info->troops[w][x][y][z] += troops->troops[w][x][y][z];
    }
    return TRUE;
    break;
  default:
    return FALSE;
  }
  return FALSE;
}

void get_best_combo(int *one, int *two, int pos)
{
  int prod, i, j, k, best = 0, sum;

  prod = (*one) * (*two);
  sum = (*one) + (*two);

  for (i = sum; i >= 0; i--) {
    for (j = (*one); j >= 0; j--) {
      for (k = (*two); k >= 0; k--) {
	if (j + k == i) {
	  best++;
	}
	if (best == pos) {
	  (*one) = j;
	  (*two) = k;
	  return;
	}
      }
    }
  }
}

int deploy_troops(struct char_data *ch, int kingdom_num, int type, int level, int quality, int amount, int task, int report)
{
  struct troop_values troops;
  int count = 0, done = 0, amount_of_type;
  int w, x, y, z, i;

  /* init the troop struct */
  TROOP_LOOP troops.troops[w][x][y][z] = 0;

  wizlog(57, "deploytroops: %s - kingnum=%d type=%d level=%d quality=%d amount=%d task=%d report=%d", GET_NAME(ch), kingdom_num, type, level, quality, amount, task, report);
  /* Do they have the troops available? */
  TROOP_TYPE_LEVEL_LOOP count += kingdom[kingdom_num].score->troops_in_barracks[type][level][y][z];
  if (count < amount) {
    send_to_char("&+RThere are not enough troops of that type in the barracks!&n\r\n", ch);
    return;
  }
  count = 0;
  y = NUM_TROOP_OFFENSE - 1;
  z = NUM_TROOP_DEFENSE - 1;
  switch (task) {
  case TASK_ACQUIRE_LAND:
    for (i = 1; i <= NUM_TROOP_OFFENSE * NUM_TROOP_DEFENSE && !done; i++) {
      get_best_combo(&y, &z, i);
      amount_of_type = kingdom[kingdom_num].score->troops_in_barracks[type][level][y][z];
      count += amount_of_type;
      wizlog(57, "Best combo was %d %d and amount found was %d", y, z, amount_of_type);
      if (count > amount) {
	amount_of_type -= (count - amount);
	done = 1;
      }
      troops.troops[type][level][y][z] += amount_of_type;
      y = NUM_TROOP_OFFENSE - 1;
      z = NUM_TROOP_DEFENSE - 1;
    }
    troops_to_room(&troops, kingdom_num, ch->in_room, TROOP_FROM_BARRACKS);
    break;
  default:
    break;

  }
}

void do_decree_acquire(struct char_data *ch, char *arg, int kingdom_num, int god_mode)
{
  char Gbuf1[MAX_STRING_LENGTH];
  char Gbuf2[MAX_STRING_LENGTH];
  char type[MAX_STR_NORMAL];
  char quality[MAX_STR_NORMAL];
  char amount[MAX_STR_NORMAL];
  char level[MAX_STR_NORMAL];
  int i, j, y, z, count, types_avail = 0;
  int amount_num = 0, type_pos, level_pos, quality_pos = 0;

  if (!ch)
    return;

  if (arg && *arg) {
    if (god_mode) {
      half_chop(arg, Gbuf1, Gbuf2);
      if (is_abbrev(Gbuf1, "confirm")) {
	acquire_land(world[ch->in_room].number, kingdom_num, KINGDOM_TYPE_PC);
	wizlog(GET_LEVEL(ch), "%s just acquired room #%d for kingdom #%d.", GET_NAME(ch), world[ch->in_room].number, kingdom_num);
	return;
      }
    }
    half_chop(arg, level, arg);
    if (arg && *arg)
      half_chop(arg, type, arg);
    if (arg && *arg)
      half_chop(arg, quality, arg);
    if (arg && *arg)
      half_chop(arg, amount, arg);
    if (type && level && quality && *type && *level && *quality) {
      for (type_pos = 0; type_pos <= NUM_TROOP_TYPES; type_pos++) {
	if (is_abbrev(type, troop_types[type_pos]))
	  break;
      }
      for (level_pos = 0; level_pos <= NUM_TROOP_LEVELS; level_pos++) {
	if (is_abbrev(level, troop_levels[level_pos]))
	  break;
      }
      if (is_abbrev(quality, "best"))
	quality_pos = 1;
      if (is_abbrev(quality, "worst"))
	quality_pos = 2;
      if (arg && *arg)
	half_chop(arg, amount, arg);
      if (amount && *amount && is_number(amount))
	amount_num = atoi(amount);
      if (type_pos < NUM_TROOP_TYPES && level_pos < NUM_TROOP_LEVELS && quality_pos && amount_num) {
	/* syntax is OK */
	deploy_troops(ch, kingdom_num, type_pos, level_pos, quality_pos, amount_num, TASK_ACQUIRE_LAND, TO_CHAR);
	return;
      }
    }
  }
  /* command failed due to syntax, print usage */
  sprintf(Gbuf1, "&+RDecree Acquire&n\r\n"
	  "&+r==============&n\r\n");
  if (god_mode) {
    sprintf(Gbuf2, "&+BOptional God Usage:&n decree acquire confirm\r\n");
    strcat(Gbuf1, Gbuf2);
  }
  sprintf(Gbuf2, "&+RUsage:&n decree acquire <&+Llevel&n> <&+Ctype&n> <&+Mquality&n> [# of troops]&n\r\n"
  "&+r------------------------------------------------------------&n\r\n"
	  "&+LTypes available in barracks:&n\r\n");
  strcat(Gbuf1, Gbuf2);
  for (i = 0; i < NUM_TROOP_TYPES; i++) {
    for (j = 0; j < NUM_TROOP_LEVELS; j++) {
      count = 0;
      TROOP_TYPE_LEVEL_LOOP count += kingdom[kingdom_num].score->troops_in_barracks[i][j][y][z];
      if (count) {
	sprintf(Gbuf2, " <&+L%s &+C%s&n> - %3d available\r\n",
		troop_levels[j], troop_types[i], count);
	strcat(Gbuf1, Gbuf2);
	types_avail++;
      }
    }
  }
  if (!types_avail)
    strcat(Gbuf1, " <&+LNONE&n>\r\n");
  strcat(Gbuf1, "&+MQuality:&n\r\n <&+Mbest&n>\r\n <&+Mworst&n>\r\n");
  send_to_char(Gbuf1, ch);
}

void do_decree_status(struct char_data *ch, char *arg, int kingdom_num, int god_mode)
{
  char Gbuf1[MAX_STRING_LENGTH];
  char Gbuf2[MAX_STRING_LENGTH];
  int i, land = 0, total_troops = 0, dep_troops = 0, barracks_troops = 0,
   transit_troops = 0, w, x, y, z;
  struct troop_info_rec *troops;

  if (!ch)
    return;

  for (i = 0; i < NUM_SECT_TYPES; i++)
    land += kingdom[kingdom_num].score->land[i];
  TROOP_LOOP total_troops += kingdom[kingdom_num].score->virtual_troops[w][x][y][z];
  TROOP_LOOP barracks_troops += kingdom[kingdom_num].score->troops_in_barracks[w][x][y][z];
  TROOP_LOOP transit_troops += kingdom[kingdom_num].score->troops_in_transit[w][x][y][z];
  for (troops = deployed_troops; troops; troops = troops->next) {
    if (troops->kingdom_num == kingdom_num) {
      TROOP_LOOP dep_troops += troops->troops[w][x][y][z];
    }
  }
  if (god_mode)
    sprintf(Gbuf1, "Status for Kingdom #%d\r\n"
	    "=======================\r\n", kingdom_num);
  else
    sprintf(Gbuf1, "Kingdom Status\r\n"
	    "==============\r\n");
  sprintf(Gbuf2, "&+YFood&n               : %d\r\n"
	  "&+LOre&n                : %d\r\n"
	  "&+WLast Payment&n       : %d\r\n"
	  "&+GTotal Land&n         : %d\r\n"
	  "&+RTotal Troops&n       : %d\r\n"
	  "&+RTroops in Barracks&n : %d\r\n"
	  "&+RTroops in Transit&n  : %d\r\n"
	  "&+RTroops Deployed&n    : %d\r\n",
	  kingdom[kingdom_num].score->food,
	  kingdom[kingdom_num].score->ore,
	  kingdom[kingdom_num].score->last_payment,
	  land,
	  total_troops,
	  barracks_troops,
	  transit_troops,
	  dep_troops);

  strcat(Gbuf1, Gbuf2);
  send_to_char(Gbuf1, ch);
}

int liberate_land(int room, int kingdom_num, int kingdom_type)
{
  int real_room = 0, append_room = 0, old_kingdom_num = 0;

  /* if not kingdom, no land for you! */
  if (!IS_KINGDOM(kingdom_num))
    return FALSE;

  /* Check if this is a real room and get number */
  if (real_room = real_room0(room)) {
    if (world[real_room].kingdom_type == kingdom_type && world[real_room].kingdom_num == kingdom_num) {
      /* assign the land attribute the new values */
      world[real_room].kingdom_type = 0;
      world[real_room].kingdom_num = 0;
      /* update the land file */
      write_pc_land(FALSE);
      /* adjust kingdom scores */
      update_kingdom_score(kingdom_num, real_room, KINGDOM_LAND_LOSS);
      return TRUE;
    }
  }
  return FALSE;
}

void do_decree_liberate(struct char_data *ch, char *arg, int kingdom_num, int god_mode)
{

  char Gbuf1[MAX_STRING_LENGTH];
  char confirm[MAX_STR_NORMAL];

  if (!ch)
    return;

  if (arg && *arg) {
    half_chop(arg, confirm, arg);
    if (confirm && *confirm) {
      if (!strcmp(confirm, "confirm")) {
	if (world[ch->in_room].kingdom_num == kingdom_num) {
	  liberate_land(world[ch->in_room].number, kingdom_num, KINGDOM_TYPE_PC);
	  wizlog(GET_LEVEL(ch), "%s has liberated room #%d from Kingdom #%d", GET_NAME(ch), world[ch->in_room].number, kingdom_num);
	  sprintf(Gbuf1, "This land has been liberated from Kingdom #%d\r\n", kingdom_num);
	} else {
	  sprintf(Gbuf1, "This land does not belong to kingdom #%d\r\n", kingdom_num);
	}
	send_to_char(Gbuf1, ch);
	return;
      }
    }
  }
  sprintf(Gbuf1, "&+RDecree Liberate&n\r\n"
	  "&+r===============&n\r\n"
	  "&+RUsage:&n decree liberate confirm\r\n"
	  "&+r------------------------------&n\r\n");
  send_to_char(Gbuf1, ch);

}

void do_decree_schedule(struct char_data *ch, char *arg, int kingdom_num, int god_mode)
{
  char Gbuf1[MAX_STRING_LENGTH];
  char Gbuf2[MAX_STRING_LENGTH];
  char name[MAX_STR_NORMAL];
  char cancel[MAX_STR_NORMAL];
  char event[MAX_STR_NORMAL];
  char confirm[MAX_STR_NORMAL];
  int purchases = 0, kingdom_number, type, level, offense, defense,
   amount, god;
  int days, hours, cost;
  P_event ev, e_save;

  if (!ch)
    return;

  if (arg && *arg) {
    half_chop(arg, cancel, arg);
    if (arg && *arg)
      half_chop(arg, event, arg);
    if (arg && *arg)
      half_chop(arg, confirm, arg);
    if (cancel && event && confirm && *cancel && *event && *confirm) {
      if (is_abbrev(cancel, "cancel") && is_number(event) && !strcmp(confirm, "confirm")) {
	FIND_EVENT_TYPE(ev, EVENT_ORDER_TROOP) {
	  sscanf(((const char *) ev->target.t_arg), "%d %d %d %d %d %d %d %s", &kingdom_number, &type, &level, &offense, &defense, &amount, &god, name);
	  if (kingdom_number != kingdom_num)
	    continue;
	  purchases++;
	  if (purchases == atoi(event)) {
	    e_save = current_event;
	    current_event = ev;
	    RemoveEvent();
	    current_event = e_save;
	    if (god_mode)
	      wizlog(GET_LEVEL(ch), "%s just cancelled a troop purchase for Kingdom #%d.", GET_NAME(ch), kingdom_num);
	    send_to_char("&-L&+RPurchase cancelled!!&n\r\n", ch);
	    return;
	  }
	}
	send_to_char("&-L&+RThat event was not found. Check the schedule again.&n\r\n", ch);
	purchases = 0;
      }
    }
  }
  send_to_char("&+RDecree Schedule&n\r\n&+r===============&n\r\n", ch);
  send_to_char("&+RUsage:&n decree schedule cancel <&+Cevent #&n> confirm\r\n", ch);
  send_to_char("&+r-----------------------------------------------&n\r\n", ch);
  send_to_char("&+MPurchase Information:&n\r\n&+m---------------------&n\r\n", ch);
  FIND_EVENT_TYPE(ev, EVENT_ORDER_TROOP) {
    cost = 0;
    sscanf(((const char *) ev->target.t_arg), "%d %d %d %d %d %d %d %s", &kingdom_number, &type, &level, &offense, &defense, &amount, &god, name);
    if (kingdom_number != kingdom_num)
      continue;
    purchases++;
    sprintf(Gbuf2, "&+C%d&n. [&+W%d&n] ", purchases, amount);
    strcpy(Gbuf1, Gbuf2);
    send_to_char(Gbuf1, ch);
    print_troop_name_to_char(ch, type, level, offense, defense, amount > 1 ? TRUE : FALSE);
    cost = cost_per_unit(type, level, offense, defense) * amount;
    days = (int) ((ev->timer * 300) / 12000);
    hours = (int) (((ev->timer * 300) - (days * 12000)) / 500);
    sprintf(Gbuf1, " ordered by &+B%s&n for &+W%d platiunum&n is due in &+Y%d&n &+ydays&n &+Y%d&n &+yhours&n.\r\n",
	    name, cost, days, hours);
    send_to_char(Gbuf1, ch);
  }
  if (!purchases)
    send_to_char("&+LNONE&n\r\n", ch);
  send_to_char("\r\n&+RDeployed Troop Roster&n\r\n&+r---------------------&n\r\n", ch);

  /*TODO: print deployed troop events when exists */
  send_to_char("&+LNONE&n", ch);
}

void do_decree(struct char_data *ch, char *arg, int cmd)
{
  int god_mode = 0;
  ush_int asc_number;
  uint asc_bits;
  char one[MAX_STRING_LENGTH];
  char rest[MAX_STRING_LENGTH];
  char Gbuf1[MAX_STR_NORMAL];
  struct stat statbuf;

  if (!ch || IS_NPC(ch))
    return;

  asc_number = GET_A_NUM(ch);
  asc_bits = GET_A_BITS(ch);

  if (IS_TRUSTED(ch))
    god_mode = 1;
  else if (!IS_KINGDOM(asc_number) || !IS_MEMBER(asc_bits) || !IS_LEADER(asc_bits)) {
    send_to_char("What? You fancy yourself a king? Go swab the floors ya lazy sot!\r\n", ch);
    return;
  }
  /* Ok, we're clear for takeoff */
  if (!arg || !(*arg)) {
    print_decree_usage(ch);
    return;
  }
  half_chop(arg, one, rest);

  if (god_mode) {
    if (is_number(one)) {
      asc_number = (ush_int) atoi(one);
      sprintf(Gbuf1, "%sasc.%u", ASC_DIR, asc_number);
      if (stat(Gbuf1, &statbuf)) {
	send_to_char("That association does not exist!\r\n", ch);
	return;
      }
      if (!IS_KINGDOM(asc_number)) {
	send_to_char("That association is not a kingdom.\r\n", ch);
	return;
      }
      half_chop(rest, one, rest);
      if (!one || !(*one)) {
	send_to_char("What would like to decree for that kingdom?\r\n", ch);
	return;
      }
    } else {
      send_to_char("You must specify an association number: decree <assoc #> ...\r\n", ch);
      return;
    }
  }
  if (is_abbrev(one, "acquire"))
    do_decree_acquire(ch, rest, asc_number, god_mode);
  else if (is_abbrev(one, "purchase"))
    do_decree_purchase(ch, rest, asc_number, god_mode);
  else if (is_abbrev(one, "schedule"))
    do_decree_schedule(ch, rest, asc_number, god_mode);
  else if (is_abbrev(one, "status"))
    do_decree_status(ch, rest, asc_number, god_mode);
  else if (is_abbrev(one, "troopinfo"))
    do_decree_troopinfo(ch, rest, asc_number, god_mode);
  else if (god_mode) {
    if (is_abbrev(one, "liberate"))
      do_decree_liberate(ch, rest, asc_number, god_mode);
    else
      print_decree_usage(ch);
  } else
    print_decree_usage(ch);
}

int get_flat_resource_type(int room)
{
  /* TODO: when land loss occurs, the type of the random stuff
     needs to be known in order to decrement the right value
   */
  switch (world[room].sector_type) {
  case SECT_FIELD:
  case SECT_WATER_SWIM:
  case SECT_UNDRWLD_WATER:
    return FLAT_RESOURCE_FOOD;
    break;
  case SECT_HILLS:
  case SECT_MOUNTAIN:
  case SECT_UNDRWLD_MOUNTAIN:
    return FLAT_RESOURCE_ORE;
    break;
  case SECT_FOREST:
  case SECT_UNDRWLD_WILD:
    if (number(0, 1))
      return FLAT_RESOURCE_FOOD;
    else
      return FLAT_RESOURCE_ORE;
    break;
  default:
    return FLAT_RESOURCE_NONE;
  }
}

void update_kingdom_score(int kingdom_num, int int_arg, int type)
{

  int resource = 0;

  if (!IS_KINGDOM(kingdom_num))
    return;

  switch (type) {
  case KINGDOM_LAND_GAIN:
    kingdom[kingdom_num].score->land[world[int_arg].sector_type]++;
    resource = get_flat_resource_type(int_arg);
    if (resource == FLAT_RESOURCE_FOOD)
      kingdom[kingdom_num].score->food++;
    else if (resource == FLAT_RESOURCE_ORE)
      kingdom[kingdom_num].score->ore++;
    break;
  case KINGDOM_LAND_LOSS:
    kingdom[kingdom_num].score->land[world[int_arg].sector_type]--;
    resource = get_flat_resource_type(int_arg);
    if (resource == FLAT_RESOURCE_FOOD)
      kingdom[kingdom_num].score->food--;
    else if (resource == FLAT_RESOURCE_ORE)
      kingdom[kingdom_num].score->ore--;
    break;
  }
}

void read_pc_land()
{
  FILE *f;
  char filename[100];
  int room, kingdom_num;

  sprintf(filename, "%s/kingdom.pc.land", KING_DIR);
  f = fopen(filename, "r");

  if (!f)
    return;			/* throw a status message here */

  while (fscanf(f, "%d %d\n", &room, &kingdom_num) != EOF) {
    world[room].kingdom_type = KINGDOM_TYPE_PC;
    world[room].kingdom_num = kingdom_num;
    update_kingdom_score(kingdom_num, room, KINGDOM_LAND_GAIN);
  }
  fclose(f);
}

/* Writes all PC kingdom land to the land file */
/* if room is 0 then write ALL rooms, else just append room info to file */
void write_pc_land(int room)
{
  FILE *f;
  char filename[100];
  int rroom;

  sprintf(filename, "%s/kingdom.pc.land", KING_DIR);
  if (room)
    f = fopen(filename, "a+");
  else
    f = fopen(filename, "w");

  if (!f)
    return;			/* throw a status message here */

  if (!room) {
    for (rroom = 0; rroom <= top_of_world; rroom++) {
      if (world[rroom].kingdom_type == KINGDOM_TYPE_PC) {
	fprintf(f, "%d %d\n", rroom, world[rroom].kingdom_num, KINGDOM_TYPE_PC);
      }
    }
  } else {
    fprintf(f, "%d %d\n", room, world[room].kingdom_num, KINGDOM_TYPE_PC);
  }

  fclose(f);
}

/* Called when a PC kingdom acquires NEW land */
int acquire_land(int room, int kingdom_num, int kingdom_type)
{
  int real_room = 0, append_room = 0, old_kingdom_num = 0;

  /* if not kingdom, no land for you! */
  if (!IS_KINGDOM(kingdom_num))
    return FALSE;

  /* Check if this is a real room and get number */
  if (real_room = real_room0(room)) {
    /* if already owned by this kingdom, bail */
    if (world[real_room].kingdom_type == kingdom_type && world[real_room].kingdom_num == kingdom_num)
      return FALSE;
    /* is this a takeover? */
    if (!world[real_room].kingdom_type)
      append_room = real_room;
    /* save old kingdom number */
    old_kingdom_num = world[real_room].kingdom_num;
    /* assign the land attribute the new values */
    world[real_room].kingdom_type = kingdom_type;
    world[real_room].kingdom_num = kingdom_num;
    /* update the land file */
    write_pc_land(append_room);
    /* adjust kingdom scores */
    update_kingdom_score(kingdom_num, real_room, KINGDOM_LAND_GAIN);
    update_kingdom_score(old_kingdom_num, real_room, KINGDOM_LAND_LOSS);
    return TRUE;
  } else {
    return FALSE;
  }
}

void init_kingdom_score(int kingdom_num)
{
  int w, x, y, z;
  struct kingdom_score_type *score;

  if (!IS_KINGDOM(kingdom_num))
    return;

  CREATE(score, struct kingdom_score_type, 1);
  bzero(score, sizeof(score));
  kingdom[kingdom_num].score = score;

/* this is more for bounds checking than anything else */
  for (w = 0; w < NUM_SECT_TYPES; w++)
    kingdom[kingdom_num].score->land[w] = 0;
  kingdom[kingdom_num].score->food = 0;
  kingdom[kingdom_num].score->ore = 0;
  kingdom[kingdom_num].score->last_payment = 0;
  TROOP_LOOP kingdom[kingdom_num].score->virtual_troops[w][x][y][z] = 0;
  TROOP_LOOP kingdom[kingdom_num].score->troops_in_barracks[w][x][y][z] = 0;
  TROOP_LOOP kingdom[kingdom_num].score->troops_in_transit[w][x][y][z] = 0;
}
