/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <math.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "house.h"
#include "graph.h"
#include "shop.h"
#include "map.h"
#include "olc.h"

#include "plugin.h"

/* external variables */
extern char *room_bits[];
extern struct char_data *char_in_room(int);
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct room_data *world;
extern struct index_data *obj_index;
extern const char *dirs[];
extern const int rev_dir[];
extern int top_of_world;
extern int top_of_zone_table;
extern struct zone_data *zone;
extern struct zone_data *zone_table;

/* local protos */
int whats_in_maproom(struct char_data *, int, int, sh_int, sh_int);
int reverse_x_cord(int room, int dir);
int reverse_y_cord(int room, int dir);
int reverse_dir(int dir);
byte DoesWallTouchAnother(int dir, int room, int origx, int origy, int x, int y);
void reload_room_maps(int room_nr);
void unload_room_maps(int zone);
void generate_mapgrid(FILE * fl, int room);
char fread_letter(FILE * fp);

// Dont use ';' as an icon, client uses it to parse the map
const struct map_icon_info map_info[NUM_MAP_SYMBOLS + 1] =
{
  {" ", "&+w", "Blank (R)"},
  {"=", "&+L", "Wall"},
  {".", "&+L", "City Street"},
  {" ", "&+w", "Normal Floor"},
  {"#", "&+y", "Closed Door (R)"},
  {"O", "&+y", "Open Door (R)"},
  {"D", "&+w", "Down Exit (R)"},
  {"U", "&+w", "Up Exit (R)"},
  {"@", "&+W", "PC me (R)"},
  {"@", "&+w", "PC other (R)"},
  {"*", "&+r", "Mob (R)"},
  {"\"", "&+g", "Field"},
  {"*", "&+g", "Tree"},
  {"^", "&+y", "Hill"},
  {"S", "&+y", "Secretdoor (R)"},
  {"+", "&+L", "Road"},
  {"*", "&+c", "Border (R)"},
  {" ", "&=bw", "Lake/Ocean"},
  {"M", "&+L", "Mountain"},
  {"%", "&+L", "Rubble"},
  {"*", "&+g", "Brush"},
  {".", "&+g", "Field 2"},
  {":", "&+L", "Cave Floor"},
  {"\"", "&+M", "Swamp"},
  {"~", "&=bw", "River"},
  {" ", "&n", "Not Blank"},
  {"F", "&+B", "Generic Furniture"},
  {"T", "&+B", "Throne"},
  {"C", "&+B", "Chair"},
  {"O", "&+B", "Table"},
  {"E", "&+B", "Bookcase"},
  {"B", "&+B", "Bed"},
  {"=", "&+B", "Counter"},
  {"D", "&+B", "Desk"},
  {"W", "&+B", "Well"},
  {".", "&+Y", "Desert Sands"},
  {"*", "&+Y", "Large Cactus"},
  {" ", "&=ww", "Tundra"},
  {"M", "&=wl", "Snow-capped Mountain"},
  {"S", "&n", "Up/Down Exit (R)"},
  {"+", "&+y", "Small Trail"},
  {"X", "&+L", "Pit/Trapdoor"},
  {"\\", "&+y", "Wall2"},
  {"/", "&+y", "Wall3"},
  {"|", "&+y", "Wall4"},
  {"_", "&+y", "Wall5"},
  {"-", "&+y", "Wall6"},
  {".", "&+y", "Wooden Floor/Deck"},
  {"O", "&+Y", "Ship Part, Good"},
  {"O", "&+y", "Ship Part, Ruined"},
  {"%", "&+R", "Lava Flow"},
  {"=", "&+y", "Wooden Wall"},
  {"=", "&+c", "Tin wall"},
  {"C", "&+Y", "Chest"},
  {"S", "&+C", "Default Ship"},
  {" ", "&n", "Border2"},
  {"O", "&+r", "Occupied Doorway"},
  {".", "&+y", "Rubble"},
  {".", "&+y", "Badlands"},
  {"/", "&+y", "Wood Roof"},

  {"X", "&=LR", "ERROR"},
  {"\n", "\n", "\n"}
};

int reverse_dir(int dir)
{

  switch (dir) {
  case NORTH:
    return SOUTH;
  case EAST:
    return WEST;
  case SOUTH:
    return NORTH;
  case WEST:
    return EAST;
  }
  return 0;
}

int reverse_x_cord(int room, int dir)
{
  int rdir;

  rdir = reverse_dir(dir);
  if (world[world[room].dir_option[dir]->to_room].dir_option[rdir])
    if (world[world[room].dir_option[dir]->to_room].dir_option[rdir]->x_cord)
      return (world[world[room].dir_option[dir]->to_room].dir_option[rdir]->x_cord);

  return 0;
}
int reverse_y_cord(int room, int dir)
{
  if (world[world[room].dir_option[dir]->to_room].dir_option[(int) rev_dir[dir]])
    if (world[world[room].dir_option[dir]->to_room].dir_option[(int) rev_dir[dir]]->y_cord)
      return (world[world[room].dir_option[dir]->to_room].dir_option[(int) rev_dir[dir]]->y_cord);

  return 0;
}
/* takes rnum */
/* icon comes in, and we either alter it or leave it alone. */
int whats_in_maproom(struct char_data *ch, int icon, int room, sh_int xcord, sh_int ycord)
{
  struct obj_data *obj, *next_obj;
  struct char_data *who, *who_next;
  int roof = FALSE;

  if (GET_Z_CORD(ch) < 0)
    return icon;

  if (!room)
    return icon;

  if (icon == MAP_WOOD_ROOF)
    roof = TRUE;

  if (world[room].contents)
    for (obj = world[room].contents; obj; obj = next_obj) {
      next_obj = obj->next_content;

      if (roof && GET_OBJ_Z_CORD(obj) < GET_Z_SIZE(room))
	continue;

      if (GET_OBJ_TYPE(obj) == ITEM_FURNITURE &&
	  GET_OBJ_X_CORD(obj) == xcord && GET_OBJ_Y_CORD(obj) == ycord) {
	switch (GET_OBJ_VAL(obj, 4)) {
	case FURN_THRONE:
	  return MAP_FURN_THRONE;
	case FURN_CHAIR:
	  return MAP_FURN_CHAIR;
	case FURN_TABLE:
	  return MAP_FURN_TABLE;
	case FURN_BOOKCASE:
	  return MAP_FURN_BOOKCASE;
	case FURN_BED:
	  return MAP_FURN_BED;
	default:
	  return MAP_FURN;
	}
      }
      if (GET_OBJ_TYPE(obj) == ITEM_SHIP &&
	  GET_OBJ_X_CORD(obj) == xcord && GET_OBJ_Y_CORD(obj) == ycord)
	return MAP_SHIP1;

      if (GET_OBJ_TYPE(obj) == ITEM_DOOR &&
	  GET_OBJ_X_CORD(obj) == xcord && GET_OBJ_Y_CORD(obj) == ycord)
	return MAP_OPEN_DOOR;
      if (GET_OBJ_TYPE(obj) == ITEM_TRASH && isname(obj->name, "boulder") &&
	  GET_OBJ_X_CORD(obj) == xcord && GET_OBJ_Y_CORD(obj) == ycord)
	return MAP_RUBBLE;
      if (GET_OBJ_TYPE(obj) == ITEM_CONTAINER && isname(obj->name, "chest") &&
	  GET_OBJ_X_CORD(obj) == xcord && GET_OBJ_Y_CORD(obj) == ycord)
	return MAP_CHEST;
      if (GET_OBJ_TYPE(obj) == ITEM_STORAGE && isname(obj->name, "chest") &&
	  GET_OBJ_X_CORD(obj) == xcord && GET_OBJ_Y_CORD(obj) == ycord)
	return MAP_CHEST;
    }
  if (world[room].people)
    for (who = world[room].people; who; who = who_next) {
      who_next = who->next_in_room;

      if (roof && GET_Z_CORD(who) < GET_Z_SIZE(room))
	continue;

      if (GET_X_CORD(who) != xcord) {
	continue;
      }
      if (GET_Y_CORD(who) != ycord) {
	continue;
      }
      if (ch == who)
	continue;
      if (!CAN_SEE(ch, who))
	continue;

      if (icon == MAP_OPEN_DOOR && who)
	return MAP_OCCUPIED_DOOR;
      if (IS_PC(who))
	return MAP_PC;
      if (IS_NPC(who))
	return MAP_MOB;
    }
  return icon;
}

/* takes rnum */
/* icon comes in, and we either alter it or leave it alone. */
int whats_in_guiroom(struct char_data *ch, int icon, int room, sh_int xcord, sh_int ycord)
{
  struct obj_data *obj, *next_obj;
  struct char_data *who, *who_next;

  if (GET_Z_CORD(ch) < 0)
    return icon;

  if (!room)
    return icon;

  if (IS_WATER_ROOM(room) && world[room].contents)
    for (obj = world[room].contents; obj; obj = next_obj) {
      next_obj = obj->next_content;
      if (GET_OBJ_TYPE(obj) == ITEM_SHIP)
	return MAP3_SHIP1;
    }
  if (world[room].people)
    for (who = world[room].people; who; who = who_next) {
      who_next = who->next_in_room;

      if (GET_X_CORD(who) != xcord) {
	continue;
      }
      if (GET_Y_CORD(who) != ycord) {
	continue;
      }
      if (ch == who)
	continue;
      if (!CAN_SEE(ch, who))
	continue;
      if (IS_PC(who))
	return MAP3_PC;
      if (IS_NPC(who))
	return MAP3_MOB;
    }
  return icon;
}

/* returns the basic floor icon for this room */
/* special contents will overwrite */
int FloorType(int next_room)
{

  switch (SECT(next_room)) {
  case SECT_CITY:
  case SECT_ROAD:
    return (MAP_CITY_STREET);
  case SECT_FIELD:
    return (MAP_FIELD);
  case SECT_FOREST:
    return (MAP_BRUSH);
  case SECT_HILLS:
    return (MAP_HILL);
  case SECT_CAVE:
    return (MAP_CAVE);
  case SECT_SWAMP:
    return (MAP_SWAMP);
  case SECT_MOUNTAIN:
    return (MAP_MOUNTAIN);
  case SECT_WATER_SWIM:
    return (MAP_RIVER);
  case SECT_WATER_NOSWIM:
    return (MAP_LAKE);
  default:
    return (MAP_NORMAL_FLOOR);
    break;
  }
}

/* decides if the dir wall in this room, touches an oposite wall in the
 * the other. This is so if this room is 10 wide, and the north room is
 * only 5, and there is no door between them, we will be able to draw only
 * half a wall in the current, while leaving the other half open to the 
 * north.
 */
byte DoesWallTouchAnother(int dir, int room, int origx, int origy, int x, int y)
{
  char Gbuf[80];

  y += origy;
  x += origx;

  sprintf(Gbuf, "dir %d, room %d, ox%d, oy%d, x%d, y%d", dir, room, origx, origy, x, y);
  log(Gbuf);
  if (dir == NORTH || dir == SOUTH) {
    if (y < (origy + (world[room].dir_option[dir]->y_cord) -
	     reverse_y_cord(room, dir))) {
      log("y is not in bounds");
      return 0;
    } else if (y > (origy + (world[room].dir_option[dir]->y_cord) -
		    reverse_y_cord(room, dir))) {
      log("y is not in bounds");
      return 0;
    } else {
      log("returning y touching");
      return 1;
    }
  } else if (dir == EAST || dir == WEST) {
    if ((x - origx - 1) < (origx + (world[room].dir_option[dir]->x_cord) -
			   reverse_x_cord(room, dir))) {
      sprintf(Gbuf, "%d - %d - 1 < %d + %d - %d", x, origx, origx, world[room].dir_option[dir]->x_cord,
	      reverse_x_cord(room, dir));
      log(Gbuf);
      return 0;
    } else if ((x - origx + 1) > (origx + (world[room].dir_option[dir]->x_cord) -
				  reverse_x_cord(room, dir))) {
      sprintf(Gbuf, "%d - %d + 1 > %d + %d - %d", x, origx, origx, world[room].dir_option[dir]->x_cord,
	      reverse_x_cord(room, dir));
      log(Gbuf);
      return 0;
    } else {
      log("returning x touching");
      return 1;
    }
  }
  log("default reached");
  return 0;
}

/* main function, the big cheese, my ticket to fame and fabulous fortunes */
void guimap_look(struct char_data *ch, sh_int size, int dif_room)
{
  char Gbuf[MAX_STRING_LENGTH];
  int i, n, x, y, what, what1, what2, dir = -1, xc, yc;
  int startx, starty, origx, origy, currx, curry, room_shape;
  int room, next_room;
  int the_map[31][31];
  char the_object[MAX_STRING_LENGTH];
  char obj_temp[MAX_STRING_LENGTH];
  int the_wall[31][31];
  int wall_type;
  struct obj_data *obj = NULL;

  if (size > 30)		/* max viewing area */
    size = 30;

  if (ch->in_room == NOWHERE)
    return;

  /* messages are given in look_room */
  if (IS_DARK(ch->in_room) && !CAN_SEE_IN_DARK(ch)) {
    return;
  } else if (IS_AFFECTED(ch, AFF_BLIND)) {
    return;
  }
  if (dif_room == -1)		/* used for various spells, etc */
    room = ch->in_room;
  else
    room = dif_room;

  if (GET_X_CORD(ch) < 0 || GET_X_CORD(ch) > world[ch->in_room].x_size - 1 ||
  GET_Y_CORD(ch) < 0 || GET_Y_CORD(ch) > world[ch->in_room].y_size - 1) {
    xc = world[ch->in_room].x_size / 2;
    yc = world[ch->in_room].y_size / 2;
  } else {
    xc = GET_X_CORD(ch);
    yc = GET_Y_CORD(ch);
  }

  for (n = 0; n <= size; n++) {
    for (i = 0; i <= size; i++) {
      the_map[n][i] = MAP_BLANK;
      the_wall[n][i] = MAP_BLANK;
    }
  }
  strcpy(the_object, "");
  strcpy(obj_temp, "");

  /* setup main rooms center, and hold the info for later use */
  /* hey, let's be safe and initialize a few more things...   */
  origx = (size / 2) - xc;
  origy = (size / 2) - yc;
  currx = world[room].x_size;
  curry = world[room].y_size;
  room_shape = world[room].shape;
  next_room = room;
  what = MAP_BLANK;
  what1 = MAP_BLANK;
  what2 = MAP_BLANK;

  for (dir = -1; dir <= NUM_OF_DIRS - 1; dir++) {

    /* first decide if we even need this loop. Don't waste time! */
    if (dir > -1 && (!world[room].dir_option[dir] ||
		     DRAW_WALL_ONLY(room, dir) ||
		     world[room].dir_option[dir]->to_room < 1 ||
		     DOOR_IS_CLOSED(room, obj, dir))) {
      continue;
    }
    /* fathoming where to center this room, 
     * based on which dir it is from the central chamber (dir -1) 
     */
    switch (dir) {
    case NOWHERE:		/* aka current room */
      startx = origx;
      starty = origy;
      break;
    case NORTH:
      startx = origx - (world[world[room].dir_option[dir]->to_room].x_size - 1);
      starty = origy + (world[room].dir_option[dir]->y_cord) - reverse_y_cord(room, dir);
      break;
    case NORTHWEST:
      startx = origx - (world[world[room].dir_option[dir]->to_room].x_size - 1);
      starty = origy - (world[world[room].dir_option[dir]->to_room].y_size - 1);
      break;
    case WEST:
      startx = origx + (world[room].dir_option[dir]->x_cord) - reverse_x_cord(room, dir);
      starty = origy - (world[world[room].dir_option[dir]->to_room].y_size - 1);
      break;
    case NORTHEAST:
      startx = origx - (world[world[room].dir_option[dir]->to_room].x_size - 1);
      starty = origy + (world[room].y_size) - 1;
      break;
    case EAST:
      startx = origx + (world[room].dir_option[dir]->x_cord) - reverse_x_cord(room, dir);
      starty = origy + (world[room].y_size) - 1;
      break;
    case SOUTHWEST:
      startx = origx + (world[room].x_size) - 1;
      starty = origy - (world[world[room].dir_option[dir]->to_room].y_size - 1);
      break;
    case SOUTHEAST:
      startx = origx + (world[room].x_size) - 1;
      starty = origy + (world[room].y_size) - 1;
      break;
    case SOUTH:
      startx = origx + (world[room].x_size) - 1;
      starty = origy + (world[room].dir_option[dir]->y_cord) - reverse_y_cord(room, dir);
      break;
    default:			/* missing or incomplete data, just skip it */
      continue;
    }

    /* Now, change the info for the new room to be drawn */
    if (dir >= 0 && world[room].dir_option[dir]->to_room > 0) {
      currx = world[world[room].dir_option[dir]->to_room].x_size;
      curry = world[world[room].dir_option[dir]->to_room].y_size;
      room_shape = (world[world[room].dir_option[dir]->to_room].shape);
      next_room = world[room].dir_option[dir]->to_room;
    }
    switch (room_shape) {
      /* room contents primarily will use next_room, so we don't change room */
    case ROOM_SHAPE_SQUARE:
    case ROOM_SHAPE_PCMAP:
      for (x = 0; x < currx; x++) {
	for (y = 0; y < curry; y++) {

	  /* draw some walls. Complex because we may only need a partial wall */
	  if (x == 0) {
	    /* no wall */
	    if (world[next_room].dir_option[NORTH] &&
		!IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR) &&
		!IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED)
		&& y != 0 && y != (curry - 1)) {
	      what = FloorType(next_room);
	      /* corner square */
	    } else if (world[next_room].dir_option[EAST] &&
		       (IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR) ||
			IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED))
		       && y == curry - 1) {
	      what = FloorType(next_room);
	      what2 = MAP2_WALL;
	      /* corner square */
	    } else if (world[next_room].dir_option[WEST] &&
		       (IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR) ||
			IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED))
		       && y == 0) {
	      what = FloorType(next_room);
	      what2 = MAP2_WALL;
	      /* next to interior room */
	    } else if (SECT(next_room) == SECT_INSIDE
		       || (world[next_room].dir_option[NORTH] && SECT(world[next_room].dir_option[NORTH]->to_room) == SECT_INSIDE)) {
	      what = FloorType(next_room);
	      what2 = MAP2_WALL;
	      /* fake wall */
	    } else if (world[next_room].dir_option[NORTH] && IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	      /* real mccoy */
	    } else if (world[next_room].dir_option[NORTH] &&
		       IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	      /* default */
	    } else {
	      what = FloorType(next_room);
	    }
	  } else if (x == (currx - 1)) {
	    if (world[next_room].dir_option[SOUTH] &&
		!IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR) &&
		!IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED)
		&& y != 0 && y != (curry - 1)) {
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[EAST] &&
		       (IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR) ||
			IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED))
		       && y == curry - 1) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[WEST] &&
		       (IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR) ||
			IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED))
		       && y == 0) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (SECT(next_room) == SECT_INSIDE
		       || (world[next_room].dir_option[SOUTH] && SECT(world[next_room].dir_option[SOUTH]->to_room) == SECT_INSIDE)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[SOUTH] && IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[SOUTH] &&
		       IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else {
	      what = FloorType(next_room);
	    }
	  } else if (y == 0) {
	    if (world[next_room].dir_option[WEST] &&
		!IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR) &&
		!IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED)
		&& x != 0 && x != (currx - 1)) {
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[NORTH] &&
		       (IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR) ||
			IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED))
		       && x == 0) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[SOUTH] &&
		       (IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR) ||
			IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED))
		       && x == currx - 1) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (SECT(next_room) == SECT_INSIDE
		       || (world[next_room].dir_option[WEST] && SECT(world[next_room].dir_option[WEST]->to_room) == SECT_INSIDE)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[WEST] && IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[WEST] &&
		       IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else {
	      what = FloorType(next_room);
	    }
	  } else if (y == (curry - 1)) {
	    if (world[next_room].dir_option[EAST] &&
		!IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR) &&
		!IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED)
		&& x != 0 && x != (currx - 1)) {
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[NORTH] &&
		       (IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR) ||
			IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED))
		       && x == 0) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[SOUTH] &&
		       (IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR) ||
			IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED))
		       && x == currx - 1) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (SECT(next_room) == SECT_INSIDE
		       || (world[next_room].dir_option[EAST] && SECT(world[next_room].dir_option[EAST]->to_room) == SECT_INSIDE)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[EAST] && IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else if (world[next_room].dir_option[EAST] &&
		       IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR)) {
	      what2 = MAP2_WALL;
	      what = FloorType(next_room);
	    } else {
	      what = FloorType(next_room);
	    }
	  } else {
	    what = FloorType(next_room);
	  }
	  /* end of wall drawing */

	  /* add some doors */
	  for (n = 0; n < 6; n++) {
	    if (world[next_room].dir_option[n]
		&& world[next_room].dir_option[n]->to_room > 0
		&& x == world[next_room].dir_option[n]->x_cord
		&& y == world[next_room].dir_option[n]->y_cord) {
	      /* check stairs first */
	      if (n == UP) {
		sprintf(obj_temp, "%s%c", obj_temp, MAP3_STAIR_UP + 53);
		what = FloorType(next_room);
	      } else if (n == DOWN) {
		sprintf(obj_temp, "%s%c", obj_temp, MAP3_STAIR_DOWN + 53);
		what = FloorType(next_room);
	      } else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
			 DOOR_IS_OPEN(next_room, obj, n) &&
			 CAN_SEE_EXIT(ch, next_room, n)) {
		what2 = MAP2_OPEN_DOOR;
		what = FloorType(next_room);
	      } else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
			 DOOR_IS_CLOSED(next_room, obj, n) &&
			 CAN_SEE_EXIT(ch, next_room, n)) {
		what2 = MAP2_CLOSED_DOOR;
		what = FloorType(next_room);
	      } else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
			 DOOR_IS_OPEN(next_room, obj, n) &&
			 !CAN_SEE_EXIT(ch, next_room, n)) {
		what2 = MAP2_SECRETDOOR;
		what = FloorType(next_room);
	      } else if (!CAN_SEE_EXIT(ch, next_room, n)) {
		what2 = MAP2_WALL;
		what = FloorType(next_room);
	      } else if (DRAW_WALL_ONLY(next_room, n)) {
		what2 = MAP2_WALL;
		what = FloorType(next_room);
	      } else {
		what = FloorType(next_room);
	      }
	    }
	  }
	  /* pass it to a contents function, which will either leave the icon
	   * alone, or change it to a mob/object etc and return the new icon
	   */
	  what1 = whats_in_guiroom(ch, what1, next_room, x, y);
	  if (what1 > MAP_ERROR)
	    what1 = MAP_BLANK;
	  if (what1 != MAP_BLANK) {
	    sprintf(obj_temp, "%s%c", obj_temp, what1 + 53);
	  }
	  /* don't ever want to overwrite yourself */
	  if (GET_X_CORD(ch) == x &&
	      GET_Y_CORD(ch) == y && dir == -1 && dif_room == -1) {
	    sprintf(obj_temp, "%s%c", obj_temp, MAP3_ME + 53);
	  }
	  /* keep everything within the map. Also, a good spot to check
	   * for light sources, etc affecting total shown.
	   */

	  if ((x + startx) < size && (y + starty) < size &&
	      (x + startx) > 0 && (y + starty) > 0) {

	    the_map[x + startx][y + starty] = what;
	    the_wall[x + startx][y + starty] = what2;
	    if (strlen(obj_temp) > 0) {
	      sprintf(the_object, "%s|%c%c%s", the_object, x + startx + 53, y + starty + 53, obj_temp);
	    }
	  }
	  what1 = MAP_BLANK;
	  what2 = MAP_BLANK;
	  strcpy(obj_temp, "");
	}
      }
      break;			/* end of SQUARE shape */
/*    case ROOM_SHAPE_PCMAP: */
    case ROOM_SHAPE_WILDERNESS:
      if (world[next_room].map == NULL) {
	CREATE(world[next_room].map, struct map_data, 1);
#ifdef NEWMAP
	CREATE(world[next_room].map->map_grid, sh_int *,
	       world[next_room].x_size + 1);
	for (i = 0; i < world[next_room].x_size + 1; i++)
	  CREATE(world[next_room].map->map_grid[i], sh_int,
		 world[next_room].y_size + 1);
#endif
      }
      for (x = 0; x < world[next_room].x_size; x++) {
	for (y = 0; y < world[next_room].y_size; y++) {
	  what = world[next_room].map->map_grid[x][y];

	  /* add some doors */
	  for (n = 0; n < 6; n++) {
	    if (world[next_room].dir_option[n]
		&& world[next_room].dir_option[n]->to_room > 0
		&& x == world[next_room].dir_option[n]->x_cord
		&& y == world[next_room].dir_option[n]->y_cord) {
	      /* check stairs first */
	      if (n == UP) {
		sprintf(obj_temp, "%s%c", obj_temp, MAP3_STAIR_UP + 53);
		what = FloorType(next_room);
	      } else if (n == DOWN) {
		sprintf(obj_temp, "%s%c", obj_temp, MAP3_STAIR_DOWN + 53);
		what = FloorType(next_room);
	      } else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
			 DOOR_IS_OPEN(next_room, obj, n) &&
			 CAN_SEE_EXIT(ch, next_room, n)) {
		what2 = MAP2_OPEN_DOOR;
		what = FloorType(next_room);
	      } else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
			 DOOR_IS_CLOSED(next_room, obj, n) &&
			 CAN_SEE_EXIT(ch, next_room, n)) {
		what2 = MAP2_CLOSED_DOOR;
		what = FloorType(next_room);
	      } else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
			 DOOR_IS_OPEN(next_room, obj, n) &&
			 !CAN_SEE_EXIT(ch, next_room, n)) {
		what2 = MAP2_SECRETDOOR;
		what = FloorType(next_room);
	      } else if (!CAN_SEE_EXIT(ch, next_room, n)) {
		what2 = MAP2_WALL;
		what = FloorType(next_room);
	      } else if (DRAW_WALL_ONLY(next_room, n)) {
		what2 = MAP2_WALL;
		what = FloorType(next_room);
	      } else {
		what = FloorType(next_room);
	      }
	    }
	  }
	  /* pass it to a contents function, which will either leave the icon
	   * alone, or change it to a mob/object etc and return the new icon
	   */
	  what1 = whats_in_guiroom(ch, what1, next_room, x, y);

	  if (what1 > MAP_ERROR)
	    what1 = MAP_BLANK;
	  if (what1 != MAP_BLANK)
	    sprintf(obj_temp, "%s%c", obj_temp, what1 + 53);

	  /* don't ever want to overwrite yourself */
	  if (GET_X_CORD(ch) == x &&
	      GET_Y_CORD(ch) == y && dir == -1 && dif_room == -1)
	    sprintf(obj_temp, "%s%c", obj_temp, MAP3_ME + 53);
	  /* keep everything within the map. Also, a good spot to check
	   * for light sources, etc affecting total shown.
	   */
	  if ((x + startx) < size && (y + starty) < size &&
	      (x + startx) > 0 && (y + starty) > 0) {

	    the_map[x + startx][y + starty] = what;
	    if (strlen(obj_temp) > 0) {
	      sprintf(the_object, "%s|%c%c%s", the_object, x + startx + 53, y + starty + 53, obj_temp);
	    }
	    what1 = MAP_BLANK;
	  }
	  what1 = MAP_BLANK;
	  strcpy(obj_temp, "");
	}
      }
      break;			/* end of wilderness shape */
      /* add other room shapes here */
    }
  }

  for (n = 1; n <= (size - 1); n++) {
    for (i = 1; i <= (size - 1); i++) {
      if (the_wall[n][i] != MAP_BLANK && the_wall[n][i] != MAP2_WALL) {
	if (the_wall[n][i] == MAP2_OPEN_DOOR) {
	  if (the_wall[n - 1][i] != MAP_BLANK) {
	    the_wall[n][i] = MAP2_OPEN_DOOR_NS;
	  }
	  if (the_wall[n][i - 1] != MAP_BLANK) {
	    the_wall[n][i] = MAP2_OPEN_DOOR_EW;
	  }
	  if (the_wall[n][i + 1] != MAP_BLANK) {
	    the_wall[n][i] = MAP2_OPEN_DOOR_EW;
	  }
	  if (the_wall[n + 1][i] != MAP_BLANK) {
	    the_wall[n][i] = MAP2_OPEN_DOOR_NS;
	  }
	}
	if (the_wall[n][i] == MAP2_CLOSED_DOOR) {
	  if (the_wall[n - 1][i] != MAP_BLANK) {
	    the_wall[n][i] = MAP2_CLOSE_DOOR_NS;
	  }
	  if (the_wall[n][i - 1] != MAP_BLANK) {
	    the_wall[n][i] = MAP2_CLOSE_DOOR_EW;
	  }
	  if (the_wall[n][i + 1] != MAP_BLANK) {
	    the_wall[n][i] = MAP2_CLOSE_DOOR_EW;
	  }
	  if (the_wall[n + 1][i] != MAP_BLANK) {
	    the_wall[n][i] = MAP2_CLOSE_DOOR_NS;
	  }
	}
      }
      if (the_wall[n][i] == MAP2_WALL) {
	wall_type = 0;
	if (the_wall[n - 1][i] != MAP_BLANK) {
	  wall_type += 1;
	}
	if (the_wall[n][i - 1] != MAP_BLANK) {
	  wall_type += 2;
	}
	if (the_wall[n][i + 1] != MAP_BLANK) {
	  wall_type += 4;
	}
	if (the_wall[n + 1][i] != MAP_BLANK) {
	  wall_type += 8;
	}
	switch (wall_type) {
	case 1:
	  the_wall[n][i] = MAP2_WALL2;
	  break;
	case 2:
	  the_wall[n][i] = MAP2_WALL1;
	  break;
	case 3:
	  the_wall[n][i] = MAP2_WALL6;
	  break;
	case 4:
	  the_wall[n][i] = MAP2_WALL1;
	  break;
	case 5:
	  the_wall[n][i] = MAP2_WALL5;
	  break;
	case 6:
	  the_wall[n][i] = MAP2_WALL1;
	  break;
	case 7:
	  the_wall[n][i] = MAP2_WALL8;
	  break;
	case 8:
	  the_wall[n][i] = MAP2_WALL2;
	  break;
	case 9:
	  the_wall[n][i] = MAP2_WALL2;
	  break;
	case 10:
	  the_wall[n][i] = MAP2_WALL7;
	  break;
	case 11:
	  the_wall[n][i] = MAP2_WALL10;
	  break;
	case 12:
	  the_wall[n][i] = MAP2_WALL4;
	  break;
	case 13:
	  the_wall[n][i] = MAP2_WALL11;
	  break;
	case 14:
	  the_wall[n][i] = MAP2_WALL9;
	  break;
	case 15:
	  the_wall[n][i] = MAP2_WALL3;
	  break;
	}
      }
    }
  }

  sprintf(Gbuf, "%s%c", PLUGIN_MAP_START, size + 53 - 2);

  for (n = 1; n <= (size - 1); n++) {
    for (i = 1; i <= (size - 1); i++) {
      sprintf(Gbuf, "%s%c", Gbuf, the_map[n][i] + 53);
    }
  }
  sprintf(Gbuf, "%s%s", Gbuf, PLUGIN_MAP_END);
  send_to_char(Gbuf, ch);

  sprintf(Gbuf, "%s%c", PLUGIN_WALL_START, size + 53 - 2);

  for (n = 1; n <= (size - 1); n++) {
    for (i = 1; i <= (size - 1); i++) {
      sprintf(Gbuf, "%s%c", Gbuf, the_wall[n][i] + 53);
    }
  }
  sprintf(Gbuf, "%s%s", Gbuf, PLUGIN_WALL_END);
  send_to_char(Gbuf, ch);

  sprintf(Gbuf, "%s%c", PLUGIN_OBJECT_START, size + 53 - 2);
  sprintf(Gbuf, "%s%s", Gbuf, the_object);
  sprintf(Gbuf, "%s%s", Gbuf, PLUGIN_OBJECT_END);
  send_to_char(Gbuf, ch);
}


/* main function, the big cheese, my ticket to fame and fabulous fortunes */
void smallmap_look(struct char_data *ch, sh_int size, int dif_room)
{
  char Gbuf[MAX_STRING_LENGTH];
  int i, n, x, y, what, dir = -1, xc, yc;
  int startx, starty, origx, origy, currx, curry, room_shape;
  int room, next_room, color, previous_color = 0, previous_what = 0;
  int the_map[31][61];
  struct obj_data *obj = NULL;
  /* int dis_bet, c_x, t_x, c_y, t_y, c_z, t_z; */

  /* Have we purged the map from memory? */
  if (zone_table[world[ch->in_room].zone].unloaded)
    reload_room_maps(ch->in_room);

  if (IS_GREY_TERM(ch->desc)) {
    guimap_look(ch, size, dif_room);
  }
  if (GET_Z_CORD(ch) > 0)	/* so swimmers dont see less and flyers see more */
    size += GET_Z_CORD(ch);

  if (size > 30)		/* max viewing area, set above on init */
    size = 30;

  if (ch->in_room == NOWHERE)
    return;

  if (dif_room == -1)		/* used for various spells, etc */
    room = ch->in_room;
  else
    room = dif_room;

  if (GET_X_CORD(ch) < 0 || GET_X_CORD(ch) > world[ch->in_room].x_size - 1 ||
  GET_Y_CORD(ch) < 0 || GET_Y_CORD(ch) > world[ch->in_room].y_size - 1) {
    xc = world[ch->in_room].x_size / 2;
    yc = world[ch->in_room].y_size / 2;
  } else {
    xc = GET_X_CORD(ch);
    yc = GET_Y_CORD(ch);
  }

  /* return, so map isnt on prompt line */
  send_to_char("\r\n", ch);
  if (IS_CLIENT_TERM(ch->desc)) {
    sprintf(Gbuf, "%s", CLIENT_MAP_PREFIX);
    send_to_char(Gbuf, ch);
  }
  for (n = 0; n <= size; n++) {
    for (i = 0; i <= (size); i++) {
      if (n == 0 || n == (size) || i == 0 || i == (size))
	the_map[n][i] = MAP_BORDER;
      else if (n == 1 || n == (size - 1) || i == 1 || i == (size - 1))
	the_map[n][i] = MAP_BORDER2;
      else
	the_map[n][i] = MAP_BLANK;
    }
  }

  /* setup main rooms center, and hold the info for later use */
  /* hey, let's be safe and initialize a few more things...   */
  origx = (size / 2) - xc;
  origy = (size / 2) - yc;
  currx = world[room].x_size;
  curry = world[room].y_size;
  room_shape = world[room].shape;
  next_room = room;
  what = MAP_BLANK;

  for (dir = -1; dir <= NUM_OF_DIRS - 1; dir++) {

    /* first decide if we even need this loop. Don't waste time! */
    if (dir > -1 && (!world[room].dir_option[dir] ||
		     DRAW_WALL_ONLY(room, dir) ||
		     world[room].dir_option[dir]->to_room < 1 ||
		     DOOR_IS_CLOSED(room, obj, dir))) {
      continue;
    }
    /* fathoming where to center this room, 
     * based on which dir it is from the central chamber (dir -1) 
     */
    switch (dir) {
    case NOWHERE:		/* aka current room */
      startx = origx;
      starty = origy;
      break;
    case NORTH:
      startx = origx - (world[world[room].dir_option[dir]->to_room].x_size - 1);
      starty = origy + (world[room].dir_option[dir]->y_cord) - reverse_y_cord(room, dir);
      break;
    case NORTHWEST:
      startx = origx - (world[world[room].dir_option[dir]->to_room].x_size - 1);
      starty = origy - (world[world[room].dir_option[dir]->to_room].y_size - 1);
      break;
    case WEST:
      startx = origx + (world[room].dir_option[dir]->x_cord) - reverse_x_cord(room, dir);
      starty = origy - (world[world[room].dir_option[dir]->to_room].y_size - 1);
      break;
    case NORTHEAST:
      startx = origx - (world[world[room].dir_option[dir]->to_room].x_size - 1);
      starty = origy + (world[room].y_size) - 1;
      break;
    case EAST:
      startx = origx + (world[room].dir_option[dir]->x_cord) - reverse_x_cord(room, dir);
      starty = origy + (world[room].y_size) - 1;
      break;
    case SOUTHWEST:
      startx = origx + (world[room].x_size) - 1;
      starty = origy - (world[world[room].dir_option[dir]->to_room].y_size - 1);
      break;
    case SOUTHEAST:
      startx = origx + (world[room].x_size) - 1;
      starty = origy + (world[room].y_size) - 1;
      break;
    case SOUTH:
      startx = origx + (world[room].x_size) - 1;
      starty = origy + (world[room].dir_option[dir]->y_cord) - reverse_y_cord(room, dir);
      break;
    default:			/* missing or incomplete data, just skip it */
      continue;
    }

    /* Now, change the info for the new room to be drawn */
    if (dir >= 0 && world[room].dir_option[dir]->to_room > 0) {
      currx = world[world[room].dir_option[dir]->to_room].x_size;
      curry = world[world[room].dir_option[dir]->to_room].y_size;
      room_shape = (world[world[room].dir_option[dir]->to_room].shape);
      next_room = world[room].dir_option[dir]->to_room;
    }
    switch (room_shape) {
      /* room contents primarily will use next_room, so we don't change room */
    case ROOM_SHAPE_SQUARE:
      for (x = 0; x < currx; x++) {
	for (y = 0; y < curry; y++) {

	  /* draw some walls. Complex because we may only need a partial wall */
	  if (x == 0) {
	    /* no wall */
	    if (world[next_room].dir_option[NORTH] &&
		!IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR) &&
		!IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED)
		&& y != 0 && y != (curry - 1))
	      what = FloorType(next_room);
	    /* corner square */
	    else if (world[next_room].dir_option[EAST] &&
		     (IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR) ||
		      IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED))
		     && y == curry - 1)
	      what = MAP_WALL;
	    /* corner square */
	    else if (world[next_room].dir_option[WEST] &&
		     (IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR) ||
		      IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED))
		     && y == 0)
	      what = MAP_WALL;
	    /* next to interior room */
	    else if (SECT(next_room) == SECT_INSIDE
		     || (world[next_room].dir_option[NORTH] && SECT(world[next_room].dir_option[NORTH]->to_room) == SECT_INSIDE))
	      what = MAP_WALL;
	    /* fake wall */
	    else if (world[next_room].dir_option[NORTH] && IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED))
	      what = MAP_WALL;
	    /* real mccoy */
	    else if (world[next_room].dir_option[NORTH] &&
		     IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR))
	      what = MAP_WALL;
	    /* default */
	    else
	      what = FloorType(next_room);
	  } else if (x == (currx - 1)) {
	    if (world[next_room].dir_option[SOUTH] &&
		!IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR) &&
		!IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED)
		&& y != 0 && y != (curry - 1))
	      what = FloorType(next_room);
	    else if (world[next_room].dir_option[EAST] &&
		     (IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR) ||
		      IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED))
		     && y == curry - 1)
	      what = MAP_WALL;
	    else if (world[next_room].dir_option[WEST] &&
		     (IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR) ||
		      IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED))
		     && y == 0)
	      what = MAP_WALL;
	    else if (SECT(next_room) == SECT_INSIDE
		     || (world[next_room].dir_option[SOUTH] && SECT(world[next_room].dir_option[SOUTH]->to_room) == SECT_INSIDE))
	      what = MAP_WALL;
	    else if (world[next_room].dir_option[SOUTH] && IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED))
	      what = MAP_WALL;
	    else if (world[next_room].dir_option[SOUTH] &&
		     IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR))
	      what = MAP_WALL;
	    else
	      what = FloorType(next_room);
	  } else if (y == 0) {
	    if (world[next_room].dir_option[WEST] &&
		!IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR) &&
		!IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED)
		&& x != 0 && x != (currx - 1))
	      what = FloorType(next_room);
	    else if (world[next_room].dir_option[NORTH] &&
		     (IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR) ||
		      IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED))
		     && x == 0)
	      what = MAP_WALL;
	    else if (world[next_room].dir_option[SOUTH] &&
		     (IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR) ||
		      IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED))
		     && x == currx - 1)
	      what = MAP_WALL;
	    else if (SECT(next_room) == SECT_INSIDE
		     || (world[next_room].dir_option[WEST] && SECT(world[next_room].dir_option[WEST]->to_room) == SECT_INSIDE))
	      what = MAP_WALL;
	    else if (world[next_room].dir_option[WEST] && IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED))
	      what = MAP_WALL;
	    else if (world[next_room].dir_option[WEST] &&
		     IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR))
	      what = MAP_WALL;
	    else
	      what = FloorType(next_room);
	  } else if (y == (curry - 1)) {
	    if (world[next_room].dir_option[EAST] &&
		!IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR) &&
		!IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED)
		&& x != 0 && x != (currx - 1))
	      what = FloorType(next_room);
	    else if (world[next_room].dir_option[NORTH] &&
		     (IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR) ||
		      IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED))
		     && x == 0)
	      what = MAP_WALL;
	    else if (world[next_room].dir_option[SOUTH] &&
		     (IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR) ||
		      IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED))
		     && x == currx - 1)
	      what = MAP_WALL;
	    else if (SECT(next_room) == SECT_INSIDE
		     || (world[next_room].dir_option[EAST] && SECT(world[next_room].dir_option[EAST]->to_room) == SECT_INSIDE))
	      what = MAP_WALL;
	    else if (world[next_room].dir_option[EAST] && IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED))
	      what = MAP_WALL;
	    else if (world[next_room].dir_option[EAST] &&
		     IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR))
	      what = MAP_WALL;
	    else
	      what = FloorType(next_room);
	  } else {
	    what = FloorType(next_room);
	  }
	  /* end of wall drawing */

	  /* add some doors */
	  for (n = 0; n < 6; n++) {
	    if (world[next_room].dir_option[n]
		&& world[next_room].dir_option[n]->to_room > 0
		&& x == world[next_room].dir_option[n]->x_cord
		&& y == world[next_room].dir_option[n]->y_cord) {
	      /* check stairs first */
	      if (n == UP && CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_STAIR_UP;
	      else if (previous_what == MAP_STAIR_UP &&
		       n == DOWN && CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_STAIR_UPDOWN;
	      else if (n == DOWN && CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_STAIR_DOWN;
	      else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
		       DOOR_IS_OPEN(next_room, obj, n) &&
		       CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_OPEN_DOOR;
	      else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
		       DOOR_IS_CLOSED(next_room, obj, n) &&
		       CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_CLOSED_DOOR;
	      else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
		       DOOR_IS_OPEN(next_room, obj, n) &&
		       !CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_SECRETDOOR;
	      else if (!CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_WALL;
	      else if (DRAW_WALL_ONLY(next_room, n))
		what = MAP_WALL;
	      else
		what = FloorType(next_room);
	    }
	  }
	  /* Add those changable objects to the array, so checks on it
	   * will work. Later, if we write it to file again, we'll
	   * just re-convert, without these objects! 
	   */
	  if (world[next_room].map)
	    world[next_room].map->map_grid[x][y] = what;

	  /* Above/beside a room */
	  if (SECT(IN_ROOM(ch)) != SECT_CAVE && SECT(next_room) != SECT_CAVE &&
	      (ROOM_FLAGGED(next_room, ROOM_INDOORS) &&
	       !IS_WALL(what) && GET_Z_SIZE(next_room) != -1) &&
	      ((GET_Z_CORD(ch) > GET_Z_SIZE(next_room)) ||	/* above */
	       (ch->in_room != next_room && GET_Z_SIZE(ch->in_room) == -1 && OUTSIDE(ch))))	/* beside */
	    what = MAP_WOOD_ROOF;

	  /* pass it to a contents function, which will either leave the icon
	   * alone, or change it to a mob/object etc and return the new icon
	   */
	  what = whats_in_maproom(ch, what, next_room, x, y);
	  if (what > MAP_ERROR)
	    what = MAP_ERROR;
	  /* don't ever want to overwrite yourself */
	  if (GET_X_CORD(ch) == x &&
	      GET_Y_CORD(ch) == y && dir == -1 && dif_room == -1)
	    what = MAP_ME;

	  /* keep everything within the map. Also, a good spot to check
	   * for light sources, etc affecting total shown.
	   */

	  if ((x + startx) < (size + 1) && (y + starty) < (size + 1) &&
	      (x + startx) > -1 && (y + starty) > -1) {
	    /* prudent coder noticed that the above is 2 less than is
	     * possible. Well, I have grand plans for a border you see...
	     */
	    if (the_map[x + startx][y + starty] == MAP_BLANK ||
		what == MAP_ME || what == MAP_MOB)
	      /* don't overwrite previously drawn, unless its a wall */
	      the_map[x + startx][y + starty] = what;
	  }
	}
      }
      break;			/* end of SQUARE shape */
    case ROOM_SHAPE_PCMAP:
    case ROOM_SHAPE_WILDERNESS:
      if (world[next_room].map == NULL) {
	CREATE(world[next_room].map, struct map_data, 1);
#ifdef NEWMAP
	CREATE(world[next_room].map->map_grid, sh_int *,
	       world[next_room].x_size + 1);
	for (i = 0; i < world[next_room].x_size + 1; i++)
	  CREATE(world[next_room].map->map_grid[i], sh_int,
		 world[next_room].y_size + 1);
#endif
      }
      for (x = 0; x < world[next_room].x_size; x++) {
	for (y = 0; y < world[next_room].y_size; y++) {
	  what = world[next_room].map->map_grid[x][y];

	  /* add some doors */
	  for (n = 0; n < 6; n++) {
	    if (world[next_room].dir_option[n]
		&& world[next_room].dir_option[n]->to_room > 0
		&& x == world[next_room].dir_option[n]->x_cord
		&& y == world[next_room].dir_option[n]->y_cord) {
	      /* check stairs first */
	      if (n == UP && CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_STAIR_UP;
	      else if (previous_what == MAP_STAIR_UP &&
		       n == DOWN && CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_STAIR_UPDOWN;
	      else if (n == DOWN && CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_STAIR_DOWN;
	      else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
		       DOOR_IS_OPEN(next_room, obj, n) &&
		       CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_OPEN_DOOR;
	      else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
		       DOOR_IS_CLOSED(next_room, obj, n) &&
		       CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_CLOSED_DOOR;
	      else if (DOOR_IS_OPENABLE(next_room, obj, n) &&
		       DOOR_IS_OPEN(next_room, obj, n) &&
		       !CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_SECRETDOOR;
	      else if (!CAN_SEE_EXIT(ch, next_room, n))
		what = MAP_WALL;
	      else if (DRAW_WALL_ONLY(next_room, n))
		what = MAP_WALL;
	    }
	  }
	  /* Add those changable objects to the array, so checks on it
	   * will work. Later, if we write it to file again, we'll
	   * just re-convert, without these objects! 
	   */
	  world[next_room].map->map_grid[x][y] = what;

	  /* Above/beside a room */
	  if ((ROOM_FLAGGED(next_room, ROOM_INDOORS) &&
	       !IS_WALL(what) && GET_Z_SIZE(next_room) != -1) &&
	      ((GET_Z_CORD(ch) > GET_Z_SIZE(next_room)) ||	/* above */
	       (ch->in_room != next_room && GET_Z_SIZE(ch->in_room) == -1 && OUTSIDE(ch))))	/* beside */
	    what = MAP_WOOD_ROOF;

	  /* Underwater viewings */
	  if (GET_Z_CORD(ch) < 0)
	    if (what != MAP_LAKE && what != MAP_RIVER)
	      what = MAP_NOTBLANK;

	  /* Some folks see the trails in the forest better */
	  if (what == MAP_BRUSH && (GET_CLASS(ch) == CLASS_RANGER ||
				    GET_CLASS(ch) == CLASS_DRUID))
	    what = MAP_FIELD;

	  /* pass it to a contents function, which will either leave the icon
	   * alone, or change it to a mob/object etc and return the new icon
	   */
	  what = whats_in_maproom(ch, what, next_room, x, y);

	  if (what > MAP_ERROR)
	    what = MAP_ERROR;
	  /* don't ever want to overwrite yourself */
	  if (GET_X_CORD(ch) == x &&
	      GET_Y_CORD(ch) == y && dir == -1 && dif_room == -1)
	    what = MAP_ME;
	  /* keep everything within the map. Also, a good spot to check
	   * for light sources, etc affecting total shown.
	   */

#if 0
	  c_x = x + startx;
	  c_y = y + starty;
	  t_x = t_y = size / 2;
	  c_z = t_z = 0;
	  dis_bet = sqrt(pow((c_x - t_x), 2) + pow((c_y - t_y), 2)
			 + pow((c_z - t_z), 2));
	  if (dis_bet < (size / 2))
#endif

	    if ((x + startx) < size && (y + starty) < (size) &&
		(x + startx) > 0 && (y + starty) > 0) {
	      if ((the_map[x + startx][y + starty] == MAP_BLANK ||
		what == MAP_WALL || what == MAP_ME || what == MAP_MOB) &&
		  (the_map[x + startx][y + starty] != MAP_ME &&
		   the_map[x + startx][y + starty] != MAP_MOB &&
		   the_map[x + startx][y + starty] != MAP_BORDER2)) {
		/* don't overwrite previously drawn, unless its a wall */
		the_map[x + startx][y + starty] = what;
		previous_what = what;
	      }
	    }
	}
      }
      break;			/* end of wilderness shape */
      /* add other room shapes here */
    }
  }

  for (n = 0; n <= (size); n++) {
    for (i = 0; i <= (size); i++) {
//      if (IS_CLIENT_TERM(ch->desc))
//	sprintf(Gbuf, "%c", the_map[n][i]);
//      else {
	sprintf(Gbuf, "%s", map_info[the_map[n][i]].icon);
	/* if this color isn't the same as last time, terminate last, add new */
	color = the_map[n][i];
	if (color != previous_color)
	  sprintf(Gbuf, "&n%s%s", map_info[the_map[n][i]].color, map_info[the_map[n][i]].icon);
	previous_color = color;
//      }

      send_to_char(Gbuf, ch);
      if (i == ((size)))
	send_to_char("\r\n", ch);
      if (i == size && n == size)
	send_to_char("&n", ch);
    }
  }
  if (IS_CLIENT_TERM(ch->desc)) {
    sprintf(Gbuf, "%s\r\n", CLIENT_SUFFIX);
    send_to_char(Gbuf, ch);
  }
}


ACMD(do_zonemap)
{
  int start = (world[ch->in_room].number / 100), room_to_show = 0;
  int x = 0, y = 0;

  if (IS_TRUSTED(ch)) {
    sprintf(buf, "STARTZONE %d\r\n", (start - 22));
    send_to_char(buf, ch);
  }
  for (y = 5; y < 25; y++) {
    if (y < 10) {
      for (x = 0; x < 30; x++) {
	if (x < 10)
	  room_to_show = (start - 22) * 100 + x + (y * 10);
	else if (x < 20)
	  room_to_show = (start - 21) * 100 + x + (y * 10);
	else
	  room_to_show = (start - 20) * 100 + x + (y * 10);


	if (room_to_show < 100000 || room_to_show > 350099 || real_room(room_to_show) <= 1)
	  room_to_show = 0;
	room_to_show = real_room(room_to_show);
	sprintf(buf, "%s%s", map_info[FloorType(room_to_show)].color, map_info[FloorType(room_to_show)].icon);
	send_to_char(buf, ch);

	if (x == 29)
	  send_to_char("\r\n", ch);
      }
    } else if (y < 20) {
      for (x = 0; x < 30; x++) {
	if (x < 10)
	  room_to_show = (start - 1) * 100 + x + ((y - 10) * 10);
	else if (x < 20)
	  room_to_show = (start) * 100 + x + ((y - 10) * 10);
	else
	  room_to_show = (start + 1) * 100 + x + ((y - 10) * 10);


	if (room_to_show < 100000 || room_to_show > 350099 || real_room(room_to_show) <= 1)
	  room_to_show = 0;
	room_to_show = real_room(room_to_show);
	if (world[room_to_show].number == world[ch->in_room].number)
	  send_to_char("&+WX&n", ch);
	else
	  sprintf(buf, "%s%s", map_info[FloorType(room_to_show)].color, map_info[FloorType(room_to_show)].icon);
	send_to_char(buf, ch);
	if (x == 29)
	  send_to_char("\r\n", ch);
      }
    } else {
      for (x = 0; x < 30; x++) {
	if (x < 10)
	  room_to_show = (start + 20) * 100 + x + ((y - 20) * 10);
	else if (x < 20)
	  room_to_show = (start + 21) * 100 + x + ((y - 20) * 10);
	else
	  room_to_show = (start + 22) * 100 + x + ((y - 20) * 10);


	if (room_to_show < 100000 || room_to_show > 350099 || real_room(room_to_show) <= 1)
	  room_to_show = 0;
	room_to_show = real_room(room_to_show);
	sprintf(buf, "%s%s", map_info[FloorType(room_to_show)].color, map_info[FloorType(room_to_show)].icon);
	send_to_char(buf, ch);
	if (x == 29)
	  send_to_char("\r\n", ch);
      }
    }
  }
}

void maplook(struct char_data *ch, sh_int size, int dif_room, int type)
{
  struct char_data *to = NULL;
  int distance = 3;

  if (type == TO_CHAR) {
    if (ch && SENDOK(ch))
      smallmap_look(ch, size, dif_room);
    return;
  }
  if (ch && ch->in_room != NOWHERE)
    to = world[ch->in_room].people;
  else {
    log("SYSERR: no valid target to maplook()!");
    return;
  }

  if (type == TO_PROXIMITY || type == TO_PROXIMITY2 || type == TO_PROXIMITY3) {
    for (; to; to = to->next_in_room) {
      if (type == TO_PROXIMITY2)
	distance *= 2;
      else if (type == TO_PROXIMITY3)
	distance *= 3;
      if (SENDOK(to) && (to != ch)	/* && 
					   WithinProximity(ch, NULL, to, NULL, NULL, -1, &distance, &dir, FALSE) */ )
	smallmap_look(ch, size, dif_room);
    }
    return;
  }
  for (; to; to = to->next_in_room)
    if (SENDOK(to) && (to != ch) &&
	(type == PROXIMITY))
      smallmap_look(ch, size, dif_room);

}

void maproom_to_mapgrid(int room)
{
  int x, y, what;
  int currx, curry, room_shape;
  int next_room;


  currx = world[room].x_size;
  curry = world[room].y_size;
  room_shape = world[room].shape;
  next_room = room;
  what = MAP_NOTBLANK;		/* This makes it so circular rooms etc, still 
				   occupy a full x*y square. We need to prevent
				   moves outside the walls. */

  switch (room_shape) {
    /* room contents primarily will use next_room, so we don't change room */
  case ROOM_SHAPE_SQUARE:

    for (x = 0; x < currx; x++) {
      for (y = 0; y < curry; y++) {

	/* draw some walls. Complex because we may only need a partial wall */
	if (x == 0) {
	  /* no wall */
	  if (world[next_room].dir_option[NORTH] &&
	      !IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR) &&
	      !IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED)
	      && y != 0 && y != (curry - 1))
	    what = FloorType(next_room);
	  /* corner square */
	  else if (world[next_room].dir_option[EAST] &&
		   (IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR) ||
		    IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED))
		   && y == curry - 1)
	    what = MAP_WALL;
	  /* corner square */
	  else if (world[next_room].dir_option[WEST] &&
		   (IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR) ||
		    IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED))
		   && y == 0)
	    what = MAP_WALL;
	  /* next to interior room */
	  else if (SECT(next_room) == SECT_INSIDE
		   || (world[next_room].dir_option[NORTH] && SECT(world[next_room].dir_option[NORTH]->to_room) == SECT_INSIDE))
	    what = MAP_WALL;
	  /* fake wall */
	  else if (world[next_room].dir_option[NORTH] && IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED))
	    what = MAP_WALL;
	  /* real mccoy */
	  else if (world[next_room].dir_option[NORTH] &&
		   IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR))
	    what = MAP_WALL;
	  /* default */
	  else
	    what = FloorType(next_room);
	} else if (x == (currx - 1)) {
	  if (world[next_room].dir_option[SOUTH] &&
	      !IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR) &&
	      !IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED)
	      && y != 0 && y != (curry - 1))
	    what = FloorType(next_room);
	  else if (world[next_room].dir_option[EAST] &&
		   (IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR) ||
		    IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED))
		   && y == curry - 1)
	    what = MAP_WALL;
	  else if (world[next_room].dir_option[WEST] &&
		   (IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR) ||
		    IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED))
		   && y == 0)
	    what = MAP_WALL;
	  else if (SECT(next_room) == SECT_INSIDE
		   || (world[next_room].dir_option[SOUTH] && SECT(world[next_room].dir_option[SOUTH]->to_room) == SECT_INSIDE))
	    what = MAP_WALL;
	  else if (world[next_room].dir_option[SOUTH] && IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED))
	    what = MAP_WALL;
	  else if (world[next_room].dir_option[SOUTH] &&
		   IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR))
	    what = MAP_WALL;
	  else
	    what = FloorType(next_room);
	} else if (y == 0) {
	  if (world[next_room].dir_option[WEST] &&
	      !IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR) &&
	  !IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED)
	      && x != 0 && x != (currx - 1))
	    what = FloorType(next_room);
	  else if (world[next_room].dir_option[NORTH] &&
		   (IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR) ||
		    IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED))
		   && x == 0)
	    what = MAP_WALL;
	  else if (world[next_room].dir_option[SOUTH] &&
		   (IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR) ||
		    IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED))
		   && x == currx - 1)
	    what = MAP_WALL;
	  else if (SECT(next_room) == SECT_INSIDE
		   || (world[next_room].dir_option[WEST] && SECT(world[next_room].dir_option[WEST]->to_room) == SECT_INSIDE))
	    what = MAP_WALL;
	  else if (world[next_room].dir_option[WEST] && IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_WALLED))
	    what = MAP_WALL;
	  else if (world[next_room].dir_option[WEST] &&
	  IS_SET(world[next_room].dir_option[WEST]->exit_info, EX_ISDOOR))
	    what = MAP_WALL;
	  else
	    what = FloorType(next_room);
	} else if (y == (curry - 1)) {
	  if (world[next_room].dir_option[EAST] &&
	      !IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR) &&
	  !IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED)
	      && x != 0 && x != (currx - 1))
	    what = FloorType(next_room);
	  else if (world[next_room].dir_option[NORTH] &&
		   (IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_ISDOOR) ||
		    IS_SET(world[next_room].dir_option[NORTH]->exit_info, EX_WALLED))
		   && x == 0)
	    what = MAP_WALL;
	  else if (world[next_room].dir_option[SOUTH] &&
		   (IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_ISDOOR) ||
		    IS_SET(world[next_room].dir_option[SOUTH]->exit_info, EX_WALLED))
		   && x == currx - 1)
	    what = MAP_WALL;
	  else if (SECT(next_room) == SECT_INSIDE
		   || (world[next_room].dir_option[EAST] && SECT(world[next_room].dir_option[EAST]->to_room) == SECT_INSIDE))
	    what = MAP_WALL;
	  else if (world[next_room].dir_option[EAST] && IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_WALLED))
	    what = MAP_WALL;
	  else if (world[next_room].dir_option[EAST] &&
	  IS_SET(world[next_room].dir_option[EAST]->exit_info, EX_ISDOOR))
	    what = MAP_WALL;
	  else
	    what = FloorType(next_room);
	} else {
	  what = FloorType(next_room);
	}
	/* end of wall drawing */

	world[next_room].map->map_grid[x][y] = what;
      }
    }
    break;			/* end of SQUARE shape */
  case ROOM_SHAPE_PCMAP:
  case ROOM_SHAPE_WILDERNESS:
    break;
    /* add other room shapes here */
  }

}

/* yes or no, is the map symbol the same in each square? */
byte SameSectorType(struct char_data *ch, int door)
{

  return YES;
}

void unload_room_maps(int zone)
{
  int counter, realcounter, i;
  int vzone = zone_table[zone].top / 100;
  struct olc_save_info *entry;

  if (zone < 150)		/* Dont unload common zones */
    return;

  if ((zone >= 450) && (zone <= 459))	/* ship rooms */
    return;

  if (zone_table[zone].unloaded == TRUE)	/* already unloaded */
    return;
/*
   sprintf(buf, "Unloading room maps for zone %d.",
   (int) (zone_table[zone].top / 100));
   log(buf);
 */

/*
   Dont unload zones that have room changes
 */
  if (olc_save_list) {
    for (entry = olc_save_list; entry; entry = entry->next) {
      if ((entry->zone == vzone) &&
	  ((int) entry->type == SCMD_OLC_REDIT))
	return;
    }
  }
  zone_table[zone].unloaded = TRUE;

  for (counter = zone_table[zone].number * 100;
       counter <= zone_table[zone].top;
       counter++) {
    realcounter = real_room(counter);
    if (realcounter >= 0 && (world[realcounter].map != NULL)) {
#ifdef NEWMAP
      for (i = 0; i < world[realcounter].x_size + 1; i++)
	FREE(world[realcounter].map->map_grid[i]);
      FREE(world[realcounter].map->map_grid);
#endif
      FREE(world[realcounter].map);
      world[realcounter].map = NULL;
    }
  }
}


void reparse_maps(FILE * fl, int virt_nr)
{
  char line[256];
  char *room_name, *room_desc;
  int real_nr = real_room(virt_nr);

  room_name = fread_string(fl, buf2);
  if (str_cmp(room_name, world[real_nr].name)) {	/* shrug */
    log("SYSERR: Read name does not equal room name.");
    exit(1);
  }
  free(room_name);		/* fread_string allocates mem..so free it */
  room_desc = fread_string(fl, buf2);
  free(room_desc);
/* 3 int lines */
  get_line(fl, buf2);
  get_line(fl, buf2);
  get_line(fl, buf2);
/* letters */
  if (world[real_nr].map == NULL) {
    for (;;) {
      if (!get_line(fl, line)) {
	sprintf(buf, "SYSERR: Format error when reloading roommaps for %d.wld\n",
		world[real_nr].zone);
	log(buf);
	exit(1);
      }
      if (*line == 'M' && *(line + 1) == '\0')
	generate_mapgrid(fl, real_nr);
      if (*line == 'D' && *(line + 2) == '\0') {
	get_line(fl, line);
	get_line(fl, line);
	get_line(fl, line);
      }
      if (*line == 'S' && *(line + 1) == '\0')	/* continue to the end of the record */
	return;
    }

  } else {
    sprintf(buf, "SYSERR: Attempting to reload Non-NULL map for room %d.",
	    world[real_nr].number);
    log(buf);
    return;
  }
}

void reload_room_maps(int real_nr)
{
  FILE *db_file;
  int nr = -1, last = 0;
  int zone = (int) (world[real_nr].number / 100);
  char line[256];

  sprintf(buf, "Reloading room maps for zone %d.", zone);
  log(buf);

  sprintf(buf1, "%d.wld", zone);

  zone_table[world[real_nr].zone].unloaded = FALSE;
  zone_table[world[real_nr].zone].age = 0;

  sprintf(buf2, "%s/%s", WLD_PREFIX, buf1);
  if (!(db_file = fopen(buf2, "r"))) {
    perror(buf2);
    exit(1);
  }
  for (;;) {
    if (!get_line(db_file, line)) {
      fprintf(stderr, "Format error after world #%d\n", nr);
      exit(1);
    }
    if (*line == '$')
      break;

    if (*line == 'T')		/* triggers come after end of room marker */
      continue;

    if (*line == '#') {
      last = nr;
      if (sscanf(line, "#%d", &nr) != 1) {
	fprintf(stderr, "Format error after world #%d\n", last);
	exit(1);
      }
      if (nr >= 9999999)
	break;
      else
	reparse_maps(db_file, nr);
    } else if (world[real_nr].shape == ROOM_SHAPE_WILDERNESS ||
	       world[real_nr].shape == ROOM_SHAPE_PCMAP) {
      sprintf(buf, "Format error in world file near room #%d", last + 1);
      sprintf(buf + strlen(buf), " Offending line: '%s'", line);
      mudlog(buf, BRF, LVL_GOD, TRUE);
      /* Error, notify */
      zone_table[world[real_nr].zone].unloaded = TRUE;
      sprintf(buf, "SYSERR: Maps for zone %d Room: %d were unable to be re-loaded.", world[real_nr].zone, world[real_nr].number);
      mudlog(buf, BRF, LVL_GOD, TRUE);
      fclose(db_file);
      return;
    }
  }

  fclose(db_file);
}
