
/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#define _OASIS_OLC_

#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "interpreter.h"
#include "comm.h"
#include "utils.h"

#include "db.h"
#include "olc.h"
#include "quest.h"
#include "dg_olc.h"

 /*
  * External data structures.
  */
extern struct obj_data *obj_proto;
extern struct char_data *mob_proto;
extern struct room_data *world;
extern int top_of_zone_table;
extern struct social_messg *soc_mess_list;
extern int top_of_socialt;
extern struct zone_data *zone_table;
extern struct descriptor_data *descriptor_list;
extern struct help_index_element *help_table;
extern int top_of_helpt;


 /*
  * External functions.
  */
extern int find_help(char *keyword);
extern void zedit_setup(struct descriptor_data *d, int room_num);
extern void zedit_save_to_disk(int zone);
extern void zedit_save_internally(struct descriptor_data *d);
extern void zedit_new_zone(struct char_data *ch, int new_zone);
extern void medit_setup_new(struct descriptor_data *d);
extern void medit_setup_existing(struct descriptor_data *d, int rmob_num);
extern void medit_save_to_disk(int zone);
extern void redit_setup_new(struct descriptor_data *d);
extern void redit_setup_existing(struct descriptor_data *d, int rroom_num);
extern void redit_save_to_disk(int zone);
extern void oedit_setup_new(struct descriptor_data *d);
extern void oedit_setup_existing(struct descriptor_data *d, int robj_num);
extern void oedit_save_to_disk(int zone);
extern void sedit_setup_new(struct descriptor_data *d);
extern void sedit_setup_existing(struct descriptor_data *d, int robj_num);
extern void sedit_save_to_disk(int zone);
extern void qedit_setup_new(struct descriptor_data *d);
extern void qedit_setup_existing(struct descriptor_data *d, int real_num);
extern void qedit_save_to_disk(int znum);
extern void hedit_save_to_disk(struct descriptor_data *d);
extern int real_shop(int vnum);
extern void free_shop(struct shop_data *shop);
extern void free_room(struct room_data *room);
extern void aedit_save_to_disk(struct descriptor_data *d);
extern int find_action(int cmd);
extern void free_quest(struct aq_data *quest);
extern void medit_free_mobile(struct char_data *mob);
extern void free_help(struct help_index_element *help);
extern void free_action(struct social_messg *action);
extern void trigedit_setup_new(struct descriptor_data *d);
extern void trigedit_setup_existing(struct descriptor_data *d, int rtrg_num);
extern int real_trigger(int vnum);
extern int already_being_edited(int cnum);
extern void setup_new_guild(struct char_data *ch, int cnum);
extern void setup_guild_edit(struct char_data *ch, int cnum);
extern int is_guildnum_already_used(int cnum);

 /*
  * Internal function prototypes.
  */
int real_zone(int number);
void create_zone_from_loads(struct descriptor_data *d, int znum);
void olc_saveinfo(struct char_data *ch);

 /*
  * Global string constants.
  */
const char *save_info_msg[8] =
{"Rooms", "Objects", "Zone info",
 "Mobiles", "Shops", "Quests", "HelpFiles", "Actions"};

 /*
  * Internal data structures.
  */
struct olc_scmd_data {
  char *text;
  int con_type;
};

struct olc_scmd_data olc_scmd_info[] =
{
  {"room", CON_REDIT},
  {"object", CON_OEDIT},
  {"room", CON_ZEDIT},
  {"mobile", CON_MEDIT},
  {"shop", CON_SEDIT},
  {"quest", CON_QEDIT},
  {"help", CON_HEDIT},
  {"action", CON_AEDIT},
  {"triggers", CON_TEDIT},
  {"guilds", CON_GEDIT},
  {"\n", 0}
};

/*------------------------------------------------------------*/

 /*
  * Exported ACMD do_olc function.
  *
  * This function is the OLC interface.  It deals with all the 
  * generic OLC stuff, then passes control to the sub-olc sections.
  */

ACMD(do_olc)
{
  int number = -1, save = 0, real_num;
  int delete = 0, create = 0;
  struct descriptor_data *d;
  char buf3[128];
  /*
   * No screwing around as a mobile.
   */
  if (IS_NPC(ch))
    return;

  /*
   * Parse any arguments.
   */
  two_arguments(argument, buf1, buf2);
  two_arguments(buf2, buf2, buf3);

  if (!*buf1) {			/* No argument given. */
    switch (subcmd) {
    case SCMD_OLC_SAVEINFO:
      olc_saveinfo(ch);
      return;
    case SCMD_OLC_REDIT:
    case SCMD_OLC_ZEDIT:
      number = world[IN_ROOM(ch)].number;
      break;
    case SCMD_OLC_OEDIT:
    case SCMD_OLC_MEDIT:
    case SCMD_OLC_QEDIT:
    case SCMD_OLC_SEDIT:
    case SCMD_OLC_TEDIT:
    case SCMD_OLC_GEDIT:
      sprintf(buf, "Specify a %s VNUM to edit.\r\n", olc_scmd_info[subcmd].text);
      send_to_char(buf, ch);
      return;
    case SCMD_OLC_HEDIT:
      send_to_char("Specify a help topic to edit.\r\n", ch);
      return;
    case SCMD_OLC_AEDIT:
      send_to_char("Specify an action to edit.\r\n", ch);
      return;
    }
  } else if (!isdigit(*buf1)) {
    if (strn_cmp("delete", buf1, 6) == 0) {
      if (!*buf2) {
	send_to_char("Delete what?\r\n", ch);
	return;
      }
      if (subcmd == SCMD_OLC_OEDIT || subcmd == SCMD_OLC_MEDIT || subcmd == SCMD_OLC_TEDIT) {
	delete = 1;
	number = atoi(buf2);
      } else if (subcmd == SCMD_OLC_AEDIT || subcmd == SCMD_OLC_HEDIT) {
	delete = 1;
	number = 0;
      }
    } else if ((strn_cmp("create", buf1, 6) == 0) && subcmd == SCMD_OLC_ZEDIT) {
      create = 1;
      number = world[IN_ROOM(ch)].number;
    } else if (strn_cmp("save", buf1, 4) == 0) {
      if (subcmd == SCMD_OLC_HEDIT || subcmd == SCMD_OLC_AEDIT) {
	save = 1;
	number = 0;
      } else if (!*buf2) {
	if (GET_OLC_ZONE(ch)) {
	  save = 1;
	  number = (GET_OLC_ZONE(ch) * 100);
	} else {
	  send_to_char("Save which zone?\r\n", ch);
	  return;
	}
      } else {
	save = 1;
	number = atoi(buf2) * 100;
      }
    } else if ((strn_cmp("open", buf1, 4) == 0) && subcmd == SCMD_OLC_ZEDIT) {
      if (!*buf2) {
	if (GET_OLC_ZONE(ch)) {
	  save = 1;
	  number = (GET_OLC_ZONE(ch) * 100);
	} else {
	  send_to_char("Open which zone?\r\n", ch);
	  return;
	}
      } else {
	save = 1;
	number = atoi(buf2) * 100;
      }
      zone_table[real_zone(number)].closed = 0;
    } else if ((strn_cmp("close", buf1, 5) == 0) && subcmd == SCMD_OLC_ZEDIT) {
      if (!*buf2) {
	if (GET_OLC_ZONE(ch)) {
	  save = 1;
	  number = (GET_OLC_ZONE(ch) * 100);
	} else {
	  send_to_char("Close which zone?\r\n", ch);
	  return;
	}
      } else {
	save = 1;
	number = atoi(buf2) * 100;
      }
      zone_table[real_zone(number)].closed = 1;
    } else if (subcmd == SCMD_OLC_HEDIT || subcmd == SCMD_OLC_AEDIT) {
      number = 0;
    } else if (subcmd == SCMD_OLC_ZEDIT && GET_LEVEL(ch) >= LVL_GOD6) {
      if ((strn_cmp("new", buf1, 3) == 0) && *buf2)
	zedit_new_zone(ch, atoi(buf2));
      else
	send_to_char("Specify a new zone number.\r\n", ch);
      return;
    } else {
      send_to_char("Yikes!  Stop that, someone will get hurt!\r\n", ch);
      return;
    }
  }
  /*
   * If a numeric argument was given, get it.
   */
  if ((number == -1) && (subcmd != SCMD_OLC_HEDIT) && (subcmd != SCMD_OLC_AEDIT))
    number = atoi(buf1);

  /*
   * Check that whatever it is isn't already being edited.
   */
  for (d = descriptor_list; d; d = d->next)
    if (d->connected == olc_scmd_info[subcmd].con_type)
      if (d->olc && OLC_NUM(d) == number && number != NOTHING) {
	if (subcmd == SCMD_OLC_HEDIT)
	  sprintf(buf, "Help files are currently being edited by %s\r\n",
		  (CAN_SEE(ch, d->character) ? GET_NAME(d->character) : "someone"));
	else if (subcmd == SCMD_OLC_AEDIT)
	  sprintf(buf, "Actions are already being editted by %s.\r\n",
		  (CAN_SEE(ch, d->character) ? GET_NAME(d->character) : "someone"));
	else
	  sprintf(buf, "That %s is currently being edited by %s.\r\n",
		  olc_scmd_info[subcmd].text, GET_NAME(d->character));
	send_to_char(buf, ch);
	return;
      }
  d = ch->desc;

  /*
   * Give descriptor an OLC struct.
   */
  CREATE(d->olc, struct olc_data, 1);

  if (subcmd != SCMD_OLC_HEDIT && subcmd != SCMD_OLC_AEDIT) {
    /*. Find the zone . */
    OLC_ZNUM(d) = real_zone(number);
    if (OLC_ZNUM(d) == -1) {
      send_to_char("Sorry, there is no zone for that number!\r\n", ch);
      FREE(d->olc);
      return;
    }
    /*
     * Everyone but IMPLs can only edit zones they have been assigned.
     */

#if 0
    if ((GET_LEVEL(ch) < LVL_BUILDER + 1) && (subcmd == SCMD_OLC_OEDIT ||
					      subcmd == SCMD_OLC_MEDIT)) {
      send_to_char("You may only edit items from the MDB, using the copyo or copym commands.\r\n", ch);
      FREE(d->olc);
      return;
    }
    if ((GET_LEVEL(ch) < LVL_IMPL) && cmd != LVL_IMPL &&
	(zone_table[OLC_ZNUM(d)].number != GET_OLC_ZONE(ch))) {
      send_to_char("You do not have permission to edit this zone.\r\n", ch);
      FREE(d->olc);
      return;
    }
#endif
  } else if ((!(GET_LEVEL(ch) < LVL_IMPL)) &&
	     (!ch->player_specials->saved.olc_zone == AEDIT_PERMISSION)) {
    send_to_char("You do not have permission to edit actions.\r\n", ch);
    FREE(d->olc);
    return;
  } else if ((!(GET_LEVEL(ch) < LVL_BUILDER)) &&
	     (!ch->player_specials->saved.olc_zone == HEDIT_PERMISSION)) {
    send_to_char("You do not have permission to edit help files.\r\n", ch);
    FREE(d->olc);
    return;
  }
  if (delete) {
    switch (subcmd) {
    case SCMD_OLC_OEDIT:
      if ((real_num = real_object(number)) >= 0) {
	OLC_NUM(d) = number;
	sprintf(buf, "Do you wish to delete object (%d) %s? ", OLC_NUM(d), obj_proto[real_num].short_description);
	send_to_char(buf, ch);
	OLC_MODE(d) = OEDIT_CONFIRM_DELETE;
	STATE(d) = CON_OEDIT;
      } else
	send_to_char("Number not valid!\r\n", ch);

      break;
    case SCMD_OLC_TEDIT:
      if ((real_num = real_trigger(number)) >= 0) {
	OLC_NUM(d) = number;
	sprintf(buf, "Do you wish to delete trigger (%d)? ", OLC_NUM(d));
	send_to_char(buf, ch);
	OLC_MODE(d) = TEDIT_CONFIRM_DELETE;
	STATE(d) = CON_TEDIT;
      } else
	send_to_char("Number not valid!\r\n", ch);

      break;
    case SCMD_OLC_MEDIT:
      if ((real_num = real_mobile(number)) >= 0) {
	OLC_NUM(d) = number;
	sprintf(buf, "Do you wish to delete mobile (%d) %s? ", OLC_NUM(d), mob_proto[real_num].player.name);
	send_to_char(buf, ch);
	OLC_MODE(d) = MEDIT_CONFIRM_DELETE;
	STATE(d) = CON_MEDIT;
      } else
	send_to_char("Number not valid!\r\n", ch);

      break;
    case SCMD_OLC_AEDIT:
      OLC_NUM(d) = 0;
      OLC_STORAGE(d) = str_dup(buf2);
      for (OLC_ZNUM(d) = 0; (OLC_ZNUM(d) <= top_of_socialt); OLC_ZNUM(d)++) {
	if (is_abbrev(OLC_STORAGE(d), soc_mess_list[OLC_ZNUM(d)].command))
	  break;
      }
      if (OLC_ZNUM(d) > top_of_socialt) {
	send_to_char("That action dont exist!\r\n", ch);
	return;
      } else {
	sprintf(buf, "Do you wish to delete the '%s' action? ", soc_mess_list[OLC_ZNUM(d)].command);
	send_to_char(buf, ch);
	OLC_MODE(d) = AEDIT_CONFIRM_DELETE;
	STATE(d) = CON_AEDIT;
      }
      break;
    case SCMD_OLC_HEDIT:
      OLC_NUM(d) = 0;
      OLC_STORAGE(d) = str_dup(buf2);
      for (OLC_ZNUM(d) = 0; (OLC_ZNUM(d) <= top_of_helpt); OLC_ZNUM(d)++) {
	if (is_abbrev(OLC_STORAGE(d), help_table[OLC_ZNUM(d)].keyword))
	  break;
      }
      if (OLC_ZNUM(d) > top_of_helpt) {
	send_to_char("This help topic dont exist!\r\n", ch);
	return;
      } else {
	if (help_table[OLC_ZNUM(d)].save == FALSE) {
	  send_to_char("You cannot delete this topic!\r\n", ch);
	  return;
	} else {
	  sprintf(buf, "Do you wish to delete the help topic on '%s'? ",
		  help_table[OLC_ZNUM(d)].keyword);
	  send_to_char(buf, ch);
	  OLC_MODE(d) = HEDIT_CONFIRM_DELETE;
	  STATE(d) = CON_HEDIT;
	}
      }
      break;
    }
    return;

  } else if (save) {
    const char *type = NULL;

    if (*buf3) {
      if (strn_cmp("wld", buf1, 3) == 0)
	type = "room";
      else if (strn_cmp("room", buf1, 4) == 0)
	type = "room";
      else if (strn_cmp("zon", buf1, 3) == 0)
	type = "zone";
      else if (strn_cmp("shp", buf1, 3) == 0)
	type = "shop";
      else if (strn_cmp("mob", buf1, 3) == 0)
	type = "mobile";
      else if (strn_cmp("obj", buf1, 3) == 0)
	type = "object";
      else if (strn_cmp("qst", buf1, 3) == 0)
	type = "quest";
      else if (strn_cmp("hlp", buf1, 3) == 0)
	type = "help";
      else if (strn_cmp("soc", buf1, 3) == 0)
	type = "social";
      else if (strn_cmp("act", buf1, 3) == 0)
	type = "social";
      else if (strn_cmp("trg", buf1, 3) == 0)
	type = "triggers";
    } else {
      switch (subcmd) {
      case SCMD_OLC_REDIT:
	type = "room";
	break;
      case SCMD_OLC_ZEDIT:
	type = "state";
	break;
      case SCMD_OLC_SEDIT:
	type = "shop";
	break;
      case SCMD_OLC_MEDIT:
	type = "mobile";
	break;
      case SCMD_OLC_OEDIT:
	type = "object";
	break;
      case SCMD_OLC_QEDIT:
	type = "quest";
	break;
      case SCMD_OLC_HEDIT:
	type = "help";
	break;
      case SCMD_OLC_AEDIT:
	type = "social";
	break;
      case SCMD_OLC_TEDIT:
	type = "triggers";
	break;
      }
    }
    if (!type) {
      send_to_char("Oops, I forgot what you wanted to save.\r\n", ch);
      return;
    }
    sprintf(buf, "Saving all %ss in zone %d.\r\n",
	    type, zone_table[OLC_ZNUM(d)].number);
    if (GET_LEVEL(ch) >= LVL_BUILDER)
      send_to_char(buf, ch);
    sprintf(buf, "OLC: %s saves %s info for zone %d.", GET_NAME(ch), type,
	    zone_table[OLC_ZNUM(d)].number);
    mudlog(buf, CMP, MAX(LVL_BUILDER, GET_INVIS_LEV(ch)), TRUE);

    switch (subcmd) {
    case SCMD_OLC_REDIT:
      redit_save_to_disk(OLC_ZNUM(d));
      break;
    case SCMD_OLC_ZEDIT:
      zedit_save_to_disk(OLC_ZNUM(d));
      break;
    case SCMD_OLC_OEDIT:
      oedit_save_to_disk(OLC_ZNUM(d));
      break;
    case SCMD_OLC_MEDIT:
      medit_save_to_disk(OLC_ZNUM(d));
      break;
    case SCMD_OLC_SEDIT:
      sedit_save_to_disk(OLC_ZNUM(d));
      break;
    case SCMD_OLC_QEDIT:
      qedit_save_to_disk(OLC_ZNUM(d));
      break;
    case SCMD_OLC_HEDIT:
      hedit_save_to_disk(d);
      break;
    case SCMD_OLC_AEDIT:
      aedit_save_to_disk(d);
      break;
    case SCMD_OLC_TEDIT:
      break;
    }
    FREE(d->olc);
    return;
  }
  OLC_NUM(d) = number;

  if (subcmd == SCMD_OLC_AEDIT) {
    OLC_NUM(d) = 0;
    OLC_PROTO(d) = 0;
    OLC_STORAGE(d) = str_dup(buf1);
    for (OLC_ZNUM(d) = 0; (OLC_ZNUM(d) <= top_of_socialt); OLC_ZNUM(d)++) {
      if (is_abbrev(OLC_STORAGE(d), soc_mess_list[OLC_ZNUM(d)].command))
	break;
    }
    if (OLC_ZNUM(d) > top_of_socialt) {
      if (find_command(OLC_STORAGE(d)) > NOTHING) {
	cleanup_olc(d, CLEANUP_ALL);
	send_to_char("That command already exists.\r\n", ch);
	return;
      }
      sprintf(buf, "Do you wish to add the '%s' action? ", OLC_STORAGE(d));
      send_to_char(buf, ch);
      OLC_MODE(d) = AEDIT_CONFIRM_ADD;
    } else {
      sprintf(buf, "Do you wish to edit the '%s' action? ", soc_mess_list[OLC_ZNUM(d)].command);
      send_to_char(buf, ch);
      OLC_MODE(d) = AEDIT_CONFIRM_EDIT;
    }
  }
  if (subcmd == SCMD_OLC_HEDIT) {
    OLC_NUM(d) = 0;
    OLC_PROTO(d) = 0;
    OLC_STORAGE(d) = str_dup(buf1);
    for (OLC_ZNUM(d) = 0; (OLC_ZNUM(d) <= top_of_helpt); OLC_ZNUM(d)++) {
/*      if (is_abbrev(OLC_STORAGE(d), help_table[OLC_ZNUM(d)].keyword)) */
      if (!str_cmp(OLC_STORAGE(d), help_table[OLC_ZNUM(d)].keyword))
	break;
    }
    if (OLC_ZNUM(d) > top_of_helpt) {
      if (find_help(OLC_STORAGE(d)) > NOTHING) {
	cleanup_olc(d, CLEANUP_ALL);
	send_to_char("That help topic already exists.\r\n", ch);
	return;
      }
      sprintf(buf, "Do you wish to add a help topic on '%s'? ", OLC_STORAGE(d));
      send_to_char(buf, ch);
      OLC_MODE(d) = HEDIT_CONFIRM_ADD;
    } else {
      sprintf(buf, "Do you wish to edit the help topic on '%s'? ",
	      help_table[OLC_ZNUM(d)].keyword);
      send_to_char(buf, ch);
      OLC_MODE(d) = HEDIT_CONFIRM_EDIT;
    }
  }
  /*
   * Steal player's descriptor start up subcommands.
   */
  switch (subcmd) {
  case SCMD_OLC_REDIT:
    if ((real_num = real_room(number)) >= 0)
      redit_setup_existing(d, real_num);
    else
      redit_setup_new(d);
    STATE(d) = CON_REDIT;
    break;
  case SCMD_OLC_ZEDIT:
    if ((real_num = real_room(number)) < 0) {
      send_to_char("That room does not exist.\r\n", ch);
      FREE(d->olc);
      return;
    }
    if (create) {
      if (!*buf2) {
	if (GET_OLC_ZONE(ch)) {
	  number = (GET_OLC_ZONE(ch));
	} else {
	  send_to_char("Create which zone?\r\n", ch);
	  return;
	}
      } else {
	number = atoi(buf2);
      }
      create_zone_from_loads(ch->desc, number);
      zedit_save_internally(ch->desc);
/*      zedit_save_to_disk(number); */
      FREE(d->olc);
      send_to_char("Zones status written to .zon file\r\n", ch);
      return;
    }
    zedit_setup(d, real_num);
    STATE(d) = CON_ZEDIT;
    break;
  case SCMD_OLC_MEDIT:
    if ((real_num = real_mobile(number)) < 0)
      medit_setup_new(d);
    else
      medit_setup_existing(d, real_num);
    STATE(d) = CON_MEDIT;
    break;
  case SCMD_OLC_TEDIT:
    if ((real_num = real_trigger(number)) < 0)
      trigedit_setup_new(d);
    else
      trigedit_setup_existing(d, real_num);
    STATE(d) = CON_TEDIT;
    break;
  case SCMD_OLC_OEDIT:
    if ((real_num = real_object(number)) >= 0)
      oedit_setup_existing(d, real_num);
    else
      oedit_setup_new(d);
    STATE(d) = CON_OEDIT;
    break;
  case SCMD_OLC_SEDIT:
    if ((real_num = real_shop(number)) >= 0)
      sedit_setup_existing(d, real_num);
    else
      sedit_setup_new(d);
    STATE(d) = CON_SEDIT;
    break;
  case SCMD_OLC_HEDIT:
    STATE(d) = CON_HEDIT;
    break;
  case SCMD_OLC_AEDIT:
    STATE(d) = CON_AEDIT;
    break;
  case SCMD_OLC_GEDIT:
    if (is_guildnum_already_used(number) == YES)
      setup_guild_edit(d->character, number);
    else
      setup_new_guild(d->character, number);
    STATE(d) = CON_GEDIT;
    break;
  case SCMD_OLC_QEDIT:
    real_num = real_quest(number);
    if (real_num >= 0)
      qedit_setup_existing(d, real_num);
    else
      qedit_setup_new(d);
    STATE(d) = CON_QEDIT;
    break;
  }
  act("$n starts using OLC.", TRUE, d->character, 0, 0, PROXIMITY);
  SET_BIT(PLR_FLAGS(ch), PLR_WRITING);
}

 /*------------------------------------------------------------*\
  Internal utilities 
 \*------------------------------------------------------------*/

void olc_saveinfo(struct char_data *ch)
{
  struct olc_save_info *entry;

  if (olc_save_list)
    send_to_char("The following OLC components need saving:-\r\n", ch);
  else
    send_to_char("The database is up to date.\r\n", ch);

  for (entry = olc_save_list; entry; entry = entry->next) {
    if (entry->type != SCMD_OLC_AEDIT) {
      sprintf(buf, " - %s for zone %d.\r\n",
	      save_info_msg[(int) entry->type],
	      entry->zone);
    } else {
      sprintf(buf, " - %s for zone %d.\r\n",
	      save_info_msg[(int) entry->type], entry->zone);
    }
    send_to_char(buf, ch);
  }
}

int real_zone(int number)
{
  int counter;

  for (counter = 0; counter <= top_of_zone_table; counter++)
    if ((number >= (zone_table[counter].number * 100)) &&
	(number <= (zone_table[counter].top)))
      return counter;

  return -1;
}

 /*------------------------------------------------------------*\
  Exported utilities 
 \*------------------------------------------------------------*/

 /*
  * Add an entry to the 'to be saved' list.
  */

void olc_add_to_save_list(int zone, byte type)
{
  struct olc_save_info *new;

  /* zone 0 seems to always wind up here, changed or not. Since
   * only those in the know would change it, let's not add it here.
   */
  if (zone == 0)
    return;
  /*
   * Return if it's already in the list.
   */
  for (new = olc_save_list; new; new = new->next)
    if ((new->zone == zone) && (new->type == type))
      return;

  CREATE(new, struct olc_save_info, 1);
  new->zone = zone;
  new->type = type;
  new->next = olc_save_list;
  olc_save_list = new;
}

 /*
  * Remove an entry from the 'to be saved' list.
  */

void olc_remove_from_save_list(int zone, byte type)
{
  struct olc_save_info **entry;
  struct olc_save_info *temp;

  for (entry = &olc_save_list; *entry; entry = &(*entry)->next)
    if (((*entry)->zone == zone) && ((*entry)->type == type)) {
      temp = *entry;
      *entry = temp->next;
      FREE(temp);
      return;
    }
}

 /*
  * Set the colour string pointers for that which this char will
  * see at color level NRM.  Changing the entries here will change 
  * the colour scheme throughout the OLC.
  */

void get_char_cols(struct char_data *ch)
{
  nrm = "&+w";
  grn = "&+g";
  cyn = "&+c";
  yel = "&+y";
  blk = "&+L";
  red = "&+r";
  blu = "&+B";
}

 /*
  * This procedure removes the '\r\n' from a string so that it may be
  * saved to a file.  Use it only on buffers, not on the original
  * strings.
  */
void strip_string(char *buffer)
{
  register char *ptr, *str;

  ptr = buffer;
  str = ptr;

  while ((*str = *ptr)) {
    str++;
    ptr++;
    if (*ptr == '\r')
      ptr++;
  }
}

 /*
  * This procdure frees up the strings and/or the structures
  * attatched to a descriptor, sets all flags back to how they
  * should be.
  */

void cleanup_olc(struct descriptor_data *d, byte cleanup_type)
{
  if (d->olc) {
    if (OLC_STORAGE(d))
      FREE(OLC_STORAGE(d));

    /* Check for a room.  */
    if (OLC_ROOM(d)) {
      /* free_room doesn't perform sanity checks, must be careful here.  */
      switch (cleanup_type) {
      case CLEANUP_ALL:
	free_room(OLC_ROOM(d));
	break;
      case CLEANUP_STRUCTS:
	FREE(OLC_ROOM(d));
	break;
      default:			/* The caller has screwed up. */
	break;
      }
    }
    if (OLC_ACTION(d)) {
      switch (cleanup_type) {
      case CLEANUP_ALL:
	free_action(OLC_ACTION(d));
	break;
      case CLEANUP_STRUCTS:
	FREE(OLC_ACTION(d));
	break;
      default:
	/* Caller has screwed up */
	break;
      }
    }
    /*
     * Check for an object.
     */
    if (OLC_OBJ(d))
      /*
       * free_obj() makes sure strings aern't part of the prototype.
       */
      free_obj(OLC_OBJ(d));

    /*
     * Check for a mob.
     */
    if (OLC_MOB(d))
      /*
       * medit_free_mobile() makes sure strings are not in the prototype.
       */
      medit_free_mobile(OLC_MOB(d));

    /*
     * Check for a zone.
     */
    if (OLC_ZONE(d)) {
      /*
       * cleanup_type is irrelevant here, FREE() everything.
       */
      FREE(OLC_ZONE(d)->name);
      FREE(OLC_ZONE(d)->cmd);
      FREE(OLC_ZONE(d));
    }
    /*
     * Check for a shop.
     */
    if (OLC_SHOP(d)) {
      /*
       * free_shop doesn't perform sanity checks, we must be careful here.
       */
      switch (cleanup_type) {
      case CLEANUP_ALL:
	free_shop(OLC_SHOP(d));
	break;
      case CLEANUP_STRUCTS:
	FREE(OLC_SHOP(d));
	break;
      default:			/* The caller has screwed up. */
	break;
      }
    }
    if (OLC_HELP(d)) {
      switch (cleanup_type) {
      case CLEANUP_ALL:
	free_help(OLC_HELP(d));
	break;
      case CLEANUP_STRUCTS:
	FREE(OLC_HELP(d));
	break;
      default:
	/* Caller has screwed up */
	break;
      }
    }
    if (OLC_QUEST(d)) {		/*. free_quest performs no sanity checks, must be carefull here . */
      switch (cleanup_type) {
      case CLEANUP_ALL:
	free_quest(OLC_QUEST(d));
	break;
      case CLEANUP_STRUCTS:
	FREE(OLC_QUEST(d));
	break;
      default:
	/*. Caller has screwed up . */
	break;
      }
    }
    /*
     * Restore descriptor playing status.
     */
    if (d->character) {
      REMOVE_BIT(PLR_FLAGS(d->character), PLR_WRITING);
      STATE(d) = CON_PLAYING;
      act("$n stops using OLC.", TRUE, d->character, 0, 0, PROXIMITY);
    }
    FREE(d->olc);
  }
}
