/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

/*
 * If you don't want a short explanation of each field in your zone files,
 * change the number below to a 0 instead of a 1.
 */
#if 1
#define ZEDIT_HELP_IN_FILE
#endif

/*
 * If you want to clear the screen before certain Oasis menus, set to 1.
 */
#define CLEAR_SCREEN	1

/*
 * Macros, defines, structs and globals for the OLC suite.
 */

#define NUM_ROOM_FLAGS 		25
#define NUM_ROOM_SECTORS	13

#define NUM_MOB_FLAGS		19
#define NUM_AFF_FLAGS		32
#define NUM_AFF2_FLAGS		32
#define NUM_AFF3_FLAGS		32
#define NUM_AFF4_FLAGS		32
#define NUM_PROT_FLAGS		32
#define NUM_DETE_FLAGS		32
#define NUM_CONT_FLAGS		32
#define NUM_ATTACK_TYPES	15
#define NUM_INT_RATINGS		11

#define NUM_ITEM_TYPES		45
#define NUM_ITEM_FLAGS		30
#define NUM_ITEM2_FLAGS		19
#define NUM_ANTI_FLAGS		12
#define NUM_ITEM_WEARS 		22
#define NUM_APPLIES		27
#define NUM_LIQ_TYPES 		16
#define NUM_SPELLS		TOP_SPELL_DEFINE + 1

#define NUM_GENDERS		3
#define NUM_SHOP_FLAGS 		2
#define NUM_TRADERS 		7
#define NUM_ACTIVATING_COMMANDS 6
/*
 * Define this to how many MobProg scripts you have.
 */
#define NUM_PROGS		13

#define LVL_BUILDER		LVL_GOD2

/*
 * Utilities exported from olc.c.
 */
void strip_string(char *);
void cleanup_olc(struct descriptor_data *d, byte cleanup_type);
void get_char_cols(struct char_data *ch);
void olc_add_to_save_list(int zone, byte type);
void olc_remove_from_save_list(int zone, byte type);

/*
 * OLC structures.
 */

struct add_points {
  int left;
  int min;
};

struct abil_data {
  int str;
  int intel;
  int wis;
  int dex;
  int con;
  int cha;
  struct add_points *pts;
};

struct olc_data {
  int mode;
  int submode;
  int zone_num;
  int number;
  int value;
  int orig_prototype;
  int value2;
  char *storage;
  int total_mprogs;
  struct char_data *mob;
  struct room_data *room;
  struct obj_data *obj;
  struct zone_data *zone;
  struct social_messg *action;
  struct shop_data *shop;
  struct extra_descr_data *desc;
  struct aq_data *quest;
  struct help_index_element *help;
  struct mob_prog_data *mprog;
  struct mob_prog_data *mprogl;
  struct path_info *path;
  struct path_info *path_list;
  int total_paths;
  struct trig_data *trig;
  int script_mode;
  int trigger_position;
  int item_type;
  struct trig_proto_list *script;
};

struct olc_save_info {
  int zone;
  char type;
  struct olc_save_info *next;
};

/*
 * Exported globals.
 */
#ifdef _OASIS_OLC_
char *nrm, *grn, *cyn, *yel, *blk, *red, *blu;
struct olc_save_info *olc_save_list = NULL;
#else
extern char *nrm, *grn, *cyn, *yel, *red, *blk, *blu;
extern struct olc_save_info *olc_save_list;
#endif
#define OLC_TRIG(d)     ((d)->olc->trig)	/* Trigger structure.   */
#define OLC_STORAGE(d)  ((d)->olc->storage)	/* For command storage  */

/*
 * Descriptor access macros.
 */
#define OLC_MODE(d) 	((d)->olc->mode)	/* Parse input mode.      */
#define OLC_SUBMODE(d) 	((d)->olc->submode)	/* Parse input submode.   */
#define OLC_NUM(d) 	((d)->olc->number)	/* Room/Obj VNUM. */
#define OLC_PROTO(d) 	((d)->olc->orig_prototype) /* what its created from */
#define OLC_VAL(d) 	((d)->olc->value)	/* Scratch variable.      */
#define OLC_VAL2(d) 	((d)->olc->value2)	/* Scratch variable.      */
#define OLC_ZNUM(d) 	((d)->olc->zone_num)	/* Real zone number.      */
#define OLC_ROOM(d) 	((d)->olc->room)	/* Room structure.        */
#define OLC_OBJ(d) 	((d)->olc->obj)		/* Object structure.      */
#define OLC_ZONE(d)     ((d)->olc->zone)	/* Zone structure.        */
#define OLC_MOB(d)	((d)->olc->mob)		/* Mob structure. */
#define OLC_SHOP(d) 	((d)->olc->shop)	/* Shop structure.        */
#define OLC_DESC(d) 	((d)->olc->desc)	/* Extra description.     */
#define OLC_STORAGE(d)  ((d)->olc->storage)	/* Command storage        */
#define OLC_ACTION(d)   ((d)->olc->action)	/*. Action structure    */
#define OLC_HELP(d)     ((d)->olc->help)	/* Help structure */
#define OLC_QUEST(d)    ((d)->olc->quest)	/* Quest structure      */
#define OLC_MTOTAL(d)	((d)->olc->total_mprogs)	/* Total mprog number.    */
#define OLC_PATH(d)    ((d)->olc->path)		/* Temporary Path.   */
#define OLC_PATH_LIST(d)   ((d)->olc->path_list)	/* Path list.     */
#define OLC_PTOTAL(d)   ((d)->olc->total_paths)
#define GET_PATH(mob) (mob_index[(mob)->nr].paths)
#define GET_PATH_BY_RNUM(rnum)  (mob_index[(rnum)].paths)

/*
 * Other macros.
 */
#define OLC_EXIT(d)	(OLC_ROOM(d)->dir_option[OLC_VAL(d)])
#define GET_OLC_ZONE(c)	((c)->player_specials->saved.olc_zone)

/*
 * Cleanup types.
 */
#define CLEANUP_ALL		(byte)	1	/* Free the whole lot.    */
#define CLEANUP_STRUCTS 	(byte)	2	/* Don't free strings.    */

/*
 * Add/Remove save list types.
 */
#define OLC_SAVE_ROOM		(byte)	0
#define OLC_SAVE_OBJ		(byte)	1
#define OLC_SAVE_ZONE		(byte)	2
#define OLC_SAVE_MOB		(byte)	3
#define OLC_SAVE_SHOP		(byte)	4
#define OLC_SAVE_QUEST		(byte)  5
#define OLC_SAVE_HELP		(byte)  6
#define OLC_SAVE_ACTION		(byte)  7

 /* Submodes of AEDIT connectedness    */
#define AEDIT_CONFIRM_SAVESTRING      0
#define AEDIT_CONFIRM_EDIT            1
#define AEDIT_CONFIRM_ADD             2
#define AEDIT_MAIN_MENU                       3
#define AEDIT_ACTION_NAME             4
#define AEDIT_SORT_AS                 5
#define AEDIT_MIN_CHAR_POS            6
#define AEDIT_MIN_VICT_POS            7
#define AEDIT_HIDDEN_FLAG             8
#define AEDIT_MIN_CHAR_LEVEL          9
#define AEDIT_NOVICT_CHAR             10
#define AEDIT_NOVICT_OTHERS           11
#define AEDIT_VICT_CHAR_FOUND         12
#define AEDIT_VICT_OTHERS_FOUND               13
#define AEDIT_VICT_VICT_FOUND         14
#define AEDIT_VICT_NOT_FOUND          15
#define AEDIT_SELF_CHAR                       16
#define AEDIT_SELF_OTHERS             17
#define AEDIT_VICT_CHAR_BODY_FOUND            18
#define AEDIT_VICT_OTHERS_BODY_FOUND          19
#define AEDIT_VICT_VICT_BODY_FOUND            20
#define AEDIT_OBJ_CHAR_FOUND          21
#define AEDIT_OBJ_OTHERS_FOUND                22
#define AEDIT_CONFIRM_DELETE        	23

/*
 * Submodes of OEDIT connectedness.
 */
#define OEDIT_MAIN_MENU              	1
#define OEDIT_EDIT_NAMELIST          	2
#define OEDIT_SHORTDESC              	3
#define OEDIT_LONGDESC               	4
#define OEDIT_ACTDESC                	5
#define OEDIT_TYPE                   	6
#define OEDIT_EXTRAS                 	7
#define OEDIT_WEAR                  	8
#define OEDIT_WEIGHT                	9
#define OEDIT_COST                  	10
#define OEDIT_COSTPERDAY            	11
#define OEDIT_TIMER                 	12
#define OEDIT_VALUE_1               	13
#define OEDIT_VALUE_2               	14
#define OEDIT_VALUE_3               	15
#define OEDIT_VALUE_4               	16
#define OEDIT_APPLY                 	17
#define OEDIT_APPLYMOD              	18
#define OEDIT_EXTRADESC_KEY         	19
#define OEDIT_CONFIRM_SAVEDB        	20
#define OEDIT_CONFIRM_SAVESTRING    	21
#define OEDIT_PROMPT_APPLY          	22
#define OEDIT_EXTRADESC_DESCRIPTION 	23
#define OEDIT_EXTRADESC_MENU        	24
#define OEDIT_LEVEL                 	25
#define OEDIT_SIZE                 	26
#define OEDIT_SPACE                 	27
#define OEDIT_CONDITION                	28
#define OEDIT_STATUS                 	29
#define OEDIT_MATERIAL                 	30
#define OEDIT_CRAFTSMANSHIP            	31
#define OEDIT_VALUE_5               	32
#define OEDIT_VALUE_6               	33
#define OEDIT_VALUE_7               	34
#define OEDIT_VALUE_8               	35
#define OEDIT_EXTRAS2                 	36
#define OEDIT_ANTI                 	37
#define OEDIT_RACE                 	38
#define OEDIT_CONFIRM_DELETE        	39
#define OEDIT_AFFECTED_BITS		40
#define OEDIT_AFFECTED_BITS2		41
#define OEDIT_AFFECTED_BITS3		42
#define OEDIT_AFFECTED_BITS4		43
#define OEDIT_PROTECTION		44
#define OEDIT_DETECTING			45
#define OEDIT_CONTROLLING		46

/*
 * Submodes of REDIT connectedness.
 */
#define REDIT_MAIN_MENU 		1
#define REDIT_NAME 			2
#define REDIT_DESC 			3
#define REDIT_FLAGS 			4
#define REDIT_SECTOR 			5
#define REDIT_EXIT_MENU 		6
#define REDIT_CONFIRM_SAVEDB 		7
#define REDIT_CONFIRM_SAVESTRING 	8
#define REDIT_EXIT_NUMBER 		9
#define REDIT_EXIT_DESCRIPTION 		10
#define REDIT_EXIT_KEYWORD 		11
#define REDIT_EXIT_KEY 			12
#define REDIT_EXIT_XCORD		13
#define REDIT_EXIT_YCORD		14
#define REDIT_EXIT_ZCORD		15
#define REDIT_EXIT_DOORFLAGS 		16
#define REDIT_EXTRADESC_MENU 		17
#define REDIT_EXTRADESC_KEY 		18
#define REDIT_EXTRADESC_DESCRIPTION 	19
#define REDIT_XSIZE			20
#define REDIT_YSIZE			21
#define REDIT_SHAPE			22
#define REDIT_MAP_SPECIALS		23
#define REDIT_DOOR_FLAGS	        24
#define REDIT_MAP		        25
#define REDIT_MAP_X			26
#define REDIT_MAP_Y			27
#define REDIT_MAP_ICON			28
#define REDIT_CURRENT_DIR		29
#define REDIT_CURRENT_SPEED		30
#define REDIT_KINGDOM_NUM		31
#define REDIT_KINGDOM_TYPE		32

/*
 * Submodes of ZEDIT connectedness.
 */
#define ZEDIT_MAIN_MENU              	0
#define ZEDIT_DELETE_ENTRY		1
#define ZEDIT_NEW_ENTRY			2
#define ZEDIT_CHANGE_ENTRY		3
#define ZEDIT_COMMAND_TYPE		4
#define ZEDIT_IF_FLAG			5
#define ZEDIT_ARG1			6
#define ZEDIT_ARG2			7
#define ZEDIT_ARG3			8
#define ZEDIT_ZONE_NAME			9
#define ZEDIT_ZONE_LIFE			10
#define ZEDIT_ZONE_TOP			11
#define ZEDIT_ZONE_RESET		12
#define ZEDIT_CONFIRM_SAVESTRING	13
#define ZEDIT_ARG4			14
#define ZEDIT_ARG5			15
#define ZEDIT_ARG6			16
#define ZEDIT_ARG7			17
#define ZEDIT_SARG1                     18
#define ZEDIT_SARG2                     19
#define ZEDIT_KINGDOM			20

/*
 * Submodes of MEDIT connectedness.
 */
#define MEDIT_MAIN_MENU              	0
#define MEDIT_ALIAS			1
#define MEDIT_S_DESC			2
#define MEDIT_L_DESC			3
#define MEDIT_D_DESC			4
#define MEDIT_NPC_FLAGS			5
#define MEDIT_AFF_FLAGS			6
#define MEDIT_AFF2_FLAGS		7
#define MEDIT_AFF3_FLAGS		8
#define MEDIT_AFF4_FLAGS		9
#define MEDIT_CONFIRM_SAVESTRING	10
#define MEDIT_NEW_ATTACK		11
#define MEDIT_ATTACK_MIN		12
#define MEDIT_ATTACK_MAX		13
#define MEDIT_ATTACK_TYPE		14
#define MEDIT_CONFIRM_DELETE        	15
#define MEDIT_ADD_SPELL			16

/*
 * Numerical responses.
 */
#define MEDIT_CLASS			18
#define MEDIT_RACE			19
#define MEDIT_NUMERICAL_RESPONSE	20
#define MEDIT_SEX			21
#define MEDIT_HITROLL			22
#define MEDIT_DAMROLL			23
#define MEDIT_NUM_HP_DICE		26
#define MEDIT_SIZE_HP_DICE		27
#define MEDIT_ADD_HP			28
#define MEDIT_AC			29
#define MEDIT_EXP			30
#define MEDIT_GOLD			31
#define MEDIT_POS			32
#define MEDIT_DEFAULT_POS		33
#define MEDIT_THAC0			34
#define MEDIT_LEVEL			35
#define MEDIT_ALIGNMENT			36
#define MEDIT_PATH			37
#define MEDIT_MAGIC_RESISTANCE          38
#define MEDIT_DELETE_ATTACK		39
#define MEDIT_CHANGE_ATTACK		40
#define MEDIT_INTELLIGENCE		41
#define MEDIT_HOMETOWN			42

/* 
 * Submodes for Path stuff.
 */
#define MEDIT_SUB_NONE			0
#define MEDIT_SUB_PATH			1
#define MEDIT_SUB_EDIT_PATH		2
#define MEDIT_SUB_EDIT_ONE_PATH		3
#define MEDIT_SUB_EDIT_TIME		4
#define MEDIT_SUB_EDIT_SELECTION	5
#define MEDIT_SUB_PURGE_SELECTION	6
#define MEDIT_SUB_TYPE			7
#define MEDIT_SUB_TIME_EDIT		8
#define MEDIT_SUB_COMMAND		9
#define MEDIT_SUB_DAY_SELECT		10
#define MEDIT_SUB_MONTH_SELECT		11
#define MEDIT_SUB_HRMIN_SELECT		12
#define MEDIT_SUB_AMPM_SELECT		13
#define MEDIT_SUB_WEEK_SELECT		14

/*
 * Submodes of SEDIT connectedness.
 */
#define SEDIT_MAIN_MENU              	0
#define SEDIT_CONFIRM_SAVESTRING	1
#define SEDIT_NOITEM1			2
#define SEDIT_NOITEM2			3
#define SEDIT_NOCASH1			4
#define SEDIT_NOCASH2			5
#define SEDIT_NOBUY			6
#define SEDIT_BUY			7
#define SEDIT_SELL			8
#define SEDIT_PRODUCTS_MENU		11
#define SEDIT_ROOMS_MENU		12
#define SEDIT_NAMELIST_MENU		13
#define SEDIT_NAMELIST			14
/*
 * Numerical responses.
 */
#define SEDIT_NUMERICAL_RESPONSE	20
#define SEDIT_OPEN1			21
#define SEDIT_OPEN2			22
#define SEDIT_CLOSE1			23
#define SEDIT_CLOSE2			24
#define SEDIT_KEEPER			25
#define SEDIT_BUY_PROFIT		26
#define SEDIT_SELL_PROFIT		27
#define SEDIT_TYPE_MENU			29
#define SEDIT_DELETE_TYPE		30
#define SEDIT_DELETE_PRODUCT		31
#define SEDIT_NEW_PRODUCT		32
#define SEDIT_DELETE_ROOM		33
#define SEDIT_NEW_ROOM			34
#define SEDIT_SHOP_FLAGS		35
#define SEDIT_NOTRADE			36

/*. Quest editor . */
#define QEDIT_MAIN_MENU                 0
#define QEDIT_CONFIRM_SAVESTRING        1
#define QEDIT_NAME                      2
#define QEDIT_DESC                      3
#define QEDIT_INFO			4
#define QEDIT_ENDING			5
#define QEDIT_QUESTMASTER               6
#define QEDIT_TYPE                      7
#define QEDIT_FLAGS                     8
#define QEDIT_TARGET                    9
#define QEDIT_REWARD_TYPE               10
#define QEDIT_REWARD                    11
#define QEDIT_NEXT			12
#define QEDIT_VALUE_0			13
#define QEDIT_VALUE_1			14
#define QEDIT_VALUE_2			15
#define QEDIT_VALUE_3			16

/*
 * Limit information.
 */
#define MAX_ROOM_NAME	75
#define MAX_MOB_NAME	70
#define MAX_OBJ_NAME	70
#define MAX_QUEST_NAME	70
#define MAX_QUEST_INFO  1024
#define MAX_QUEST_ENDING 1024
#define MAX_ROOM_DESC	2048
#define MAX_EXIT_DESC	512
#define MAX_EXTRA_DESC  1024
#define MAX_MOB_DESC	1024
#define MAX_OBJ_DESC	1024

/* Submodes of HEDIT connectedness      */
#define HEDIT_CONFIRM_SAVESTRING	0
#define HEDIT_CONFIRM_EDIT		1
#define HEDIT_CONFIRM_ADD		2
#define HEDIT_MAIN_MENU			3
#define HEDIT_ENTRY			4
#define HEDIT_MIN_LEVEL			5
#define HEDIT_CONFIRM_DELETE        	6

/* hedit permission */
#define HEDIT_PERMISSION		1000
#define AEDIT_PERMISSION		2000

/* Submodes of PEDIT */
#define PEDIT_QSEX			0
#define PEDIT_QRACE			1
#define PEDIT_QCLASS			2
#define PEDIT_QEXDESC			3
#define PEDIT_QABIL_STR			4
#define PEDIT_QABIL_INTEL		5
#define PEDIT_QABIL_WIS			6
#define PEDIT_QABIL_DEX			7
#define PEDIT_QABIL_CON			8
#define PEDIT_QABIL_CHA			9
#define PEDIT_QREROLL			10
#define PEDIT_QRERANK			11
#define PEDIT_ADD_POINTS		12
#define PEDIT_INTROD1			13
#define PEDIT_INTROD2			14
#define PEDIT_INTROD3			15
#define PEDIT_ALIGNMENT                 16
