
/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "spells.h"
#include "handler.h"
#include "db.h"
#include "events.h"

extern struct room_data *world;
extern struct obj_data *object_list;
extern struct char_data *character_list;
extern struct index_data *obj_index;

extern struct weather_data weather_info;
extern struct descriptor_data *descriptor_list;
extern struct zone_data *zone_table;

extern int mini_mud;

extern struct default_mobile_stats *mob_defaults;
extern char weapon_verbs[];
extern int *max_ac_applys;
extern struct apply_mod_defaults *apmd;

void clearMemory(struct char_data *ch);
void act(char *str, int i, struct char_data *c, struct obj_data *o,
	 void *vict_obj, int j);

void damage(struct char_data *ch, struct char_data *victim,
	    int damage, int weapontype, int which, int location);

void weight_change_object(struct obj_data *obj, int weight);
void add_follower(struct char_data *ch, struct char_data *leader);
int dice(int number, int size);
extern struct spell_info_type spell_info[];


struct char_data *read_mobile(int, int);

/* First Level Magic-User Spells */

AROUTINE(r_armor)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_ARMOR;
  af->level = level;
  af->duration = -1;
  af->modifier = 8 + level;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_charm)
{
  struct affected_type *af;

// checks that MUST fail

  if (vict == ch) {
    send_to_char("You contemplate how charming you are.\r\n", ch);
    return 0;
  }
  if (circle_follow(vict, ch)) {
    send_to_char("Sorry, following in circles cannot be allowed.\r\n", ch);
    return 0;
  }
  if (vict->master && (vict->master != ch))
    stop_follower(vict);

  if (!vict->master)
    add_follower(vict, ch);

  CREATE(af, struct affected_type, 1);

  af->type = saved;
  af->level = level;
  af->duration = 10 * (25 - GET_INT(vict)) / 3;
  af->bitvector = AFF_CHARM;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_detect_magic)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_DETECT_MAGIC;
  af->level = level;
  af->duration = 2 * level;
  af->detecting = DETECTING_MAGIC;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_gaze_reflection)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_GAZE_REFLECTION;
  af->level = level;
  af->duration = 2 + (level * 1);
  af->bitvector2 = AFF2_GAZE_REFLECTION;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_protection_from_evil)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_PROT_FROM_EVIL;
  af->level = level;
  af->duration = 2 * GET_LEVEL(ch);
  af->protection = PROT_FROM_EVIL;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_protection_from_good)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_PROT_FROM_GOOD;
  af->level = level;
  af->duration = 2 * GET_LEVEL(ch);
  af->protection = PROT_FROM_GOOD;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_shield)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_SHIELD;
  af->level = level;
  af->duration = 5 * level;
  af->bitvector2 = AFF2_SHIELD;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_shocking_grasp)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_SHOCKING_GRASP;
  af->level = level;
  af->duration = level;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_sleep)
{
  struct affected_type af;

  af.type = SPELL_SLEEP;
  af.level = level;
  af.duration = 5 * level;
  af.bitvector = AFF_SLEEP;
  affect_to_char(vict, &af);

  if (FIGHTING(vict))
    stop_fighting(vict);
  if (AWAKE(vict)) {
    act("&+W$n goes to sleep.", TRUE, vict, 0, 0, PROXIMITY);
    SET_POS(vict, GET_POS(vict) + STAT_SLEEPING);
    update_pos(vict);
  }
  return 1;
}

AROUTINE(r_stinking_cloud)
{
  struct affected_type af;

  af.type = SPELL_SLEEP;
  af.level = level;
  af.duration = 5 * level;
  af.bitvector = AFF_SLEEP;
  affect_to_char(vict, &af);

  if (FIGHTING(vict))
    stop_fighting(vict);
  if (AWAKE(vict)) {
    act("&+W$n falls down gasping and choking.", TRUE, vict, 0, 0, PROXIMITY);
    SET_POS(vict, POS_PRONE + STAT_SLEEPING);
    update_pos(vict);
  }
  return 1;
}

AROUTINE(r_spider_climb)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_SPIDER_CLIMB;
  af->level = level;
  af->duration = 3 * (level * 1);
  af->bitvector2 = AFF2_SPIDER_CLIMB;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

/* Second Level Magic-User Spells */

AROUTINE(r_blindness)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_BLINDNESS;
  af->level = level;
  af->bitvector = AFF_BLIND;
  af->duration = -1;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_blur)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_BLUR;
  af->level = level;
  af->duration = 3 + GET_LEVEL(ch);
  af->bitvector3 = AFF3_BLUR;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_detect_evil)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_DETECT_EVIL;
  af->level = level;
  af->duration = 5 * level;
  af->detecting = DETECTING_EVIL;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_detect_good)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_DETECT_GOOD;
  af->level = level;
  af->duration = 5 * level;
  af->detecting = DETECTING_GOOD;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_detect_invisibility)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_DETECT_INVISIBILITY;
  af->level = level;
  af->duration = 5 * level;
  af->detecting = DETECTING_INVIS;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_invisibility)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_INVISIBILITY;
  af->level = level;
  af->duration = 1440;
  af->bitvector = AFF_INVISIBLE;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_levitate)
{
  struct affected_type *af, *af2 = NULL;

  CREATE(af, struct affected_type, 1);

  af->modifier = GET_ID(vict);

  af->type = SPELL_LEVITATE;
  if (ch == vict)
    af->bitvector = AFF_LEVITATE;
  else {
    CREATE(af2, struct affected_type, 1);
    af2->modifier = GET_ID(ch);
    af2->bitvector = AFF_LEVITATE;
    af2->type = SPELL_LEVITATE;
    af->level = level;
    af2->duration = 10 * GET_LEVEL(ch);
    affect_to_char(vict, af2);
    FREE(af2);
  }
  af->duration = 10 * GET_LEVEL(ch);
  affect_to_char(ch, af);

  FREE(af);
  return 1;
}

/* 
   AROUTINE(mirror_image)
 */

AROUTINE(r_ray_of_enfeeblement)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);
  af->type = SPELL_RAY_OF_ENFEEBLEMENT;
  af->level = level;
  af->duration = level;
  af->modifier = -(GET_STR(vict) - 5);
  af->location = APPLY_STR;
  affect_to_char(vict, af);
  FREE(af);

  return 1;
}

AROUTINE(r_strength)
{
  struct affected_type *af;
  int mod, mod2;

  switch (GET_CLASS(vict)) {
  case CLASS_MAGIC_USER:
  case CLASS_ILLUSIONIST:
    mod = dice(1, 4);
    break;
  case CLASS_RANGER:
  case CLASS_FIGHTER:
  case CLASS_PALADIN:
  case CLASS_MULTI_C_F:
  case CLASS_MULTI_C_R:
  case CLASS_MULTI_F_T:
  case CLASS_MULTI_F_M:
  case CLASS_MULTI_F_I:
  case CLASS_MULTI_F_M_T:
  case CLASS_MULTI_C_F_M:
    mod = dice(1, 8);
    break;
  case CLASS_CLERIC:
  case CLASS_DRUID:
  case CLASS_THIEF:
  case CLASS_BARD:
  case CLASS_MULTI_C_M:
  case CLASS_MULTI_C_I:
  case CLASS_MULTI_I_T:
  case CLASS_MULTI_M_T:
  case CLASS_MULTI_C_T:
    mod = dice(1, 6);
    break;
  default:
    mod = dice(1, 8);
  }

  mod2 = MAX(0, 18 - GET_STR(vict));
  mod2 = MIN(mod, mod2);
  if (GET_STR(vict) > 18)
    mod2 = mod;
  mod -= mod2;
#if 0
  if ((mod > 0) && IS_WARRIOR(vict)) {
    mod *= 10;
    af2.type = SPELL_STRENGTH;
    af->level = level;
    af2.modifier = MIN(100 - GET_ADD(vict), mod);
    af2.duration = 60 * GET_LEVEL(ch);
    af2.location = APPLY_ADD;
    affect_to_char(vict, &af2);
  }
#endif

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_STRENGTH;
  af->level = level;
  af->modifier = MIN(mod2, 25);
  af->duration = 60 * GET_LEVEL(ch);
  af->location = APPLY_STR;

  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

/* Third Level Magic-User Spells */

AROUTINE(r_blink)
{
  struct affected_type *af;
  struct blink_event_data *be;
  EVENTFUNC(blink_event);

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_BLINK;
  af->level = level;
  af->duration = GET_LEVEL(ch);
  af->bitvector3 = AFF3_BLINK;
  affect_to_char(vict, af);

  CREATE(be, struct blink_event_data, 1);
  be->ch = vict;
  GET_BLINK(vict) = event_create(blink_event, be, PASSES_PER_SEC * SECS_PER_ROUND / 2);
  FREE(af);
  return 1;
}

AROUTINE(r_fly)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_FLY;
  af->level = level;
  af->bitvector = AFF_FLY;
  af->duration = 10 * (dice(1, 6) + GET_LEVEL(ch));
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_haste)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_HASTE;
  af->level = level;
  af->duration = GET_LEVEL(ch) + 3;
  af->bitvector = AFF_HASTE;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_infravision)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_INFRAVISION;
  af->level = level;
  af->duration = (GET_LEVEL(ch) * 60) + 120;
  af->bitvector = AFF_INFRAVISION;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_mass_invisibility)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_MASS_INVISIBILITY;
  af->level = level;
  af->duration = 1440;
  af->bitvector = AFF_INVISIBLE;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_slow)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_SLOW;
  af->level = level;
  af->duration = GET_LEVEL(ch) + 3;
  af->bitvector2 = AFF2_SLOW;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_water_breathing)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_WATER_BREATHING;
  af->level = level;
  af->duration = (GET_LEVEL(ch) * 60) + (dice(1, 4) * 60);
  af->bitvector = AFF_WATERBREATH;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_wraithform)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_WRAITHFORM;
  af->level = level;
  af->duration = GET_LEVEL(ch) * 2;
  af->bitvector4 = AFF4_WRAITHFORM;
  af->bitvector3 = AFF3_PLUSONE;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

/* Fourth Level Magic-User Spells */

AROUTINE(r_cold_shield)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_COLD_SHIELD;
  af->level = level;
  af->bitvector3 = AFF3_COLDSHIELD;
  af->duration = 2 + GET_LEVEL(ch);
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_fire_shield)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_FIRE_SHIELD;
  af->level = level;
  af->bitvector2 = AFF2_FIRESHIELD;
  af->duration = 2 + GET_LEVEL(ch);
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

AROUTINE(r_minor_globe_of_invulnerability)
{
  struct room_affected_type *af;

  CREATE(af, struct room_affected_type, 1);
  af->type = SPELL_MINOR_GLOBE_OF_INVULNERABILITY;
  af->level = level;
  af->id = GET_ID(ch);
  af->duration = level;
  CREATE(af->loc, struct location_data, 1);
  af->loc->x_cord = GET_X_CORD(ch);
  af->loc->y_cord = GET_Y_CORD(ch);
  af->loc->z_cord = GET_Z_CORD(ch);
  af->loc->real_room = ch->in_room;
  affect_to_room(ch->in_room, af);

  FREE(af->loc);
  FREE(af);
  sprintf(buf, "%d", world[ch->in_room].spell_affects->loc->real_room);
  log(buf);
  return 1;
}

/* Fifth Level Magic-User Spells */

AROUTINE(r_feeblemind)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);

  af->type = SPELL_FEEBLEMIND;
  af->level = level;
  af->duration = -1;
  af->modifier = -(GET_INT(vict) - 3);
  af->location = APPLY_INT;
  affect_to_char(vict, af);

  FREE(af);
  return 1;
}

/* Sixth Level Magic-User Spells */
/*
   AROUTINE(r_globe_of_invulnerability)
   {
   return 1;
   }
 */
/* Eighth Level Magic-User Spells */

AROUTINE(r_power_word_blind)
{
  struct affected_type *af;

  CREATE(af, struct affected_type, 1);
  af->type = SPELL_POWER_WORD_BLIND;
  af->level = level;
  af->bitvector3 = AFF_BLIND;
  af->duration = (dice(1, 4) + 1) * ((saved) ? 1 : 10);
  affect_to_char(vict, af);
  FREE(af);
  return 1;
}
