/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#define __PROMPT_C__

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "shop.h"

extern struct descriptor_list *descriptor_list;
extern struct room_data *world;
extern struct char_data *character_list;
extern int is_introd(struct char_data *ch, struct char_data *vict);
extern char *dirs[];
extern char *position_types[];

char *make_prompt(struct descriptor_data *d)
{
  static char prompt[256];
  struct char_data *char_fighting = NULL, *tank = NULL;
  int percent, char_prompt = 0;

  /* Note, prompt is truncated at MAX_PROMPT_LENGTH chars (structs.h ) */

  if (d->str)
    strcpy(prompt, "] ");
  else if (d->showstr_count) {
    sprintf(prompt,
	    "\r\n&n[ Return to continue, (q)uit, (r)efresh, (b)ack, or page number (%d/%d) ]",
	    d->showstr_page, d->showstr_count);
  } else if (!d->connected) {
    *prompt = '\0';
    if (d->character)
      char_prompt = IS_PC(d->character) ? d->character->player_specials->saved.prompt : d->original->player_specials->saved.prompt;

    if (IS_ANSI_TERM(d))
      strcpy(prompt, "\033[32m<");
    else
      strcpy(prompt, "<");
#if 0
    /* infobar prompt */
    if (IS_SET(d->character->specials.act, PLR_SMARTPROMPT)) {
      UpdateScreen(d->character, 0);
      sprintf(prompt, "&+R>&n");
      send_to_char(prompt, d->character);
      return;
    }
#endif
    if ((IS_PC(d->character) || IS_PC(d->original)) && char_prompt) {
      if (IS_SET(char_prompt, PROMPT_HIT)) {
	if (IS_ANSI_TERM(d)) {
	  if (GET_MAX_HIT(d->character) > 0)
	    percent = (100 * GET_HIT(d->character)) / GET_MAX_HIT(d->character);
	  else
	    percent = -1;

	  if (percent >= 66) {
	    sprintf(prompt + strlen(prompt), "\033[0;32m %dh", d->character->points.hit);
	  } else if (percent >= 33) {
	    sprintf(prompt + strlen(prompt), "\033[33m %dh", d->character->points.hit);
	  } else if (percent >= 15) {
	    sprintf(prompt + strlen(prompt), "\033[31m %dh", d->character->points.hit);
	  } else {
	    if (IS_SET(char_prompt, PROMPT_WARN) && d->term_type >= TERM_MSP && percent <= 10)
	      sprintf(prompt + strlen(prompt), "!!SOUND(low_hits* T=combat)\033[31;40;1;5m %dh", d->character->points.hit);
	    else
	      sprintf(prompt + strlen(prompt), "\033[31;40;1;5m %dh", d->character->points.hit);
	  }
	} else {
	  sprintf(prompt + strlen(prompt), " %dh", d->character->points.hit);
	}
      }
      if (IS_SET(char_prompt, PROMPT_MAX_HIT)) {
	if (IS_ANSI_TERM(d))
	  sprintf(prompt + strlen(prompt), "\033[0;32m/%dH", GET_MAX_HIT(d->character));
	else
	  sprintf(prompt + strlen(prompt), "/%dH", GET_MAX_HIT(d->character));
      }
      if (IS_SET(char_prompt, PROMPT_FACE)) {
	if (IS_ANSI_TERM(d))
	  sprintf(prompt + strlen(prompt), "\033[0;35m Facing: \033[0;32m%s", (dirs[GET_FACING(d->character)]));
	else
	  sprintf(prompt + strlen(prompt), " Facing: %s", (dirs[GET_FACING(d->character)]));
      }
      if (IS_SET(char_prompt, PROMPT_POSITION)) {
	if (IS_ANSI_TERM(d))
	  sprintf(prompt + strlen(prompt), "\033[0;35m Position: \033[0;32m%s", (position_types[GET_POS(d->character)]));
	else
	  sprintf(prompt + strlen(prompt), " Position: %s", (position_types[GET_POS(d->character)]));
      }
      if (IS_SET(char_prompt, PROMPT_TWOLINE) &&
	  (char_prompt & (PROMPT_HIT | PROMPT_MAX_HIT))) {
	if (IS_ANSI_TERM(d))
	  sprintf(prompt + strlen(prompt), "\033[0;32m >\r\n\033[32m<");
	else
	  sprintf(prompt + strlen(prompt), " >\r\n<");
      }
      /* the prompt elements only active while fighting */
      char_fighting = FIGHTING(d->character);
      if (char_fighting && (d->character->in_room == char_fighting->in_room)) {

	/* TANK elements only active if... */
	if ((tank = char_fighting->char_specials.fighting) && tank != d->character &&
	    (d->character->in_room == tank->in_room)) {

	  if (IS_SET(char_prompt, PROMPT_TANK_NAME)) {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), " \033[0;34;1mT:\033[0m %s",
		      ((d->character != tank) && !CAN_SEE(d->character, tank)) ? "someone" :
		 (IS_PC(tank) ? ((tank)->player.name) : GET_NAME(tank)));
	    else
	      sprintf(prompt + strlen(prompt), " T: %s",
		      ((d->character != tank) && !CAN_SEE(d->character, tank)) ? "someone" :
		 (IS_PC(tank) ? ((tank)->player.name) : GET_NAME(tank)));
	  }
	  if (IS_SET(char_prompt, PROMPT_TANK_COND)) {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), "\033[0;34;1m TC:");
	    else
	      sprintf(prompt + strlen(prompt), " TC:");
	    if (GET_MAX_HIT(tank) > 0) {
	      percent = (100 * GET_HIT(tank)) / GET_MAX_HIT(tank);
	    } else {
	      percent = -1;
	    }
	    if (percent >= 100) {
	      if (IS_ANSI_TERM(d))
		sprintf(prompt + strlen(prompt), "\033[0;32m excellent");
	      else
		sprintf(prompt + strlen(prompt), " excellent");
	    } else if (percent >= 90) {
	      if (IS_ANSI_TERM(d))
		sprintf(prompt + strlen(prompt), "\033[0;33m few scratches");
	      else
		sprintf(prompt + strlen(prompt), " few scratches");
	    } else if (percent >= 75) {
	      if (IS_ANSI_TERM(d))
		sprintf(prompt + strlen(prompt), "\033[0;33;40;1m small wounds");
	      else
		sprintf(prompt + strlen(prompt), " small wounds");
	    } else if (percent >= 50) {
	      if (IS_ANSI_TERM(d))
		sprintf(prompt + strlen(prompt), "\033[0;35;40;1m few wounds");
	      else
		sprintf(prompt + strlen(prompt), " few wounds");
	    } else if (percent >= 30) {
	      if (IS_ANSI_TERM(d))
		sprintf(prompt + strlen(prompt), "\033[0;35m nasty wounds");
	      else
		sprintf(prompt + strlen(prompt), " nasty wounds");
	    } else if (percent >= 15) {
	      if (IS_ANSI_TERM(d))
		sprintf(prompt + strlen(prompt), "\033[0;31;40;1m pretty hurt");
	      else
		sprintf(prompt + strlen(prompt), " pretty hurt");
	    } else if (percent >= 0) {
	      if (IS_ANSI_TERM(d))
		sprintf(prompt + strlen(prompt), "\033[0;31m awful");
	      else
		sprintf(prompt + strlen(prompt), " awful");
	    } else {
	      if (IS_ANSI_TERM(d))
		sprintf(prompt + strlen(prompt), "\033[0;31m bleeding, close to death");
	      else
		sprintf(prompt + strlen(prompt), " bleeding, close to death");
	    }
	  }
	}
	if (IS_SET(char_prompt, PROMPT_ENEMY)) {
	  if (IS_ANSI_TERM(d))
	    sprintf(prompt + strlen(prompt), " \033[0;31mE:\033[0m %s",
		    (!CAN_SEE(d->character, char_fighting) || !is_introd(d->character, char_fighting)) ?
		    "someone" :
		    (IS_PC(char_fighting) ? ((char_fighting)->player.name) : GET_NAME(char_fighting)));
	  else
	    sprintf(prompt + strlen(prompt), " E: %s",
		    (!CAN_SEE(d->character, char_fighting) || !is_introd(d->character, char_fighting)) ?
		    "someone" :
		    (IS_PC(char_fighting) ? ((char_fighting)->player.name) : GET_NAME(char_fighting)));
	}
	if (IS_SET(char_prompt, PROMPT_ENEMY_COND)) {
	  if (GET_MAX_HIT(char_fighting) > 0) {
	    percent = (100 * GET_HIT(char_fighting)) / GET_MAX_HIT(char_fighting);
	  } else {
	    percent = -1;
	  }
	  if (IS_ANSI_TERM(d))
	    sprintf(prompt + strlen(prompt), " \033[0;31mEC:\033[0m");
	  else
	    sprintf(prompt + strlen(prompt), " EC:");
	  if (percent >= 100) {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), "\033[0;32m excellent");
	    else
	      sprintf(prompt + strlen(prompt), " excellent");
	  } else if (percent >= 90) {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), "\033[0;33m few scratches");
	    else
	      sprintf(prompt + strlen(prompt), " few scratches");
	  } else if (percent >= 75) {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), "\033[0;33;40;1m small wounds");
	    else
	      sprintf(prompt + strlen(prompt), " small wounds");
	  } else if (percent >= 50) {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), "\033[0;35;40;1m few wounds");
	    else
	      sprintf(prompt + strlen(prompt), " few wounds");
	  } else if (percent >= 30) {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), "\033[0;35m nasty wounds");
	    else
	      sprintf(prompt + strlen(prompt), " nasty wounds");
	  } else if (percent >= 15) {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), "\033[0;31;40;1m pretty hurt");
	    else
	      sprintf(prompt + strlen(prompt), " pretty hurt");
	  } else if (percent >= 0) {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), "\033[0;31m awful");
	    else
	      sprintf(prompt + strlen(prompt), " awful");
	  } else {
	    if (IS_ANSI_TERM(d))
	      sprintf(prompt + strlen(prompt), "\033[0;31m bleeding, close to death");
	    else
	      sprintf(prompt + strlen(prompt), " bleeding, close to death");
	  }
	}
      }
      if (IS_SET(char_prompt, PROMPT_VIS) && IS_TRUSTED(d->character)) {
	if (IS_ANSI_TERM(d))
	  sprintf(prompt + strlen(prompt), "\033[0;35m");
	sprintf(prompt + strlen(prompt), " Vis: %d", d->character->player_specials->saved.invis_level);
      }
      if (PLR_FLAGGED(d->character, PLR_AFK)) {
	if (IS_ANSI_TERM(d))
	  sprintf(prompt + strlen(prompt), "\033[0m (\033[31;40;1mAFK\033[0m)");
	else
	  sprintf(prompt + strlen(prompt), " (AFK)");
      }
    }
    if (IS_ANSI_TERM(d))
      sprintf(prompt + strlen(prompt), "\033[0;32m>\033[0m ");
    else
      sprintf(prompt + strlen(prompt), " > ");
  } else
    *prompt = '\0';

  return prompt;
}

ACMD(do_prompt)
{
  if (IS_NPC(ch))
    return;

  one_argument(argument, buf);

  if (*buf) {

    if (!str_cmp("twoline", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_TWOLINE)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_TWOLINE);
	send_to_char("Twoline prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_TWOLINE);
	send_to_char("Twoline prompt turned &+gON&N.\r\n", ch);
      }
    } else if (!str_cmp("warn", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_WARN)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_WARN);
	send_to_char("Audio warning turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_WARN);
	send_to_char("Audio warning turned &+gON&N.\r\n", ch);
      }
#if 0
    } else if (!str_cmp("screen", buf)) {
      if (IS_SET(ch->specials.act, PLR_SMARTPROMPT))
	InitScreen(ch);
#endif
    } else if (!str_cmp("hits", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_HIT)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_HIT);
	send_to_char("Hits prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_HIT);
	send_to_char("Hits prompt turned &+gON&N.\r\n", ch);
      }
    } else if (!str_cmp("maxhits", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_MAX_HIT)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_MAX_HIT);
	send_to_char("Maxhits prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_MAX_HIT);
	send_to_char("Maxhits prompt turned &+gON&N.\r\n", ch);
      }
    } else if (!str_cmp("tankname", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_TANK_NAME)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_TANK_NAME);
	send_to_char("Tank name prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_TANK_NAME);
	send_to_char("Tank name prompt turned &+gON&N.\r\n", ch);
      }
    } else if (!str_cmp("tankcond", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_TANK_COND)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_TANK_COND);
	send_to_char("Tank condition prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_TANK_COND);
	send_to_char("Tank condition prompt turned &+gON&N.\r\n", ch);
      }
    } else if (!str_cmp("enemy", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_ENEMY)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_ENEMY);
	send_to_char("Enemy prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_ENEMY);
	send_to_char("Enemy prompt turned &+gON&N.\r\n", ch);
      }
    } else if (!str_cmp("enemycond", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_ENEMY_COND)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_ENEMY_COND);
	send_to_char("Enemy condition prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_ENEMY_COND);
	send_to_char("Enemy condition prompt turned &+gON&N.\r\n", ch);
      }
    } else if ((!str_cmp("vis", buf)) && (GET_LEVEL(ch) >= LVL_GOD)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_VIS)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_VIS);
	send_to_char("Visibility prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_VIS);
	send_to_char("Visibility prompt turned &+gON&N.\r\n", ch);
      }
    } else if (!str_cmp("face", buf) || !str_cmp("turn", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_FACE)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_FACE);
	send_to_char("Facing prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_FACE);
	send_to_char("Facing prompt turned &+gON&N.\r\n", ch);
      }
    } else if (!str_cmp("position", buf)) {
      if (IS_SET(ch->player_specials->saved.prompt, PROMPT_POSITION)) {
	REMOVE_BIT(ch->player_specials->saved.prompt, PROMPT_POSITION);
	send_to_char("Position prompt turned &+rOFF&N.\r\n", ch);
      } else {
	SET_BIT(ch->player_specials->saved.prompt, PROMPT_POSITION);
	send_to_char("Position prompt turned &+gON&N.\r\n", ch);
      }
    } else if (!str_cmp("all", buf)) {
      ch->player_specials->saved.prompt =
	  (PROMPT_HIT | PROMPT_MAX_HIT | PROMPT_WARN |
	   PROMPT_TANK_NAME | PROMPT_TANK_COND | PROMPT_ENEMY |
	   PROMPT_ENEMY_COND | PROMPT_TWOLINE);
      if (GET_LEVEL(ch) >= LVL_GOD)
	ch->player_specials->saved.prompt |= PROMPT_VIS;
    } else if (!str_cmp("off", buf)) {
      if (ch->player_specials->saved.prompt)
	send_to_char("Turning off prompt.\r\n", ch);
      else
	send_to_char("But you aren't prompting anything!\r\n", ch);
      ch->player_specials->saved.prompt = 0;
    }
    return;
  } else
    send_to_char("Syntax: prompt <option>\r\n"
      "Note: You must type the full name of the option listed below.\r\n"
		 "Options:all|off|hits|maxhits|slots|maxslots|\r\n"
	     "        tankname|tankcond|enemy|enemycond|twoline|warn\r\n"
		 "        vis|face|position\r\n", ch);
  return;
}
