/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "shop.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "house.h"
#include "rand_items.h"
#include "dg_scripts.h"

/* constants for generating short descs */
/* 100 max each so far */
char *appearance_descs_pc[100];
char *shape_descs_pc[100];
char *modifier_descs_pc[100];
int num_appearances_pc;
int num_shapes_pc;
int num_modifiers_pc;
char *appearance_descs_mob[100];
char *shape_descs_mob[100];
char *modifier_descs_mob[100];
char *job_descs_mob[200];
int num_jobs_mob;
int num_appearances_mob;
int num_shapes_mob;
int num_modifiers_mob;

/* external variables */
extern struct char_data *char_in_room(int);
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct room_data *world;
extern struct index_data *obj_index;
extern struct index_data *mob_index;
extern const char *dirs[];
extern const char rev_dir[];
extern int top_of_world;
extern int top_of_zone_table;
extern int map_g_modifier;
extern int map_e_modifier;
extern struct zone_data *zone;
extern struct zone_data *zone_table;
extern char *strip_color(const char *string);
extern void generate_desc(struct char_data *ch);
extern void outfit_random_mobs(struct char_data *ch);
extern char *random_name(void);
extern char *race_names[];
extern char *class_names[];
extern char *race_types[];
extern void create_and_give_money(struct char_data *ch, int amount, int type);
extern struct obj_data *create_random_magic(void);
extern struct shop_data *shop_index;
extern int top_shop;
byte IsShopKeeper(struct char_data *keeper);

/* local protos */
struct obj_data *ran_obj(struct char_data *, ulong);
void random_gems(struct obj_data *, int);
void random_jewelry(struct obj_data *, int);
void CheckEqWorthUsing(struct char_data *ch, struct obj_data *obj);
char *random_name(void);
struct obj_data *CreateMerchantItem(void);
extern struct char_data *mob_proto;
extern void search_replace(char *string, const char *find, const char *replace);

/* read in and alloc the array sizes for shape, appearance, and modifier arrays */
void boot_desc_data_pc(void)
{
  FILE *f;
  int count;
  char Gbuf[80];

  /* appearance first */
  if (!(f = fopen("descs/appearance.pc", "r")))
    return;
  count = 0;
  Gbuf[0] = '\0';
  while (Gbuf[0] != '$') {
    fgets(Gbuf, 100, f);
    Gbuf[strlen(Gbuf) - 1] = '\0';
    appearance_descs_pc[count] = str_dup(Gbuf);
    count++;
  }
  num_appearances_pc = count - 1;
  fclose(f);
  count = 0;
  Gbuf[0] = '\0';
  if (!(f = fopen("descs/shapes.pc", "r")))
    return;
  while (Gbuf[0] != '$') {
    fgets(Gbuf, 100, f);
    Gbuf[strlen(Gbuf) - 1] = '\0';
    shape_descs_pc[count] = str_dup(Gbuf);
    count++;
  }
  num_shapes_pc = count - 1;
  fclose(f);
  count = 0;
  Gbuf[0] = '\0';
  if (!(f = fopen("descs/modifiers.pc", "r")))
    return;
  while (Gbuf[0] != '$') {
    fgets(Gbuf, 100, f);
    Gbuf[strlen(Gbuf) - 1] = '\0';
    modifier_descs_pc[count] = str_dup(Gbuf);
    count++;
  }
  num_modifiers_pc = count - 1;
  fclose(f);
  vlogf("boot_desc_data_pc appearances = %d shapes = %d modifiers = %d", num_appearances_pc, num_shapes_pc, num_modifiers_pc);
}

void boot_desc_data_mob(void)
{
  FILE *f;
  int count;
  char Gbuf[80];

  /* appearance first */
  if (!(f = fopen("descs/appearance.mob", "r"))) {
    log("SYSERR: appearance.mob unloadable");
    return;
  }
  count = 0;
  Gbuf[0] = '\0';
  while (Gbuf[0] != '$') {
    fgets(Gbuf, 100, f);
    Gbuf[strlen(Gbuf) - 1] = '\0';
    appearance_descs_mob[count] = str_dup(Gbuf);
    count++;
  }
  num_appearances_mob = count - 1;
  fclose(f);
  count = 0;
  Gbuf[0] = '\0';
  if (!(f = fopen("descs/shapes.mob", "r"))) {
    log("SYSERR: shapes.mob unloadable");
    return;
  }
  while (Gbuf[0] != '$') {
    fgets(Gbuf, 100, f);
    Gbuf[strlen(Gbuf) - 1] = '\0';
    shape_descs_mob[count] = str_dup(Gbuf);
    count++;
  }
  num_shapes_mob = count - 1;
  fclose(f);
  count = 0;
  Gbuf[0] = '\0';
  if (!(f = fopen("descs/modifiers.mob", "r"))) {
    log("SYSERR: modifiers.mob unloadable");
    return;
  }
  while (Gbuf[0] != '$') {
    fgets(Gbuf, 100, f);
    Gbuf[strlen(Gbuf) - 1] = '\0';
    modifier_descs_mob[count] = str_dup(Gbuf);
    count++;
  }
  num_modifiers_mob = count - 1;
  fclose(f);
  count = 0;
  Gbuf[0] = '\0';
  if (!(f = fopen("descs/jobs.mob", "r"))) {
    log("SYSERR: jobs.mob unloadable");
    return;
  }
  while (Gbuf[0] != '$') {
    fgets(Gbuf, 100, f);
    Gbuf[strlen(Gbuf) - 1] = '\0';
    job_descs_mob[count] = str_dup(Gbuf);
    count++;
  }
  num_jobs_mob = count - 1;
  fclose(f);
  vlogf("boot_desc_data_mob jobs = %d appearances = %d shapes = %d modifiers = %d", num_jobs_mob, num_appearances_mob, num_shapes_mob, num_modifiers_mob);
}

char *generate_job(struct char_data *ch)
{
  return (job_descs_mob[number(0, num_jobs_mob - 1)]);
}

char *generate_shape(struct char_data *ch)
{
  if (IS_PC(ch))
    return (shape_descs_pc[number(0, num_shapes_pc - 1)]);
  else
    return (shape_descs_mob[number(0, num_shapes_mob - 1)]);
}

char *generate_appear(struct char_data *ch)
{
  if (IS_PC(ch))
    return (appearance_descs_pc[number(0, num_appearances_pc - 1)]);
  else
    return (appearance_descs_mob[number(0, num_appearances_mob - 1)]);
}

char *generate_modif(struct char_data *ch)
{
  char *Gbuf;
  int flag = 0;

  while (!flag) {
    if (IS_PC(ch))
      Gbuf = str_dup(modifier_descs_pc[number(0, num_modifiers_pc - 1)]);
    else
      Gbuf = str_dup(modifier_descs_mob[number(0, num_modifiers_mob - 1)]);
    flag = 1;
    /* no bearded elves */
    if (((GET_RACE(ch) == RACE_ELF) || (GET_RACE(ch) == RACE_HALFELF)) &&
	strstr(Gbuf, "beard"))
      flag = 0;
    /* or female beards */
    if ((GET_SEX(ch) == SEX_FEMALE) &&
	strstr(Gbuf, "beard"))
      flag = 0;
  }
  return (Gbuf);
}

#define VOWELL(a) (((a) == 'a') || ((a) == 'e') || ((a) == 'i') || ((a) == 'o') || ((a) == 'u'))

void generate_desc(struct char_data *ch)
{
  char Gbuf[80];
  char Gbuf2[80];
  char Gbuf3[80];
  int num;
  num = number(0, 9);

  *buf = '\0';

  /* Too many humans in town, lets call them men/women/boys/girls instead */
  if (IS_NPC(ch) && GET_RACE(ch) == RACE_HUMAN) {
    if (GET_SEX(ch) == SEX_MALE)
      if (number(0, 2))
	strcpy(buf, "man");
      else
	strcpy(buf, "boy");
    else if (GET_SEX(ch) == SEX_FEMALE)
      if (number(0, 2))
	strcpy(buf, "woman");
      else
	strcpy(buf, "girl");
    else
      sprintf(buf, "%s", race_types[(int) GET_RACE(ch)]);

    if (number(0, 2))
      sprintf(buf, "%s", generate_job(ch));
    if (GET_MOB_ORIG_VNUM(ch) == RANDOM_MERCHANT)
      if (number(0, 1))
	sprintf(buf, "merchant");	/* expand this one */
      else
	sprintf(buf, "peddler");
    else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_SAILOR)
      sprintf(buf, "sailor");
    else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_BEGGAR)
      sprintf(buf, "beggar");

  } else
    sprintf(buf, "%s", race_types[(int) GET_RACE(ch)]);

  if (IS_MOB(ch))
    UNCAP(buf);

  switch (num) {
  case 0:			/* just shape */
  case 1:
    sprintf(Gbuf2, "%s", generate_shape(ch));
    sprintf(Gbuf, "%s %s %s", VOWELL(Gbuf2[0]) ? "An" : "A", Gbuf2, buf);
    sprintf(Gbuf3, "%s %s %s", VOWELL(Gbuf2[0]) ? "an" : "a", Gbuf2, buf);
    break;
  case 2:			/* just modif */
  case 3:
    sprintf(Gbuf, "A %s with %s", buf, generate_modif(ch));
    sprintf(Gbuf3, "a %s", buf);
    break;
  case 4:			/* just appearance */
  case 5:
    sprintf(Gbuf2, "%s", generate_appear(ch));
    sprintf(Gbuf, "%s %s %s", VOWELL(Gbuf2[0]) ? "An" : "A", Gbuf2, buf);
    sprintf(Gbuf3, "%s %s %s", VOWELL(Gbuf2[0]) ? "an" : "a", Gbuf2, buf);
    break;
  case 6:
    sprintf(Gbuf2, "%s", generate_shape(ch));
    sprintf(Gbuf, "%s %s %s with %s",
	  VOWELL(Gbuf2[0]) ? "An" : "A", Gbuf2, buf, generate_modif(ch));
    sprintf(Gbuf3, "%s %s %s",
	    VOWELL(Gbuf2[0]) ? "an" : "a", Gbuf2, buf);
    break;
  case 7:			/*s+a */
    sprintf(Gbuf2, "%s", generate_shape(ch));
    sprintf(Gbuf, "%s %s, %s %s",
	    VOWELL(Gbuf2[0]) ? "An" : "A", Gbuf2, generate_appear(ch),
	    buf);
    sprintf(Gbuf3, "%s %s, %s %s",
	    VOWELL(Gbuf2[0]) ? "an" : "a", Gbuf2, generate_appear(ch),
	    buf);
    break;
  case 8:			/*m+a */
    sprintf(Gbuf2, "%s", generate_appear(ch));
    sprintf(Gbuf, "%s %s %s with %s",
	  VOWELL(Gbuf2[0]) ? "An" : "A", Gbuf2, buf, generate_modif(ch));
    sprintf(Gbuf3, "%s %s %s",
	    VOWELL(Gbuf2[0]) ? "an" : "a", Gbuf2, buf);
    break;
  case 9:			/*m+a+s */
    sprintf(Gbuf2, "%s", generate_shape(ch));
    sprintf(Gbuf, "%s %s, %s %s with %s",
	    VOWELL(Gbuf2[0]) ? "An" : "A", Gbuf2, generate_appear(ch),
	    buf, generate_modif(ch));
    sprintf(Gbuf3, "%s %s, %s %s",
	 VOWELL(Gbuf2[0]) ? "an" : "a", Gbuf2, generate_appear(ch), buf);
    break;
  }				/* case */

  /* Will this work? Changing descrips seems to cause FREE() errors
   * upon player quitting.
   */
  if (IS_PC(ch)) {
    if (ch->player.short_descr != NULL)
      FREE(ch->player.short_descr);
    ch->player.short_descr = NULL;
    ch->player.short_descr = str_dup(Gbuf);
  }
  /* if mob, we should be adding an ending to the long */
  if (!IS_PC(ch) && !isname(ch->player.name, "RANDOM")) {
    /* give them a proper name, and add it as part of their alias */
    /* these could possibly be parsed by a script or whatever, for speech */
    sprintf(Gbuf2, "%s %s RANDOM %s %s", random_name(), buf,
       class_names[(int) GET_CLASS(ch)], race_names[(int) GET_RACE(ch)]);
    ch->player.name = NULL;
    ch->player.name = str_dup(Gbuf2);
    ch->player.long_descr = NULL;
    ch->player.long_descr = str_dup(Gbuf);
    ch->player.short_descr = NULL;
    ch->player.short_descr = str_dup(Gbuf3);
    ch->player.description = NULL;
  }
}

/* this is for character like mobs (cityfolk) who need a random name */
void randomize_mob(struct char_data *ch)
{
  struct obj_data *obj = NULL;
  int shop_nr, n, x, i, vnum, rnum;
  trig_data *trig;

  SPECIAL(merchant);
  SPECIAL(nudge_nudge);

  if (RANDOM_MOB(ch)) {
    GET_HOMETOWN(ch) = KINGDOM_GREYHAWK;
    GET_MOB_TREASURE(ch) = -1;
    SET_ALIGNMENT(ch, number(1, 9));
    GET_SEX(ch) = number(1, 2);
    if (GET_MOB_ORIG_VNUM(ch) == RANDOM_HUMAN) {
      if (!number(0, 3))	/* 25% of being classed */
	GET_CLASS(ch) = number(0, 8);
      GET_LEVEL(ch) = number(1, 10);	/* should be okay for a city */
      GET_HPDIE(ch) = 8;
      GET_HPPLUS(ch) = 0;
    } else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_H_MAGE) {
      GET_LEVEL(ch) = number(1, 10);	/* should be okay for a city */
      GET_HPDIE(ch) = 4;
      GET_HPPLUS(ch) = 0;
      GET_INT(ch) = number(12, 18);
    } else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_THIEF) {
      GET_RACE(ch) = number(1, 7);	/* human to half-orc */
      SET_ALIGNMENT(ch, ALIGN_NEUTRAL_EVIL);
      GET_LEVEL(ch) = number(1, 10);	/* should be okay for a city */
      GET_HPDIE(ch) = 6;
      GET_HPPLUS(ch) = 0;
      GET_DEX(ch) = number(12, 18);
    } else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_HALFELF) {
      if (!number(0, 3))	/* 25% of being classed */
	GET_CLASS(ch) = CLASS_MAGIC_USER;
      else if (!number(0, 1))
	GET_CLASS(ch) = CLASS_RANGER;
      GET_LEVEL(ch) = number(3, 6);	/* should be okay for a city */
      GET_HPDIE(ch) = 8;
      GET_HPPLUS(ch) = 0;
    } else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_ELF) {
      if (!number(0, 1))
	GET_CLASS(ch) = CLASS_MAGIC_USER;
      else if (!number(0, 1))
	GET_CLASS(ch) = CLASS_RANGER;
      GET_LEVEL(ch) = number(5, 12);
      GET_HPDIE(ch) = 8;
      GET_HPPLUS(ch) = 0;
    } else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_DWARF) {
      if (!number(0, 2))
	GET_CLASS(ch) = CLASS_FIGHTER;
      GET_LEVEL(ch) = number(1, 5);	/* should be okay for a city */
      GET_HPDIE(ch) = 8;
      GET_HPPLUS(ch) = 0;
    } else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_HALFLING) {
      if (!number(0, 3))	/* 25% of being classed */
	GET_CLASS(ch) = CLASS_THIEF;
      else if (!number(0, 1))
	GET_CLASS(ch) = CLASS_FIGHTER;
      GET_LEVEL(ch) = number(3, 6);	/* should be okay for a city */
      GET_HPDIE(ch) = 8;
      GET_HPPLUS(ch) = 0;
    } else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_GNOME) {
      if (!number(0, 3))	/* 25% of being classed */
	GET_CLASS(ch) = CLASS_MAGIC_USER;
      else if (!number(0, 3))
	GET_CLASS(ch) = CLASS_ILLUSIONIST;
      else if (!number(0, 1))
	GET_CLASS(ch) = CLASS_FIGHTER;
      GET_LEVEL(ch) = number(3, 6);	/* should be okay for a city */
      GET_HPDIE(ch) = 8;
      GET_HPPLUS(ch) = 0;
    } else if (GET_MOB_ORIG_VNUM(ch) == RANDOM_HALFORC) {
      GET_CLASS(ch) = CLASS_FIGHTER;
      GET_LEVEL(ch) = number(1, 6);	/* should be okay for a city */
      GET_HPDIE(ch) = 8;
      GET_HPPLUS(ch) = 0;
    }
    /* all random mobs get this stuff */
    generate_desc(ch);
    outfit_random_mobs(ch);
    sprintf(buf, "wear all");
    command_interpreter(ch, buf);
    create_and_give_money(ch, number(5, 15), MONEY_COPPER);
    create_and_give_money(ch, number(0, 5), MONEY_SILVER);
    if (GET_LEVEL(ch) > 5)
      create_and_give_money(ch, number(0, GET_LEVEL(ch)), MONEY_GOLD);

    if (ch->carrying) {
      for (obj = ch->carrying; obj; obj = obj->next_content) {
	if (GET_OBJ_TYPE(obj) == ITEM_CONTAINER)
	  CheckEqWorthUsing(ch, obj);
      }
      obj = NULL;		/* for later on */
      for (n = 0; n < NUM_WEARS; n++)
	if (GET_EQ(ch, n) && GET_OBJ_TYPE(GET_EQ(ch, n)) == ITEM_CONTAINER)
	  CheckEqWorthUsing(ch, GET_EQ(ch, n));
    }
    /* proc assignments */
    if (isname("merchant", ch->player.name) || isname("peddler", ch->player.name)) {
      /* see if they are a shop first */
      if (!mob_index[GET_MOB_RNUM(ch)].func)
	mob_index[GET_MOB_RNUM(ch)].func = merchant;
      else {
	for (shop_nr = 0; shop_nr < top_shop; shop_nr++)
	  if (SHOP_KEEPER(shop_nr) == GET_MOB_RNUM(ch))
	    SHOP_FUNC(shop_nr) = merchant;
      }

      /* give them something to sell */
      i = number(0, 2);
      for (x = 0; x <= i; x++) {
	obj = CreateMerchantItem();
	if (obj)
	  obj_to_char(obj, ch);
      }
    }
    if (GET_RACE(ch) == RACE_HUMAN && !mob_index[GET_MOB_RNUM(ch)].func && !number(0, 50))
      mob_index[GET_MOB_RNUM(ch)].func = nudge_nudge;

/*** end of adding procs ***/

    /* If town = GreyHawk? */
    vnum = 3000 + number(1, 8);	/* this is how many random scripts we have made */
    rnum = real_trigger(vnum);
    if ((rnum >= 0) && (trig = read_trigger(rnum))) {

      if (!SCRIPT(ch))
	CREATE(SCRIPT(ch), struct script_data, 1);
      add_trigger(SCRIPT(ch), trig, -1);

    }
  }
}

struct obj_data *CreateMerchantItem(void)
{
  struct obj_data *obj = NULL;
  int vnum = 0;

  switch (number(0, 12)) {	/* up this when more are done */
  case 0:
  case 1:
  case 2:
  case 3:
    vnum = RANDOM_ART;
    break;
  case 4:
    vnum = RANDOM_GEM;
    break;
  case 5:
  case 6:
  case 7:
    vnum = RANDOM_POTION;
    break;
  case 8:
  case 9:
    vnum = RANDOM_SCROLL;
    break;
  case 10:
    vnum = RANDOM_RING;
    break;
  case 11:
    vnum = RANDOM_WAND;
    break;
  case 12:
    vnum = RANDOM_ROD;
    break;
  case 13:
    vnum = RANDOM_JEWELRY;
    break;
  case 14:
    vnum = RANDOM_CLOAK;
    break;
  case 15:
    vnum = RANDOM_BOOT;
    break;
  case 16:
    vnum = RANDOM_BELT;
    break;
  case 17:
    vnum = RANDOM_BAG;
    break;
  case 18:
    vnum = RANDOM_INSTRUMENT;
    break;
  case 19:
    vnum = RANDOM_MAGIC_WEAPON;
    break;
  case 20:
  case 21:
    vnum = RANDOM_MUNDANE_WEAPON;
    break;
  default:
    vnum = 0;
  }
  if (vnum)
    obj = read_object(vnum, VIRTUAL);

  return obj;
}

void name_mobile(struct char_data *mob)
{
  char new_name[256];

  /* strip %rand from alias here */
  sprintf(new_name, "%s%s", random_name(), mob->player.name);
  search_replace(new_name, "%rand", "");
  mob->player.name = NULL;
  mob->player.name = str_dup(new_name);
}
