
/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "interpreter.h"
#include "spells.h"
#include "handler.h"
#include "comm.h"
#include "events.h"
#include "db.h"
#include "skills.h"

struct skill_info_type skill_info[TOTAL_SKILLS];

/* Assign the spells on boot up */
void assign_skill(const char *Name, int skl, byte Type, ulong allowed,
		  byte number_slot, byte stat, byte check_modif)
{
  int i;

  if (allowed & SKILL_CLASS_ALL) {
    for (i = 0; i < NUM_CLASSES; i++)
      skill_info[skl].can_use[i] = TRUE;
  } else {

    if (allowed & SKILL_CLASS_CLERIC)
      skill_info[skl].can_use[CLASS_CLERIC] = TRUE;
    if (allowed & SKILL_CLASS_DRUID)
      skill_info[skl].can_use[CLASS_DRUID] = TRUE;
    if (allowed & SKILL_CLASS_FIGHTER)
      skill_info[skl].can_use[CLASS_FIGHTER] = TRUE;
    if (allowed & SKILL_CLASS_PALADIN)
      skill_info[skl].can_use[CLASS_PALADIN] = TRUE;
    if (allowed & SKILL_CLASS_RANGER)
      skill_info[skl].can_use[CLASS_RANGER] = TRUE;
    if (allowed & SKILL_CLASS_MAGIC_USER)
      skill_info[skl].can_use[CLASS_MAGIC_USER] = TRUE;
    if (allowed & SKILL_CLASS_ILLUSIONIST)
      skill_info[skl].can_use[CLASS_ILLUSIONIST] = TRUE;
    if (allowed & SKILL_CLASS_THIEF)
      skill_info[skl].can_use[CLASS_THIEF] = TRUE;
    if (allowed & SKILL_CLASS_BARD)
      skill_info[skl].can_use[CLASS_BARD] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_C_M)
      skill_info[skl].can_use[CLASS_MULTI_C_M] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_F_M)
      skill_info[skl].can_use[CLASS_MULTI_F_M] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_F_T)
      skill_info[skl].can_use[CLASS_MULTI_F_T] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_M_T)
      skill_info[skl].can_use[CLASS_MULTI_M_T] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_F_M_T)
      skill_info[skl].can_use[CLASS_MULTI_F_M_T] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_C_R)
      skill_info[skl].can_use[CLASS_MULTI_C_R] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_C_F)
      skill_info[skl].can_use[CLASS_MULTI_C_F] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_I_T)
      skill_info[skl].can_use[CLASS_MULTI_I_T] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_C_F_M)
      skill_info[skl].can_use[CLASS_MULTI_C_F_M] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_C_T)
      skill_info[skl].can_use[CLASS_MULTI_C_T] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_F_I)
      skill_info[skl].can_use[CLASS_MULTI_F_I] = TRUE;
    if (allowed & SKILL_CLASS_MULTI_C_I)
      skill_info[skl].can_use[CLASS_MULTI_C_I] = TRUE;


  }

  skill_info[skl].name = (Name);
  skill_info[skl].num_slot_req = number_slot;
  skill_info[skl].stat_use = stat;
  skill_info[skl].check_mod = check_modif;
  skill_info[skl].type = Type;

}

#define S_C   SKILL_CLASS_CLERIC
#define S_D   SKILL_CLASS_DRUID
#define S_F   SKILL_CLASS_FIGHTER
#define S_P   SKILL_CLASS_PALADIN
#define S_R   SKILL_CLASS_RANGER
#define S_MU  SKILL_CLASS_MAGIC_USER
#define S_I   SKILL_CLASS_ILLUSIONIST
#define S_T   SKILL_CLASS_THIEF
#define S_B   SKILL_CLASS_BARD
#define S_CM  SKILL_CLASS_MULTI_C_M
#define S_FM  SKILL_CLASS_MULTI_F_M
#define S_FT  SKILL_CLASS_MULTI_F_T
#define S_MT  SKILL_CLASS_MULTI_M_T
#define S_FMT SKILL_CLASS_MULTI_F_M_T
#define S_CR  SKILL_CLASS_MULTI_C_R
#define S_CF  SKILL_CLASS_MULTI_C_F
#define S_IT  SKILL_CLASS_MULTI_I_T
#define S_CFM SKILL_CLASS_MULTI_C_F_M
#define S_CT  SKILL_CLASS_MULTI_C_T
#define S_FI  SKILL_CLASS_MULTI_F_I
#define S_CI  SKILL_CLASS_MULTI_C_I
#define SKILL_MCLASS_THIEF SKILL_CLASS_THIEF | SKILL_CLASS_MULTI_F_T | SKILL_CLASS_MULTI_M_T | SKILL_CLASS_MULTI_F_M_T | SKILL_CLASS_MULTI_I_T | SKILL_CLASS_MULTI_C_T
#define SKILL_MCLASS_FIGHTER S_C | S_F | S_P | S_R | S_CM | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM | S_FI

void mag_assign_skills(void)
{
  int i, j;

  for (i = 0; i < TOTAL_SKILLS; i++) {
    for (j = 0; j < NUM_CLASSES; j++)
      skill_info[i].can_use[j] = FALSE;

    skill_info[i].name = NULL;
    skill_info[i].type = SKILL_TYPE_NOT_DEFINED;
    skill_info[i].num_slot_req = 0;
    skill_info[i].stat_use = SKILL_STAT_NONE;
    skill_info[i].check_mod = 0;
  }

  assign_skill("Backstab", SKILL_BACKSTAB, SKILL_TYPE_MISC, SKILL_MCLASS_THIEF, 0, SKILL_STAT_NONE, 0);
  assign_skill("Hide", SKILL_HIDE, SKILL_TYPE_THIEF, SKILL_MCLASS_THIEF, 0, SKILL_STAT_NONE, 0);
  assign_skill("Pick Lock", SKILL_PICK_LOCK, SKILL_TYPE_THIEF, SKILL_MCLASS_THIEF | SKILL_CLASS_BARD, 0, SKILL_STAT_NONE, 0);
  assign_skill("Sneak", SKILL_SNEAK, SKILL_TYPE_THIEF, SKILL_MCLASS_THIEF, 0, SKILL_STAT_NONE, 0);
  assign_skill("Steal", SKILL_STEAL, SKILL_TYPE_THIEF, SKILL_MCLASS_THIEF, 0, SKILL_STAT_NONE, 0);
  assign_skill("Listen", SKILL_LISTEN, SKILL_TYPE_THIEF, SKILL_MCLASS_THIEF | SKILL_CLASS_BARD, 0, SKILL_STAT_NONE, 0);
  assign_skill("Find trap", SKILL_FIND_TRAP, SKILL_TYPE_THIEF, SKILL_MCLASS_THIEF, 0, SKILL_STAT_NONE, 0);
  assign_skill("Climb wall", SKILL_CLIMB_WALL, SKILL_TYPE_THIEF, SKILL_MCLASS_THIEF | SKILL_CLASS_BARD, 0, SKILL_STAT_NONE, 0);
  assign_skill("Read language", SKILL_READ_LANGUAGE, SKILL_TYPE_THIEF, SKILL_MCLASS_THIEF | SKILL_CLASS_BARD, 0, SKILL_STAT_NONE, 0);

  assign_skill("Kick", SKILL_KICK, SKILL_TYPE_NON_WEAPON, SKILL_MCLASS_FIGHTER, 2, SKILL_STAT_NONE, 0);

  assign_skill("Style_weapon_and_shield", SKILL_STYLE_WEAPON_SHIELD, SKILL_TYPE_WEAPON, S_C | S_D | S_F | S_P | S_R | S_T | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM | S_CT | S_FI | S_CI, 0, SKILL_STAT_NONE, 0);
  assign_skill("Style_one-handed_weapon", SKILL_STYLE_ONE_HANDED_WEAPON, SKILL_TYPE_WEAPON, S_C | S_D | S_F | S_P | S_R | S_T | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM | S_CT | S_FI | S_CI, 0, SKILL_STAT_NONE, 0);
  assign_skill("Style_two-handed_weapon", SKILL_STYLE_TWO_HANDED_WEAPON, SKILL_TYPE_WEAPON, S_C | S_D | S_F | S_P | S_R | S_T | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM | S_CT | S_FI | S_CI, 0, SKILL_STAT_NONE, 0);
  assign_skill("Style_two_weapons", SKILL_STYLE_TWO_WEAPON, SKILL_TYPE_WEAPON, S_C | S_D | S_F | S_P | S_R | S_T | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM | S_CT | S_FI | S_CI, 0, SKILL_STAT_NONE, 0);
  assign_skill("Style_missile", SKILL_STYLE_MISSILE, SKILL_TYPE_WEAPON, S_C | S_D | S_F | S_P | S_R | S_T | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM | S_CT | S_FI | S_CI, 0, SKILL_STAT_NONE, 0);

  assign_skill("Bastard sword", SKILL_BASTARD_SWORD, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_CM | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Battle axe", SKILL_BATTLE_AXE, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Bow", SKILL_BOW, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Broad sword", SKILL_BROAD_SWORD, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Club", SKILL_CLUB, SKILL_TYPE_WEAPON, S_FI | S_CI | S_CT | S_C | S_D | S_F | S_P | S_R | S_T | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 0, SKILL_STAT_NONE, 0);
  assign_skill("Crossbow", SKILL_CROSSBOW, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Dagger", SKILL_DAGGER, SKILL_TYPE_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_MU | S_I | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Dart", SKILL_DART, SKILL_TYPE_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_MU | S_I | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Flail", SKILL_FLAIL, SKILL_TYPE_WEAPON, S_FI | S_CI | S_CT | S_C | S_F | S_P | S_R | S_B | S_CM | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 0, SKILL_STAT_NONE, 0);
  assign_skill("Hammer", SKILL_HAMMER, SKILL_TYPE_WEAPON, S_FI | S_CI | S_CT | S_C | S_F | S_P | S_R | S_B | S_CM | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 0, SKILL_STAT_NONE, 0);
  assign_skill("Hand axe", SKILL_HAND_AXE, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Harpoon", SKILL_HARPOON, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Javelin", SKILL_JAVELIN, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_CM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Khopesh", SKILL_KHOPESH, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Knife", SKILL_KNIFE, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_MU | S_I | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Lance", SKILL_LANCE, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Long sword", SKILL_LONG_SWORD, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Mancatcher", SKILL_MANCATCHER, SKILL_TYPE_WEAPON, S_FI | S_CI | S_CT | S_C | S_F | S_P | S_R | S_B | S_CM | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 0, SKILL_STAT_NONE, 0);
  assign_skill("Morning star", SKILL_MORNING_STAR, SKILL_TYPE_WEAPON, S_FI | S_CI | S_CT | S_C | S_F | S_P | S_R | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_CFM, 0, SKILL_STAT_NONE, 0);
  assign_skill("Mace", SKILL_MACE, SKILL_TYPE_WEAPON, S_FI | S_CI | S_CT | S_C | S_F | S_P | S_R | S_B | S_CM | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 0, SKILL_STAT_NONE, 0);
  assign_skill("Pick", SKILL_PICK, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Polearm", SKILL_POLEARM, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Rapier", SKILL_RAPIER, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Saber", SKILL_SABER, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Staff", SKILL_STAFF, SKILL_TYPE_WEAPON, S_FI | S_CI | S_CT | S_C | S_F | S_P | S_R | S_MU | S_I | S_T | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 0, SKILL_STAT_NONE, 0);
  assign_skill("Scimitar", SKILL_SCIMITAR, SKILL_TYPE_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Scrouge", SKILL_SCROUGE, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Sickle", SKILL_SICKLE, SKILL_TYPE_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Sling", SKILL_SLING, SKILL_TYPE_WEAPON, S_FI | S_CI | S_CT | S_C | S_F | S_P | S_R | S_MU | S_I | S_T | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 0, SKILL_STAT_NONE, 0);
  assign_skill("Spear", SKILL_SPEAR, SKILL_TYPE_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_MT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Staff sling", SKILL_STAFF_SLING, SKILL_TYPE_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Shield", SKILL_SHIELD, SKILL_TYPE_WEAPON, S_C | S_D | S_F | S_P | S_R | S_CR | S_CF, 0, SKILL_STAT_NONE, 0);
  assign_skill("Short sword", SKILL_SHORT_SWORD, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_IT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Trident", SKILL_TRIDENT, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Two-handed sword", SKILL_TWO_HANDED_SWORD, SKILL_TYPE_WEAPON, S_FI | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT, 0, SKILL_STAT_NONE, 0);
  assign_skill("Whip", SKILL_WHIP, SKILL_TYPE_WEAPON, S_FI | S_CI | S_CT | S_C | S_F | S_P | S_R | S_B | S_CM | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 0, SKILL_STAT_NONE, 0);


  assign_skill("Language", SKILL_LANGUAGE, SKILL_TYPE_MISC, SKILL_CLASS_ALL, 1, SKILL_STAT_INT, 0);

  assign_skill("Animal_handling", SKILL_ANIMAL_HANDLING, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_INT, 0);
  assign_skill("Animal_training", SKILL_ANIMAL_TRAINING, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_WIS, -1);
  assign_skill("Blacksmithing", SKILL_BLACKSMITHING, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_WIS, 0);
  assign_skill("Cooking", SKILL_COOKING, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_INT, 0);
  assign_skill("Fishing", SKILL_FISHING, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_WIS, -1);
  assign_skill("Leather working", SKILL_LEATHER_WORKING, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_INT, 0);
  assign_skill("Riding_land", SKILL_RIDING_LAND, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_WIS, 3);
  assign_skill("Riding_airborn", SKILL_RIDING_AIRBORN, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 2, SKILL_STAT_WIS, -2);
  assign_skill("Rope use", SKILL_ROPE_USE, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_DEX, 0);
  assign_skill("Seamanship", SKILL_SEAMANSHIP, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_DEX, 1);
  assign_skill("Swimming", SKILL_SWIMMING, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_STR, 0);
  assign_skill("Tailor", SKILL_TAILOR, SKILL_TYPE_NON_WEAPON, SKILL_CLASS_ALL, 1, SKILL_STAT_DEX, -1);

  assign_skill("Healing", SKILL_HEALING, SKILL_TYPE_NON_WEAPON, S_FI | S_CI | S_C | S_D | S_P | S_CM | S_CR | S_CF | S_CFM, 2, SKILL_STAT_WIS, -2);

  assign_skill("Appraising", SKILL_APPRAISING, SKILL_TYPE_NON_WEAPON, S_CT | S_T | S_B | S_FT | S_MT | S_FMT | S_IT, 1, SKILL_STAT_INT, 0);
  assign_skill("Disguise", SKILL_DISGUISE, SKILL_TYPE_NON_WEAPON, S_CT | S_T | S_B | S_FT | S_MT | S_FMT | S_IT, 1, SKILL_STAT_CHA, -1);
  assign_skill("Forgery", SKILL_FORGERY, SKILL_TYPE_NON_WEAPON, S_CT | S_T | S_B | S_FT | S_MT | S_FMT | S_IT, 1, SKILL_STAT_DEX, -1);
  assign_skill("Jumping", SKILL_JUMPING, SKILL_TYPE_NON_WEAPON, S_CT | S_T | S_B | S_FT | S_MT | S_FMT | S_IT, 1, SKILL_STAT_STR, 0);
  assign_skill("Reading lips", SKILL_READING_LIPS, SKILL_TYPE_NON_WEAPON, S_CT | S_T | S_B | S_FT | S_MT | S_FMT | S_IT, 2, SKILL_STAT_INT, -2);

  assign_skill("Armorer", SKILL_ARMORER, SKILL_TYPE_NON_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 2, SKILL_STAT_INT, -2);
  assign_skill("Bowyer", SKILL_BOWYER, SKILL_TYPE_NON_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 2, SKILL_STAT_DEX, -1);
  assign_skill("Endurance", SKILL_ENDURANCE, SKILL_TYPE_NON_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 2, SKILL_STAT_CON, 0);
  assign_skill("Hunting", SKILL_HUNTING, SKILL_TYPE_NON_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 1, SKILL_STAT_WIS, -1);
  assign_skill("Mountaineering", SKILL_MOUNTAINEERING, SKILL_TYPE_NON_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 1, SKILL_STAT_NONE, 0);
  assign_skill("Running", SKILL_RUNNING, SKILL_TYPE_NON_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 1, SKILL_STAT_CON, -6);
  assign_skill("Survival", SKILL_SURVIVAL, SKILL_TYPE_NON_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 2, SKILL_STAT_INT, 0);
  assign_skill("Tracking", SKILL_TRACKING, SKILL_TYPE_NON_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 2, SKILL_STAT_WIS, 0);
  assign_skill("Weaponsmithing", SKILL_WEAPONSMITHING, SKILL_TYPE_NON_WEAPON, S_FI | S_D | S_F | S_P | S_R | S_B | S_FM | S_FT | S_FMT | S_CR | S_CF | S_CFM, 3, SKILL_STAT_INT, -3);

  assign_skill("Herbalism", SKILL_HERBALISM, SKILL_TYPE_NON_WEAPON, S_FI | S_CI | S_CT | S_C | S_D | S_P | S_R | S_MU | S_I | S_B | S_CM | S_FM | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 2, SKILL_STAT_INT, -2);
  assign_skill("Musical instrument", SKILL_MUSICAL_INSTRUMENT, SKILL_TYPE_NON_WEAPON, S_FI | S_CI | S_CT | S_C | S_D | S_P | S_T | S_B | S_CM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 1, SKILL_STAT_DEX, -1);
  assign_skill("Navigation", SKILL_NAVIGATION, SKILL_TYPE_NON_WEAPON, S_FI | S_CI | S_CT | S_C | S_D | S_F | S_P | S_R | S_MU | S_I | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 1, SKILL_STAT_INT, -2);
  assign_skill("Reading writing", SKILL_READING_WRITING, SKILL_TYPE_NON_WEAPON, S_FI | S_CI | S_CT | S_C | S_D | S_P | S_R | S_MU | S_I | S_B | S_CM | S_FM | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 1, SKILL_STAT_INT, 1);
  assign_skill("Religion", SKILL_RELIGION, SKILL_TYPE_NON_WEAPON, S_FI | S_CI | S_CT | S_C | S_D | S_P | S_R | S_MU | S_I | S_B | S_CM | S_FM | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 1, SKILL_STAT_WIS, 0);
  assign_skill("Spellcraft", SKILL_SPELLCRAFT, SKILL_TYPE_NON_WEAPON, S_FI | S_CI | S_CT | S_C | S_D | S_P | S_R | S_MU | S_I | S_B | S_CM | S_FM | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 1, SKILL_STAT_INT, -2);
  assign_skill("Blind fighting", SKILL_BLIND_FIGHTING, SKILL_TYPE_NON_WEAPON, S_FI | S_CT | S_D | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 2, SKILL_STAT_NONE, 0);
  assign_skill("Gaming", SKILL_GAMING, SKILL_TYPE_NON_WEAPON, S_FI | S_CT | S_D | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 1, SKILL_STAT_CHA, 0);
  assign_skill("Set snares", SKILL_SET_SNARES, SKILL_TYPE_NON_WEAPON, S_FI | S_CT | S_D | S_F | S_P | S_R | S_T | S_B | S_FM | S_FT | S_MT | S_FMT | S_CR | S_CF | S_IT | S_CFM, 1, SKILL_STAT_DEX, -1);
  assign_skill("Gem cutting", SKILL_GEM_CUTTING, SKILL_TYPE_NON_WEAPON, S_FI | S_CI | S_CT | S_R | S_MU | S_I | S_T | S_B | S_CM | S_FM | S_FT | S_MT | S_FMT | S_CR | S_IT | S_CFM, 2, SKILL_STAT_DEX, -2);

}

#undef SKILL_MCLASS_FIGHTER
#undef S_C
#undef S_D
#undef S_F
#undef S_P
#undef S_R
#undef S_MU
#undef S_I
#undef S_T
#undef S_B
#undef S_CM
#undef S_FM
#undef S_FT
#undef S_MT
#undef S_FMT
#undef S_CR
#undef S_CF
#undef S_IT
#undef S_CFM
#undef S_CT
#undef S_FI
#undef S_CI

void init_skills(struct char_data *ch)
{
  int i;

  for (i = 0; i < MAX_SKILLS + 1; i++)
    SET_SKILL(ch, i, 0);

}

void list_skills(struct char_data *ch)
{

  int i;
  sprintf(buf, "&+YYou know of the following skills:&N\r\n\r\n");

  strcpy(buf2, buf);

  for (i = 1; i < TOTAL_SKILLS; i++) {
    if (GET_SKILL(ch, i) > 0 && skill_info[i].name) {
      if (skill_info[i].type == SKILL_TYPE_WEAPON) {
	sprintf(buf, "%-30s ", skill_info[i].name);
	switch (GET_SKILL(ch, i)) {
	case 1:
	  strcat(buf, "&+Wbasic&N\r\n");
	  break;
	case 2:
	  strcat(buf, "&+Yproficient&N\r\n");
	  break;
	case 3:
	  strcat(buf, "&+Bspecialist&N\r\n");
	  break;
	case 4:
	  strcat(buf, "&+Rmaster&N\r\n");
	  break;
	case 5:
	  strcat(buf, "&+Mhigh master&N\r\n");
	  break;
	case 6:
	  strcat(buf, "&+Cgrand master&N\r\n");
	  break;
	}			//switch

      }				//if

      else {
	sprintf(buf, "%-30s %d\r\n", skill_info[i].name, GET_SKILL(ch, i));
      }				//else

      strcat(buf2, buf);
    }				//if

  }				//for

  page_string(ch->desc, buf2, 1);
}

void list_skills_for_practice(struct char_data *ch)
{
  int i, cost;
  sprintf(buf, "&+YThese are the skills you can practice:&N\r\n");
  strcat(buf, "--------------------------------------\r\n");
  strcat(buf, "Skill                          Cost\r\n\r\n");
  strcpy(buf2, buf);

  for (i = 1; i < TOTAL_SKILLS; i++) {

    if (skill_info[i].name && skill_info[i].type == SKILL_TYPE_THIEF) {
      cost = skill_info[i].num_slot_req;
      if (skill_info[i].can_use[(int) GET_CLASS(ch)]) {
	cost *= 2;
	sprintf(buf, "%-30s   (thief)\r\n", skill_info[i].name);
	strcat(buf2, buf);
      }				//if

    }				//if

    else if (skill_info[i].name && skill_info[i].type == SKILL_TYPE_NON_WEAPON) {
      cost = skill_info[i].num_slot_req;
      if (skill_info[i].can_use[(int) GET_CLASS(ch)] == FALSE)
	cost *= 2;
      sprintf(buf, "%-30s %d (non-weapon)\r\n", skill_info[i].name, cost);
      strcat(buf2, buf);
    }				//else if

    else if (skill_info[i].name && skill_info[i].type == SKILL_TYPE_WEAPON) {
      if (skill_info[i].can_use[(int) GET_CLASS(ch)]) {
/*
   if ((GET_SKILL(ch, i) == 0) || (GET_SKILL(ch, i) > 0 && GET_CLASS(ch) == CLASS_FIGHTER)) {
 */
	sprintf(buf, "%-30s 1 (weapon)\r\n", skill_info[i].name);
	strcat(buf2, buf);
/*
   }    //if
 */
      }				//if

    }				//else if

  }				//for

  page_string(ch->desc, buf2, 1);
}				//list_skills_for_practice

int find_skill_num(char *Name)
{
  int i = 0, ok;
  char *temp, *temp2;
  char first[256], first2[256];

  for (i = 1; i < TOTAL_SKILLS; i++) {

    if (!skill_info[i].name)
      continue;

    if (is_abbrev(Name, (char *) skill_info[i].name))
      return i;

    ok = 1;
    temp = any_one_arg((char *) skill_info[i].name, first);
    temp2 = any_one_arg(Name, first2);
    while (*first && *first2 && ok) {
      if (!is_abbrev(first2, first))
	ok = 0;
      temp = any_one_arg(temp, first);
      temp2 = any_one_arg(temp2, first2);
    }				//while

    if (ok && !*first2)
      return i;
  }				//for

  return -1;
}				//find_skill_num


ACMD(do_skill)
{
  list_skills(ch);
}


ACMD(do_practice)
{
  int i, a, nb_lang = 0;
  int cost;
  int non_fighter_max = FALSE;

  extern char *language_names[LAST_TALK + 1];
  extern struct int_app_type int_app[];

  one_argument(argument, arg);

  if (!*arg) {
    list_skills_for_practice(ch);
    return;
  }				//if

  if ((i = find_skill_num(arg)) < 0) {
    send_to_char("Practice what?\r\n", ch);
    return;
  }				//if

  /* language skills */
  if (i == SKILL_LANGUAGE) {

    for (a = 1; a < LAST_TALK; a++) {
      if (GET_TALK(ch, a) > 0) {
	nb_lang++;
      }				//if

    }				//for

    if (nb_lang > int_app[GET_INT(ch)].max_lang) {
      send_to_char("You cannot learn anymore language!\r\n", ch);
      return;
    }				//if

    if (GET_NON_WEAPON_PROF(ch) < 1) {
      send_to_char("You dont have enough non-weapon slot to practice!\r\n", ch);
      return;
    }				//if

    two_arguments(argument, arg, buf);

    for (a = 1; a < 11; a++) {
      if (!str_cmp(buf, language_names[a])) {
	if (GET_TALK(ch, a) > 0) {
	  send_to_char("You already know that language!\r\n", ch);
	  return;
	}			//if

	GET_TALK(ch, a) = 1;
	GET_NON_WEAPON_PROF(ch)--;
	sprintf(buf, "You now know %s.\r\n", language_names[a]);
	send_to_char(buf, ch);
	return;
      }				//if

    }				//for

    send_to_char("What language do you want to learn?\r\n", ch);
    return;
  }				//if

  /* thief type skills */
  if (skill_info[i].type == SKILL_TYPE_THIEF) {
    if (skill_info[i].can_use[(int) GET_CLASS(ch)]) {
      two_arguments(argument, arg, buf);
      if (!isdigit(*buf) || (!(cost = atoi(buf)))) {
	send_to_char("You need to specify a number of point!\r\n", ch);
	return;
      }				//if

      if (GET_CLASS(ch) != CLASS_BARD) {
	if (GET_LEVEL(ch) == 1) {
	  if (cost > 30) {
	    send_to_char("You cannot put more than 30 points to one skill!\r\n", ch);
	    return;
	  }			//if

	}			//if

	else {
	  if (cost > 15) {
	    send_to_char("You cannot put more than 15 points to one skill!\r\n", ch);
	    return;
	  }			//if

	}			//else

      }				//if

      if (cost <= GET_THIEF_SKILL(ch)) {
	if (GET_SKILL(ch, i) + cost > 99) {
	  send_to_char("Maximum skill is 99!\r\n", ch);
	  return;
	}			//if

	GET_SKILL(ch, i) += cost;
	GET_THIEF_SKILL(ch) -= cost;
	sprintf(buf, "You practice %s.\r\n", skill_info[i].name);
	send_to_char(buf, ch);
	return;
      }				//if

      else {
	send_to_char("You dont have enough thief skill slot to practice!\r\n", ch);
	return;
      }				//else

    }				//if

    else {
      send_to_char("Sorry you cant practice that!\r\n", ch);
      return;
    }				//else

  }				//if

  /* weapon type skills */
  if (skill_info[i].type == SKILL_TYPE_WEAPON) {
    if (i == SKILL_STYLE_WEAPON_SHIELD || i == SKILL_STYLE_ONE_HANDED_WEAPON ||
     i == SKILL_STYLE_TWO_HANDED_WEAPON || i == SKILL_STYLE_TWO_WEAPON ||
	i == SKILL_STYLE_MISSILE) {

      if (GET_SKILL(ch, SKILL_STYLE_WEAPON_SHIELD) > 1 ||
	  GET_SKILL(ch, SKILL_STYLE_ONE_HANDED_WEAPON) > 1 ||
	  GET_SKILL(ch, SKILL_STYLE_TWO_HANDED_WEAPON) > 1 ||
	  GET_SKILL(ch, SKILL_STYLE_TWO_WEAPON) > 1 ||
	  GET_SKILL(ch, SKILL_STYLE_MISSILE) > 1)
	non_fighter_max = TRUE;

      if (skill_info[i].can_use[(int) GET_CLASS(ch)]) {
	if (i == SKILL_STYLE_ONE_HANDED_WEAPON) {
	  if (GET_SKILL(ch, i) >= 3) {
	    send_to_char("You are already at max.\r\n", ch);
	    return;
	  }			//if

	}			//if

	else {
	  if (GET_SKILL(ch, i) >= 2) {
	    send_to_char("You are already at max.\r\n", ch);
	    return;
	  }			//if

	}			//else

	if (GET_SKILL(ch, i) == 0 || (GET_CLASS(ch) == CLASS_FIGHTER ||
				      GET_CLASS(ch) == CLASS_RANGER || GET_CLASS(ch) == CLASS_PALADIN)) {

	  if (GET_WEAPON_PROF(ch) > 0) {
	    GET_SKILL(ch, i)++;
	    GET_WEAPON_PROF(ch)--;
	    sprintf(buf, "You practice your %s skill.\r\n", skill_info[i].name);
	    send_to_char(buf, ch);
	  }			//if

	  else {
	    send_to_char("You need a weapon slot to practice!\r\n", ch);
	  }			//else

	  return;
	}			//if

	else {
	  if (non_fighter_max) {
	    send_to_char("You cannot specialize in more than one style.\r\n", ch);
	  }			//if

	  else {
	    if (GET_WEAPON_PROF(ch) > 0) {
	      GET_SKILL(ch, i)++;
	      GET_WEAPON_PROF(ch)--;
	      sprintf(buf, "You practice your %s skill.\r\n", skill_info[i].name);
	      send_to_char(buf, ch);
	    }			//if

	    else {
	      send_to_char("You need a weapon slot to practice!\r\n", ch);
	    }			//else

	  }			//else

	}			//else

      }				//if

      else {
	send_to_char("Sorry you cant practice that!\r\n", ch);
      }				//else

      return;
    }				//if

    if (skill_info[i].can_use[(int) GET_CLASS(ch)]) {
/* Why?
   if ((GET_SKILL(ch, i) == 0) || (GET_SKILL(ch, i) > 0 && GET_CLASS(ch) == CLASS_FIGHTER)) {
 */
      if (GET_CLASS(ch) == CLASS_FIGHTER) {
	if (GET_SKILL(ch, i) == 3 && GET_LEVEL(ch) < 6) {
	  send_to_char("You must be 6th to get master at weapon.\r\n", ch);
	  return;
	}			//if

	if (GET_SKILL(ch, i) == 4 && GET_LEVEL(ch) < 12) {
	  send_to_char("You must be 12th to get high master at weapon.\r\n", ch);
	  return;
	}			//if

	if (GET_SKILL(ch, i) == 5 && GET_LEVEL(ch) < 16) {
	  send_to_char("You must be 16th to get grand master at a weapon.\r\n", ch);
	  return;
	}			//if

	if (GET_SKILL(ch, i) > 5) {
	  send_to_char("You are already grand master, cant improve beyond that.\r\n", ch);
	  return;
	}			//if

      }				//if

      else if (IS_WARRIOR(ch)) {
	if (GET_SKILL(ch, i) == 2 && GET_LEVEL(ch) < 6) {
	  send_to_char("You must be 6th to get specialist at weapon.\r\n", ch);
	  return;
	}			//if

	if (GET_SKILL(ch, i) == 3 && GET_LEVEL(ch) < 12) {
	  send_to_char("You must be 12th to get master at weapon.\r\n", ch);
	  return;
	}			//if

	if (GET_SKILL(ch, i) > 3) {
	  send_to_char("You cant improve beyond this level.", ch);
	  return;
	}			//if

      }				//else if

      else {
	if (GET_SKILL(ch, i) == 1 && GET_LEVEL(ch) < 12) {
	  send_to_char("You must be 12th to get proficient at weapon.\r\n", ch);
	  return;
	}			//if

	if (GET_SKILL(ch, i) > 1) {
	  send_to_char("You cant improve beyond this level.\r\n", ch);
	  return;
	}			//if

      }				//else

      if (GET_WEAPON_PROF(ch) > 0) {
	GET_SKILL(ch, i)++;
	GET_WEAPON_PROF(ch)--;
	sprintf(buf, "You practice your %s skill.\r\n", skill_info[i].name);
	send_to_char(buf, ch);
      }				//if

      else {
	send_to_char("You need a weapon slot to practice!\r\n", ch);
      }				//else
      /*
         }    //if
         else {
         send_to_char("Only true fighter can become specialist or better!\r\n", ch);
         }    //else
       */
    }				//if

    else
      send_to_char("Sorry you cant practice that!\r\n", ch);
    return;
  }				//if

/* non-weapon type skills */
  if (skill_info[i].type == SKILL_TYPE_NON_WEAPON) {
    cost = skill_info[i].num_slot_req;
    if (skill_info[i].can_use[(int) GET_CLASS(ch)] == FALSE)
      cost *= 2;
    if (GET_NON_WEAPON_PROF(ch) >= cost) {
      GET_SKILL(ch, i)++;
      GET_NON_WEAPON_PROF(ch)--;
      sprintf(buf, "You practice %s.\r\n", skill_info[i].name);
      send_to_char(buf, ch);
    }				//if

    else
      send_to_char("You dont have enough non-weapon slot to practice!\r\n", ch);
  }				//if

}				//do_practice

int check_skill(struct char_data *ch, int Skill, int Modif)
{
  int roll, i = 0;

  if (!ch)
    return FALSE;

  if (GET_SKILL(ch, Skill) == 0)
    return FALSE;

  if (skill_info[Skill].type == SKILL_TYPE_THIEF) {
    return FALSE;
  }
  if (skill_info[Skill].type == SKILL_TYPE_NON_WEAPON) {
    roll = number(1, 20);
    switch (skill_info[Skill].stat_use) {
    case SKILL_STAT_STR:
      i = GET_STR(ch);
      break;
    case SKILL_STAT_DEX:
      i = GET_STR(ch);
      break;
    case SKILL_STAT_CON:
      i = GET_STR(ch);
      break;
    case SKILL_STAT_INT:
      i = GET_STR(ch);
      break;
    case SKILL_STAT_WIS:
      i = GET_STR(ch);
      break;
    case SKILL_STAT_CHA:
    }
    i += skill_info[Skill].check_mod;
    i += Modif;
    if (roll <= i) {
      return TRUE;
    } else {
      return FALSE;
    }
  }
  return FALSE;
}


void improve_skill(struct char_data *ch, int Skill)
{
  char buf1[MAX_STRING_LENGTH];

  sprintf(buf1, "&+CYour %s skill improves&n", skill_info[Skill].name);
  GET_SKILL(ch, Skill)++;
  send_to_char(buf1, ch);
}				//improve_skill
