/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "spells.h"
#include "handler.h"
#include "db.h"

#ifndef DEBUG_GRANOR
#define DEBUG_GRANOR 1
#endif

extern struct room_data *world;
extern struct obj_data *object_list;
extern struct char_data *character_list;
extern struct cha_app_type cha_app[];
extern struct int_app_type int_app[];
extern struct index_data *obj_index;

extern struct weather_data weather_info;
extern struct descriptor_data *descriptor_list;
extern struct zone_data *zone_table;

extern int mini_mud;

extern struct default_mobile_stats *mob_defaults;
extern char weapon_verbs[];
extern int *max_ac_applys;
extern struct apply_mod_defaults *apmd;
extern struct spell_info_type spell_info[];

void clearMemory(struct char_data *ch);
void act(char *str, int i, struct char_data *c, struct obj_data *o,
	 void *vict_obj, int j);

void damage(struct char_data *ch, struct char_data *victim,
	    int damage, int weapontype, int which, int location);

void weight_change_object(struct obj_data *obj, int weight);
void add_follower(struct char_data *ch, struct char_data *leader);

byte is_enemy(struct char_data *ch, struct char_data *victim)
{
  struct char_data *c_leader = NULL, *v_leader = NULL;
  struct group_type *cfolo = NULL, *vfolo = NULL;

  if (!ch || !victim)
    return FALSE;

  if (GET_STAT(victim) <= STAT_DEAD) {
    return FALSE;
  }
  if (ch == victim)
    return FALSE;

  if ((ch->char_specials.fighting == victim) || (victim->char_specials.fighting == ch))
    return TRUE;


  if (IS_TRUSTED(victim) && (victim->char_specials.fighting != ch))
    return FALSE;

  v_leader = victim;
  c_leader = ch;

  if (v_leader->leader)
    v_leader = v_leader->leader;

  if (c_leader->leader)
    c_leader = c_leader->leader;

  if (((IS_FIGHTING(c_leader) == v_leader) || (IS_FIGHTING(v_leader) ==
		c_leader)) && (IS_GROUP(c_leader) || IS_GROUP(v_leader)))
    return TRUE;

  if (c_leader->group)
    cfolo = c_leader->group->next;

  while (cfolo) {
    if (v_leader->group)
      vfolo = v_leader->group->next;
    while (vfolo) {
#ifdef DEBUG_GRANOR
      sprintf(buf, "Cfolo: %s, Vfolo: %s\r\n",
	      GET_NAME(cfolo->follower), GET_NAME(vfolo->follower));
      log(buf);
#endif
      if (((IS_FIGHTING(cfolo->follower) == vfolo->follower) ||
	   (IS_FIGHTING(vfolo->follower) == cfolo->follower)) &&
	  (IS_GROUP(cfolo->follower) || IS_GROUP(vfolo->follower)))
	return TRUE;
      if (((IS_FIGHTING(c_leader) == vfolo->follower) ||
	   (IS_FIGHTING(vfolo->follower) == c_leader)) &&
	  (IS_GROUP(c_leader) || IS_GROUP(vfolo->follower)))
	return TRUE;
      vfolo = vfolo->next;
    }
    cfolo = cfolo->next;
  }

  if (v_leader && c_leader && (v_leader == c_leader))
    return FALSE;

  return TRUE;
}

byte affectarea(struct char_data * caster, struct char_data * vict,
	   struct location_data * loc2, struct char_data * ch, int spell)
{
  struct location_data loc;
  int direction = -1, rge = 0, shape = -1, who_hits, rnge = 0;
  byte vict_dead = 0;
  int pc_class = IS_MAGE_SPELL(spell) ? CLASS_MAGIC_USER : CLASS_CLERIC;

  if (loc2) {
    vict_dead = 1;
    loc.real_room = loc2->real_room;
    loc.x_cord = loc2->x_cord;
    loc.y_cord = loc2->y_cord;
    loc.z_cord = loc2->z_cord;
    loc.angle = loc2->angle;
  }
  rge = spell_info[spell].area_of_affect + (GET_LEVEL(caster) *
					spell_info[spell].area_by_level);
  shape = spell_info[spell].shape;
  who_hits = spell_info[spell].hits;

  if ((who_hits == HIT_ENEMY) && !is_enemy(caster, ch))
    return FALSE;
  if ((who_hits == HIT_FRIEND) && is_enemy(caster, ch))
    return FALSE;

  if (vict) {
    if (GET_STAT(vict) <= STAT_DEAD) {
      vict_dead = 1;
      loc.real_room = vict->in_room;
      loc.x_cord = GET_X_CORD(vict);
      loc.y_cord = GET_Y_CORD(vict);
      loc.z_cord = GET_Z_CORD(vict);
      loc.angle = GET_FACING(vict);
    }
  }
  if (!vict_dead) {
    switch (shape) {
    case SHAPE_SPHERE:
      if (victWithinRangeofChar(vict, ch, rge)) {
	if (IS_MINOR_GLOBED(ch) &&
	    (GET_SPELL_LEVEL(spell, pc_class) < 4)) {
	  act("$n's spell dissapates before affecting $N.", FALSE, caster,
	      NULL, ch, TO_OTHERS);
	  act("Your spell dissapates before affecting $N.", FALSE, caster,
	      NULL, ch, TO_CHAR);
	  act("$n's spell dissapates before affecting you.", FALSE, caster,
	      NULL, ch, TO_VICT);
	  return FALSE;
	}
	return TRUE;
      }
      break;
    case SHAPE_CONE:
      if (victWithinRangeofChar(vict, ch, rge)
	  && WithinAngle(vict, NULL, ch, NULL, 30, 30)) {
	if (IS_MINOR_GLOBED(ch) &&
	    (GET_SPELL_LEVEL(spell, pc_class) < 4)) {
	  act("$n's spell dissapates before affecting $N.", FALSE, caster,
	      NULL, ch, TO_OTHERS);
	  act("Your spell dissapates before affecting $N.", FALSE, caster,
	      NULL, ch, TO_CHAR);
	  act("$n's spell dissapates before affecting you.", FALSE, caster,
	      NULL, ch, TO_VICT);
	  return FALSE;
	}
	return TRUE;
      }
      break;
    case SHAPE_LINE:
      direction = GET_FACING(caster);
      if (victWithinRangeofChar(vict, ch, rge) &&
	  (angleBetweenCharAndVict(vict, ch) == direction)) {
	if (IS_MINOR_GLOBED(ch) &&
	    (GET_SPELL_LEVEL(spell, pc_class) < 4)) {
	  act("$n's spell dissapates before affecting $N.", FALSE, caster,
	      NULL, ch, TO_OTHERS);
	  act("Your spell dissapates before affecting $N.", FALSE, caster,
	      NULL, ch, TO_CHAR);
	  act("$n's spell dissapates before affecting you.", FALSE, caster,
	      NULL, ch, TO_VICT);
	  return FALSE;
	}
	return TRUE;
      }
      break;
    case SHAPE_SQUARE:
      if (((GET_X_CORD(vict) - GET_X_CORD(ch)) < (rge / 2)) &&
	  ((GET_X_CORD(vict) - GET_X_CORD(ch)) > (-rge / 2)) &&
	  ((GET_Y_CORD(vict) - GET_Y_CORD(ch)) < (rge / 2)) &&
	  ((GET_Y_CORD(vict) - GET_Y_CORD(ch)) > (-rge / 2))) {
	if (IS_MINOR_GLOBED(ch) &&
	    (GET_SPELL_LEVEL(spell, pc_class) < 4)) {
	  act("$n's spell dissapates before affecting $N.", FALSE, caster,
	      NULL, ch, TO_OTHERS);
	  act("Your spell dissapates before affecting $N.", FALSE, caster,
	      NULL, ch, TO_CHAR);
	  act("$n's spell dissapates before affecting you.", FALSE, caster,
	      NULL, ch, TO_VICT);
	  return FALSE;
	}
	return TRUE;
      }
      break;
    default:
      break;
    }
  } else {
    switch (shape) {
    case SHAPE_SPHERE:
      if (WithinRange(ch, &loc, rge, &rnge)) {
	sprintf(buf, "Victim: %s    Range: %d\r\n", GET_NAME(ch), rnge);
	send_to_char(buf, caster);
	return TRUE;
      }
      break;
    case SHAPE_CONE:
      if (WithinRange(ch, &loc, rge, &rnge) &&
	  WithinAngle(NULL, &loc, ch, NULL, 30, 30)) {
	sprintf(buf, "Victim: %s    Range: %d\r\n", GET_NAME(ch), rnge);
	send_to_char(buf, caster);
	return TRUE;
      } else {
	sprintf(buf, "Victim: %s, Angle: %d, Angle of spell: %d \r\nRange: %d, Range_of_spell: %d\r\n",
	     GET_NAME(ch), angle_offset(NULL, &loc, ch, NULL), loc.angle,
		rnge, rge);
	send_to_char(buf, caster);
      }
      break;
    case SHAPE_LINE:
      direction = loc.angle;
      if (WithinRange(ch, &loc, rge, &rnge)) {
	sprintf(buf, "Victim: %s    Range: %d\r\n", GET_NAME(ch), rnge);
	send_to_char(buf, caster);
	return TRUE;
      }
      break;
    case SHAPE_SQUARE:
      if (((loc.x_cord - GET_X_CORD(ch)) < (rge / 2)) &&
	  ((loc.x_cord - GET_X_CORD(ch)) > (-rge / 2)) &&
	  ((loc.y_cord - GET_Y_CORD(ch)) < (rge / 2)) &&
	  ((loc.y_cord - GET_Y_CORD(ch)) > (-rge / 2)))
	return TRUE;
      break;
    default:
      break;
    }
  }
  return FALSE;
}

int spelltype_to_damtype(int spelltype)
{

  switch (spelltype) {
  case SPELL_SPIRIT_HAMMER:
    return TYPE_BLUDGEON;
  case SPELL_DELAYED_BLAST_FIREBALL:
  case SPELL_WALL_OF_FIRE:
  case SPELL_GLYPH_WARDING:
  case SPELL_FLAME_STRIKE:
  case SPELL_PRODUCE_FLAME:
  case SPELL_FIRE_TRAP:
  case SPELL_HEAT_METAL:
  case SPELL_FIRE_STORM:
  case SPELL_SUNRAY:
  case SPELL_AFFECT_NORMAL_FIRES:
  case SPELL_BURNING_HANDS:
  case SPELL_PROD_FIRE:
  case SPELL_WALL_FIRE:
  case SPELL_EXPLOSIVE_RUNES:
  case SPELL_FIREBALL:
  case SPELL_FLAME_ARROW:
  case SPELL_FLAMING_SPHERE:
    return TYPE_FIRE;
  case SPELL_BLADE_BARRIER:
    return TYPE_SLASH;
  case SPELL_EARTHQUAKE:
  case SPELL_MELF_MINUTE_METEORS:
  case SPELL_METEOR_SWARM:
    return TYPE_BLOW;
  case SPELL_CHILL_METAL:
  case SPELL_WALL_OF_ICE:
  case SPELL_CONE_OF_COLD:
  case SPELL_ICE_STORM:
    return TYPE_COLD;
  case SPELL_CHAIN_LIGHTNING:
  case SPELL_CALL_LIGHTNING:
  case SPELL_LIGHTNING_BOLT:
  case SPELL_SHOCKING_GRASP:
    return TYPE_ELECTRICITY;
  case SPELL_WALL_THORNS:
  case SPELL_MAGIC_MISSILE:
    return TYPE_PIERCE;
  case SPELL_WALL_OF_FOG:
  case SPELL_MELF_ACID_ARROW:
    return TYPE_ACID;
  case SPELL_BIGBY_CLENCHED_FIST:
  case SPELL_BIGBY_CRUSHING_HAND:
  case SPELL_BIGBY_GRASPING_HAND:
  case SPELL_BIGBY_FORCEFUL_HAND:
  case SPELL_BIGBY_INTERPOSING_HAND:
  case SPELL_SPECTRAL_HAND:
    return TYPE_PUNCH;
/* spell, special case?  */
  case SPELL_DESTRUCTION:
  case SPELL_DISINTEGRATE:
  case SPELL_SHATTER:
  case SPELL_CRYSTALBRITTLE:
    return TYPE_DESTROY;
  }
  return TYPE_HIT;
}
